/** Wszystko co moze zostac zabite, zniszczone lub zdemolowane.
 */
public class Actor extends TriggerObject
{
    // typy wydawanych halasow
    final static int _NOISE_UNKNOWN = 0;

    //////////////
    // atrybuty //
    //////////////

    // akcje odpalane kiedy aktor ginie
    public Action[] aOnDie = null;//#\ACTOR;Actions preformed when actor dies
    
    /** Poziom zycia Actor'a <br>
     * gdy zmniejsza sie do zera, Actor umiera (wolane jest {@link #Die()})
     */
    public float fHealth = 100.0f;//#ACTOR
    
    /** True jesli niezywy */
    boolean bDead = false;	// SG

    /** maksymalny poziom zycia jaki aktor moze osiaganac po zazyciu drugow
     * not SG
     */
    public float fMaxHealth = 1000.0f; //#ACTOR

    // Identyfikatory metod dla timehandlerow
    int METHOD_UpdateNameSpritePos = 0; // not SG

    /** Czas co jaki bedzie sprawdzane wejscie w hotspoty
     */
    float fCheckHotSpotsInterval = 0.1f; // not SG
    
    /** Klasa hotspota na jaka Actor reaguje.
     */
    String sHotSpotClass = null; // not SG 

    /** Tablica hotspotow o klasie jw.
     */
    CheckHotSpot[] aCheckHotSpots = null; // not SG
	
	/** Jezeli cDamagingType jest = EnumDamageType._ALL, to obiekt moze byc zniszczony przez
	 * uszkodzenie dowolnego typu, jezeli nie, to obiekt
	 * moze byc zniszczony tylko przez typ, ktory jest wyspecyfikowany 
	 */
	public EnumDamageType cDamagingType = new EnumDamageType(EnumDamageType._ALL);//#ACTOR;Hurting damage type
   
    ////////////
    // metody //
    ////////////

	Actor()
	{
		super();
		
        bActiveTrigger = false;
	}
	
    void OnCreate()
    {
		bSG = true;
		
        super.OnCreate();
		
        InitializeMeshNameSprite();
        
        // inicjalizacja systemu hotspotow dla aktora
        InitHotSpotSystem();
    }
    
    /** Odebranie podanej wartosci energii. Energia jest zawsze >=0. Mozna tu pozniej dodac handler
     * OnDie lub jakos tak.
     * @param fDamage
     */
    public void Damage(float fDamage)
    {
        if (fHealth >= 0.0f )
        {
            fHealth-=fDamage;
            if(fHealth <= 0.0f)
            {
                fHealth=0.0f;
                
                // wolamy umieranie aktora
                Die();
            }
        }
    }

	/** zadawanie damage okreslonego typu */
	public void Damage(float fDamage, int nDamageType)
    {
		if (!(GetsDamageIfType(nDamageType)))
			return;
		Damage(fDamage);
    }
	
	/** czy obiekt dostaje damage okreslonego typu */
	boolean GetsDamageIfType(int nDamageType)
	{
		if ( (cDamagingType.GetValue() == EnumDamageType._ALL) ||
			 (nDamageType == cDamagingType.GetValue()) )
			return true;
		
		return false;
	}
    
    /** Wolane gdy Actor umiera ({#link #fHealth} spadlo do zera) */
    public void Die() 
    {
        bDead = true;
        InvokeActionChain(aOnDie);
    }
    
    /** [native] Halasuje na podanym poziomie. Wywoluje handler OnHearNoise() dla obiektow, ktore sa
     * w zasiegu slyszenia obiektu. Zasieg ten zalezy od wlasciwosci eteru znajdujacego sie na levelu i
     * jest ustalany w edytorze.
     * @param fLoudness glosnosc halasu
     * @param nNoiseType typ dzwieku wprowadzony w celu rozrozniania co akurat zabrzeczalo
     */
    public native void MakeNoise(float fLoudness, int nNoiseType);

    /** [native] Halasuje na podanym poziomie. Wywoluje handler OnHearNoise() dla obiektow, ktore sa
     * w zasiegu slyszenia obiektu. Zasieg ten zalezy od wlasciwosci eteru znajdujacego sie na levelu i
     * jest ustalany w edytorze.
     * @param vPosition pozycja, na ktorej odpali sie ten noise
     * @param fLoudness glosnosc halasu
     * @param nNoiseType typ dzwieku wprowadzony w celu rozrozniania co akurat zabrzeczalo
     */
    public native void MakeNoiseOnPosition(Vector vPosition, float fLoudness, int nNoiseType);
    
    /** Handler otrzywany gdy w okolicy obiektu pojawil sie jakis halas.
     * @param Noisy okresla kto spowodowal halas.
     * @param fLooudness poziom halasu uszlyszany przez Aktora.
     * @param nNoiseType rodzaj halasu
     */
    //OnHearNoise(Actor Noisy, float fNoiseLevel, int nNoiseType)

    /** Metoda zadaje innym aktorom blastDamage. Jako inicjator damage'u podawany
     * jest aktor na rzecz ktorego metoda jest wywolana.
     * @param fBlastDamage wartosc odejmowanego damage'a w epicentrum blasta
     * @param fPhSimDamage wartosc obrazen przekazywana do symulacji fizyki po smierci "klienta", im wieksza wartosc, tym bardziej postac "fruwa"
     * @param vPos pozycja z ktorej blast jest odpalany
     * @param fBlastRange zasieg dzialania blastu
     * @param bExcludeActor flaga mowi czy blast ma byc odjety aktorowi dla ktorego jest wolana ta metdoa     
     * @param cExcludeObj obiekt na ktory ma nie dzialac blast
     */
    public void MakeBlastDamage(float fBlastDamage, float fPhSimDamage, Vector vPos, float fBlastRange, boolean bExcludeActor, Actor cExcludeObj)
    {
        Actor cActor = null;
        ControlObject[] aObjects = GetObjectsInRadiusFromClass(fBlastRange, "Actor");
        Vector vSpeed = null;
        for(int i=0; i<aObjects.length; i++)
        {
            cActor = (Actor)aObjects[i]; 
            if( cExcludeObj != cActor )
            {
                // sprawdzamy czy sobie nie odejmowac energii
                if(bExcludeActor) if( cActor==this ) continue;
                
                // umarlaki nie maja co miec zabieranego damagu
                if(cActor.bDead) continue;
                
                float fRange = vPos.Distance(cActor.GetPosition());
                if (fRange < fBlastRange)
                {
                    float fDamage = (fBlastRange - fRange) / fBlastRange;
                    float fPhDamage = fDamage;
                    fDamage *= fBlastDamage;
                    fPhDamage *= fPhSimDamage;
                    // liczymy kierunek predkosci
                    vSpeed = cActor.GetPosition();
                    vSpeed.Sub(vPos);
                    vSpeed.Normalize();
                    vSpeed.Mul(100000.0f);
                    
                    // w odleglosci 30% zasiegu dzialania bulleta i tak zabiera energie
                    if(fRange > fBlastRange*0.3f)
                    {
                        // liczymy trace'a czy czasem cos nie zaslania obiektu
                        Vector vActorMin = new Vector();
                        Vector vActorMax = new Vector();
                        cActor.GetMeshExtents(vActorMin, vActorMax);
                    
                        vActorMin.fX = 0; vActorMin.fZ = 0;
                    
                        // inaczej sprawdzamy czy nie schowal sie ktos za przeszkoda
                        int nSamples = 4;
                        float fStep = (vActorMax.fY - vActorMin.fY) / ((float)nSamples);
                        float fDamageStep = fDamage/((float)nSamples);
                        float fPhDamageStep = fPhDamage/((float)nSamples);
                        for(int j=0; j<nSamples; ++j)
                        {
                            if(!CheckTrace(cActor, new Vector(), vActorMin))
                            {
                                // nie ma trafienia, wiec odbieramy mniej energii
                                fDamage -= fDamageStep;
                                fPhDamage -= fPhDamageStep;
                            }
                            vActorMin.fY += fStep;
                        }
                    }
                    
                    cActor.TakeDamage(fDamage, fPhDamage, this, Vector.Up, vSpeed, EBones._HEAD ); // nie wiem, dlaczego ta wersja, ale niech zostanie...
                }
            }
        }
    }
    
    //////////////////////////////
    // obsluga hotspotow aktora //
    //////////////////////////////
    
    public void InitHotSpotSystem()
    {
        if( sHotSpotClass == null ) return;
        
        // pobieramy wszystkie HotSpoty dla gracza z levelu
        GameObject [] objects = GetObjectsFromClass( sHotSpotClass );
        
        // tworzymy tablice z punktami
        aCheckHotSpots=new CheckHotSpot[objects.length];
        
        // kasujemy wszystkie znalezione hotspoty z levelu
        for(int i=0; i<aCheckHotSpots.length; i++)
        {
            HotSpot cHot = (HotSpot)objects[i];
            aCheckHotSpots[i]=new CheckHotSpot();
            aCheckHotSpots[i].cHotSpot = cHot;
            aCheckHotSpots[i].fLastDistance2 = 1e30f;
            aCheckHotSpots[i].nHotSpotID = cHot.GetUniqueID();
            
            // przeliczamy radius2
            cHot.fRadius2 = cHot.fRadius * cHot.fRadius;
            
            // kasujemy, zeby sie hotspot nie renderowal
            cHot.DeactivateOnModule();
            cHot.EnableRendering(false);
            cHot.SetFlags(EFlags._NO_ANIMATION);
        }
        
        // wlaczamy wolanie sprawdzania hotspotow co jakis czas
        if ( Role == ERole._AUTHORITY )
            CallMethodEvery("CheckHotSpots", fCheckHotSpotsInterval);
    }
    
    /** Metoda usuwa podanego HotSpota z systemu wykrywania wejscia w hotSpot
     */
    public void DeleteHotSpot(HotSpot cHotSpotToDel)
    {
        for (int i=0; i<aCheckHotSpots.length; i++)
        {
            if( aCheckHotSpots[i].cHotSpot == cHotSpotToDel )
            {
                cHotSpotToDel.delete();
                aCheckHotSpots[i].cHotSpot = null;
            }
        }
    }
    
    /** Metoda usuwa HotSpota po nazwie
     */
    public void DeleteHotSpot(String sHotSpotToDel)
    {
        for (int i=0; i<aCheckHotSpots.length; i++)
        {
            HotSpot cHot = aCheckHotSpots[i].cHotSpot;
            if( cHot!=null && cHot.GetName().compareTo(sHotSpotToDel)==0 )
            {
                //CHECK nie powinno byc robione ! aCheckHotSpots[i].cHotSpot.delete();
                aCheckHotSpots[i].cHotSpot = null;
            }
        }
    }
    
    /** Metoda wolana co jakis czas w celu sprawdzenia czy gracz nie wszedl w jakis obszar
     */
    public void CheckHotSpots()
    {
        Vector vPos = GetPosition();
        
        // przgladamy wszystkie hotspoty
        for (int i=0; i<aCheckHotSpots.length; i++)
        {
            // sprawdzamy czy hotspot nie jest wylaczony
            if(aCheckHotSpots[i].cHotSpot == null) continue;
            
            // wyliczamy odleglosc gracza od hotspota
            float fDistance2 = vPos.Distance2(aCheckHotSpots[i].cHotSpot.GetPosition());
            
            //CrashLog("Check <"+i+"> with distance <"+fDistance2+">\n");
            
            if ((aCheckHotSpots[i].fLastDistance2 >  aCheckHotSpots[i].cHotSpot.fRadius2) &&
                (fDistance2                        <= aCheckHotSpots[i].cHotSpot.fRadius2))
            {
                //CrashLog("Enter <"+i+">\n");
                EnteredHotSpot(i);
                CallEnterHotSpot( this, aCheckHotSpots[i].cHotSpot );
            }
            
            // powtorne sprawdzenie, bo ktos mogl w handlerze skasowac hotspota
            if(aCheckHotSpots[i].cHotSpot == null) continue;
            
            if ((aCheckHotSpots[i].fLastDistance2 <= aCheckHotSpots[i].cHotSpot.fRadius2) &&
                (fDistance2                        >  aCheckHotSpots[i].cHotSpot.fRadius2))
            {
                //CrashLog("Leave <"+i+">\n");
                LeftHotSpot(i);
                CallLeaveHotSpot( this, aCheckHotSpots[i].cHotSpot );
            }
            
            aCheckHotSpots[i].fLastDistance2 = fDistance2;
        }
    }

    /** Metoda wolana gdy Actor wchodzi w HotSpot
     * @param nHotSpot numer HotSpota z tablicy aCheckHotSpots
     */
    void EnteredHotSpot(int nHotSpot)
    {
    }
    
    /** Metoda wolana gdy Actor wychodzi z HotSpota
     * @param nHotSpot numer HotSpota z tablicy aCheckHotSpots
     */
    void LeftHotSpot(int nHotSpot)
    {
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
    
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[Actor.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunks._CHK_ACTOR)
        {
            bDead = cFC.LoadBool();
			
			if (bDead == true)
				LoadMeshElements(cFC);
			
			super.SGLoadChunk(cFC);
			
            // odczytujemy energie
            fHealth = cFC.LoadFloat();
            fMaxHealth = cFC.LoadFloat();
			
			// typ zniszczenia, ktory moze wywolywac uszkodzenie obiektu
			cDamagingType = new EnumDamageType(cFC.LoadInt());            
        }
		
		cFC.delete();
    }

    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Actor.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunks._CHK_ACTOR, cFCParent);
		
        cFC.SaveBool( bDead );
		
		if (bDead == true)
			SaveMeshElements(cFC);				
		
		super.SGSaveChunk(cFC);

		// zapisujemy energie
        cFC.SaveFloat( fHealth );
        cFC.SaveFloat( fMaxHealth );

		// typ zniszczenia, ktory moze wywolywac uszkodzenie obiektu
		cFC.SaveInt( cDamagingType.GetValue() );
        		        
        cFC.delete();
    }
	
    //*********************************************************
	//				ONLY FOR DEBUGING PURPOSE
    //*********************************************************

    
    /** Sprite-napis z nazwa mesha 
     */
    Sprite cNameSpr = null;
    
    /** flaga mowiaca czy maja byc wyswietlane nazwy meshy
     */
    boolean bIsNameSprVisible = false;
    
    /** co jaki czas ma byc updatowana pozycja spritea z nazwa mesha
     */
    float fNameSprUpdateTime = 0.3f;
    
    /** odleglosc na jaka trzeba sie zblizyc zeby wyswietlil sie nazwa
     */
    float fNameSprMinDistance = 5000;
    
    /** kat kamery w jakim pokazuja sie napisy (w stopniach)
     */
    float fNameSprMaxAngle = 40;
    
    /** Wartosci przeliczone przy inicjalizacji  (InitializeMeshNameSprite)
     */
    float fNameSprMinDistance2;
    float fNameSprMaxAngleCos;
    
    /** rozmiary sprite'a z nazwa mesha
     */
    int nSprSizeX = 100;
    int nSprSizeY = 50;
    
    /** nazwa mesha 
     */
    String sName = null;
    
    //---------------------------------------------------------------------------------------
    /** metoda inicjalizujaca sprite'a z nazwa mesha
     */
    void InitializeMeshNameSprite()
    {
        if (METHOD_UpdateNameSpritePos == 0)
            METHOD_UpdateNameSpritePos = RegisterCallMethod("UpdateNameSpritePos");
        
        if (bIsNameSprVisible)
        {
            fNameSprMinDistance2 = fNameSprMinDistance * fNameSprMinDistance;
            fNameSprMaxAngleCos = (float)Math.cos(fNameSprMaxAngle * Math.PI / 180.f);
            bIsNameSprVisible = false;
            
            //tworze sprite'a z nazwa mesha
            cNameSpr = (Sprite)CreateObject("Sprite");
		    cNameSpr.SetNoTexture();
		    cNameSpr.ProcessMouse(false);
            //cDistanceSprText.MoveTop();
            cNameSpr.SetSize(new Vector(nSprSizeX, nSprSizeY,0));
            cNameSpr.SetPos(new Vector(200,200,0));
            cNameSpr.SetTextColor( 1.0f,1.0f,1.0f, 1.0f );

            cNameSpr.SetFont(Fonts.FONT_MSSANSSERIF_08);
            cNameSpr.SetTextPosFlags(ESpriteText.MULTILINE);

            //sName = GetName();
		    AddSprite(cNameSpr);
            CrashLog("METHOD_UpdateNameSpritePos activated\n");
                     
            CallMethodEvery(METHOD_UpdateNameSpritePos, fNameSprUpdateTime);
        }
    }
    
    //---------------------------------------------------------------------------------------
    /** metoda kasujaca sprite'a z nazwa mesha
     */
    void DeleteMeshNameSprite()
    {
        if (bIsNameSprVisible)
        {
            RemoveSprite(cNameSpr);
            cNameSpr.delete();
            cNameSpr = null;
            DisableCallMethod(METHOD_UpdateNameSpritePos);
        }           
    }
    //---------------------------------------------------------------------------------------    
    
    /** pomocnicza: zapamietanie czy sprite widoczny
     */
    boolean bNameSprIsShown = false;
    /** pomocnicza: zapamietanie wyswietlanego tekstu - zeby go nie generowac jeszcze raz
     */
    String sNameSprDisplayedTxt = new String();
    /** metoda udpatujaca pozycje spritea z nazwa mesha
     */
    void UpdateNameSpritePos()
    {
        if (IsInCameraNameSprRange())
        {
            Vector vMin = new Vector();
            Vector vMax = new Vector();

            //TODO FIXIT Add camera GetScreenExtents(vMin, vMax);
            cNameSpr.SetPos(new Vector(vMin.fX,vMin.fY - nSprSizeY,0));
            
            String sTxt = GetInfoText();
            if (sTxt.compareTo(sNameSprDisplayedTxt) != 0)
            {
                cNameSpr.DisplayText(sTxt);
                sNameSprDisplayedTxt = sTxt;
            }
                
            if (!bNameSprIsShown)
            {
                cNameSpr.Show();
                bNameSprIsShown = true;
            }
        }
        else if (bNameSprIsShown)
        {
            cNameSpr.Hide();
            bNameSprIsShown = false;
        }
    }
    
    boolean IsInCameraNameSprRange()
    {
        Camera cCamera = GetModule().GetActiveCamera();
        Vector vCameraPos = cCamera.GetPosition(),
               vMeshPos = GetPosition();

        // najpierw sprawdz odleglosc
        if (vCameraPos.Distance2(vMeshPos) < fNameSprMinDistance2)
        {
            Vector vMeshCamVect = vCameraPos;
            vMeshCamVect.Sub(vMeshPos);
            vMeshCamVect.Normalize();
            // a teraz - kat
            if (vMeshCamVect.Dot(cCamera.GetForwardVector()) > fNameSprMaxAngleCos)
                return true;
        }
        return false;
    }
    //---------------------------------------------------------------------------------------    
    /** metoda liczy kat miedzy wekorem forward kamery a wektorem od kamery do mesha
     */
/*    float ComputeAngle()
    {
        Camera cam = GetModule().GetActiveCamera();
        Vector cam_pos = cam.GetPosition();
        Vector mesh_pos = GetPosition();  
        Vector cam_mesh_vect = new Vector(cam_pos.fX - mesh_pos.fX, 0, cam_pos.fZ - mesh_pos.fZ);
        cam_mesh_vect.Normalize();
        Vector cam_forward = cam.GetForwardVector();
        cam_forward.Normalize();
        
        float cos_alfa = cam_mesh_vect.Dot(cam_forward);
        double kat_alfa = java.lang.Math.acos((double)cos_alfa);
        double kat_degree = (kat_alfa*180.0)/java.lang.Math.PI;
        
        return (float)kat_degree;
        
    }
*/
    
    //---------------------------------------------------------------------------------------    
    /** metoda liczy odleglosc kamery od obiektu
     */
/*    float ComputeDistFromCamera()
    {
        Camera cam = GetModule().GetActiveCamera();
        Vector cam_pos = cam.GetPosition();
        Vector mesh_pos = GetPosition();
        
        float fDistance = cam_pos.Distance(mesh_pos);
        
        return fDistance;
        
    }
    */
    
    //---------------------------------------------------------------------------------------   
    /** metoda buduje odpowiedni string (rozne parametry, ktory ma byc wyswietlany nad aktorem)
     */
    String GetInfoText()
    {
        String txt = new String("nazwa_AI : \n"+ this.GetName());
        return txt;
        
    }
    //---------------------------------------------------------------------------------------   

    /** TODO: Comment
     *  INFO: Do pokrycia
     */
    public void TriggerObjectNotification(TriggerObject trigger, ExecutionInfo currentExecutionInfo)
    {
        trigger.ExecutionStepDone(this, currentExecutionInfo);
    }
 
    /**
     * Do nadpisania w PlayerNetBaseInventory
     */
    boolean ServerVerifyActionExecute( TriggerObject cTrigger )
    {
        return true;
    }

}

/** Struktura trzymajaca informacje o hotspotach
 */
class CheckHotSpot
{
    /** Ostatnio wyliczona odleglosc obiektu od hotspota
     */
    float fLastDistance2;
    
    int nHotSpotID;
    
    /** Hotspot
     */
    HotSpot cHotSpot;
}