/** Klasa obslugujaca pojedyncza kamere w grze*/
class Camera extends GameObject
{
    private Vector m_vVolPosition = new Vector();
	    
    /**[native]  Tworzy baze kamery na podstawie wektorow  w przod, 
     * do gory, pozycja
     * @param Forward wektor w przod (<code>Vector</code>)
     * @param Up wektor do gory (<code>Vector</code>)
     * @param Pos pozycja (<code>Vector</code>)*/
    native void FromForwardUpPos(Vector Forward, Vector Up, Vector Pos);

    /**[native] Ustawia pozycje kamery
     * @param Pos nowa pozycja kamery (<code>Vector</code>)
     * */
    native void SetPosition(Vector Pos);
	
	/**[native] Ustawia odleglosc od ktorej zaczyna sie 
	 * renderowanie (liczac od kamery w przod). 
	 * @param Near okresla jak daleko od kamery zaczyna sie obszar renderowany (<code>float</code>)
	 * @see #SetClipFar(float Far)
	 */
	native void SetClipNear(float Near);

	/**[native] Pobiera odleglosc od ktorej zaczyna sie 
	 * renderowanie (liczac od kamery w przod). 
	 */
	native float GetClipNear();
	
	/**[native]  Ustawia zakres widzenia.
	 * @param Far okresla jak daleko od kamery konczy sie obszar renderowany (<code>float</code>)
	 * @see #SetClipNear(float Near)*/
	native void SetClipFar(float Far);

	/**[native] Pobiera zakres widzenia (jak daleko od kamery konczy sie obszar renderowany).
	 */
	native float GetClipFar();
	
    /**[native] Ustawienie Field of View czyli pola widzenia lub inaczej kata widzenia. Czlowiek widzi
     * aktywnie okolo 60 stopni, poza tym katem zauwaza jedynie ruch.
     * Ustawia wartosc LODZoom w kamerze na 1.0f.
     * @param wartosc kata widzenia w stopniach(<code>float</code>)
     */
    native void SetFOV(float fFOV);
    
    /**[native] Pobranie Field of View czyli pola widzenia lub inaczej kata widzenia (w stopniach).
     */
    native float GetFOV();
    
    /**[native] Ustawia wartosc zoomu swiata w kamerze. Ustawiac na wartosc zoomu
     * kamery w trybach snajperki, lornetki, itp.
     * @param wartosc zoomu
     */
    public native void SetFarZoom(float fFarZoom);
    
    /**[native] Ustawia bazowego FOVa. Dla takiego FOVa maja byc dobrane wszystkie
     * parametry wyobru LOD obiektow, detalu itp.
     * @param fBaseZoom bazowy fov
     */
    native void SetBaseFOV(float fBaseZoom);
    
    // pobranie pozycji z macierzy, nie wiem jak zrobic
	/**[native] Pobiera wektor "w prawo" bazy kamery 
	 * @return wektor "w prawo" bazy kamery (<code>Vector</code>)*/
	native Vector GetRightVector();

	/**[native] Pobiera wektor "w gore" bazy kamery
	 * @return wektor "w gore" bazy kamery (<code>Vector</code>)*/
    native Vector GetUpVector();
	
	/**[native] Pobiera wektor "w przod" bazy kamery
	 * @return wektor "w przod" bazy kamery (<code>Vector</code>)*/
    native Vector GetForwardVector();
	
	/**[native] Pobiera pozycje kamery w swiecie
	 * @return pozycja kamery (<code>Vector</code>)*/
    native Vector GetPosition();

    /**[native] Pobiera pozycje kamery w swiecie
	 * @param vPos wektor (<code>Vector</code>) do ktorego zostanie wpisana pozycja kamery */
    native void GetPosition(Vector vPos);
    
    /**[native] Pobiera pozycje kamery w swiecie do lokalnego atrybutu
     * do wykorzystania zaraz po zawolaniu.
	 * @return pozycja kamery (<code>Vector</code>)*/
    public Vector GetPositionVolatile()
    {
        GetPosition(m_vVolPosition);
        return m_vVolPosition;
    }
    
    /**[native] Obrot kamery o fRoll wedlug podanej osi (wektor[x,y,z])
     * @param fRoll obrot o kat w stopniach (<code>float</code>)
     * @param x wspolrzedna x wektora osi obrotu (<code>float</code>)
     * @param y wspolrzedna y wektora osi obrotu (<code>float</code>)
     * @param z wspolrzedna z wektora osi obrotu (<code>float</code>)*/
    native void Rotate(float fRoll, float x, float y, float z);
    
   
    /** Handler wolany w momencie podlaczenia kamery do widoku
     * @param iViewID identifikator widoku
     */
    //public void OnAttachedToView(int iViewID);

    /** Handler wolany w momencie odlaczenia kamery od widoku
     * @param iViewID identifikator widoku
     */
    //public void OnDetachedFromView(int iViewID);

    /**[native] Przeksztalca punkt z ukladu swiata do wspolrzednych ekranowych.
     * Clampuje pozycje punktu do frustuma kamery
     * @param vPointInWorld punkt w swiecie
     * @return punkt na ekranie (x,y,0)
     */
    public native final Vector PointToScreenClampToFrustum(Vector vPointInWorld);

	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[Camera.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunks._CHK_CAMERA)
        {
			super.SGLoadChunk(cFC);
			
			m_vVolPosition = cFC.LoadVector();
					
			/*
			Vector vUp = cFC.LoadVector();
			Vector vForward = cFC.LoadVector();
			Vector vPosition = cFC.LoadVector();	
			
			FromForwardUpPos(vForward,vUp,vPosition);			
			*/
        }
		
		cFC.delete();
	}
	
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Camera.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunks._CHK_CAMERA, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		cFC.SaveVector( m_vVolPosition );
	
		/*
		cFC.SaveVector(GetUpVector());
		cFC.SaveVector(GetForwardVector());
		cFC.SaveVector(GetPosition());								
		*/
		
		cFC.delete();
	}	 
}
