/** Flagi w ControlObject, ktore sluza przelaczaniu roznych opcji dla niego
 */
public class EFlags
{
    final static int _NO_FLAGS             =0x0000;
    
    /** Default=1
     * Wlacza wszelkie kolizje dla obiektu.
     * <br>Moze byc zmieniana w dowolnym momencie.
     */
    final static int _COLLISION            =0x0001;
    
    /** Default=0
     * Animacje sa updejtowane, ale macierze w hierachii mesha nie sa zmieniane.
     * <br>UWAGA zmiana tej flagi daje efekt tylko przed InitObject() lub przy ustawieniu flagi w edytorze.
     */
    final static int _NO_ANIMATION_REFRESH =0x0002;
    
    /** Default=1
     * Wylacza updejt animacji dla obiektu.
     * <br>UWAGA zmiana tej flagi daje efekt tylko przed InitObject() lub przy ustawieniu flagi w edytorze.
     */
    final static int _NO_ANIMATION         =0x0004;
    
    /** Default=1
     * Wlacza kolizje promieni z obiektami.
     * <br>Moze byc zmieniana w dowolnym momencie.
     */
    final static int _BLOCK_TRACE          =0x0008;
	
    /** Default=0
     * Jeeli wlaczona, to przy probie wejscia obiektu na niedozwolone miejsce w pathmapie
     * <br>obiekt sie nie przesuwa.
     */
    final static int _STOPPED_BY_PATHFINDER=0x0010;
	
    /** Default=1
     * Jeeli wlaczona, obiekty poruszajace sie za pomoca elipsoidy (w tym particle)
     * koliduja z tym obiektem, 
     */
    final static int _BLOCK_MOVE           =0x0020;
    
    /**Default=1
     * Steruje symulacja fizyki ciala.
     */
    final static int _NO_BODY_PHYSICS      =0x0040;
    
    /**Default=0
     * Czy obiekt posiada portale
     */
    final static int _PORTAL_OBJECT        =0x0080;

    /**Default=0
     * Czy obiekt rzuca cienie
     */
    final static int _CAST_SHADOWS        =0x0100;

    /**Default=0
     * rysuje obiekt na koncu w czystym z-buforze (ale przed sprite'ami)
     */
    final static int _ALWAYS_ON_TOP       =0x0200;

    /**Default=0
     * obiekt posiada portale outdoorowe
     */
    final static int _OUTDOOR_PORTAL_OBJECT   =0x0400;

    /**Default=0
     * obiekty rysujace sie zawsze, niezaleznie od widocznosci w systemie portali
     */
    final static int _PORTAL_INDEPENDENT      =0x0800;

    /**Default=1
     * Czy na obiekt maja byc rzucane cienie
     */
    final static int _RECEIVE_SHADOWS        =0x1000;

    /* Default=1
     * Gdy flaga ustawiona to obiekt ma kolizje z fx particlami
     * wprowadzone, zeby nie wszystkie obiekty mogly byc obryzgiwane krwia
     * lub w innych blizej nie okreslonych celach
     * domyslnie ta flaga jest ustawiona dla wszystkich obiektow
     */
    final static int _FXPARTICLE_COLLISION   = 0x2000;
    
    /* Default = 0
     * Wlacza liczenie dynamicznej cubemapy do enva
     */
    final static int _DYNAMIC_CUBE_ENVMAP   = 0x4000;
    
    /* HACK: jesli nie ustawiona, to particle nie sa wyrzucane ponad teren jesli
     * sie znajda pod nim. UWAGA! Flaga brana pod uwage przy wolaniu metody
     * PhysicsCreate(), wiec gdy ustawi sie jej pozniej nie przyniesie ona skutku.
     * Default = 0
     */
    final static int _PHYSICS_COL_CORRECTION = 0x8000;

    // obiekt jest podlaczony do obiektu z portalami
    final static int _LINKED_TO_PORTAL_OBJECT = 0x20000;
    
    /* Obiekt dostaje OnPhysicsUpdate niezaleznie od tego, czy fizyka jest aktywna.
    /* (przydatne w przypadku, kiedy uzywamy jakiejs swojej fizyki zaimplementowanej
    /* w javie, ktora wymaga stalego ziarna symulacji)
    */
    final static int _FORCE_ONPHYSICSUPDATE  = 0x80000;

    /** Obiekty maja defaultowo wlaczona ta flage.
     * Gdy jest wylaczona to obiekt zachowuje sie jakby mial flage NO_ANIMATION
     * oraz nie jest wolany dla niego zaden updejt.
     * Flaga sluzy zaznaczeniu, ktore obiekty tylko sie renderuja.
     */
    final static int _NOT_STATIC             = 0x100000;

    /*
     * flaga powoduje ze obiekty sa cieniowane lightmapa terenu (ustawiana w edytorze)
     * flaga ta automatycznie jest zapalana dla obiektow zawierajacych bip'edy
     * UWAGA: flage ta nalezy ustawiac dopiero po zaladowaniu mesha do model objektu. 
     *        LoadMesh moze ustawic te flage po swojemu, tzn.ustawic na true jesli biped,
     *        false jesli nie biped
     */
    final static int USE_TERRAIN_LIGHTMAP_FOR_LIGHTING = 0x10000;
	/*  Defaultowo wlaczona. Okresla, czy obiekt ma byc uwzgledniany przy generowaniu map
	 *  pathfinder'a.
	 */
	final static int _PATHFINDER_VISIBLE	= 0x200000;

    /** Okresla czy obiekty maja sie Cullowac jesli ich wielkosc jest mniejsza
      * od parametru ustalonego w levelu.
      */
    final static int _NO_CULL_BY_SIZE       = 0x400000;

    // obiekty rysujace sie zawsze, niezaleznie od widocznosci w systemie portali
    // flaga ustawiana tylko poprzez kolizje i operacje na hierarchii
    // nie ustawiac z zewnatrz!
    //final static int _INHERITED_PORTAL_INDEPENDENT = 0x800000;

    // obiekt musi miec ustawiana ta flage, zeby kolidowac z particlami
    // defaultowo wlaczona
    final static int _BLOCK_PARTICLES       = 0x01000000;

    // flaga mowiaca czy obiekt jest potencjalnym celem dla AI
    // defaultowo wylaczona
    final static int _AI_TARGET             = 0x02000000;

    // flaga musi byc ustawiona jesli obiekt ma sie odbijac (np. w wodzie)
    // defaultowo wlaczona
    final static int _IS_REFLECTED          = 0x04000000;

    static String GetFlagName(int nFlag)
    {
        String sRet = "__UNKNOWN FLAG____";
        switch (nFlag)
        {
            case _NO_FLAGS:
                sRet = "_NO_FLAGS";
                break;
            case _COLLISION:
                sRet = "_COLLISION";
                break;
            case _NO_ANIMATION_REFRESH:
                sRet = "_NO_ANIMATION_REFRESH";
                break;
            case _NO_ANIMATION:
                sRet = "_NO_ANIMATION";
                break;
            case _BLOCK_TRACE:
                sRet = "_BLOCK_TRACE";
                break;
            case _STOPPED_BY_PATHFINDER:
                sRet = "_STOPPED_BY_PATHFINDER";
                break;
            case _BLOCK_MOVE:
                sRet = "_BLOCK_MOVE";
                break;
            case _NO_BODY_PHYSICS:
                sRet = "_NO_BODY_PHYSICS";
                break;    
            case _PORTAL_OBJECT:
                sRet = "_PORTAL_OBJECT";
                break;
            case _CAST_SHADOWS:
                sRet = "_CAST_SHADOWS";
                break;    
            case _ALWAYS_ON_TOP:
                sRet = "_ALWAYS_ON_TOP";
                break;
            case _OUTDOOR_PORTAL_OBJECT:
                sRet = "_OUTDOOR_PORTAL_OBJECT";
                break;    
            case _PORTAL_INDEPENDENT:
                sRet = "_PORTAL_INDEPENDENT";
                break;
            case _RECEIVE_SHADOWS:
                sRet = "_RECEIVE_SHADOWS";
                break;    
            case _FXPARTICLE_COLLISION:
                sRet = "_FXPARTICLE_COLLISION";
                break;
            case _PHYSICS_COL_CORRECTION:
                sRet = "_PHYSICS_COL_CORRECTION";
                break;
            case _FORCE_ONPHYSICSUPDATE:
                sRet = "_FORCE_ONPHYSICSUPDATE";
                break;
            case _NOT_STATIC:
                sRet = "_NOT_STATIC";
                break;
			case _PATHFINDER_VISIBLE:
				sRet = "_PATHFINDER_VISIBLE";
				break;
            case _NO_CULL_BY_SIZE:
                    sRet = "_NO_CULL_BY_SIZE";
                    break;
            case _BLOCK_PARTICLES:
                    sRet = "_BLOCK_PARTICLES";
                    break;
            case _AI_TARGET:
                    sRet = "_AI_TARGET";
                    break;
            case _IS_REFLECTED:
                    sRet = "_IS_REFLECTED";
                    break;
        }
        return sRet;
    }

}
