/** Klasa bazowa uzywana do definiowania sekwencji efektow.
 *  Kazdy efekt musi byc potomkiem klasy FX
 * */

public class FX extends GameObject
{
	/**[native] Rozpoczyna definicje nowej sekwencji 
	 * @param sFXSequence nazwa sekwencji  (<code>String</code>)     
	 */  
	native final void Sequence(String sFXSequence);
	/**[native] Definiuje teksture uzyta dla efektu
	 * @param sTextureName nazwa tekstury - jest nazwa tekstury, ktora zostala wczesniej zdefiniowana (<code>String</code>)     
	 */ 
	native final void Texture(String sTextureName);
	/**[native] Rozpoczyna definicje nowego efektu eksplozji w ramach sekwencji     
	 */   
	native final void Expl();
	
	/**[native] Rozpoczyna definicje nowego efektu sladu  (definicja globalna - nie ma wplywu na sekwencje)
	 * * @param sTraceName nazwa sladu  (<code>String</code>)     
	 */   
	native final void Trace(String sTraceName);
	/**[native] Rozpoczyna definicje nowego efektu sladu, ktory jest docinany do geometrii  (definicja globalna - nie ma wplywu na sekwencje)
	 * * @param sTraceName nazwa sladu  (<code>String</code>)     
	 */       
	native final void TraceClipped(String sTraceName);        
	
	/**[native] Rozpoczyna definicje nowego efektu sladu, ktory jest rzutowany na geometrie  (definicja globalna - nie ma wplywu na sekwencje)
	 * * @param sTraceName nazwa sladu  (<code>String</code>)     
	 */       
	native final void TraceProjected(String sTraceName);        

	/**[native] Rozpoczyna definicje nowego efektu oslepienia  (definicja globalna - nie ma wplywu na sekwencje)
	 * * @param sBlindName nazwa efektu  (<code>String</code>)     
	 */   
	native final void Blind(String sTraceName);    
	/**[native] Rozpoczyna definicje nowego efektu swiatla    
	 */   
	native final void Light();

    /**[native] Ustawia priorytet rysowania FX-u eksplozji, im mniejszy priorytet tym szybciej rysuje
     * sie FX. Domyslnym priorytetem jest EFXDrawPriority::_DRAW_SEQUENCES.
     * @param iPriority - liczba z zakresu 0..15. @see EFXDrawPriority.
    */
    native final void DrawPriority( int iPriority );
	
	/**[native] Ustawia, badz zeruje, flage symetrycznosci efektu 
	 *@param isSymetricCoords okresla wartosc flagi symetrycznosci efektu (<code>int</code>)     
	 */      
	native final void SymCoords(int isSymetricCoords);
	native final void SymCoordsDyn(String strSymetricCoords);
	/**[native] Ustawia, badz zeruje, flage wyrownania efektu do plaszczyzny kamery
	 *@param isTowardsToCamera okresla wartosc flagi wyrowania efektu (<code>int</code>)     
	 */      
	native final void TowardsToCamera(int isTowardsToCamera);
	native final void TowardsToCameraDyn(String strTowardsToCamera);
	/**[native] Rozpoczyna definicje pozycji efektu. Okresla kontekst wywolania funkcji v
	 */          
	native final void Position();
	
	/**[native] Rozpoczyna definicje przesuniecia efektu wzgledem kamery. Okresla kontekst wywolania funkcji v. Dziala tylko dla efektow majacych ustawiona flage TowardsToCamera
	 */          
	native final void Offset();

	/**[native] Rozpoczyna definicje zakresu pozycji efektu. Okresla kontekst wywolania funkcji v.
	 *@param  minVal najmniejsza wartosc z zakresu (<code>Vector</code>)
	 *@param  maxVal najwieksza wartosc z zakresu (<code>Vector</code>)          
	 */               
	native final void RandomPosition();
	
	native final void RandomAnimPosition();    
	
	/**[native] Rozpoczyna definicje obrotu efektu. Okresla kontekst wywolania funkcji v
	 */              
	native final void Rotation();
	/**[native] Rozpoczyna definicje zakresu obrotu efektu. Okresla kontekst wywolania funkcji v.
	 *@param  minVal najmniejsza wartosc z zakresu (<code>Vector</code>)
	 *@param  maxVal najwieksza wartosc z zakresu (<code>Vector</code>)          
	 */                   
	native final void RandomRotation();
	
	native final void RandomAnimRotation();    
	/**[native] Definiuje losowy wspolczynnik, przez ktory bedzie przemnozony kat obrotu.
	 * @param minFactor najmniejsza dopuszczalna wartosc wspolczynnika (<code>double</code>)
	 * @param maxFactor najwieksza dopuszczalna wartosc wspolczynnika (<code>double</code>)
	 */
	native final void RandomRotationFactor(double minFactor, double maxFactor);
	
	/**[native] Rozpoczyna definicje rozmiaru efektu. Okresla kontekst wywolania funkcji v
	 */              
	native final void Size();
	/**[native] Rozpoczyna definicje zakresu rozmiaru efektu. Okresla kontekst wywolania funkcji v.
	 *@param  minValX najmniejsza wartosc rozmiaru wzdluz osi X z zakresu (<code>Vector</code>)
	 *@param  maxValX najwieksza wartosc rozmiaru wzdluz osi X z zakresu (<code>Vector</code>)
	 *@param  minValY najmniejsza wartosc rozmiaru wzdluz osi Y z zakresu (<code>Vector</code>)
	 *@param  maxValY najwieksza rozmiaru wzdluz osi Y wartosc z zakresu (<code>Vector</code>)                     
	 */               
	native final void RandomSize();
	
	native final void RandomAnimSize();     
	/**[native] Rozpoczyna definicje przezroczystosci efektu. Okresla kontekst wywolania funkcji v
	 */              
	native final void Alpha();
	/**[native] Rozpoczyna definicje zakresu przezroczystosci efektu. 
	 *@param  minVal najmniejsza wartosc z zakresu (<code>Vector</code>)
	 *@param  maxVal najwieksza wartosc z zakresu (<code>Vector</code>)          
	 */                       
	native final void RandomAlpha();
	
	native final void RandomAnimAlpha();
	/**[native] Definiuje wlasciwosc efektu okreslona wczesniejszym wywolaniem metody <code>Alpha()</code>
	 */              
	native final void v(double x);
	native final void vDyn(String x);          
	/**[native] Definiuje wlasciwosc efektu okreslona wczesniejszym wywolaniem metody <code>Size()</code> 
	 */                  
	native final void v(double x, double y);
	native final void vDyn(String x, String y);
	/**[native] Definiuje wlasciwosc efektu okreslona wczesniejszym wywolaniem metod <code>Position()</code> lub <code>Rotation()</code> 
	 */                  
	native final void v(double x, double y, double z);
	native final void vDyn(String x, String y, String z);
	/**[native] Definiuje wlasciwosc efektu okreslona wczesniejszym wywolaniem metod <code>BlindColor()</code> 
	 */                  
	native final void v(double x, double y, double z, double w);
	native final void vDyn(String x, String y, String z, String w);
	/**[native] Ustawia czas opoznienia (w ramach sekwencji) efektow, ktorych definicje nastapia po wywolaniu tej metody. Wielokrotne wywolanie metody powoduje, ze czas opoznienia jest sumowany.
	 *@param delay okresla wartosc opoznienia (w milisekundach)uruchomienia efektow sekwencji (<code>double</code>)     
	 */          
	native final void Delay(double delay);   
	/**[native] Ustawia czas trwania (w milisekundach) efektu w ramach sekwencji
	 *@param duration okresla czas trwania efektu w ramach sekwencji(<code>double</code>)     
	 */             
	native final void Duration(double duration);
	native final void DurationDyn(String strDuration);
	/**[native] Ustawia czas trwania (w milisekundach) sekwencji efektow
	 *@param duration okresla czas trwania sekwencji efektow (<code>double</code>)     
	 */                 
	native final void SeqDuration(double duration);
	native final void SeqDurationDyn(String strDuration);
	/**[native] Ustawia czas (w milisekundach) po jakim wykonanie efektu zostanie powtorzone
	 *@param repeatAfter okresla czas powtarzania efektu (<code>double</code>)     
	 */                     
	native final void Repeat(double repeatAfter);
	native final void RepeatDyn(String strRepeatAfter);
	/**[native] Ustawia ilosc powtorzen animacji tekstury efektu w czasie trwania efektu
	 *@param repeatCount okresla ilosc powtorzen animacji (<code>double</code>)     
	 */
	native final void TextureRepeat(double repeatCount);                         
	/**[native] Ustawia zasieg swiatla
	 *@param range zasieg swiatla (<code>double</code>)     
	 */                             
	native final void LightRange(double range);
	native final void LightRangeDyn(String strRange);
	/**[native] Rozpoczyna definicje koloru swiatla
	 */                             
	native final void LightColor();
	
	/**[native] Rozpoczyna definicje koloru efektu oslepienia. Okresla kontekst wywolania funkcji v
	 */              
	native final void BlindColor();

	/**[native] Okresla czy efekt oslepienia ma byc renderowany na koncu, po wszystkich innych efektach
	 * Blind plane jest wtedy mnozacy
	 */              
	native final void BlindRenderLast(int nLast);

	/**[native] Okresla rodzaj blendowania efektu oslepienia
	 * @param nBlend - 0 dodajacy, 1 miksujacy, 2 mnozacy
	 * Miksujacy jest renderowany w Render levelu, dodajacy i mnozacy w PostRender
	 */              
	native final void BlindBlendType(int nBlend);
	
	/**[native] blokuje mgle dla danego fx'a
	 */
	native final void NoFog();

	/**[native] Definiuje tryb blendowania 
	 */                
	native final void BlendMode(int nBlendMode);
	native final void BlendModeDyn(String strBlendMode);
	
	/** Tryb blendowania przez mieszanie kolorow - domyslny*/
	public static final int _BLEND_MIX  = 0;
	/** Tryb blendowania przez dodawanie kolorow*/
	public static final int _BLEND_ADD  = 1;
	/** Tryb blendowania przez mnozenie kolorow*/   
	public static final int _BLEND_MUL  = 2;
	
	/**[native] Okresla, ze w trakcie wykonywania sekwencji efektow 
	 * zostanie wykonany efekt sladu o podanej nazwie
	 * @param sFXTraceName nazwa efektu  (<code>String</code>)     
	 */                              
	native final void ExecuteFXTrace(String sFXTraceName);
	native final void ExecuteFXTraceDyn(String sFXTraceName);
	/**[native] Okresla, ze w trakcie wykonywania sekwencji efektow 
	 * zostanie wykonana sekwencja o podanej nazwie
	 * @param sFXSeqName nazwa efektu  (<code>String</code>)     
	 */                              
	native final void ExecuteFX(String sFXSeqName);
	native final void ExecuteFXDyn(String sFXSeqName);
	/**[native] Rozpoczyna definicje deszczu
	 * @param sFXRainName nazwa efektu deszcu (<code>String</code>)     
	 */                              
	native final void Rain(String sFXRainName); 
    /**[native] Okresla sposob w jaki rysowane beda krople.
        @param mode - tryb rysowania @see ERainMode;
        ERainMode._DRAW_SQUARE_SPRITES - Prostokaty spasowane z krawedziami ekranu 
    obrocone przodem w strone obserwatora. Np. platki sniegu.
        ERainMode._DRAW_DIRECTED_QUADS - Prostokaty skierowane w kierunku ruchu,
    obrocone przodem do obserwatora. Np. krople deszczu.
    Domyslnym trybem rysowania jest _DRAW_SQUARE_SPRITES
    */
    native final void RainType( int mode );

	/**[native] Definiuje ilosc kropel deszczu.
	 * @param iCapacity ilosc kropli deszczu.
	 */
	native final void Capacity(int iCapacity);
	native final void CapacityDyn(String strCapacity);

	/**[native] Definiuje gestosc deszczu - ile procent czastek ma byc
     * wyswietlanych.
	 * @param density gestosc deszczu w procentach [0..100]
	 */
	native final void Density(double density);
	native final void DensityDyn(String strDensity);

	/**[native] Definiuje kierunek opadow deszczu 
	 * @param dir kierunek opadow deszczu  (<code>Vector</code>)     
	 */                               
	native final void Direction(Vector dir);
	native final void DirectionDyn(String dirX,String dirY,String dirZ);
	/**[native] Definiuje zakres predkosci opadow deszczu 
	 * @param speed predkosc opadow deszczu  (<code>double</code>)     
	 */                               
	native final void RainMinVelocity(double speed);  
	native final void RainMinVelocityDyn(String strSpeed);
	/**[native] Definiuje zakres predkosci opadow deszczu 
	 * @param speed predkosc opadow deszczu  (<code>double</code>)     
	 */                               
	native final void RainMaxVelocity(double speed);  
	native final void RainMaxVelocityDyn(String strSpeed);

	/**[native] Definiuje modyfikacje kierunku opadow deszczu 
	 * @param angle kat najwiekszego dopuszczalnego odchylenia kierunku 
     * opadow deszczu [stopnie]
	 */
	native final void DirVariationAngle(double fAngle);
	native final void DirVariationAngleDyn(String sAngle );
    /**[native] Szybkosc obrotow turbulencji [rot/sec]
     * @param fSpeed szybkosc obracania sie czastki po spirali.
    */
    native final void RainTurbulencySpeed( double fSpeed );
    native final void RainTurbulencySpeedDyn( String sSpeed );
    /**[native] Okresla sredni promien spirali zataczanej przez czastke.
     * @param fAmplitude promien spirali zataczanej przez czastke.[cm]
    */
    native final void RainTurbulencyAmplitude( double fAmplitude );
    native final void RainTurbulencyAmplitudeDyn( String sAmplitude );

	/**[native] Zakres dzialania deszczu.
     * Deszcz pada w szescianie ktorego srodek znajduje sie zawsze(!) 
     * na pozycji aktualnej kamery. Daje zludzenie bezkresnej przestrzeni deszczu.
	 * @param fSize dlugosc boku szescianu.
	 */
	native final void RainCubeSize( double fSize );
	native final void RainCubeSizeDyn( String sSize );

	/**[native] Rozpoczyna definicje systemu partikli
	 * @param sFXName nazwa systemu partikli  (<code>String</code>)     
	 */                               
	native final void ParticleSystem(String sFXName);
	/**[native] Definiuje wielkosc emitera czastek
	 * @param cx rozmiar emitera wzdluz osi X  (<code>double</code>)
	 * @param cy rozmiar emitera wzdluz osi Y  (<code>double</code>)    
	 */                               
	native final void EmiterSize(double cx,double cy);
	native final void EmiterSizeDyn(String cx,String cy);
	/**[native] Definiuje czas zycia emitera czastek
	 * @param lifeTime czas zycia emitera w milisekundach  (<code>double</code>)  
	 */                               
	native final void EmiterLifeTime(double lifeTime);
	native final void EmiterLifeTimeDyn(String strLifeTime);
	/**[native] Definiuje ilosc czastek, ktore wymemituje emiter
	 * @param nCount ilosc czastek  (<code>int</code>)  
	 */                               
	native final void EmiterParticleCount(int nCount);
	native final void EmiterParticleCountDyn(String strCount);
	/**[native] Okresla sposob interpretacji zadanej ilosci czastek. Jezeli emiter nie jest ciagly to jednorazowo zostanie
	 * wygenerowana ilosc czastek okreslona metoda EmiterParticleCount(). Jezeli emiter jest ciagly to przez caly czas swojego zycia bedzie generowal strumien czastek
	 * zawierajacy ilosc czastek okreslona metoda EmiterParticleCount().
	 * @param nCount ilosc czastek  (<code>int</code>)  
	 */                               
	native final void EmiterContinuous(int nContinuous);//czy emiter generuje ciagly strumien czastek
	native final void EmiterContinuousDyn(String strContinuous);
	/**[native] Definiuje czas zycia pojedynczej czastki. Czastka zyje nie dluzej niz emiter.
	 * @param lifeTime czas zycia czastki w milisekundach  (<code>double</code>)  
	 */                               
	native final void ParticleLifeTime(double lifeTime);
	native final void ParticleLifeTimeDyn(String strLifeTime);
	/**[native] Definiuje zakres katow o jaki moga byc odchylane kierunki poczatkowe czastek.Kierunek odchylenia bedzie losowany z zakresu
	 * (-angleX/2,angleX/2),(-angleY/2,angleY/2)
	 * @param angleX kat odchylenia w plaszczyznie XZ  (<code>double</code>)
	 * @param angleY kat odchylenia w plaszczyznie YZ  (<code>double</code>)    
	 */                               
	native final void ParticleDirVariation(double angleX,double angleY);
	native final void ParticleDirVariationDyn(String strAngleX,String strAngleY);
	/**[native] Definiuje mase czastki. 
	 * @param mass masa czastki w kilogramach  (<code>double</code>)  
	 */                               
	native final void ParticleMass(double mass);
	native final void ParticleMassDyn(String strMass);
	/**[native] Definiuje predkosc poczatkowa jaka zostanie nadana kazdej czastce. 
	 * @param velocity predkosc czastki w m/s  (<code>double</code>)  
	 */                               
	native final void ParticleVelocity(double velocity);
	native final void ParticleVelocityDyn(String strVelocity);
	
	/**[native] Definiuje zakres losowej predkosci poczatkowej jaka zostanie nadana kazdej czastce. 
	 * @param velocityMin minimalna wartosc predkosci czastki w m/s  (<code>double</code>)
	 * @param velocityMax minimalna wartosc predkosci czastki w m/s  (<code>double</code>)    
	 */                               
	native final void ParticleRandomVelocity(double velocityMin, double velocityMax);
	
	/**[native] Definiuje sile wypadkowa jaka bedzie dzialac na kazda czastke. 
	 * @param force sila dzialaja na czastke w N (<code>Vector</code>)  
	 */                               
	native final void ParticleForce(Vector force);
	native final void ParticleForceDyn(String forceX,String forceY,String forceZ);
	/**[native] Definiuje typ czastki. Nalezy uzyc _PARTICLE_QUAD, _PARTICLE_MESH lub _PARTICLE_PIECE_MESH
	 * @param nType typ czastki (<code>int</code>)  
	 */                               
	native final void Particle(int nType);
	/**[native] Okresla czy i jaki efekt sladu zostanie uruchomiony w przypadku wykrycia kolizji czastki z innymi obiektami
	 * @param sFXTraceName nazwa efektu sladu (<code>String</code>)  
	 */                               
	native final void ParticleTrace(String sFXTraceName);
	native final void ParticleTraceDyn(String sFXTraceName);
	
	/**[native] Okresla czy efekt sladu zostanie przypiety pod obiekt czy nie
	 * @param nLinkToObject flaga okreslajaca czy przypinac efekt sladu pod obiekt (<code>int</code>)  
	 */                               
	native final void ParticleTraceLinkToObject(int nLinkToObject);
	/** typ czastki - animowany prostokat
	 */
	
	native final void ParticleTail();
	native final void TailRepeat(double repeatAfter);
	native final void TailRepeatDyn(String strRepeatAfter);
	
	native final void ParticleTrailLength(int iLength); 
	native final void ParticleTrailWidth(float fWidth);
	native final void ParticleTrailTexture(String sTexture);
	native final void ParticleTrailBlendMode(int iBlendMode);
	native final void ParticleTrailLengthtenSpeed(float fSpeed); //z zakresu [0, 1]
	native final void ParticleTraceTraceEvery(int iEvery);//0 nie trejsuje, 1 trejsuje kazda 2 - co druga itd.
	native final void ParticleReflectionTraceEvery(int iEvery);
	native final void ParticleReflectionSpeedAttenuation(float fAttenuation); 
	
	public static final int _PARTICLE_QUAD  = 1;
	/** typ czastki - mesh
	 */
	public static final int _PARTICLE_MESH  = 2;
	/** typ czastki - mesh, ktory zostanie podzielony na kawalki - pojedyncza czastka bedzie reprezentowana przez jeden kawalek mesha
	 */
	public static final int _PARTICLE_PIECE_MESH  = 3;
	
	public static final Vector _GRAVITY  = new Vector(0.0f,-9.8f,0.0f);
	
	/** Rozpoczyna definicje efektu mesh. Okresla kontekst wywolan przyszlych funkcji
	 */ 
	native final void Mesh(); 
	/** Definiuje plik zawierajacy dane mesha (3DA)
	 */ 
	native final void MeshFile(String sFXMesh);
	native final void MeshFileDyn(String sFXMesh);
	/** Definiuje na ile kawalkow wzdluz kazdej osi ma zostac podzielony mesh
	 * @param pieces ilosc kawalkow mesha wzdluz kazdej osi (<code>String</code>)  
	 */ 
	native final void MeshPieces(Vector pieces);
	
	/**[native] Rozpoczyna definicje skalowania efektu mesha. Okresla kontekst wywolania funkcji v
	 */               
	native final void Scale();
	
	native final void MeshPart(String sFXMesh);
	
	/** Wylacza test glebokosci dla efektu Expl()
	 * Domyslnie test jest WLACZONY
	 */ 
	native final void DepthTestDisable();

	/**[native] Rozpoczyna definicje efektu flary.
	 * @param sFXLensFlare nazwa efektu sluzaca do jego uruchomienia(<code>String</code>)  
	 */ 
	native final void LensFlare(String sFXLensFlare);
	
	/**[native] blokuje testowania promienia dla zadanej flary (poprawia wydajnosc, 
	 * ale flara nie znika gdy chowa sie za obiekty)
	 */
	native final void FlareNoRaytrace();
	
	/**[native] Definiuje ilosc "soczewek" skladajacych sie na flare.
	 * @param nLensCount ilosc soczewek (<code>int</code>)  
	 * Uwaga : Ilosc soczewek nie moze byc mniejsza od ilosci klatek animacji zdefiniowanych
	 * dla tekstury przypisanej do flary
	 */               
	native final void FlareLensCount(int nLensCount);
	
	/**[native] Okresla czasy znikania i pojawiania sie flary w przypadku jej zasloniecia i odsloniecia.  
	 * @param fadeInTime czas w milisekundach pojawiania sie flary (<code>double</code>)
	 * @param fadeOutTime czas w milisekundach znikania flary (<code>double</code>)
	 */
	native final void FlareFade(double fadeInTime, double fadeOutTime);
	
	/**[native] Okresla zasiegi widzialnosci flary.  
	 * @param fRange zasieg pelnej widocznosci flary (<code>double</code>)
	 * @param fAttenuationRange zasieg okrelsajcy odleglosc na jakiej flara zostanie wygaszona  (<code>double</code>)
	 * Jezeli fAttenuationRange = 0 to flara zostanie wygaszona natychmiast po przekroczeniu zakresu fRange
	 */
	native final void FlareRange(double fRange, double fAttenuationRange); 
	
	/**[native] Okresla wielkosc soczewek flary.  
	 * @param fSize wielkosc pierwszej (najblizszej pozycji flary) soczewki (<code>double</code>)
	 * @param fSizeAdd wielkosc o jaka bedzie zwiekszana wartosc fSize dla kazdej kolejnej soczewki (<code>double</code>)  
	 */
	native final void FlareLensSize(double fSize, double fSizeAdd);
	
	/**[native] Okresla specjalna pierwsza flare
	 * @param sTexture nazwa tekstury pierwszej flary
	 * @param fSize rozmiar pierwszej flary
	 */
	native final void FlareFirstLen(String sTexture, float fSize);
}
