public class FileChunk
{
    /** Identyfikator chunka w po stronie C++.
     *  Nie jest to identyfikator chunka zapisywany na dysk !!!
     */
    public int nCPPID;
	
	/** Obiekt do robienia SaveGame'ow. Pole to jest inicjalizowane automatycznie
     * w kontruktorze SGController.
     */
	static SGController cSG = null;

	final static byte _LOGTYPE_NONE = 0;
	final static byte _LOGTYPE_LOGR = 1;
	final static byte _LOGTYPE_CRASHLOG = 2;
	
	static boolean bSaveActive = true;
	static byte nLogsActive = _LOGTYPE_NONE;
	
	public native void  _SetActive(boolean b);
	
	public void  SetActive(boolean b) { bSaveActive = b; _SetActive(b); };
    
    /** Skasowanie chunka. Gdy chunk jest do zapisu to zapisuje odpowiednia
     * dlugosc pliku na dysku. Gdy chunk jest do odczytu to powoduje przeskoczenie
     * w pliku na koniec chunka, zeby czytac dane za nim.
     */
    public native void delete();
    
    ////////////////////////////////////////
    // zapisywanie danych do Chunka
    ////////////////////////////////////////
    
    /** Zapisuje podanego stringa do pliku. Ograniczenie dlugosci stringa
     * do 2048 wpisane jako stala w enginie.
     * @param s tekst do zapisania
     */
    public native void _SaveString(String s);
	
	public void SaveString(String s)
	{
		if (bSaveActive == false) return;
		SaveBool(s!=null);
		if (s!=null) _SaveString(s);
	}	
    
    /** Zapisuje podanego  bytea na dysk.
     */
    public native void _SaveByte(byte i);
	
	public void SaveByte(byte i)
	{
		if (bSaveActive == false) return;
		_SaveByte(i);
	}
    
    /** Zapisuje podanego inta na dysk.
     */
    public native void _SaveInt(int i);
	
	public void SaveInt(int i)
	{
		if (bSaveActive == false) return;
		_SaveInt(i);
	}
    
    /** Zapisuje podanego longa na dysk.
     */
    public native void _SaveLong(long i);
	
	public void SaveLong(long i)
	{
		if (bSaveActive == false) return;
		_SaveLong(i);
	}
    
    /** Zapisuje podanego boola na dysk.
     */
    public native void _SaveBool(boolean b);
	
	public void SaveBool(boolean b)
	{
		if (bSaveActive == false) return;
		_SaveBool(b);
	}	
    
    /** Zapisuje podanego floata na dysk.
     */
    public native void _SaveFloat(float f);
	
	public void SaveFloat(float f)
	{
		if (bSaveActive == false) return;
		_SaveFloat(f);
	}
    
    /** Zapisuje podanego wektora na dysk.
     */
    native void _SaveVector(Vector v);
	
	public void SaveVector(Vector v)
	{
		if (bSaveActive == false) return;
		SaveBool(v!=null);
		if (v!=null) _SaveVector(v);
	}
    
    /** Blokuje chunka do zapisu. Przy probie zapisu do niego
     * wywali sie asercja i komunikat na crash.log
     */
    public native void LockSave();
    
    ////////////////////////////////////////
    // odczytywanie danych do Chunka
    ////////////////////////////////////////
    
    /** Odczytuje z pliku stringa.
     */
    public native String _LoadString();
	
	public String LoadString()
	{
		return (LoadBool()) ? _LoadString() : null;
	}		
    
    /** Odczytuje z pliku inta.
     */
    public native byte LoadByte();
    
    /** Odczytuje z pliku inta.
     */
    public native int LoadInt();
    
    /** Odczytuje z pliku longa.
     */
    public native long LoadLong();
    
    /** Odczytuje z pliku boola.
     */
    public native boolean LoadBool();
    
    /** Odczytuje z pliku floata.
     */
    public native float LoadFloat();
    
    /** Odczytuje z pliku wektora.
     */
    public native Vector _LoadVector();
	
	public Vector LoadVector()
	{
		return (LoadBool()) ? _LoadVector() : null;
	}	
    
    /** Okresla ile jeszcze danych jest do odczytania w Chunku
     * @return ilosc w bajtach danych do przeczytania
     */
    public native int GetDataToRead();
    
    /** Pobranie identyfikatora chunka
     */
    public native int GetID();
	
    /** Zapisanie referencji do obiektu.
     */				
	public void SaveObjectRef(ISaveGame obj)
	{
		cSG.SaveObjectRef(this,obj);
	}
	
    /** Odczytanie referencji na obiekt.
     */					
	public ISaveGame LoadObjectRef()
	{
		return cSG.LoadObjectRef(this);
	}
	
    /** Zapisanie wektora referencji do obiektow.
     */		
    public void SaveObjectsVectorRef(java.util.Vector vRefs)
	{
		cSG.SaveObjectsVectorRef(this,vRefs);
	}
	
    /** Odczytanie wektora referencji na obiekty.
     */			
	java.util.Vector LoadObjectsVectorRef()
	{
		return cSG.LoadObjectsVectorRef(this);
	}
	
    /** Zapisanie wektora stanu obiektow.
     */		
    public void SaveObjectsVectorFull(java.util.Vector vRefs)
	{
		if (vRefs == null)
		{
			SaveInt(-1);
			FileChunk.Log("[FileChunk.SaveObjectsVector] Vector is null. \n");
			return;
		}
			
		SaveInt(vRefs.size());
		FileChunk.Log("[FileChunk.SaveObjectsVector] Size <"+vRefs.size()+">\n");
		for (int i=0; i < vRefs.size(); i++)
		{
			SaveBool(vRefs.elementAt(i) != null);			
			if (vRefs.elementAt(i) != null)
			{
				SaveString(vRefs.elementAt(i).getClass().getName());											
				SaveBool((vRefs.elementAt(i) instanceof GameObject) && ((GameObject)vRefs.elementAt(i)).IsCreatedFromEngine());
				((ISaveGame)vRefs.elementAt(i)).SGSaveChunk(this);
			}
		}
	}
	
    /** Odczytanie wektora stanu obiektw.
     */			
	public void LoadObjectsVectorFull(java.util.Vector vRefs, GameObject cObj)
	{
		int nSize = LoadInt();
		
		if (nSize == -1)
		{
			FileChunk.Log("[FileChunk.LoadObjectVector] Vector is null.\n");
			vRefs = null;
			return;
		}
		
		vRefs = new java.util.Vector();						
		vRefs.setSize(nSize);
				
		FileChunk.Log("[FileChunk.LoadObjectVector] Size <"+nSize+">\n");
		
		for (int i=0; i < vRefs.size(); i++)
		{
			String sClass = LoadString();
			if (!LoadBool())
				vRefs.setElementAt(GameObject.CreateJavaObject(sClass),i);					
			else
				vRefs.setElementAt(cObj.CreateObject(sClass),i);					
				
			((ISaveGame)vRefs.elementAt(i)).SGLoadChunk(this);		
		}
	}			
	
    /** Zapisanie wektora stanu obiektow.
     */		
    public void SaveObjectsVector(java.util.Vector vRefs)
	{
		if (vRefs == null)
		{
			SaveInt(-1);
			FileChunk.Log("[FileChunk.SaveObjectsVector] Vector is null. \n");
			return;
		}
			
		SaveInt(vRefs.size());
		FileChunk.Log("[FileChunk.SaveObjectsVector] Size <"+vRefs.size()+">\n");
		for (int i=0; i < vRefs.size(); i++)
			((ISaveGame)vRefs.elementAt(i)).SGSaveChunk(this);
	}
	
    /** Odczytanie wektora stanu obiektw.
     */			
	public void LoadObjectsVector(java.util.Vector vRefs)
	{
		int nSize = LoadInt();
		
		if (nSize == -1)
		{
			FileChunk.Log("[FileChunk.LoadObjectVector] Vector is null.\n");
			vRefs = null;
			return;
		}
		
		if (vRefs == null)
			GameObject.CrashLog("[FileChunk.LoadObjectsVector] ERROR! : Vector is null !\n");
		
		if (nSize != vRefs.size())
			GameObject.CrashLog("[FileChunk.LoadObjectsVector] ERROR! : Size is not the same like this during saving !");
		
		FileChunk.Log("[FileChunk.LoadObjectVector] Size <"+nSize+">\n");
		
		for (int i=0; i < vRefs.size(); i++)
			((ISaveGame)vRefs.elementAt(i)).SGLoadChunk(this);		
	}	
	
    /** Zapisanie tablicy obiektow.
     */		
    public void SaveObjectsTable(ISaveGame[] objs)
	{
        SaveInt((objs==null) ? -1 : objs.length);
        if(objs==null) return;
        
		for (int i=0; i < objs.length; i++)
			if (objs[i] != null)
			{
				SaveInt(1);
				objs[i].SGSaveChunk(this);
			}
			else
			{
				SaveInt(-1);
			}
	}
	
	
	public ISaveGame LoadObject(ISaveGame obj, GameObject creator)
	{
		if (LoadBool())
		{
			boolean bGameObject = LoadBool();
            
            boolean bCreatedFromEngine = false;
            if (bGameObject)
                bCreatedFromEngine = LoadBool();

			String sClassName =  LoadString();
			
			if (obj == null)  
			{
				if (bGameObject)
				{
					if (creator == null)
						GameObject.CrashLog("[FileChunk.LoadObject] Creator is null. I can not create object <"+sClassName+">\n");
					else
                    {
                        if (bCreatedFromEngine)
                            obj = creator.CreateObject(sClassName);					
                        else 
                            obj = (ISaveGame)GameObject.CreateJavaObject(sClassName);
                    }
				}
				else
					obj = (ISaveGame)GameObject.CreateJavaObject(sClassName);				
			}
				
			if (obj.getClass().getName().compareTo(sClassName)!=0)
			{
				GameObject.CrashLog("[FileChunk.LoadObject] ERROR! Expected class <"+sClassName+">\n");
				Thread.dumpStack();					
			}
			if (bGameObject && (obj!=null))
				((GameObject)obj).bSGLoaded = true;
			obj.SGLoadChunk(this);
			return obj;
		}
		return null;
	}	
	
	public void SaveObject(ISaveGame obj)
	{
		SaveBool(obj != null);
		if (obj != null)
		{
			SaveBool(obj instanceof GameObject);
            
            if (obj instanceof GameObject)
                SaveBool(((GameObject)obj).IsCreatedFromEngine());
         
			SaveString(obj.getClass().getName());
			obj.SGSaveChunk(this);
		}
	}
	
    /** Odczytanie tablicy obiektw.
     */			
	void LoadObjectsTable(ISaveGame[] objs)
	{
		int nSize = LoadInt();
		
        if(nSize == -1)
        {
            // byl null
            if(objs != null)
            {
                GameObject.CrashLog("[FileChunk.LoadObjectsTable] ERROR! Table should be null but has some elements!");
            }
            return;
        }
        
		if (objs.length != nSize)
			GameObject.CrashLog("[FileChunk.LoadObjectsTable] ERROR! Incorrect size of table");
		
		for (int i=0; i < objs.length; i++)
			if (LoadInt() == 1)
			{
				objs[i].SGLoadChunk(this);
			}
			else
			{
				if (objs[i] != null)
					GameObject.CrashLog("[FileChunk.LoadObjectsTable] ERROR! Expected null");
			}	
	}
	
		
	public static void SetActiveLog(byte nLogType)
	{
		nLogsActive = nLogType;
	}
	
	public static void Log(String s)
	{
		switch (nLogsActive)
		{
		case _LOGTYPE_LOGR:
			GameObject.LogR(s);
			break;
		case _LOGTYPE_CRASHLOG:
			GameObject.CrashLog(s);
			break;
		}
	}
}
