import java.lang.reflect.*;

/**
 * Podstawowa klasa w grze, z niej dziedziczy wiekszosc pozostalych klas.
 * Zawiera: <br>
 *	- mechanizm obslugi stanow <br>
 *	- obsluge handlerow <br>
 *  - obsluge sieci i replikacje <br>
 *  - podstawowe metody engine'u <br>
 * Kazdy obiekt javy powiazany z C++ musi dziedziczyc po GameObject.
 * 
 */

import java.util.*;

public class GameObject implements Comparable, ISaveGame, IJDC_AttrValue
{
    /** Komentarz do klasy pokazywany w edytorze */
    String sEditorComment = "";
    
   /**Aktualny stan obiektu*/
   State CurrentState = null;

   /** ID obiektu po stronie C++ (wykorzystywane przez metody native w C++)*/
   private int ThisID = 0;

   /** Flaga ustawiana w obiekcie, gdy jest kasowany po stronie enginu po
     * zawolaniu delete()
     */
   public boolean bDeletedInEngine = false;
    
   /** Rola obiektu w sieci, mozliwe wartosci w klasie ERole. Jest defaultowo AUTHORITY, zeby
    * w single player wszystko bylo symulowane
    * TODO COMMENTS */
   public int Role=ERole._AUTHORITY;
   
   /** Flaga do sieci okreslajaca czy obiekt zostal juz wyspawnowany
    * po "drugiej" stronie
    * 
    * TODO COMMENTS */
   //public boolean bNetSpawned;
   
   /** Flaga do sieci okreslajaca czy obiekt aktualnie jest wstepnie replikowany
    * 
    * TODO COMMENTS */
   public boolean bNetInit;

   /** Flaga do sieci okreslajaca czy obiekt jest wlasnoscia playera
    * 
    * TODO COMMENTS */
   public boolean bNetIsOwner = true;
   
   /** Flaga do sieci okreslajaca czy obiekt jest istotny na kliencie
    * 
    * TODO COMMENTS */
   public float fNetRelevantLevel = 1.0f;
   float GetNetRelevant(){ return fNetRelevantLevel; }
   
   float fNetForceRelevantLevel = -1.0f;
   void  SetForceRelevantLevel( float fLevel ){ fNetForceRelevantLevel = fLevel; }
   
   /** Flaga do sieci okreslajaca czy obiekt jest playerem
    * 
    * TODO COMMENTS */
   //public boolean bNetIsPlayer;

   /**Ustawia {@link #ThisID} na <code>ID</code>
    * @param ID identyfikator obiektu (<code>int</code>)*/
   void SetThisID(int ID) { ThisID = ID; }

   int GetThisID() { return ThisID; }
   
   /** Sprawdzenie, czy obiekt zostal prawidlowo stworzony w engine
    */
   public boolean IsCreatedFromEngine() { return ThisID != 0; }
   
   /** [native] Powiadamiania CScriptManagera o zmianie stanu*/
   native void StateChanged(State s);

   /** [native] Powoduje ignorowanie handlera (nie jest w ogole wywolywany),
    * ignorowanie dziala do nastepnej zmiany stanu
    * @param sName nazwa handlera (<code>String</code>)
    * */
   native void IgnoreHandler(String sName);

   /** [native] Wypisuje tekst na OutputDebugString, ale tylko w wersji debug enginu
    * @param Text tekst na OutputDebugString ('\n' nie jest dodawane) (<code>String</code>)
    */
   native static void Log(String Text);

   /** [native] Wypisuje tekst na OutputDebugStringa nie zaleznie od wersji enginu
    * @param Text tekst na OutputDebugString ('\n' nie jest dodawane) (<code>String</code>)
    */
   native static void LogR(String Text);
   
   /** [native] Wypisuje tekst na OutputDebugString. Z przodu dopisywane jest miejsce w kodzie
    * z ktorego zostal ten tekst zawolany w formacie [Class.Method]
    * @param Text tekst na OutputDebugString ('\n' nie jest dodawane) (<code>String</code>)
    */
   native static void LogD(String Text);
   
   /** [native] Wypisuje tekst na OutputDebugString. Z przodu dopisywane jest miejsce w kodzie
    * z ktorego zostal ten tekst zawolany w formacie [Class.Method::Line]
    * @param Text tekst na OutputDebugString ('\n' nie jest dodawane) (<code>String</code>)
    */
   native static void LogDL(String Text);
   
   /** [native] Wypisuje tekst na OutputDebugString i dodaje na koncu "\n"
    * @param Text tekst na OutputDebugString ('\n' jest dodawane) (<code>String</code>)
    */
   native static void LogLine(String Text);

   /** [native] Wypisuje tekst na OutputDebugString i do pliku Crash.log - sytuacje wyjatkowe
    * @param Text tekst do zapisania w pliku Crash.log ('\n' nie jest dodawane) (<code>String</code>)
    */
   native static void CrashLog(String Text);

   /** [native] Wypisuje podany tekst do pliku o podanej nazwie
    * @param FileName nazwa pliku (<code>String</code>)
    * @param Text tekst do zapisania w pliku (<code>String</code>)
    */
   native static void LogFile(String FileName, String Text);

   static void printStackTrace(String Text)
   {
       LogR(Text+"\n");
       Thread.dumpStack();
   }

   /**[native] Zwraca identyfikator wersji enginu
    */
   public native static String GetEngineVersion();
   
   /** By Czesiu - zapytaj zanim zaczniesz bluzgac, Fajko!
    */
   static public native boolean IsLeaning_();
   static public native boolean IsLeaning2_();
   static public native boolean IsLeaning3_();
   
   /* [native] Zwraca ilosc zaallokowanej w C++ pamieci.
    * Projekt musi byc skompilowany z odblokowanym memdumpem!
    */
   public native int GetUsedMemSize();
   
   /** [native] Zrzuca informacje o zaalokowanej pamieci do pliku o zadanej w parametrze nazwie 
    * @param Filename nazwa pliku (<code>String</code>)
    */
   native static void DumpMemory(String Filename/*, boolean bAll*/);

   /** [native] Zrzuca informacje o teksturach w pamieci do pliku o zadanej w parametrze nazwie 
    * @param Filename nazwa pliku (<code>String</code>)
    */
   native static void DumpTextures(String Filename);

   /** [native] Zrzuca informacje o meshach w pamieci do pliku o zadanej w parametrze nazwie 
    * @param Filename nazwa pliku (<code>String</code>)
    */
   native static void DumpMeshes(String Filename);
   
   /** [native] Zwraca true, jezeli podany plik isnieniej w systemi plikow.
    * Czyli na dsyku w podanym katalogu, lub w zipie, czy gdzie tam mialby byc
    */
   native static boolean DoesFileExist(String FileName);
   
   /** [native] Kasuje obiekt odpowiadajacy temu GameObject po stronie C++
    * UWAGA! Obiekt podlaczony do hierarchii nie odlacza sie co powoduje crash!!!!
    * Trzeba najpierw takiemu obiektowi zawolac DetachFromParent().
    * */
   native void delete();

   /** [native] Pobiera nazwe obiektu*/
   native String GetName();
   
   /**[native] Ustawia nazwe obiektu
    * @param name nowa nazwa obiektu (<code>String</code>)
    */
   native void SetName(String name);

   
   /** Ustawia stan wg nazwy, korzysta z nowej szybkiej metody w CPP
    * @param sStatName nazwa nowego stanu (<code>String</code>)
    */
    public native void SetStateNew(String sStateName);

    boolean bNewSetState = true;
    public void SetState(String sStateName)
    {
		//FileChunk.Log("[!SetState!] ClassName = <"+this.getClass().getName()+"> Name = <"+this.GetName()+"> State = <"+sStateName+"> \n");
        if(bNewSetState)
            SetStateNew(sStateName);
        else
            SetStateOld(sStateName);
    }
    
    
    /** Metoda pomocnicza do nowej metody ustawiania stanu/
     * NIE WOLAC BEZPOSREDNIO Z JAVY!!!!!!
     */
    public boolean CreateStateObject(Class InnerClass, Class OuterClass)
    {
        try
        {
            Class[] paramTypes =  new Class[]{ OuterClass };
            Constructor innerConstr = InnerClass.getConstructor( paramTypes );
            Object[] initArgs = new Object[]{ this };
            CurrentState = (State) innerConstr.newInstance( initArgs );
                
            StateChanged(CurrentState);
            CurrentState.OnEnter();
            
        } catch (Throwable eee) { 
            return false;
        }
        
        return true;
    }
   
    static int iSetStateProfile = 0;
    
   /** Ustawia stan wg nazwy
    * @param sStatName nazwa nowego stanu (<code>String</code>)
    */
    public void SetStateOld(String sStateName)
    {
        if(iSetStateProfile == 0)
        {
            iSetStateProfile = ProfileAdd("SetState", "Old JAVA");
        }
        
        ProfileBegin(iSetStateProfile);
        
        boolean bLogs = false;
        
        Class InnerClass = null;
        Object Outer = CurrentState;

        Class[] paramTypes;
              
        try
        {
            if(bLogs) { LogR("Szukamy stanu w stanie <"); LogR(sStateName); LogR(">\n"); }
            // szukamy stanu w stanie
            if(CurrentState == null) throw new Throwable();
            InnerClass = Class.forName(CurrentState.getClass().getName() + "$" + sStateName);
            paramTypes =  new Class[]{ this.getClass() };
        }
        catch (Throwable e)
        {
            // szukamy pokolei w klasach idac od obecnej w gore hierarchii do bazowej
            
            boolean bFoundState=false;
            
            Class someClass=this.getClass();
            paramTypes =  new Class[]{ someClass };
            
            while(!bFoundState && someClass!=null)
            {
                try
                {
                    if(bLogs) { LogR("Szukamy stanu <"); LogR(sStateName); LogR("> w klasie <"); LogR(someClass.getName()); LogR(">\n"); }
                    
                    InnerClass = Class.forName(someClass.getName() + "$" + sStateName);
                    paramTypes =  new Class[]{ someClass };
                    bFoundState=true;
                }
                catch(Throwable ee)
                {
                    // zmieniamy klase na bazowa
                    someClass=someClass.getSuperclass();
                }
            }
            
            if(someClass!=null)
            {
                Outer = this;
                try
                {
                    Object[] initArgs = new Object[]{ Outer };
                    Constructor innerConstr = InnerClass.getConstructor( paramTypes );
                    CurrentState = (State) innerConstr.newInstance( initArgs );
                    StateChanged(CurrentState);
                    CurrentState.OnEnter();
                }
                catch (Throwable eee) 
                { 
                    LogR("Blad przy przelaczaniu do stanu <"); LogR(sStateName); LogR(">\n"); 
                    eee.printStackTrace();
                }
            }
            else
            {
                LogR("No such state <"+sStateName+"> in class tree <"+getClass().getName()+">\n");
            }           
        }
        
        ProfileEnd(iSetStateProfile);
    }

   /** NIE WOLNO WOLAC TEGO Z JAVY !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    * Ustawia stan wg nazwy, ale nie wola OnEnter. Jest to skopiowana metoda SetState ze skasowana
    * linijka CurrentState.OnEnter().
    * @param sStatName nazwa nowego stanu (<code>String</code>)
    */
    public void SetStateNoEnter(String sStateName)
    {
        Class InnerClass = null;
        Object Outer = CurrentState;

        Class[] paramTypes;
              
        try
        {
            //LogR("Szukamy stanu w stanie <"+sStateName+">\n");
            // szukamy stanu w stanie
            InnerClass = Class.forName(CurrentState.getClass().getName() + "$" + sStateName);
            paramTypes =  new Class[]{ this.getClass() };
        }
        catch (Throwable e)
        {
            // szukamy pokolei w klasach idac od obecnej w gore hierarchii do bazowej
            
            boolean bFoundState=false;
            
            Class someClass=this.getClass();
            paramTypes = new Class[]{ someClass };
            
            while(!bFoundState && someClass!=null)
            {
                try
                {
                    //LogR("Szukamy stanu <"+sStateName+"> w klasie <"+someClass.getName()+">\n");
                    
                    InnerClass = Class.forName(someClass.getName() + "$" + sStateName);
                    paramTypes =  new Class[]{ someClass };
                    bFoundState=true;
                }
                catch(Throwable ee)
                {
                    // zmieniamy klase na bazowa
                    someClass=someClass.getSuperclass();
                }
            }
            
            if(someClass!=null)
            {
                Outer = this;
                try
                {
                    Object[] initArgs = new Object[]{ Outer };
                    Constructor innerConstr = InnerClass.getConstructor( paramTypes );
                    CurrentState = (State) innerConstr.newInstance( initArgs );
                    StateChanged(CurrentState);
                }
                catch (Throwable eee) 
                { 
                    //LogR("Blad przy przelaczaniu do stanu <"+sStateName+">\n"); 
                    e.printStackTrace();
                }
            }
            else
            {
                LogR("No such state <> in class tree <>");
            }           
        }
    }
    
    /** [native] Metoda wywoywana przy rejestracji pl obiektu, nigdy nie wolana bezposrednio, wykorzystuje ja {@link #RegisterFields()}
     * @param sName nazwa pola (<code>String</code>)
     * @param sType nazwa typu pola
     * @param sSignature okresla sygnature pola, gdy pole jest standardowe == null
     * @param bAbstract okresla czy typ pola jest abstrakcyjny
     * @param bArray okresla czy pole jest tablica
     * @param bEnum okresla czy pole jest enumem
     * @param bStatic okresla czy pole jest statyczne
     */
    static native void RegisterField(String sName, String sType, String sSignature, boolean bAbstract, boolean bArray, boolean bEnum, boolean bStatic);
   
    /** Rejestruje wszystkie pola publiczne podanej klasy, jest wolana przez engine*/
    static public void RegisterFields(Class c)
    {
        Field[] fields = c.getFields();

        //if(c.getName().equals("EnumSides"))
        //    CrashLog("Class '"+c.getName()+"'\n");

        for (int i = 0; i < fields.length; ++i)
        {
            Class FieldType = fields[i].getType();
            String sSignature = null;
            boolean bAbstract = false;
            boolean bEnum = Enum.class.isAssignableFrom(FieldType);
            boolean bArray = FieldType.isArray();
            boolean bStatic = Modifier.isStatic(fields[i].getModifiers());
            boolean bInterface = FieldType.isInterface();

            if(!bArray)
            {
                // dla typow zlozonych podajemy
                if(!FieldType.isPrimitive() &&
                   !FieldType.getName().equals("java.lang.String") &&
                   !FieldType.getName().equals("Vector") &&
                   !FieldType.getName().equals("GameObject") &&
                   !FieldType.getName().equals("java.lang.Object") &&
                   !FieldType.getName().equals("ObjectRefList")
                   || bEnum
                   )
                {
                    sSignature = "L"+ FieldType.getName() + ";";
                    //CrashLog("sSignature='"+sSignature+"'\n");
                }
                bAbstract = Modifier.isAbstract(FieldType.getModifiers());
            }
            else
            {
                // sprawdzamy typ elementu
                Class ArrayElemType = FieldType.getComponentType();
                
                if(!ArrayElemType.isPrimitive())
                {
                    sSignature = "[L"+ ArrayElemType.getName() + ";";
                    //CrashLog("Klasa <"+c.getName()+"> atrybut <"+fields[i].getName()+"> typ tablicowy element <"+ArrayElemType.getName()+">\n");
                }
                
                bAbstract = Modifier.isAbstract(ArrayElemType.getModifiers()) && 
							!Modifier.isInterface(ArrayElemType.getModifiers());
            }

            // interfejsy sa traktowane jak normalne klasy
            if(bInterface)
            {
                //CrashLog("Class '"+c.getName()+"' Field '"+fields[i].getName()+"' bAbstract="+bAbstract+"\n");
                bAbstract = false;
            }
            
            //if(c.getName().equals("EnumSides"))
            //   CrashLog("Class '"+c.getName()+"' Field '"+fields[i].getName()+"' bAbstract="+bAbstract+"\n");
            RegisterField(fields[i].getName(), FieldType.toString(), sSignature, bAbstract, bArray, bEnum, bStatic);
        }
        
    }
    
    /** Returns field of specified name for specified name.
     * If field not found returns null
     */
    Field JDC_GetDeclaredField(Class cFieldClass, String sFieldName)
    {
        if(cFieldClass == null) return null;
        if(sFieldName == null) return null;
        
        Field[] fields = cFieldClass.getDeclaredFields();
        for (int i = 0; i < fields.length; ++i)
            if(fields[i].getName().equals(sFieldName))
                return fields[i];
        
        return JDC_GetDeclaredField(cFieldClass.getSuperclass(), sFieldName);
    }
    
    /** Called by native engine code when specified field should be
     * shown in Java Debug Console.
     */
    public String JDC_GetFieldValue(String sFieldName)
    {
        Field cField = JDC_GetDeclaredField(getClass(), sFieldName);
        if(cField == null) return "No such field";
        
        Class cType = cField.getType();
        if( cType.isPrimitive() )
        {
            //TODO it should never happen
            return "PRIVMITVE TYPE, VALUE UNKNOWN";
        }
        else
        {
            Object cValue = null;
            try {
                cValue = cField.get(this);
            } catch(IllegalAccessException e) {}
            
            if(cValue instanceof IJDC_AttrValue)
            {
                IJDC_AttrValue cAV = (IJDC_AttrValue)cValue;
                return cAV.JDC_GetValue();
            }
            else
            {
                // type with no special interface
                return ((cValue==null) ? "null" : cValue.toString());
            }
        }
    }
    
    /** Method called when object is shown in java debug console as attribute
     */
    public String JDC_GetValue()
    {
        return toString()+" "+"<"+GetName()+":"+GetUniqueID()+":"+GetNetId()+">";
    }
    
	/////////////////////////
    // wywolanie handlerow //
    /////////////////////////
	
	/** [native] Wywoluje podana w stringu metode z danymi parametrami
	 * @param funcname nazwa wolanej metody (<code>String</code>)
	 * @param Params parametry wolanej metody (<code>{@link HandlerParams}</code>)
	 * */
    native final void CallHandler(String funcname, HandlerParams Params);

    /**[native] Pobranie identyfikatora handlera. Gdy taki handler jeszcze nie istnieje
     * to jest rejestrowany.
     * @return identyfikator handlera do czestego wolania handlera
     */
    native final int GetHandlerID(String funcname);
    
    /** [native] Wywoluje podany w stringu handler bez parametrow
	 * @param funcname nazwa wolanej metody (<code>String</code>)
	 * */
    native final void CallHandler(String funcname);
    
    /** [native] Wywoluje handler o podanym identyfikatorze bez parametrow
	 * @param funcname identyfikator pobrany meotda {@link #GetHandlerID(String funcname)}
	 * */
    native final void CallHandler(int funcname);
    
    /**[native] Wywoluje handler OnEnterHotSpot dla podanego HotSpota.
     * @param Adventurer aktor, ktory wkracza w hotspot
     * @param Point punkt w ktory wchodzi aktor
     */
    native final void CallEnterHotSpot(Actor Adventurer, HotSpot Point);
    
    /**[native] Wywoluje handler OnLeaveHotSpot dla podanego HotSpota.
     * @param Adventurer aktor, ktory wychodzi z punktu
     * @param Point punkt z ktorego wychodzi aktor
     */
    native final void CallLeaveHotSpot(Actor Adventurer, HotSpot Point);
    
    /////////////////////////////////////////////////////////////
    // metody/handlery standardowe
    // musza byc tutaj wszystkie, nawet jesli nic nie robia!
    /////////////////////////////////////////////////////////////
    
    /** Handler wolany przed ustworzeniem obiektu podczas ladowania levelu.
     * Sluzy okresleniu czy obiekt ma byc utworzony.
     * @param nID identyfikator obiektu utworzonego.
     * @return gdy zwroci false to obiekt nie bedzie utworzony.
     */
    public boolean PreCreate(int nID)
    {
        return true;
    }
    
    void OnCreate()
    {
    }
    
    void OnCreate(GameObject creator)
    {
    }
    
    void OnPostCreate()
    {
    }
    
    void OnDestroy()
    {
    }

    void SetDefaultAttributes()
    {
    }
    
    void OnLoadChunk(int chunk, int datasize, MapInputStream stream)
    {
    }
    
	/////////////////////////////
    // Informacje o ekranie	   //
    /////////////////////////////
	/**[native] Pobiera szerokosc okna gry
	 * @return szerokosc okna gry w pikselach
	 */
    native static int GetScreenWidth();

	/**[native] Pobiera wysokosc okna gry
	 * @return wysokosc okna gry w pikselach
	 */
	native static int GetScreenHeight();

	/////////////////////////////
    // wywolania czasowe metod //
    /////////////////////////////
	
    /**[native] Rejestracja metody w obiekcie do wolania w handlerze czasowym.
     * UWAGA Identyfikator jest przydzielany dla levelu. Proba wywolania metody
     * z pobranym identyfikatorem dla innego levelu zakonczy sie wywaleniem komunikatu
     * bledu(prawdopodobnie na CrashLog).
     * @param funcname nazwa metody ktora chcemy wywolac (<code>String</code>)
     * @return identyfikator uzywany w CallMethodIn oraz CallMethodEvery
     */
    native final int RegisterCallMethod(String funcname);
    
	/** [native] Wywoluje podana metode na koncu ramki (po symulacji wszystkich leveli)
	 * @param funcname nazwa metody ktora chcemy wywolac (<code>String</code>)
	 */
    native public final void CallOnFrameEnd(String funcname);
    
	/** [native] Wywoluje podana metode na koncu ramki (po symulacji wszystkich leveli)
	 * @param nFuncID identyfikator wczesniej zarejestrowanej funkcji (<code>int</code>)
	 */
    native public final void CallOnFrameEnd(int nFuncID);

    /** [native] Wywoluje podana metode po okreslonym czasie
	 * @param funcname nazwa metody ktora chcemy wywolac (<code>String</code>)
	 * @param seconds czas opoznienia w sekundach (<code>float</code>)
	 */
    native final void CallMethodIn(String funcname, float seconds);
    
    /** [native] Wywoluje podana metode po okreslonym czasie
	 * @param nFuncID identyfikator wczescniej zarejstrowanej funkcji (<code>int</code>)
	 * @param seconds czas opoznienia w sekundach (<code>float</code>)
	 */
    native final void CallMethodIn(int nFuncID, float seconds);
	
    /** [native] Wywoluje podana metode po okreslonym czasie
     * Czas wywolania zalezny jest od istotnosci obiektu i przyjmuje wartosci
     * fMinTime dla istotnosci = 1 i fMaxTime dla istotnosci = 0
	 * @param funcname nazwa metody ktora chcemy wywolac (<code>String</code>)
	 * @param fMinTime min czas wywolania (<code>float</code>)
	 * @param fMaxTime max czas wywolania (<code>float</code>)
	 * @see #DisableCallMethod(int nFuncID)
	 */
    native final void CallMethodInMinMax(String funcname, float fMinTime, float fMaxTime);

    /** [native] Wywoluje podana metode po okreslonym czasie
     * Czas wywolania zalezny jest od istotnosci obiektu i przyjmuje wartosci
     * fMinTime dla istotnosci = 1 i fMaxTime dla istotnosci = 0
	 * @param nFuncID identyfikator wczescniej zarejstrowanej funkcji (<code>int</code>)
	 * @param fMinTime min czas wywolania (<code>float</code>)
	 * @param fMaxTime max czas wywolania (<code>float</code>)
	 * @see #DisableCallMethod(int nFuncID)
	 */
    native final void CallMethodInMinMax(int nFuncID, float fMinTime, float fMaxTime);

	/** [native] Wywoluje podana metode okresowo co podany czas (pierwsze wywolanie po podanym czasie).
	 * @param funcname nazwa metody ktora chcemy wywolac (<code>String</code>)
	 * @param seconds okres wywolan w sekundach (<code>float</code>)
	 * @see #DisableCallMethod(String fucname)
	 */
    native final void CallMethodEvery(String funcname, float seconds);
    
    /** [native] Wywoluje podana metode okresowo co podany czas z opoznieniem (pierwsze wywolanie po 
     * podanym czasie plus fDelayTime).
	 * @param funcname nazwa metody ktora chcemy wywolac (<code>String</code>)
	 * @param seconds okres wywolan w sekundach (<code>float</code>)
	 * @param fDelayTime opoznienie pierwszego wywolania metody (<code>float</code>)
	 * @see #DisableCallMethod(String fucname)
	 */
    native final void CallMethodEvery(String funcname, float seconds, float fDelayTime);
    
    /** [native] Wywoluje podana metode okresowo co podany czas.
     * Czestotliwosc wolania zalezna jest od istotnosci obiektu i przyjmuje wartosci
     * fMinTime dla istotnosci = 1 i fMaxTime dla istotnosci = 0
	 * @param funcname nazwa metody ktora chcemy wywolac (<code>String</code>)
	 * @param fMinTime min okres wywolan (<code>float</code>)
	 * @param fMaxTime max okres wywolan (<code>float</code>)
	 * @see #DisableCallMethod(int nFuncID)
	 */
    native final void CallMethodEveryMinMax(String funcname, float fMinTime, float fMaxTime);

    /** [native] Wywoluje podana metode okresowo co podany czas.
     * Czestotliwosc wolania zalezna jest od istotnosci obiektu i przyjmuje wartosci
     * fMinTime dla istotnosci = 1 i fMaxTime dla istotnosci = 0
	 * @param funcname nazwa metody ktora chcemy wywolac (<code>String</code>)
	 * @param fMinTime min okres wywolan (<code>float</code>)
	 * @param fMaxTime max okres wywolan (<code>float</code>)
	 * @param fDelayTime opoznienie pierwszego wywolania metody (<code>float</code>)
	 * @see #DisableCallMethod(int nFuncID)
	 */
    native final void CallMethodEveryMinMax(String funcname, float fMinTime, float fMaxTime, float fDelayTime);
    
    /** [native] Wywoluje podana metode okresowo co podany czas (pierwsze wywolanie po podanym czasie).
	 * @param nFuncID identyfikator wczescniej zarejstrowanej funkcji (<code>int</code>)
	 * @param seconds okres wywolan w sekundach (<code>float</code>)
	 * @see #DisableCallMethod(int nFuncID)
	 */
    native final void CallMethodEvery(int nFuncID, float seconds);

    /** [native] Wywoluje podana metode okresowo co podany czas z opoznieniem (pierwsze wywolanie po 
     * podanym czasie plus fDelayTime).
	 * @param nFuncID identyfikator wczescniej zarejstrowanej funkcji (<code>int</code>)
	 * @param seconds okres wywolan w sekundach (<code>float</code>)
	 * @param fDelayTime opoznienie pierwszego wywolania metody (<code>float</code>)
	 * @see #DisableCallMethod(String fucname)
	 */
    native final void CallMethodEvery(int nFuncID, float seconds, float fDelayTime);
    
    /** [native] Wywoluje podana metode okresowo co podany czas.
     * Czestotliwosc wolania zalezna jest od istotnosci obiektu i przyjmuje wartosci
     * fMinTime dla istotnosci = 1 i fMaxTime dla istotnosci = 0
	 * @param nFuncID identyfikator wczescniej zarejstrowanej funkcji (<code>int</code>)
	 * @param fMinTime min okres wywolan (<code>float</code>)
	 * @param fMaxTime max okres wywolan (<code>float</code>)
	 * @see #DisableCallMethod(int nFuncID)
	 */
    native final void CallMethodEveryMinMax(int nFuncID, float fMinTime, float fMaxTime);

    /** [native] Wywoluje podana metode okresowo co podany czas.
     * Czestotliwosc wolania zalezna jest od istotnosci obiektu i przyjmuje wartosci
     * fMinTime dla istotnosci = 1 i fMaxTime dla istotnosci = 0
	 * @param nFuncID identyfikator wczescniej zarejstrowanej funkcji (<code>int</code>)
	 * @param fMinTime min okres wywolan (<code>float</code>)
	 * @param fMaxTime max okres wywolan (<code>float</code>)
	 * @param fDelayTime opoznienie pierwszego wywolania metody (<code>float</code>)
	 * @see #DisableCallMethod(int nFuncID)
	 */
    native final void CallMethodEveryMinMax(int nFuncID, float fMinTime, float fMaxTime, float fDelayTime);

    /** [native] Powoduje przerwanie wolania funkcji zarejstrowanej przez CallMetohodIn i 
     * CallMethodEvery
     * @param fucname Nazwa metody jaka ma byc wolana. Metoda ta zachowuje sie jak handler tzn.
     * moze byc pokrywana w stanach GameObjectu.
     */
    native final void DisableCallMethod(String fucname);
    
    /** [native] Powoduje przerwanie wolania funkcji zarejstrowanej przez CallMetohodIn i 
     * CallMethodEvery
     * @param nFuncID Nazwa metody jaka ma byc wolana. Metoda ta zachowuje sie jak handler tzn.
     * moze byc pokrywana w stanach GameObjectu.
     */
    native final void DisableCallMethod(int nFuncID);
    
    /**[native] Powoduje przerwanie wolania wszystkich funkcji zarejstrowanych przez CallMetohodIn 
     * i CallMethodEvery.
     */
    native final void DisableCallMethods();
	
    /**[native] Powoduje przywrcenie wolania wszystkich funkcji zarejstrowanych przez CallMetohodIn 
     * i CallMethodEvery a usunitych przez DisableCallMethods().
     */
    native final void EnableCallMethods();	
    
    /** [native] Powoduje wstrzymanie wolania metod po CallMethodIn
     * i CallMethodEvery.
     */
    native final void FreezeCallMethods();
    
    /** [native] Odmraza wolanie metod zatrzymanych wczesniej przez 
     * FreezeCallMethods()
     */
    native final void UnfreezeCallMethods();
                                        
    /** [native] Powoduje wstrzymanie wolania podanej metody przez CallMethodIn
     * i CallMethodEvery.
     */
    native final void FreezeCallMethod(int nFuncID);
    
    /** [native] Odmraza wolanie podanej metody zatrzymanej wczesniej przez 
     * FreezeCallMethod[s]
     */
    native final void UnfreezeCallMethod(int nFuncID);

    
    //////////////////////////////////////
    // funkcje zwiazane z mysza i 3D	//
    //////////////////////////////////////

	/** [native] Dla podanej kamery zwraca punkt w swiecie 3D bedacy odbiciem
	 * punktu (iPointX, iPointY) na ekranie.
     * @param Cam kamera
	 * @param iPointX wspolrzedna X punktu na ekranie  (<code>int</code>)
	 * @param iPointY wspolrzedna Y punktu na ekranie  (<code>int</code>)
	 */
	native final Vector GetOnScreenPoint(Camera Cam, int iPointX, int iPointY);

	/** [native] Dla podanej kamery zwraca wektor w swiecie 3D bedacy
	 * "kierunkiem kliknicia" z punktu (iPointX, iPointY) na ekranie.
     * @param Cam kamera
	 * @param iPointX wspolrzedna X punktu na ekranie  (<code>int</code>)
	 * @param iPointY wspolrzedna Y punktu na ekranie  (<code>int</code>)
	 */
	native final Vector GetOnScreenVector(Camera Cam, int iPointX, int iPointY);
	
	/** [native] Dla podanej kamery zwraca punkt na ekranie dla podanego
	 * punktu 3D w swiecie
	 * @param Cam kamera
	 * @param iPointInWorld punkt w swiecie
	 * @return punkt na ekranie (wspolrzedne x i y wektora)
	 */
	native final Vector PointToScreen(Camera Cam, Vector vPointInWorld);


    /** [native] Ustawienie zasiegu widzenia obiektow.
     * @param fBlendStart poczatek wygaszania widocznosci obiektow
     * @param fBlendEnd zasieg widzenia obiektow, powyzej tej odleglosci od kamery obiekt jest klipowany
     * 
     * Obiekty wyswietlane sa w odleglosci od 0 do fBlendEnd od obserwatora, przy czym
     * miedzy fBlendStart a fBlendEnd obiekt jest polprzezroczysty. Stopien polprzezroczystosi
     * zalezy liniowo od odleglosci obiektu od fBlendStart
     * 
     * fBlendFactor = (DisatanceToObserver - fBlendStart)/(fBlendEnd - fBlendStart)
     * 
     * stad parametry przekazywane do funkcji musza spelniac:
     * fBlendStart > 0 i fBlendEnd > 0 i fBlendStart < fBlendEnd
     */
    public native final void SetObjectsVisibilityRange(float fBlendStart, float fBlendEnd);
    
    /** [native] Pobranie odleglosci, od jakiej obiekty zaczna sie blendowac.
     * (odpowiednik pierwszego parametru metody SetObjectsVisibilityRange)
     * @return odleglosc, od ktorej obiekty zaczna sie blendowac
     */
    public native final float GetObjectsVisibilityRangeBlend();
	
    /** [native] Pobranie odleglosci, od jakiej obiekty zaczna znikac.
     * (odpowiednik drugiego parametru metody SetObjectsVisibilityRange)
     * @return odleglosc, od ktorej obiekty beda clipowane
     */
    public native final float GetObjectsVisibilityRangeFade();
	
    //////////////////////////////////////
    // tworzenie nowych obiektow w grze //
    //////////////////////////////////////

    /** [native] Tworzy obiekt i go inicjalizuje. <br>
     * Utworzony obiekt jest przypisany do modulu w ktorym znajduje
     * sie obiekt ktory wywolal ta metode.<br>
     * Jezeli ta metoda wywolana jest z modulu to obiekt jest przypisany
     * do tego modulu.<br>
     * Jesli chcemy zmienic standardowe atrybuty obiektu to nalezy 
     * wywolac {@link #CreateObjectNoInit(String sObjectClass)}, 
     * zmienic atrybuty i nastepnie wywolac {@link #InitObject()}.
     * @param sObjectClass nazwa klasy ktorej obiekt tworzymy
     * @return Stworzony obiekt o typie podanym w parametrze (null w razie niepowodzenia)
     */
    final native GameObject CreateObject(String sObjectClass);
    final native GameObject CreateObject(int nClassID);

    /** [native] Dziala analogicznie do {@link #CreateObject(String sObjectClass)} ale dotyczy
     * obiektow w grze sieciowej, ktore sa replikowane, a maja byc utworzone dla konkretnego 
     * clienta. nClientID na serwerze wynosi 0.
     * @param sObjectClass nazwa klasy ktorej obiekt tworzymy
     * @return Stworzony obiekt o typie podanym w parametrze (null w razie niepowodzenia)
     */
    final native GameObject CreateObjectNet(String sObjectClass, int nClientID);
    final native GameObject CreateObjectNet(int nClassID, int nClientID);
    
    /** [native] Dziala analogicznie do {@link #CreateObject(String sObjectClass)} ale dotyczy
     * obiektow w grze sieciowej, ktore zwykle sa replikowane i powinny byc na serwerze
     * dodane do replication managera, natomiast w tym przypadku maja one
     * bys stworzone tylko lokalnie (omijamy replication managera)
     * @param sObjectClass nazwa klasy ktorej obiekt tworzymy
     * @return Stworzony obiekt o typie podanym w parametrze (null w razie niepowodzenia)
     */
    final native GameObject CreateObjectLocal(String sObjectClass);
    
    /**[native] Pobranie identyfikatora klasy w celu przysieszenia metod operujacych na klasach.
     * Identyfikator ten ma zastapic nazwe przekazywana stringiem do metod natyvnych.
     * @param sClassName nazwa klasy dla ktorej identyfikator jest pobierany
     */
    final native int GetClassID(String sClassName);
    
    /** [native] Dziala analogicznie do {@link #CreateObject(String sObjectClass)}
     * ale nie inicjalizuje obiektu po utworzeniu, trzeba to zrobi samemu
     * po modyfikacji atrybutw wywoujc {@link #InitObject()}
     * @param sObjectClass nazwa klasy ktorej obiekt tworzymy
     * @return Stworzony obiekt o typie podanym w parametrze (null w razie niepowodzenia)
     */
    final native GameObject CreateObjectNoInit(String sObjectClass);
    
    /** [native] Dziala analogicznie do {@link #CreateObject(String sObjectClass)}
     * ale nie inicjalizuje obiektu po utworzeniu, trzeba to zrobi samemu. Dodatkowo
     * ustawia ID obiektu na podany. Jezeli inny obiekt ma juz takie ID to przenumerowuje
     * je, aby nie bylo dwoch obiektow o takich samych ID. Metoda powstala specjalnie do
     * robienia SaveGame'ow.
     * po modyfikacji atrybutw wywoujc {@link #InitObject()}
     * @param sObjectClass nazwa klasy ktorej obiekt tworzymy
     * @param nUniqueID unikalny ID do nadania obiektowi
     * @return Stworzony obiekt o typie podanym w parametrze (null w razie niepowodzenia)
     */
    final native GameObject CreateObjectNoInit(String sObjectClass, int nUniqueID);
    
    /** [native] Dziala analogicznie do {@link #CreateObjectNoInit(String sObjectClass)}
     * ale dotyczy obiektow w grze sieciowej, ktore sa tworzone dla konkretnego clienta.
     * @param sObjectClass nazwa klasy ktorej obiekt tworzymy
     * @param nClientID identyfikator klienta dla ktorego tworzony jest obiekt.
     * @return Stworzony obiekt o typie podanym w parametrze (null w razie niepowodzenia)
     */
    final native GameObject CreateObjectNoInitNet(String sObjectClass, int nClientID);
	
    /**[native] Pobranie identyfikatora obiektu unikalnego w obrebie levelu. Na levelu
     * nie powtorzy sie ten numerek nigdy.
     */
    public final native int GetUniqueID();
    /**[native] Ustawienie nowego identyfikatora obiektu unikalnego w obrebie levelu. Na levelu
     * nie powtorzy sie ten numerek nigdy.
     */	
	public final native int SetUniqueID();
    
	/**[native] Inicjalizuje obiekt utworzony przez 
	 * {@link #CreateObjectNoInit(String sObjectClass)}
	 */
    final native void InitObject();

    /**[native] Powoduje przypisanie podanemu obiektowi modulu takiego jaki ma obiekt ktory wola ta metode.
     * @param sAcquiredObject obiekt jakiemu ma byc ustawiony modul
     */
    final native void AcquireObjectModule(GameObject sAcquiredObject);
    
    /**[native] Laduje modul z pliku. Klasa tworzonego modul zapisana jest w pliku
     * dlatego moze byc problematyczne kastowanie go do typu innego niz Module po odczycie.
     * <BR>Jezeli obiekt ma zapisanego Chunka wczytywanego do Javy jest wolana dla niego metoda
     * public void OnLoadChunk(int ChkID, int nDataSize, java.io.DataInputStream cDataStream);. Poprzez
     * Stream mozna odczytac dane z pliku dotyczace tego chunka.
     * @param sModuleFile nazwa pliku z ktorego bedzie modul odczytany
     * @return Stworzony obiekt modulu z odczytwanymi danymi. Typ modulu jest zapisany rowniez w pliku
     */
    final native Module LoadModule(String sModuleFile);

	
    /**[native] Powoduje uruchomnienie efektu (sekwencji) on nazwie zadanej parametrem
     * @param sFXName nazwa zdefiniowanej sekwencji (ta podawana jako parametr wywolania metody Sequence("nazwa sekwencji")) (<code>String</code>)
     * @param pos pozycja FX'a: jezeli efekt nie jest przypiety do obiektu gry tzn. jest uruchamiany metoda nie przyjmujaca ani ControlObject ani meshelementu, 
     *  to pozycja pos efektu jest wyrazona w ukladzie swiata. Jezeli efekt jest przypiety do obiektu, to pozycja jest wyrazona
     *  w ukladzie tego obiektu. Jezeli dodatkowo podany zostal mesh element, to pozycja jest wyrazona w ukladzie mesh elementu (<code>Vector</code>)
     * Orientacja sekwencji jest zgodna z orientacja swiata lub orientacja obiektu (jezeli sekwencja zostala przypisana do obiektu), lub
     * orientacja mesh elementu (jezeli sekwencja zostala przypisana do mesh elementu danego obiektu)
     */
    public native int ExecuteFX(String sFXName,Vector pos);    
    public native int ExecuteFX(ControlObject ctrlObj,String sFXName,Vector pos);
    public native int ExecuteFX(ControlObject ctrlObj,int nMeshElement,String sFXName,Vector pos);
    public native int ExecuteFX(ControlObject ctrlObj,String strMeshElement,String sFXName,Vector pos);
    
    /**[native] Powoduje  uruchomnienie efektu on nazwie zadanej parametrem
     * @param sFXName nazwa  - jest nazwa klasy dzedziczonej z FX (<code>String</code>)
     * @param pos pozycja FX'a (<code>Vector</code>)
     * @param normal normalna do plaszczyzny FX'a (<code>Vector</code>)
     * Patrz opis funkcji ExecuteFX i ExecuteFXTrace (jezeli chodzi o normalna)
     */
    public native int ExecuteFX(String sFXName,Vector pos,Vector normal);
    ////TEMP[TK]: int dla odroznienia od fcji pobierajacej takie same parametry w ControlObject
    public native int ExecuteFX(ControlObject ctrlObj,String sFXName,Vector pos,Vector normal);
    public native int ExecuteFX(ControlObject ctrlObj,int nMeshElement,String sFXName,Vector pos,Vector normal);
    public native int ExecuteFX(ControlObject ctrlObj,String strMeshElement,String sFXName,Vector pos,Vector normal);
    public native boolean DeleteFXSequence(int nFXID);
    
    
   /**[native] Powoduje  uruchomnienie efektu sladu/nalepki o nazwie zadanej parametrem.
     * Nalepka jest to prostokat przyklejany do obiektow ktore napotka na swojej drodze.
     * Jezeli nalepka jest typu TraceClipped to zostanie docieta do trojkatow napotkanego 
     * obiektu, tak by nie wychodzila poza jego obrys. Najczesciej nalepki sluza do symulowania 
     * plam krwi i sladow po pociskach. Nalepka porusza sie razem z obiektem do ktorego jest przylepiona.
     * Uwaga: nie nalepia sie na trojkaty odwrocone tylem lub prostopadle do jej kierunku!
     * Uwaga: 2003-02-25 nalepki obcinane nalepiaja sie na teren, drzewa i statyczne lub 
     * ruchome obiekty, nie nalepiaja sie na skinowane(morfowane) meshe!
     * @param sFXTraceName nazwa zdefiniowanego Trace'a (ta podawana jako parametr wywolania metody Trace("nazwa trejsa") lub TraceClipped ("nazwa trejsa")(<code>String</code>)
     * @param pos pozycja FX'a (uklad odniesienia wybierany tak samo jak w przypadku powyzszych funkcji)(<code>Vector</code>)
     * @param normal normalna do plaszczyzny FX'a - definiuje orientacje plaszczyzny trejsa. Orientacja ukladu jest 
     *  wyznaczana w oparciu o wektor normal i wektor [0, 1, 0] lub wektor [1, 0, 0], jezeli wektor normal jest bliski wektorowi [0, 1, 0]. 
     *  Jezeli trejs jest podlaczony do obiektu lub mesh elementu, to orientacja jest wyrazona ukladzie tego obiektu lub mesh elementu. (<code>Vector</code>)
     */
    public native int ExecuteFXTrace(String sFXTraceName,Vector pos,Vector normal);
    public native int ExecuteFXTrace(ControlObject ctrObj,String sFXTraceName,Vector pos,Vector normal);
    public native int ExecuteFXTrace(ControlObject ctrObj,int nMeshElement,String sFXTraceName,Vector pos,Vector normal);
    public native int ExecuteFXTrace(ControlObject ctrObj,String strMeshElement,String sFXTraceName,Vector pos,Vector normal);
    
    /**[native] Powoduje uruchomnienie efektu sladu/nalepki rzutowanej na geometrie o nazwie zadanej parametrem.
     * Rzutuje sie wylacznie na geometrie zawierajaca bipeda, czyli praktycznie tylko dla postaci. Specjalnie
     * przygotowana dla obiektow uzywajacych skinowanej geometrii(skin/physique).
     * @param sFXTraceName nazwa zdefiniowanego Trace'a (ta podawana jako parametr wywolania metody TraceProjected("nazwa trejsa")(<code>String</code>)
     * @param pos pozycja FX'a (uklad odniesienia wybierany tak samo jak w przypadku powyzszych funkcji)(<code>Vector</code>)
     * @param normal normalna do plaszczyzny FX'a - definiuje orientacje plaszczyzny trejsa. Wybierana tak samo jak dla powyzszych funkcji(<code>Vector</code>)
     * @param nBoneID numer mesh elementu bedacego koscia, na ktorej fejsy bedzie rzucony trejs(<code>int</code>)
     */    
    public native int ExecuteFXTraceProjected(ControlObject ctrObj, int nBoneID, String sFXTraceName,Vector pos,Vector normal);
    public native int ExecuteFXTraceProjected(ControlObject ctrObj, int nMeshElement, int nBoneID, String sFXTraceName, Vector pos, Vector normal);
    public native int ExecuteFXTraceProjected(ControlObject ctrObj, String strMeshElement,int nBoneID, String sFXTraceName, Vector pos, Vector normal);

    /**[native] Powoduje  uruchomnienie efektu sladu rzutowanego na teren
     * @param sFXTraceName nazwa zdefiniowanego Trace'a (ta podawana jako parametr wywolania metody TraceProjected("nazwa trejsa")(<code>String</code>)
     * @param pos pozycja FX'a (uklad odniesienia wybierany tak samo jak w przypadku powyzszych funkcji)(<code>Vector</code>)
     * @param normal normalna do plaszczyzny FX'a - definiuje orientacje plaszczyzny trejsa. Wybierana tak samo jak dla powyzszych funkcji(<code>Vector</code>)          
     */    
    public native int ExecuteFXTerrainTraceProjected(String sFXTraceName,Vector pos,Vector normal);    
    
    /**[native] Powoduje bezwarunkowe usuniecie efektu sladu z systemu efektow
     * 
     * @param nFXID numer trejsa do usuniecia (zwrocony przez wywolanie ExecuteFXTrace...) (<code>int</code>)     
    */    
    public native boolean DeleteFXTrace(int nFXID);
        
    /**[native] Powoduje  uruchomnienie efektu oslepienia on nazwie zadanej parametrem
     * @param sFXBlindName nazwa  - jest nazwa zdefiniowanego Blind'a (<code>String</code>)
     * @param fDelay opoznienie w ms uruchomienia efektu oslepienia (<code>float</code>)
     */
     public native int ExecuteFXBlind(String sFXBlindName,float fDelay);
     
    /**[native] Powoduje bezwarunkowe usuniecie efektu oslepienia z systemu efektow
     * 
     * @param nFXID numer efektu sladu do usuniecia (zwrocony przez wywolanie ExecuteFXBlind) (<code>int</code>)     
    */    
    public native boolean DeleteFXBlind(int nFXID);
    

     /**[native] Dodaje nowy obiekt deszczu do systemu
     * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)     
     */
     public native void AddRain(String sFXRainName);
     
     /**[native] Rozpoczyna opady deszczu o zadanej nazwie. 
      * Jezeli dany deszcz nie zostal wczesniej dodany to jest on dodawany automatycznie
      * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)     
     */
     public native void StartRain(String sFXRainName);
     
     /**[native] Przerywa opady deszczu o podanej nazwie
     * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)     
     */
     public native void StopRain(String sFXRainName);

    /**[native] Ustawia flage sortowania kropel podczas rysowania.*/
    public native void FXRainSetSortDrops(String sFXRainName, boolean bSort);

    /**[native] Ustawia sposob rysowania kropel deszczu. @see ERainMode*/
    public native void FXRainSetDrawMode( String sFXRainName, int iDrawMode );
     
    /**[native] Zmienia gestosc deszczu o podanej nazwie
    * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)
    * @param density - gestosc deszczu [0..100], 0 - krople sie nie rysuja,
    * 50 - polowa kropli, 100 - rysuja sie wszystkie krople;
    */
    public native void FXRainSetDensity(String sFXRainName,double density);

    /**[native] Zmienia ilosc kropel deszczu o podanej nazwie.
    * Ta ilosc kropel bedzie rysowana dla density 100
    * @param sFXRainName nazwa zdefiniowanego deszczu
    * @param iCapacity maksymalna ilosc kropel w deszczu.
    */  
    public native void FXRainSetCapacity(String sFXRainName, int iCapacity);
     
     /**[native] Zmienia predkosc padania deszczu o podanej nazwie
     * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)
     * @param speed predkosc deszczu (<code>double</code>)          
     */
    public native void FXRainSetMinVelocity(String sFXRainName,double speed);
    public native void FXRainSetMaxVelocity(String sFXRainName,double speed);

    /** [native] Zmienia predkosc obrotow sprajtow kropel wokol srodka. Jednostka
    sa obroty na sekunde.*/
    public native void FXRainSetDropsRotation( String sFXRainName, 
        double fMin, double fMax );

     /**[native] Zmienia kierunek padania deszczu o podanej nazwie
     * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)
     * @param dir kierunek padania deszczu (<code>Vector</code>)          
     */
    public native void FXRainSetDirection(String sFXRainName,Vector dir);
     
     /**[native] Zmienia wartosc modyfikacji kierunku padania deszczu o podanej nazwie
     * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)
     * @param angle o ile stopni kierunek deszczu moze odchylic sie od RainDirection.
     */
    public native void FXRainSetDirVariationAngle( String sFXRainName, double angle );
     
     /**[native] Zmienia wielkosc czasteczek padajacego deszczu o podanej nazwie
     * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)
     * @param cx szerokosc czasteczki deszczu(<code>double</code>)
     * @param cy wysokosc czasteczki deszczu(<code>double</code>)                    
     */
    public native void FXRainSetDropSize(String sFXRainName,double cx,double cy);
     
     /**[native] Zmienia przezroczystosc padajacego deszczu o podanej nazwie
     * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)
     * @param alpha przezroczystosc deszczu (1.0 - nieprzezroczysty)(<code>double</code>)          
     */
    public native void FXRainSetAlpha(String sFXRainName,double alpha);
    /** W jakiej odleglosci krople zaczynaja robic sie przezroczyste.
    Alfa kropli zaczyna zchodzic ponizej 1.0, by w odleglosci CubeSize/2 osiagnac 0.0
    @param fDistance odleglosc w centymetrach.
    */
    public native void FXRainSetFadeDistanceStart(String sFXRainName, double fDistance);
    /** Najwiekszy mozliwy promien spirali po ktorej porusza sie zwichrowana czasteczka.
     *
    */
    public native void FXRainSetTurbulencyAmplitude( String sFXRainName, double fAmplitude );
    /** Szybkosc obrotow podczas turbulencji.[rot/s]
    */
    public native void FXRainSetTurbulencySpeed( String sFXRainName, double fSpeed );

    /**[native] Zmienia wielkosc szescianu w ktorym pada deszcz. Kamera zawsze znajduje sie
     * w jego srodku.
     * @param sFXRainName nazwa zdefiniowanego deszczu (<code>String</code>)
     * @param fSize dlugosc boku szescianu w ktorym symulowany jest deszcz
    */
    public native void FXRainSetCubeSize(String sFXRainName, double fSize);

    /** Zmienia material z ktorym rysowane beda krople deszczu. Domyslnym jest
    'data\ */
    public native void FXRainSetMaterial( String sFXRainName, String sMaterialName );
    /** Ustawia ile rzedow i kolumn ramek miesci sie w teksturze dla rysowania kropel.
     * Jezeli tekstura posiada wiecej niz jedna ramke to te ramki moga zostac uzyte do 
     * rysowania kropel. Im wiecej ramek tym wiecej "rodzajow" kropel.
     * @param iFramesX ile rzedow ramek.
     * @param iFramesY ile kolumn ramek.
    */
    public native void FXRainSetTextureFrames( String sFXRainName, int iFramesX, int iFramesY );
    /** Prawdopodobienstwo generowania plusku w momencie uderzenia kropli o teren/wode.
     * Wartosci nalezy podawac w procentach od 0.0 do 100.0. Wartoscia domyslna jest 0 .
    */
    public native void FXRainSetSplashProbability( String sFXRainName, double fProbability );
    /** Ustawia nazwe sekwencji efektu uruchamianego w momencie uderzenia kropli o wode.
    */
    public native void FXRainSetWaterSplash( String sFXRainName, String sSplash );
    /** Ustawia nazwe sekwencji efektu uruchamianego w momencie uderzenia kropli o teren/budynek.
    */
    public native void FXRainSetConcreteSplash( String sFXRainName, String sSplash );

    /** Dla podanej pozycji zwraca wysokosc z mapy wysokosci deszczu. */
    public native float FXRainHMapGetHeight( float x, float z );

    /** Dla podanego prostokata zwraca najmniejsza znaleziona wysokosc z mapy wysokosci deszczu.*/
    public native float FXRainHMapGetMinimalHeight( float xMin, float xMax, float zMin, float zMax );

    /**[native] Uruchamia system partikli o nazwie zadanej parametrem
     * @param sFXParticleSystemName nazwa systemu partikli(<code>String</code>)
     * Patrz komentarze do metod ExecuteFX
    */
         
    public native int ExecuteParticleSystem(String sFXParticleSystemName,Vector pos,Vector normal);
    public native int ExecuteParticleSystem(ControlObject ctrlObj,String sFXParticleSystemName,Vector pos,Vector normal);
    public native int ExecuteParticleSystem(ControlObject ctrlObj,int nMeshElement,String sFXParticleSystemName,Vector pos,Vector normal);
    public native int ExecuteParticleSystem(ControlObject ctrlObj,String strMeshElement,String sFXParticleSystemName,Vector pos,Vector normal);
    
    public native int ExecuteParticleSystem(String sFXParticleSystemName,Vector pos,Vector baseX,Vector baseY,Vector baseZ);
    public native int ExecuteParticleSystem(ControlObject ctrlObj,String sFXParticleSystemName,Vector pos,Vector baseX,Vector baseY,Vector baseZ);
    public native int ExecuteParticleSystem(ControlObject ctrlObj,int nMeshElement,String sFXParticleSystemName,Vector pos,Vector baseX,Vector baseY,Vector baseZ);
    public native int ExecuteParticleSystem(ControlObject ctrlObj,String strMeshElement,String sFXParticleSystemName,Vector pos,Vector baseX,Vector baseY,Vector baseZ);
    
    /**[native] Uruchamia efekt kawalka mesha 
     * @param refMesh obiekt zawierajacy mesha, z ktorego zostana "wyciagniete" dane, ktore posluza do utworzenia
     * kawalka mesha. Mesh musi zawierac bipeda
     * @param nBoneID numer mesh elementu bedacego koscia. Mesh utworzony w efekcie dzialania metody 
     * bedzie zawieral tylko trojkaty przypisane do kosci o zadanym numerze(<code>int</code>)
     * @param sFXMeshName nazwa efektu mesha (<code>String</code>)
     * @param Vector pos - pozycja efektu (<code>Vector</code>)
     * @param Vector normal - normalna sluzaca do definiowania orientacji mesha (<code>Vector</code>)
     * @param ctrObj obiekt pod ktory przypiety jest efekt
     * @param nMeshElement mesh element pod ktory przypiety jest efekt (<code>int</code>)
     * Patrz komentarze do metod ExecuteFX
    */
    public native int ExecuteFXMeshPart(ControlObject refMesh, int nBoneID, Vector vSpeed, float fMass, String sFXMeshName, Vector pos, Vector normal);    
    public native int ExecuteFXMeshPart(ControlObject ctrObj, ControlObject refMesh, int nBoneID, Vector vSpeed, float fMass, String sFXMeshName,Vector pos,Vector normal);
    public native int ExecuteFXMeshPart(ControlObject ctrObj, int nMeshElement, ControlObject refMesh, int nBoneID, Vector vSpeed, float fMass, String sFXMeshName, Vector pos, Vector normal);
    public native int ExecuteFXMeshPart(ControlObject ctrObj, String strMeshElement, ControlObject refMesh, int nBoneID, Vector vSpeed, float fMass, String sFXMeshName, Vector pos, Vector normal);
    
    /**[native] Usuwa z systemu FXow efekt kawalka mesha 
     * @param nFXID identyfikator efektu mesha zwracany przez funkcje ExecuteFXMeshPart
     */
     
    public native boolean DeleteFXMeshPart(int nFXID);
    
    /**[native] Powoduje uruchomnienie efektu flary o nazwie zadanej parametrem
     * @param sFXLensFlareName nazwa zdefiniowanej lens flary (ta podawana jako parametr wywolania metody LensFlare("nazwa flary")) (<code>String</code>)
     * @param pos pozycja flary wyrazona w ukladzie swiata (<code>Vector</code>)
    */
    public native int ExecuteFXLensFlare(String sFXLensFlareName, Vector pos);    
    
    /**[native] Powoduje uruchomnienie efektu flary o nazwie zadanej parametrem
     * @param ctrObj obiekt pod ktory zostanie przypieta flara
     * @param sFXLensFlareName nazwa zdefiniowanej lens flary (ta podawana jako parametr wywolania metody LensFlare("nazwa flary")) (<code>String</code>)
     * @param pos pozycja flary wyrazona w ukladzie ctrObj (<code>Vector</code>)     
    */
    public native int ExecuteFXLensFlare(ControlObject ctrObj, String sFXLensFlareName,Vector pos);
    
    /**[native] Powoduje uruchomnienie efektu flary o nazwie zadanej parametrem
     * @param ctrObj obiekt pod ktory zostanie przypieta flara
     * @param nMeshElement identyfikator mesh elementu pod ktory zostanie przypieta flara  (<code>int</code>)
     * @param sFXLensFlareName nazwa zdefiniowanej lens flary (ta podawana jako parametr wywolania metody LensFlare("nazwa flary")) (<code>String</code>)
     * @param pos pozycja flary wyrazona w ukladzie mesh elementu nMeshElement nalezacego do ctrObj (<code>Vector</code>)     
    */
    public native int ExecuteFXLensFlare(ControlObject ctrObj, int nMeshElement,String sFXLensFlareName, Vector pos);
    
    /**[native] Powoduje uruchomnienie efektu flary o nazwie zadanej parametrem
     * @param ctrObj obiekt pod ktory zostanie przypieta flara
     * @param strMeshElement nazwa mesh elementu pod ktory zostanie przypieta flara (<code>String</code>)
     * @param sFXLensFlareName nazwa zdefiniowanej lens flary (ta podawana jako parametr wywolania metody LensFlare("nazwa flary")) (<code>String</code>)
     * @param pos pozycja flary wyrazona w ukladzie mesh elementu strMeshElement nalezacego do ctrObj (<code>Vector</code>)     
    */
    public native int ExecuteFXLensFlare(ControlObject ctrObj, String strMeshElement, String sFXLensFlareName, Vector pos);
    
    /**[native] Usuwa z systemu FXow efekt flary 
     * @param nFXID identyfikator efektu flary zwracany przez funkcje ExecuteFXLensFlare
     */     
    public native boolean DeleteFXLensFlare(int nFXID);
    
     /**[native] Przepina FXy z obiektu src do dst
      * Obiekty musza byc utworzone na bazie takiego samego mesha
    */
    public native void RelinkFX(ControlObject dst, ControlObject src);
    
    /**[native] Przepina FXy z obiektu src do this
      * Obiekty musza byc utworzone na bazie takiego samego mesha
    */
    public native void RelinkFX(ControlObject src);


    //////////////////////////////////////////////////////////////////////////
    // System Sladow kol na terenie, do stworzonego sladu odwolujemy sie przez
    // id, dodajac punkty tworzymy tasme ktora naklada sie na geometrie terenu.
    // 

    /** [native] jak duzo sladow moze pomiescic system zanim najstarsze slady zaczna znikac.*/
    public native void FXTerrainTraceSetCapacity( int iCapacity );
    /** [native] zwraca maksymalna ilosc sladow obslugiwana przez system. Domyslnie 16.*/
    public native int FXTerrainTraceGetCapacity();

    /** [native] Usuwa z systemu wszystkie slady.
    UWAGA: Usuwa takze slady aktualnie edytowane!*/
    public native void FXTerrainTraceClearAll();

    /** [native] Ustawia na jakiej wysokosci nad ziemia maja rysowac sie slady. Przy zbyt duzym 
    offsecie widac ze slady wisza nad ziemia, a przy zbyt malym pojawiaja sie zet-aliasy.*/
    public native void FXTerrainTraceSetPolygonOffset( float fPolygonOffset );
    /** [native] Zwraca na jakiej wysokosci nad ziemia rysowane sa slady.*/
    public native float FXTerrainTraceGetPolygonOffset();

    /** [native] rozpoczyna nowy slad. Slad nie moze zostac skasowany dopoki nie zostanie 
    zamkniety. @see FXTerrainTraceEnd().
    @param sFXTexture nazwa zdefiniowanej FXTekstury.
    @param fLifetime dlugosc zycia sladu w sekundach. Wartosc 0.f oznacza ze slad ma istniec 
        w systemie dopoki jest to mozliwe.
    @param fFadeLength ile sekund ma trwac zanikanie sladu np: FXTerrainTraceBegin("xxx.xxx", 10, 2);
        oznacza ze slad bedzie trwac 10 sekund, a przez ostatnie dwie sekundy ma
        zanikac do kompletnej przezroczystosci.
    @return id nowego sladu, jezeli nie mozna stworzyc sladu zwraca -1.
    */
    public native int FXTerrainTraceBegin( String sFXTextureName, float fLifetime, 
        float fFadeLength );

    /** [native] Zamyka edycje sladu. Od tej chwili nie mozna juz uzywac tego sladu
        poniewaz moze zostac usuniety jezeli ilosc wszystkich sladow przekroczy 
        maksymalna dopuszczalna. @see GetCapacity().
    */
    public native void FXTerrainTraceEnd( int iTrace );

    /** [native] Ustawia tiling U dla nastepnego punktu. Jednostka jest centymetr / tiling,
        mp. SetTraceURemap(100.f) oznacza ze jedna szerokosc tekstury zmiesci sie
        w 100cm szerokosci sladu. Dla nowego sladu domyslna wartoscia jest 100.0f.
        Wywolywac wylacznie pomiedzy BeginTrace()/EndTrace().
    */
    public native void FXTerrainTraceSetURemap( int iTrace, float fCmsPerTile );
    /** [native] Ustawia tiling V dla nastepnego punktu. Jednostka jest centymetr / tiling,
        mp. SetTraceVRemap(100.f) oznacza ze jedna wysokosc tekstury zmiesci sie 
        w 100cm dlugosci sladu. Dla nowego sladu domyslna wartoscia jest 100.0f.
        UWAGA: wspolrzedna V pierwszego punktu jest zawsze rowna 0.f . Wartosc V dla 
        kolejnych punktow zalezy od odleglosci do punktu poprzedniego i aktualnego przemapowania.
        Wywolywac wylacznie pomiedzy BeginTrace()/EndTrace().
    */
    public native void FXTerrainTraceSetVRemap( int iTrace, float fCmsPerTile );

    /** [native] Ustawia szerokosc sladu w nastepnym punkcie.
        Dla nowego sladu domyslna wartoscia jest 100.0f.
        @param fWidth szerokosc sladu w centymetrach.
        Wywolywac wylacznie pomiedzy BeginTrace()/EndTrace().
    */
    public native void FXTerrainTraceSetWidth( int iTrace, float fWidth );

    /** [native] Ustawia kolor w nastepnym punkcie. Jezeli ta wartosc nie zostanie ustawiona,
        nastepny punkt "odziedziczy" kolor po poprzednim.
        Dla nowego sladu domyslna wartoscia jest (1, 1, 1, 1).
        Wywolywac wylacznie pomiedzy BeginTrace()/EndTrace().
    */
    public native void FXTerrainTraceSetColor( int iTrace, float fR, float fG, float fB, float fA );

    /** [native] Doklada nowy punkt do sladu. Punkt kopiuje aktualne ustawienia
        z koloru, szerokosci i przemapowania UV. Slad zaczyna sie rysowac dopiero po dodaniu
        pierwszych dwoch punktow, kazdy nastepny punkt dodaje nowego quada.
        Wywolywac wylacznie pomiedzy BeginTrace()/EndTrace().
        UWAGA: Ustawilem blokade ktora nie dodaje nowego punktu jezeli ten
        ma zostac postawiony na pozycji takiej samej jak punkt poprzedni!
    */
    public native void FXTerrainTraceAddPoint( int iTrace, Vector vPoint );

    ////////////////////////////////////
    
    /**[native] Dodaje obiekt do grida modulu w ktorym zostal obiekt stworzony. Powoduje to pokazanie sie
     * obiektu na module, gdy obiekt jest ControlObjectem. Dla GameObjectow powoduje wywolywanie metody
     * OnFrame i OnPostUpdate dla nich.
     * 
	 */
    public /*final*/ native void ActivateOnModule();
   
    /**[native] Usuwa obiekt z grida modulu w ktorym zostal obiekt stworzony. Powoduje to znikniecie
     * obiektu z wyswietlania modulu dla ControlObjecta. Dla GameObjecta powoduje przestanie wolania dla
     * niego OnFrame i OnPosUpdate.
	 */
    public /*final*/ native void DeactivateOnModule();
    
    /**[native] Usuwa obiekt z modulu w ktorym zostal stworzony.
     * Dziala niezaleznie od tego czy obiekt byl dodany metoda GameObject::ActivateOnModule
     * czy ControlObject::ActivateOnModuleGridIndependent
	 */
    public /*final*/ native void DeactivateOnModuleAlways();

	public final native boolean IsActiveOnModule();
	
    
    ///////////////////////
	// metody pomocnicze //
	///////////////////////

    /**[native] Ustawienie defaultowegych zasiegow klipowania w kamerach.
     * @param fNear ustawienie near clipping plane tak jak w metodzie Camera.SetClipNear
     * @param fFar ustawienie far clipping plane tak jak w metodzie Camera.SetClipFar
     */
    public native static void SetDefaultClipping(float fNear, float fFar);
    
    /** [native] Halasuje na podanym poziomie w podanym miejscu. Wywoluje handler Actor.OnHearNoise(...)
     * dla obiektow, ktore sa w zasiegu slyszenia obiektu. Zasieg ten zalezy od wlasciwosci eteru 
     * znajdujacego sie na levelu i jest ustalany w edytorze.
     * @param fLoudness glosnosc halasu
     * @param nNoiseType rodzaj wydawanego halasu
     */
    public native void MakeNoise(Vector vPos, float fLoudness, int nNoiseType);
    
    /** [native] =true gdy gra jest uruchomiona w trybie serwera dedykowanego tzn. nie renderuje nic i
     * ma ona tylko symulowac sieciowa rozgrywke.
     */
    final native static boolean IsDedicatedServer();
    
    /** [native] wola natychmiastowo renderowanie gry, wprowadzone w celu wymuszania
     * renderowania gry np. przy wczytywaniu kolejnych czesci levelu
     */
    final native void ForceRender();

    /**[native] Wlacza zapisywanie na dysk nazwy mesha wczytywanego do gry.
     */
    final native static void LogLoadingMeshes();
    
    /**[native] Natychmiastowe wyjscie z gry bez wzgledu na sytuacje.
     */
    final native static void ExitGame();

    /** [native] Zwraca modul do ktorego jest przypisany obiekt. Gdy obiekt nie jest przypisany do zadnego
     * to zwraca null.
     */
    final native Module GetModule();

    /**[native] Podaje czas trwania ostatniego updejtu symulacji w milisekundach
     */
    final native static float GetLastSimTime();
    
    /**[native] Podaje czas trwania ostatniego renderowania w milisekundach
     */
    final native static float GetLastRenderTime();
    
    /**[native] Podaje czas trwania ostatniej ramki gry tzn. updejtu symulacji i renderowania razem.
     * Czas zwracany w milisekundach
     */
    final native static float GetLastFrameTime();
    
	//////////////////////////////////////////////////////
	// rejestrowanie obiektow do pobierania kontrolerow //
	//////////////////////////////////////////////////////
	
    /** [native] Rejestruje zadanie wysylania danemu obiektowi zdarzen 
     * z klawiszy
     * @see #UnregisterKeyRequest()
     */
    native void RegisterKeyRequest();

    /** [native] Anuluje zadanie wysylania danemu obiektowi zdarzen 
     * z klawiszy
     * @see #RegisterKeyRequest()
     */
    native void UnregisterKeyRequest();
    
    /** [native] Czy obiekt zarejestrowal sie do odbioru zdarzen od klawiatury 
     * @see #RegisterKeyRequest()
     */
	final native boolean IsRegisteredKeyRequest();
	
    /** [native] Rejestruje zadanie wysylania danemu obiektowi zdarzen 
     * z klawiszy, _na wylacznosc_ pozostale rejestracje zostaja "zamrozone"
     * Wykorzystywane w sieci do trybu chat'a
     * @see #UnregisterKeyRequestExclusive()
     */
    final native void RegisterKeyRequestExclusive();

    /** [native] Anuluje zadanie wysylania danemu obiektowi zdarzen 
     * z klawiszy na wylacznosc, powoduje przywrocenie wczesniejszych requestow
     * @see #RegisterKeyRequestExclusive()
     */
    final native void UnregisterKeyRequestExclusive();
    
    /** [native] Czy obiekt zarejestrowal sie do odbioru zdarzen od klawiatury w trybie wylacznosci
     * @see #RegisterKeyRequest()
     */
    final native boolean IsRegisteredKeyRequestExclusive();

	/** [native] Rejestruje zadanie wysylania danemu obiektowi zdarzen 
     * zwiazanych z poruszaniem mysza
     * @see #UnregisterMouseAxisRequestExclusive()
     */
    final native void RegisterMouseAxisRequest();
    
    /** [native] Czy obiekt zarejestrowal sie do odbioru zdarzen od myszy (ruch)
     * @see #RegisterKeyRequest()
     */
    final native boolean IsRegisteredMouseAxisRequest();
	
    /** [native] Czysci kolejke zdarzen z inputow, wcisniete i nieobsluzone klawisze zostana zapomniane*/
    final native static void ClearInputQueue();
    
    
	/** [native] Rejestruje zadanie wysylania danemu obiektowi zdarzen 
     * zwiazanych z klawiszami myszy
     * Skutkuje wolaniem handera "public void OnInputMouseButton(int nButton, int nState){}"
     * @see #UnregisterMouseButtonRequest()
     */
	final native void RegisterMouseButtonRequest();
	
    /** [native] Czy obiekt zarejestrowal sie do odbioru zdarzen od myszy (przyciski)
     * @see #RegisterMouseButtonRequest()
     */
    final native boolean IsRegisteredMouseButtonRequest();
	
	/** [native] Anuluje zadanie wysylania danemu obiektowi zdarzen 
     * zwiazanych z poruszaniem mysza
     * @see #RegisterMouseAxisRequest()
     */
    final native void UnregisterMouseAxisRequest();
    
    /** [native] Anuluje zadanie wysylania danemu obiektowi zdarzen 
     * zwiazanych z klawiszami myszy
     * @see #RegisterMouseButtonRequest()
     */
	final native void UnregisterMouseButtonRequest();

    
    /** [native] Przerywa aktualne przetwarzanie inputow, kolejne handlery inputowe
     * beda ewentualnie zawolane dopiero w nastepnej ramce
     */
    native void BreakInputProcessing();
        
            
    
    /**[native] Zwraca liczbe joystickow w systemie
     */
    final native int GetJoysticksCount();
    
	/** [native] Rejestruje zadanie wysylania danemu obiektowi zdarzen 
     * zwiazanych z poruszaniem joystickiem
     * @see #UnregisterJoyAxisRequest()
     */
    final native void RegisterJoyAxisRequest();
	
	/** [native] Rejestruje zadanie wysylania danemu obiektowi zdarzen 
     * zwiazanych z przyciskami joysticka
     * @see #UnregisterJoyButtonRequest()
     */
    final native void RegisterJoyButtonRequest();

	/** [native] Anuluje zadanie wysylania danemu obiektowi zdarzen 
     * zwiazanych z poruszaniem joystickiem
     * @see #RegisterJoyAxisRequest()
     */
	final native void UnregisterJoyAxisRequest();
	
	/** [native] Anuluje zadanie wysylania danemu obiektowi zdarzen 
     * zwiazanych z przyciskami joysticka
     * @see #RegisterJoyButtonRequest()
     */
    final native void UnregisterJoyButtonRequest();

    /** [native] Wola handler OnInputKey dla obiektu. Wprowadzone dla usprawnienia kodowania sieci.
     */
    final native void CallOnInputKey(int nKeyCode, int nKeyState, int nKeyAscii);
    
    // zarejestrowane obiekty powinny posiadac handler:
    //void OnInputKey(int nKeyCode, int nKeyState, int nKeyAscii){}

    // OnInputMouseAxis(int nAxis, int nDelta)
    //void OnInputMouseButton(int nButton, int nState){} //not implemented

	// void OnInputJoyAxis(int nJoy, int nAxis, float fDelta) {}
    // void OnInputJoyButton(int nJoy, int nButton, boolean bState) {}

    // Zwraca stan klawisza zarowno z klawiatury jak z myszy czy joysticka
    final native boolean GetKeyState(int nKey);

    /**[native] Zwraca znak ascii odpowiadajacy podanemu klawiszowi
     * Gdy == 0 to znaczy, ze klawisz jest dziwny
     */
    final static native char GetKeyAscii(int nKey);
    
    //pozycja myszy we wspolrzednych ekranowych 640x480
    final native int GetMousePos(int nAxis); //not implemented

    // pozycja joya od ? do ?
    final native int GetJoyPos(int nAxis); //not implemented

    // interfejs do FFB --> do zrobienia

	///////////////////////////////////////////////////////
	// metody pobierania obiektow systemu gry GameOjects //
	//////////////////////////////////////////////////////
	
    /**[native] Zwraca tablice wszystkich obiektow Z CALEJ GRY o podanej lub dziedziczacej klasie<br>
	 *  UWAGA ta metoda przeglada WSZYSTKIE obiekty z gry - to moze byc wolne ! 
	 * @param sClassName nazwa klasy, ktorej obiektow szukamy (<code>string</code>)
	 * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>GameObject []</code>)
	 */
	final native static GameObject [] GetObjectsFromClass(String sClassName);
    
    /**[native] Zwraca tablice wszystkich obiektow Z PODANEGO MODULU o podanej lub dziedziczacej klasie<br>
	 *  UWAGA ta metoda przeglada WSZYSTKIE obiekty z gry - to moze byc wolne ! 
	 * @param sClassName nazwa klasy, ktorej obiektow szukamy (<code>string</code>)
	 * @param cModule modul z ktorego obiekty maja byc wyciagniete
	 * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>GameObject []</code>)
	 */
	final native static GameObject [] GetObjectsFromClassModule(String sClassName, Module cModule);
    
	/**[native] Zwraca tablice wszystkich obiektow o podanej klasie. Gdy zawolane na
	 * rzecz obiektu to sprawdza obiekty z jego levelu. Gdy zawolane dla GameObject.
	 * sprawdza wszystkie obiekty z calej gry.
	 * UWAGA ta metoda przeglada WSZYSTKIE obiekty z gry - to moze byc wolne ! 
	 * @param sClassName nazwa klasy, ktorej obiektow szukamy (<code>string</code>)
	 * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>GameObject []</code>)
	 */
	final native static GameObject [] GetObjectsOfClass(String sClassName);
	
	/**[native] Zwraca wszystkie obiekty o podanej nazwie Z CALEJ GRY <br>
	 * UWAGA ta metoda przeglada WSZYSTKIE obiekty z gry - to moze byc wolne ! 
	 * @param sObjectName nazwa obiektu (<code>string</code>)
	 * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>GameObject []</code>)
	 */
    final native GameObject [] GetObjectsByName(String sObjectName);
	
	/**[native] Zwraca jeden obiekt o podanej nazwie Z CALEJ GRY - jesli jest kilka to zwroci pierwszy lepszy<br>
	* UWAGA ta metoda przeglada WSZYSTKIE obiekty z gry - to moze byc wolne ! 
	* Metoda sprawdza obiekty z levelu do ktorego nalezy obiekt na rzecz jakiego zostala
	* zawolana. Gdy chcesz sprawdzic wszystkie obiekty musisz stosowac skladnie:
	* GameObject.GetObjectByName("nazwa_obiektu") zamiast zwyklego zawolania.
	* @param sObjectName nazwa obiektu (<code>string</code>)
	* @return znaleziony obiekt (<code>GameObject</code>)
	*/
    final native static GameObject GetObjectByName(String sObjectName);
    
    /**[native] Pobranie obiektow w podanych extentach
     * @param vCorner1 wierzcholek boxa extentow (<code>Vector</code>)
     * @param vCorner2 przeciwlegly wierzcholek boxa extentow (<code>Vector</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    final native ControlObject [] GetObjectsInExtents(Vector vCorner1, Vector vCorner2);
	
    /**[native] Pobranie obiektu w extentach. Dodatkowo pozwala okreslic klase 
     * obiektu, gdy klasa == NULL to nie bierze tego parametru pod uwage.
     * Zwraca pierwszy obiekt podanej klasy, nie wiadomo dokladnie ktory.
     * @param vCorner1 wierzcholek boxa extentow (<code>Vector</code>)
     * @param vCorner2 przeciwlegly wierzcholek boxa extentow (<code>Vector</code>)
     * @param nClassName identyfikator klasy pobrany metoda GetClassID (<code>String</code>)
     * @return obiekt podanej klasy
     */
    public final native ControlObject GetObjectInExtentsOfClass(Vector vCorner1, Vector vCorner2, int nClassName);

    /**[native] Pobranie obiektow w podanych extentach, dodatkowo pozwala okreslic klase obiektow, 
     * gdy klasa == NULL to nie bierze tego parametru pod uwage.
     * @param vCorner1 wspolrzedna min boxa (<code>Vector</code>)
     * @param vCorner2 wspolrzedna max boxa (<code>Vector</code>)
     * @param sClassName nazwa klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (<code>ControlObject []</code>)
     */
    // TODO final native ControlObject [] GetObjectsInExtents(Vector vCorner1, Vector vCorner2, String sClassName);
	
    /**[native] Pobranie obiektow w promieniu od podanej pozycji, dodatkowo pozwala 
     * okreslic klase obiektow, gdy klasa == NULL to nie bierze tego parametru pod uwage.
     * Zwracane sa obiekty dokladnie podanej klasy.
     * @param vPos pozycja wokol ktorej szukac w promieniu (<code>Vector</code>)
     * @param fRadius wielkosc promienia (<code>float</code>)
     * @param sClassName nazwa klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    final native ControlObject [] GetObjectsInRadiusOfClass(Vector vPos, float fRadius, String sClassName);

    /**[native] Pobranie obiektow w promieniu od podanej pozycji, dodatkowo pozwala 
     * okreslic klase obiektow, gdy klasa == NULL to nie bierze tego parametru pod uwage.
     * Zwracane sa obiekty potomne od podanej klasy.
     * @param vPos pozycja wokol ktorej szukac w promieniu (<code>Vector</code>)
     * @param fRadius wielkosc promienia (<code>float</code>)
     * @param sClassName nazwa klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    public final native ControlObject [] GetObjectsInRadiusFromClass(Vector vPos, float fRadius, String sClassName);
    
    /**[native] Pobranie obiektow w promieniu od podanej pozycji, dodatkowo pozwala 
     * okreslic flagi i klase obiektow, gdy klasa == NULL to nie bierze tego parametru pod uwage.
     * Zwracane sa obiekty potomne od podanej klasy.
     * @param vPos pozycja wokol ktorej szukac w promieniu (<code>Vector</code>)
     * @param fRadius wielkosc promienia (<code>float</code>)
     * @param iFlags flagi z EFlags (<code>int</code>)
     * @param sClassName nazwa klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    public final native ControlObject [] GetObjectsInRadiusOfFlagsFromClass(Vector vPos, float fRadius, int iFlags, String sClassName);
    
    /**[native] Pobranie obiektow w promieniu od podanej pozycji, dodatkowo pozwala 
     * okreslic klase obiektow, gdy klasa == NULL to nie bierze tego parametru pod uwage.
     * Zwracane sa obiekty potomne od podanej klasy.
     * @param vPos pozycja wokol ktorej szukac w promieniu (<code>Vector</code>)
     * @param fRadius wielkosc promienia (<code>float</code>)
     * @param nClassID identyfikator klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    final native ControlObject [] GetObjectsInRadiusFromClass(Vector vPos, float fRadius, int nClassID);

    /**[native] Pobranie obiektow z frustuma danej kamery
     * @param camera kamera (<code>String</code>)
     * @param fRange zasieg frustuma, jesli jest ujemny to uzyty jest oryginalny 
     * zasieg kamery (<code>String</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    final public native ControlObject [] GetObjectsInFrustum(Camera camera, float fRange);

    /** [native] Sprawdza, czy w podanych extentach sa jakies face'y terenu */
    final public native boolean AreTerrainFacesInExtents(Vector vMin, Vector vMax);
    
    /**[native] Zwraca obiekt o podanym identyfikatorze sieciowym
	* @param nNetId identyfikator szukanego obiektu (<code>int</code>)
	* @return znaleziony obiekt (<code>GameObject</code>)
	*/
    final native GameObject GetObjectByNetId(int nNetId);
    
    /**[native] Zwraca tablice klas, ktore dziedzicza z podanej klasy. Zawsze zwraca tablice.
     * @param sBaseClassName nazwa klasy z jakiej maja dziedziczyc zwracane klasy
     * @return tablica klas, gdy nie ma klas dziedziczacych to zwraca tablice o rozmiarze zero
     */
    static native public java.lang.Class[] GetClassesFromClass(String sBaseClassName);
    
    /**[native] Zwraca id sieciowe obiektu
	* @return identyfikator sieciowy obiektu (<code>int</code>)
	*/
    public final native int GetNetId();
    
    ////////////////////////////////
	// metody obslugujace kolizje //
	////////////////////////////////
    
    /**[native] Oblicza kolizje z terenem dla promienia (vEnd - vStart).
     * @param vStart poczatek promienia dla ktorego liczymy kolizje (<code>Vector</code>)
     * @param vEnd koniec promienia (<code>Vector</code>)
     * @return gdy nie ma kolizji to zwraca NULL
     * gdy jest kolizja to struktura Collision zawiera informacje o niej*/
    public final native Collision TraceTerrain(Vector vStart, Vector vEnd);
	
    /**[native] Oblicza kolizje z obiektami dla promienia (vEnd - vStart).
     * @param vStart poczatek promienia dla ktorego liczymy kolizje (<code>Vector</code>)
     * @param vEnd koniec promienia (<code>Vector</code>)
     * @return gdy nie ma kolizji to zwraca NULL
     * gdy jest kolizja to struktura Collision zawiera informacje o niej*/
    public final native Collision TraceObjects(Vector vStart, Vector vEnd);

    /**[native] Oblicza kolizje z obiektami dla promienia (vEnd - vStart).
     * @param vStart poczatek promienia dla ktorego liczymy kolizje (<code>Vector</code>)
     * @param vEnd koniec promienia (<code>Vector</code>)
     * @param Exclude obiekt, ktorego nie sprawdzac przy kolizji (<code>ControlObject</code>)
     * @return gdy nie ma kolizji to zwraca NULL
     * ( gdy jest kolizja to struktura Collision zawiera informacje o niej, a pozycja
     *   vEnd jest ustawiana w miejscu kolizji (obcinanie trace'a) )
     * */
    public final native Collision TraceObjectsExclude(Vector vStart, Vector vEnd, ControlObject Exclude);

    /**[native] Oblicza kolizje z obiektami dla promienia (vEnd - vStart).
     * @param vStart poczatek promienia dla ktorego liczymy kolizje (<code>Vector</code>)
     * @param vEnd koniec promienia (<code>Vector</code>)
     * @param iFlags flagi jakie musi miec obiekt, zeby byc branym pod uwage przy liczeniu kolizji
     * @return gdy nie ma kolizji to zwraca NULL
     * ( gdy jest kolizja to struktura Collision zawiera informacje o niej, a pozycja
     *   vEnd jest ustawiana w miejscu kolizji (obcinanie trace'a) )* 
     * */
    public final native Collision TraceObjectsWithFlags(Vector vStart, Vector vEnd, int iFlags);

    /**[native] Oblicza kolizje z obiektami dla promienia (vEnd - vStart).
     * @param vStart poczatek promienia dla ktorego liczymy kolizje (<code>Vector</code>)
     * @param vEnd koniec promienia (<code>Vector</code>)
     * @param Exclude obiekt, ktorego nie sprawdzac przy kolizji (<code>ControlObject</code>)
     * @param iFlags flagi jakie musi miec obiekt, zeby byc branym pod uwage przy liczeniu kolizji
     * @return gdy nie ma kolizji to zwraca NULL
     * gdy jest kolizja to struktura Collision zawiera informacje o niej*/
    public final native Collision TraceObjectsWithFlagsExclude(Vector vStart, Vector vEnd, ControlObject Exclude, int iFlags);

    /**[native] Oblicza kolizje z drzewami dla promienia (vEnd - vStart).
     * @param vStart poczatek promienia dla ktorego liczymy kolizje (<code>Vector</code>)
     * @param vEnd koniec promienia (<code>Vector</code>)
     * @return gdy nie ma kolizji to zwraca NULL
     * gdy jest kolizja to struktura Collision zawiera informacje o niej*/
	public final native Collision TraceTrees(Vector vStart, Vector vEnd);
    
    /**[native] Oblicza kolizje z obiektami dla promienia (vEnd - vStart).
     * @param cColl struktura w ktorej umieszczone sa informacje o kolizji, moze byc null
     * @param vStart poczatek promienia dla ktorego liczymy kolizje (<code>Vector</code>)
     * @param vEnd koniec promienia (<code>Vector</code>)
     * @param iCOFlags flagi jakie musi miec obiekt, zeby byc branym pod uwage przy liczeniu kolizji
     * @param iSkipFaceFlags trojkaty z flagami podanymi w tym polu nie beda kolidowac (patrz EFaceFlags)
     * @return true gdy byla kolizja, vEnd jest ustawiana w miejscu kolizji (obcinanie trace'a) 
     */
    public final native boolean TraceObjectsWithFaceFlags(Collision cColl, Vector vStart, Vector vEnd, int iCOFlags, int iSkipFaceFlags);

    /**[native] Oblicza kolizje z obiektami dla promienia (vEnd - vStart).
     * @param cColl struktura w ktorej umieszczone sa informacje o kolizji, moze byc null
     * @param vStart poczatek promienia dla ktorego liczymy kolizje (<code>Vector</code>)
     * @param vEnd koniec promienia (<code>Vector</code>)
     * @param Exclude obiekt, ktorego nie sprawdzac przy kolizji (<code>ControlObject</code>)
     * @param iCOFlags flagi jakie musi miec obiekt, zeby byc branym pod uwage przy liczeniu kolizji
     * @param iSkipFaceFlags trojkaty z flagami podanymi w tym polu nie beda kolidowac (patrz EFaceFlags)
     * @return true gdy byla kolizja, vEnd jest ustawiana w miejscu kolizji (obcinanie trace'a) 
     */
    public final native boolean TraceObjectsWithFaceFlags(Collision cColl, Vector vStart, Vector vEnd, ControlObject Exclude, int iCOFlags, int iSkipFaceFlags);

    /**Zwraca kolizje z najblizsza przeszkoda (sprawdza teren, drzewa i obiekty)
     * @param vStart poczatek promienia dla ktorego liczymy kolizje (<code>Vector</code>)
     * @param vEnd koniec promienia (<code>Vector</code>)
     * @param Exclude obiekt, ktorego nie sprawdzac przy kolizji (<code>ControlObject</code>)
     * UWAGA!!! vEnd jest ustawiany na wartosc punktu kolizji !!! 
     */
    Collision TraceForCollision(Vector vStart, Vector vEnd, ControlObject cExclude)
    {
        Collision stCollTerrain=TraceTerrain(vStart, vEnd);

        Collision stCollTrees=TraceTrees(vStart, vEnd);

        Collision stCollObjects = null;
        if (cExclude != null)
            stCollObjects = TraceObjectsExclude(vStart, vEnd, cExclude);
        else
            stCollObjects = TraceObjects(vStart, vEnd);
        
        if(stCollObjects != null)
            return stCollObjects;
        if (stCollTrees != null)
            return stCollTrees;
        if (stCollTerrain != null)
            return stCollTerrain;
        
        return null;
    }
	
	
	/**[native]
	 * @param quality : 0 - normal, 1 - skipmip1, 2 - skipmip2, 3 - skipmip2 no detail
	 */
	final native void TerrainSetQuality( int quality );
	
	
    /** [native] Podaje wysokosc terenu w podanym punkcie.
     * @param fX wspolrzedna X terenu
     * @param fZ wspolrzedna Y terenu
     */
    final native float TerrainHeight(float fX, float fZ);

    
    /** [native] Podaje wysokosc terenu oraz normalna w podanym punkcie.
     * @param fX wspolrzedna X terenu
     * @param fZ wspolrzedna Y terenu
     * @param vNormal wektor, w ktorym po powrocie z metody bedzie normalna terenu w punkcie
     */
    final native float TerrainHeightNormal(float fX, float fZ, Vector vNormal);
    
    
    /** [native] Podaje wielkosc terenu
     * @return (fX, fZ) - rozmiar terenu, fY - wysokosc terenu
     */
    final native Vector GetTerrainSize();
    
    ///////////////////////////////////////////
    // metody do pobierania czasu			 //
    ///////////////////////////////////////////

	/**[native] Pobiera aktualny czas gry (czyli czas od poczatku gry minus przerwy. 
     */
    final native float GetTime();

    /**[native] Pobiera aktualny czas z licznika systemowego. Liczy w sekundach.
     * TO NIE JEST CZAS GRY ! Czas ten plynie ciagle tzn. nie jest stopowany przez
     * StopTime().
     */
    final native static float GetAbsoluteTime();

    /**[native] Ustawia maksymalny czas trwania ramki (w milisekundach) 
     * domysla wartosc 1000ms = 1s
     */
    final native void SetMaxTimerUpdate(int iMaxUpdate);
    
    /**Stara nazwa metody GetAbsoluteTime() zachowana dla zgodnosci z poprzednimi wersjami*/
    final static float GetTimeNow()
    {
        return GetAbsoluteTime();
    }
    
    /**Zwraca czas systemowy w milisekundach*/
    final static float GetSystemTime()
    {
        return (float)java.lang.System.currentTimeMillis();
    }
      
    /**
     * Ustawia mnoznik predkosci zegara globalnego
     */
    static native void SetGlobalTimerSpeedUp( float fSpeedUp );
    
    ////////////////////////////////
    // obsluga plikow skrytpowych //
    ////////////////////////////////
    
    /** [native] Parsuje podany plik i zwraca keyword do odczytu danych.
     * Po wczytwaniu nalezy sprawdzic czy nie zostal zwrocony null. Jesli tak to znaczy, ze nie 
     * bylo podanego pliku. Gdy w odczytany keyword jest poprawny to sprawdzamy czy nID w nim jest ==0.
     * Gdy jest znaczy to, ze plik zle sie sparsowal (blad skladni).
     * @param sFileName nazwa pliku do sparsowania
     * @return keyword do odczytywania danych, gdy =null to znaczy ze blad odczytu pliku lub nie ma pliku
     */
    final native static ParseKeyword ParseFile(String sFileName);
    final native static ParseKeyword ParseFileDef(String sFileName,String sDef);
    
    //////////////////////////////////////////////
	// metody obslugujace szukanie drogi dla AI //
	//////////////////////////////////////////////
	
    /**[native] Ustawia parametry opisujace prace pathfindera (reinicjalizacja)
     * @param nMaxSteps maksymalna ilosc krokow wykonywanych przy szukaniu sciezki (<code>int</code>)
     * @param bStraightLines true gdy znaleziona sciezka ma byc optymalizowana (linie tnace katy proste) (<code>boolean</code>)
     */
    final native void ConfigurePathFinder(int nMaxSteps, boolean bStraightLines);
    
    /**[native] Znajduje sciezke po ktorej AI moze przejsc od vStart do vEnd.
	 * @param vStart punkt startowy (<code>Vector</code>)
	 * @param vEnd punkt docelowy (tam AI ma dojsc) (<code>Vector</code>)
	 * @return tablica punktow tworzacych sciezke omijajaca przeszkody
	 */
    final native Vector [] FindPath(Vector vStart, Vector vEnd);

	/**[native] Mowi, czy dany punkt na pathmapie mozna przejsc (czy nie jest
	 * zajety np. przez AI, drzewo lub inna przeszkode)
	 * @param vPosition punkt do sprawdzenia (<code>Vector</code>)
	 * @return czy przez punkt mozna przejsc
	 */
	final native boolean IsPlaceMoveable(Vector vPosition);
	
	/**[native] Jak powyzszy, ale z wielkoscia stampa
	 * @param vPosition punkt do sprawdzenia (<code>Vector</code>)
	 * @param vPosition punkt do sprawdzenia (<code>int</code>)
	 * @return czy przez punkt mozna przejsc
	 */
	final native boolean IsPlaceMoveable(Vector vPosition, int iStampSize);

    
    /*************************************************
     * 
     *      Obsgluga sieci
     * 
     *************************************************/
    
    /**
     * System replikacji zmiennych 
     * 
     * Z definicji zmienna replikuje sie w trybie unreliable,
     * tzn nie jest gwarantowane "dostarczenie" wartosci
     * zmiennej na drugiego hosta
     * Do replikacji w innych trybach (reliable, always) sluza
     * osobne metody
     */
    
    /**[native] Rejestracja zbioru zmiennych w systemie
     * Dodaje zbior zmiennych podany w stringu do replikacji (domyslnym
     * trybem jest unreliable, tryby i parametry replikacji mozna zmieniac
     * matodami SetReplicationSet (patrz ponizej)
     * @param sVariableSet nazwy zmiennych oddzielane ';' lub ',', zmienne musza 
     * byc publiczne, wiecej informacji {@link <a href = sieci.html>tu</a>}
     * @return identyfikator zbioru zmiennych, wykorzystywany w 
     * metodzie {@link #Replicate(int nReplicationSet) lub metodach
     * ustalajacych parametry zaczynajacych sie od SetReplicationSet}
     */
    native int NewReplicationSet(String sVariableSet);
    
    /**[native] Definiuje tryb replikacji podanego seta
     * Ustawia set w trybie replikacji reliable
     * @param nSetId - identyfikator seta otrzymany przy rejestracji
     * za pomoca metody {@link #NewReplicationSet(String sVariableSet)
     */
    native void SetReplicationSetModeReliable(int nSetId);
    
    /**[native] Definiuje tryb replikacji podanego seta
     * Ustawia set w trybie replikacji always
     * Uwaga: tryb ten moze tez byc wymuszony za pomoca metody
     * {@link #NewReplicationSet(String sVariableSet))
     * @param nSetId - identyfikator seta otrzymany przy rejestracji
     * za pomoca metody {@link #SetReplicationSetMaxPeriod(int nSetId,float fMaxPeriod))
     */
    native void SetReplicationSetModeAlways(int nSetId);

    /**[native] Definiuje parametry replikacji podanego seta
     * Zmienne seta bedza replikowana nie rzadziej niz fMaxPeriod
     * Stosowac mozna takze w kombinacji z metoda 
     * {@link #SetReplicationSetMaxPeriodUnrelevantMul(int nSetId,float fUnrelevantMul))
     * Uwaga: Jesli ograniczona jest przepustowosc lacza, to nie kazdy 
     * obiekt moze byc brany pod uwage do replikacji przy kazdej ramce 
     * replikacji. To moze spowodowac, ze maksymalny czas pomiedzy replikacjami
     * zmiennych moze sie wydluzyc (do czasu az obiekt "dostanie" sie do replikacji)
     * @param nSetId - identyfikator seta otrzymany przy rejestracji
     * za pomoca metody {@link #NewReplicationSet(String sVariableSet)
     */
    native void SetReplicationSetMaxPeriod(int nSetId,float fMaxPeriod);
    
    /**[native] Definiuje parametry replikacji podanego seta
     * Rozszerzenie parametru ustalanego w metodzie 
     * {@link #SetReplicationSetMaxPeriod(int nSetId,float fMaxPeriod))
     * Zmienne seta bedza replikowana nie rzadzej niz fTime liczony
     * wg wzoru: 
     * fMaxPeriod + ( 1.0f - "aktualna relavantnosc" ) * fUnrelevantMul;
     * @param nSetId - identyfikator seta otrzymany przy rejestracji
     * za pomoca metody {@link #NewReplicationSet(String sVariableSet)
     */
    native void SetReplicationSetMaxPeriodUnrelevantMul(int nSetId,float fUnrelevantMul);

    
    /**[native] Definiuje parametry replikacji podanego seta
     * Zmienne seta bedza replikowana nie czesciej niz fMinPeriod
     * Stosowac mozna takze w kombinacji z metoda 
     * {@link #SetReplicationSetMinPeriodUnrelevantMul(int nSetId,float fUnrelevantMul))
     * Uwaga: Wyzszy priorytet ma ustawianie _maksymalnego czasu_ pomiedzy 
     * replikacjami, zatem jesli efektywnie wyliczone fMaxPeriod
     * bedzie mniejsze niz fMinPeriod to zmienne beda sie replikowac
     * czesciej niz to wynika z fMinPeriod
     * @param nSetId - identyfikator seta otrzymany przy rejestracji
     * za pomoca metody {@link #NewReplicationSet(String sVariableSet)
     */
    native void SetReplicationSetMinPeriod(int nSetId,float fMinPeriod);
    
    /**[native] Definiuje parametry replikacji podanego seta
     * Rozszerzenie parametru ustalanego w metodzie 
     * {@link #SetReplicationSetMinPeriod(int nSetId,float fMinPeriod))
     * Zmienne seta bedza replikowana nie czesciej niz fTime liczony
     * wg wzoru: 
     * fMinPeriod + ( 1.0f - "aktualna relavantnosc" ) * fUnrelevantMul;
     * @param nSetId - identyfikator seta otrzymany przy rejestracji
     * za pomoca metody {@link #NewReplicationSet(String sVariableSet)
     */
    native void SetReplicationSetMinPeriodUnrelevantMul(int nSetId,float fUnrelevantMul);
    

    /**[native] Replikuje zmienne z podanego zbioru jesli: 
     * - zmienne te zmienily swa wartosc od ostatniej replikacji oraz
     *   uplynal czas ustawiony dla seta przez metody SetReplicationSetMinPeriodXXX
     * - uplynal czas ustawiony dla seta przez metody SetReplicationSetMaxPeriodXXX
     * @param nSetId - identyfikator seta otrzymany przy rejestracji
     * za pomoca metody {@link #NewReplicationSet(String sVariableSet)
     */
    native void Replicate(int nSetId);
    
    /**[native] Replikuje zmienne z podanego zbioru niezaleznie od
    * tego czy zmienne te zmienily swa wartosc od ostatniej replikacji, czy nie
    * oraz ile czasu uplynely od poprzedniej replikacji.
    * @param nSetId - identyfikator seta otrzymany przy rejestracji
    * za pomoca metody {@link #NewReplicationSet(String sVariableSet)
    */
    native void ForceReplicate(int nSetId);

    
    /**[native] Zmienia ownera sieciowego obiektu
     * Domyslny owner jest ustawiany automatycznie przez engine podczas
     * tworzenia obiektu.
     * Ustawia ownera obiektu takiego jakiego ma obiekt podany w paremetrze<br>
     * @param cSrcOwner obiekt z ktorego mamy przepisac ownera (<code>GameObject</code>)
     * Uwaga! Nie w zadnym wypadku wolac tej metody podczas replikacji obiektow!!!
     */
    native void ChangeNetOwner(GameObject cSrcOwner);
    
    /**[native] Ustawia priorytet sieciowy klasy<br>
     * im wiekszy tym klasa czesciej niz inne
     * bedzie wysylala dane do sieci
     * @param fPrior nowy priorytet sieciowy (<code>float</code>)
     */
    native void SetNetPriority(float fPrior);
    
    /**[native] Ustawia minimalny poziom relewantnosci klasy<br>
     * @param fRelevantLevel nowa minimalna wartosc relewantnosci (<code>float</code>)
     */
    native void SetMinRelevantLevel(float fRelevantLevel);
    
    
    /**[native] SERVER i CLIENT 
     * Wyciaga z podanego seta obiektu sieciowego _czas z ktorego 
     * pochodzila_ ostatnia replikacja _przychodzaca_ z sieci  
     * Uwaga! jest to czas w ktorym na zdalneym hoscie replikacja
     * byla wysylana, a nie czas w ktorym doszla do lokalnego hosta!
     * @param identyfikator seta ktory nas interesuje
     * @return czas gry w ktorym nastapila replikacja - jesli jeszcze 
     * set nie zostal ani razu zreplikowany na tego hosta to zwraca -1.0f
     */
    native float GetLastIncomingReplicationTime( int nSet );
    
    /**
     * Wymuszenie oproznienia wszystkich buforow wychodzacych
     * czyli defacto wyslanie wszystkich oczekujacych tam danych
     */
    native void FlushReplicationManagerBuffers();
    
    
    /**[native] SERVER Wyciaga z repliacation managera
     * aktualnego pinga wskazanego klienta
     */
    native int  GetClientLatency( int nCliendId );
    
    /**
     * [native] SERVER Wywala z gry podanego klienta
     */
    native void ForceKickClient( int nCliendId );
    
    /**[native] SERVER 
     * Wymusza maksymalna ilosc danych wysylanych
     * do kazdego z klientow w kazdej sekundzie
     * Jesli podamy 0, to przepustowosc nie bedzie 
     * uwzgledniana (na razie bedzie wysylane "tyle ile sie da")
     */
    native void  SetClientsBandwidth( int nBytesPerSecond );
    /**
     * System komunikatow 
     * 
     * Uwaga, komunikaty sa z definicji reliable
     * zatem MessageNew tworzy komunikat reliable
     * natomiast do unreliable sluzy metoda MessageNewUnreliable
     */
    
    // metody pomocne przy tworzeniu i wysylaniu komunikatu
    
    /**[native] Tworzy nowy komunikat w trybie reliable
     * o podanym identyfikatorze
    * @param nId - identyfikator komunikatu (<code>int</code>)
    */
    native void MessageNew( int nId );
    
    /**[native] Tworzy nowy komunikat w trybie ureliable o 
     * podanym identyfikatorze
     * @param nId - identyfikator komunikatu (<code>int</code>)
     */
    native void MessageNewUnreliable( int nId );
    
    /**[native] Dodaje do komunikatu podana typu byte
    * @param byValue - wartosc dodawana do komunikatu (<code>byte</code>)
    */
    native void MessagePackByte( byte byValue );

    /**[native] Dodaje do komunikatu podana wartosc calkowita
    * @param nValue - wartosc dodawana do komunikatu (<code>int</code>)
    */
    native void MessagePackInt( int nValue );
    
    /**[native] Dodaje do komunikatu podana wartosc zmienno przecinkowa
    * @param fValue - wartosc dodawana do komunikatu (<code>float</code>)
    */
    native void MessagePackFloat( float fValue );
    
    /** 
     * Dodatkowe metody operujace na komunikatach
     */
    
    /* Vector pakowany jest jako 3 floaty w kolejnosci fX, fY, fZ */
    native void MessagePackVector( Vector v );
    native Vector MessageUnPackVector();
    
    final void MessagePackBoolean( boolean bValue )
    {
        MessagePackByte( bValue ? (byte)1 : (byte)0 );
    }
    
    final boolean MessageUnPackBoolean()
    { 
        return (MessageUnPackByte() != 0);
    }
    
    /**[native] Dodaje do komunikatu podana wartosc String
    * @param sText - wartosc dodawana do komunikatu (<code>String</code>)
    */
    native void MessagePackString( String sText );
    
    /** Dodaje do komunikatu podana wartosc String jednak
     * tylko wtedy, gey jest ona != null. 
    * @param sText - wartosc dodawana do komunikatu (<code>String</code>)
    */
    void MessagePackStringBool( String sText )
    {
        MessagePackBoolean( sText != null );
        if ( sText != null )
            MessagePackString( sText );
    }
    
    String MessageUnPackStringBool(String sNullReplace)
    {
        if ( MessageUnPackBoolean() )
            return MessageUnPackString();
        return sNullReplace;
    }
    
    void MessagePackStringBoolTable( String[] aTexts )
    {
        MessagePackStringBoolTable( aTexts, -1 );
    }
    void MessagePackStringBoolTable( String[] aTexts, int nMax )
    {
        int nCnt = ((aTexts != null)&&(aTexts.length > 0)) ? 
                   aTexts.length : 0;
        if ( (nMax > -1) && (nCnt > nMax) )
            nCnt = nMax;
        MessagePackInt( nCnt );
        for ( int i=0; i<nCnt; ++i )
            MessagePackStringBool( aTexts[ i ] );
    }
    
    String[] MessageUnPackStringBoolTable(String sNullReplace)
    {
        String [] aTexts = null;
        int nCnt = MessageUnPackInt();
        if ( nCnt > 0 ) 
        {
            aTexts = new String[ nCnt ];
            for ( int i=0; i<nCnt; ++i )
                aTexts[ i ] = MessageUnPackStringBool( sNullReplace );
        }
        return aTexts;
    }
    
    /**[native] Wysyla aktualny komunikat do managera replikacji,
     * nastepnie manager replikacji rozesle go na wszystkie inne maszyny na 
     * ktorych znajduje sie obiekt wysylajacy komunikat, tam z kolei
     * manager replikacji dostarczy komunikat do danego obiektu wywolujac
     * handler OnMessage()
     * MessageSend() powinno zawsze byc poprzedzone MessageNew( int nId )
     * i ewentualnie dowolna kombinacja: 
     * MessagePackInt( int nValue ); MessagePackFloat( float fValue ); itp
    */
    native void MessageSend();

    
    /**[native] Wysyla aktualny komunikat do managera replikacji,
     * nastepnie manager replikacji rozesle go na wszystkie inne maszyny na 
     * ktorych znajduje sie obiekt wysylajacy komunikat, tam z kolei
     * manager replikacji dostarczy komunikat do danego obiektu wywolujac
     * handler OnMessage()
     * MessageSend() powinno zawsze byc poprzedzone MessageNew( int nId )
     * i ewentualnie dowolna kombinacja: 
     * MessagePackInt( int nValue ); MessagePackFloat( float fValue ); itp
     * @param nSendType - sposob rozeslania komunikatu (<code>int</code>),
     *      EMsg.ALL        - 0 - 
     *      EMsg.OWNER      - 1 - 
     *      EMsg.NOTOWNER   - 2 - 
     *      EMsg.EXCLUSIVE  - 3 -
     * Uwaga, do wysylania w trybie EMsg.EXCLUSIVE sluzy osobna procka (patrz nizej)
     * Po stronie klienta mozna wysylac tyko do serwera (nie ma komunikacji
     * miedzy klientami!)
    */
    native void MessageSend(int nSendType);

    /** Dziala dokladnie tak jak MessageSend() z ta roznica, ze
     *  komunikat wysylany jest tylko do jednego (podanego) hosta
     * @param nHostId - identyfikator docelowego hosta (<code>int</code>),
     * 
     */
    native void MessageSendHost(int nHostId);
    
    /** Dziala dokladnie tak jak MessageSendHost z ta roznica, ze
     *  komunikat wysylany jest do wszystkich oprocz jednego (podanego) hosta
     * @param nHostId - identyfikator "zabronionego" hosta (<code>int</code>),
     */
    native void MessageSendNotHost(int nHostId);

    
    // metody przydatne przy obsludze przychodzacego komunikatu (w OnMessage() )
    
    /**[native] Pobiera z komunikatu jego identyfikator
    * @return - identyfikator komunikatu (<code>int</code>)
    */
    native int MessageGetId();
    
    /**[native] Pobiera z komunikatu identyfikator hosta ktory go wyslal
    * @return - identyfikator hosta ktory wyslal komunikat (<code>int</code>)
    */
    native int MessageGetSenderId();
    
    /**[native] Pobiera z komunikatu wartosc typu byte
    * @return - odczytana z komunikatu wartosc (<code>byte</code>)
    */
    native byte MessageUnPackByte();
    
    /**[native] Pobiera z komunikatu wartosc calkowita
    * @return - odczytana z komunikatu wartosc (<code>int</code>)
    */
    native int MessageUnPackInt();
    
    /**[native] Pobiera z komunikatu wartosc zmienno przecinkowa
    * @return - odczytana z komunikatu wartosc (<code>float</code>)
    */
    native float MessageUnPackFloat();
    
    /**[native] Pobiera z komunikatu wartosc String
    * @return - odczytana z komunikatu wartosc (<code>String</code>)
    */
    native String MessageUnPackString();
        
    /**[native] Powinno byc wolane na zakonczenie odbierania wiadomosci z messega.
     */
    native void MessageEnd();
    
    /**[native] Powinno byc wolane przed MessageEnd jak rezygnujemy 
     * z obslugi komunikatu bez odczuty danych z messega.
     */
    native void MessageSkipData();
    
    void MessageCancel()
    {
        LogR("Komunikat "+MessageGetId()+" obiektu "+this+" nie zostal obsluzony!\n");
        MessageSkipData();
        MessageEnd();
    }

    byte PackBooleanToByte(byte nFlags, int nBitNumber, boolean bValue)
    {
        byte nValue = bValue ? (byte)1 : (byte)0;
        if      (nBitNumber == 0)  nFlags |= nValue;
        else if (nBitNumber == 1)  nFlags |= (nValue * 2);
        else if (nBitNumber == 2)  nFlags |= (nValue * 4);
        else if (nBitNumber == 3)  nFlags |= (nValue * 8);
        else if (nBitNumber == 4)  nFlags |= (nValue * 16);
        else if (nBitNumber == 5)  nFlags |= (nValue * 32);
        else if (nBitNumber == 6)  nFlags |= (nValue * 64);
        else if (nBitNumber == 7)  nFlags |= (nValue * 128);
        return nFlags;
    }
    
    boolean UnpackBooleanFromByte(byte nFlags, int nBitNumber)
    {
        if      (nBitNumber == 0)  return ((nFlags & 0x01) != 0);
        else if (nBitNumber == 1)  return ((nFlags & 0x02) != 0);
        else if (nBitNumber == 2)  return ((nFlags & 0x04) != 0);
        else if (nBitNumber == 3)  return ((nFlags & 0x08) != 0);
        else if (nBitNumber == 4)  return ((nFlags & 0x10) != 0);
        else if (nBitNumber == 5)  return ((nFlags & 0x20) != 0);
        else if (nBitNumber == 6)  return ((nFlags & 0x40) != 0);
        else if (nBitNumber == 7)  return ((nFlags & 0x80) != 0);
        return false;
    }

    /** [native] klient:Startuje wyrownywanie czasu z serwera
     */
    native void StartCompensingTime();
    
    /** [native] klient:Wstrzymuje wyrownywanie czasu z serwera
     */
    native void StopCompensingTime();
    
    /** [native] klient:Informuje o tym czy aktualnie trwa komensacja
     */
    native boolean IsCompensingTime();
    
    /** [native] klient:Informuje o czasie do skompensowania
     */
    native float GetTimeToCompense();
    
    /** [native] klient:Informuje o poprzednim czasie do skompensowania
     * ktory zostal zlecony timerowi
     */
    native float GetLastTimeToCompense();
        
    /** [native] klient:Zwraca informacje czy w polaczeniu
     * z serwerem nie ma zadnych problemow
     */
    native boolean IsServerResponding();
        
    /** [native] serwer: Wlacza/wylacza obsluge relewantnosci obiektow
     * jesli true to serwer bedzie dla kazdego obiektu "ocenial" czy jest
     * on istotny dla klienta i na tej podstawie podejmowal decyzje
     * o spawnowaniu do na kliencie
     * jesli false to wszystkoe obiekty sieciowe domyslnie sa istotne
     * na wszystkich klientach
     */
    native void EnableCheckRelevant(boolean bEnable);
    
    /** [native] serwer: zwraca info czy obslugiwana jest relewantnosc obiektow
     */
    native boolean IsCheckRelevantEnabled();
    

    /**
     * [native] Wlaczenie/wylaczenie akceptowania przychodzacych 
     * relikacji zmiennych z sieci
     * Domyslnie true
     */
    native void EnableIncomingReplication(boolean bEnable);
    
    /**
     * [native] Sprawdzenie czy aktualnie sa  akceptowane 
     * przychodzace relikacje zmiennych z sieci
     * Domyslnie true
     */
    native boolean IsIncomingReplicationEnabled();
    
    // metody pomocne przy testowaniu sieci

    /**[native] Wysyla z javy komunikat do managera replikacji
     * W chwili obecnej metoda ta ma charakter roboczy i jest
     * wykorzystywana tylko do testow (np przelaczanie trybow
     * pracy rep mana, wyswietlanie logow o stanie obiektow
     * w rep manie)
     * @param nMsgId - identyfikator komunikatu
     * 0 - wyswietla liste obiektow sieciowych zarejestrowanych w rep manie (nazwa klasy, NetId i OwnerId)
     * 1 - j.w. z tym ze lista jest posortowana wg NetId obiektow
     * 2 - j.w. z tym ze lista jest posortowane wg OwnerId
     */
    native void ReplicationManagerMsg( int nMagId );


    /** Perform console command like in left tilde Java console in game
     */
    static native void PerformConsoleCommand( String sCommand );
    
    /**
     * Zwraca stringa z informacjami o obiekkcie 
     * przydatnymi do logowania, powinno zostac nadpisane
     * w klasach potomnych calem "uzupelnienia" informacji
     */  
    String GetNetInfo() 
    { 
        return ""+this+" NetId:"+GetNetId(); 
    }

    
    // metody pomocne przy profilowaniu sieci
    
    /** [native] klient/serwer: poczatek profilowanego kodu
     */
    native void ProfileNetJavaBegin();
    /** [native] klient/serwer: koniec profilowanego kodu
     */
    native void ProfileNetJavaEnd();
       
    
    // metody pomocne przy tworzeniu i wysylaniu komunikatu niezaleznych
    
    /** [native] Rejestruje obiekt, ktory pragnie odbierac 
     *  niezalezne komunikaty
     */
    native void RegisterMessager();
    
    /** [native] Usuwa rejestracje obieku, ktory odbieral
     *  niezalezne komunikaty
     */
    native void UnregisterMessager();

    /**[native] Tworzy nowy komunikat o podanym identyfikatorze
    * @param nId - identyfikator komunikatu (<code>int</code>)
    */
    native void FreeMessageNew( int nId );
    
    /**[native] Dodaje do komunikatu podana wartosc typu byte
    * @param byValue - wartosc dodawana do komunikatu (<code>byte</code>)
    */
    native void FreeMessagePackByte( byte byValue );

    /**[native] Dodaje do komunikatu podana wartosc calkowita
    * @param nValue - wartosc dodawana do komunikatu (<code>int</code>)
    */
    native void FreeMessagePackInt( int nValue );
    
    /**[native] Dodaje do komunikatu podana wartosc zmienno przecinkowa
    * @param fValue - wartosc dodawana do komunikatu (<code>float</code>)
    */
    native void FreeMessagePackFloat( float fValue );
    
    /**[native] Dodaje do komunikatu podana wartosc String
    * @param sValue - wartosc dodawana do komunikatu (<code>String</code>)
    */
    native void FreeMessagePackString( String sValue );
    
    void FreeMessagePackBoolean( boolean bValue )
    {
        FreeMessagePackByte( bValue ? (byte)1 : (byte)0 );
    }
        
    
    /**[native] Wysyla aktualny komunikat do Messager'a,
     * nastepnie nastepnie Messager wysle go zgodnie z podanymi warunkami
     * u odbircow zostanie on dostarczony wszystkim zarejestrowanym obiektom
     * poprze Messagera (wywolany zostanie handler OnFreeMessage())
     * MessageSend() powinno zawsze byc poprzedzone MessageNew()
     * i ewentualnie dowolna kombinacja: 
     * MessagePackInt( int nValue ); MessagePackFloat( float fValue ); itp
     * @param nSendType - sposob rozeslania komunikatu (<code>int</code>),
     *      EFreeMsg.TYPE_HOST      - 0 - wyslanie do okreslonego hosta
     *      EFreeMsg.TYPE_BROADCAST - 1 - rozeslanie w sieci lokalnej, parametr sHostName ignorowany
     *      EFreeMsg.TYPE_MULTICAST - 2 - rozeslanie do wszystkich zarejestrowanych hostow, parametr sHostName ignorowany
     * @param sHostName - nazwa hosta odbiorcy (<code>String</code>)
     * @param nPorte - wartosc portu docelowaego (<code>int</code>)
    */
    native void FreeMessageSend(int nSendType, String sHostName, int nPort);

    
    // metody przydatne przy obsludze przychodzacego komunikatu niezaleznego (w OnFreeMessage())
    
    /**[native] Pobiera z komunikatu jego identyfikator
    * @return - identyfikator komunikatu (<code>int</code>)
    */
    native int FreeMessageGetId();
    
    /**[native] Pobiera z komunikatu nazwe hosta nadawcy
    * @return - nazwa hosta nadawcy (<code>String</code>)
    */
    native String FreeMessageGetHost();
   
    /**[native] Pobiera z komunikatu port hosta nadawcy
    * @return - port hosta nadawcy (<code>int</code>)
    */
    native int FreeMessageGetPort();
    
    /**[native] Pobiera z komunikatu wartosc typu byte
    * @return - odczytana z komunikatu wartosc (<code>byte</code>)
    */
    native byte FreeMessageUnPackByte();

    /**[native] Pobiera z komunikatu wartosc calkowita
    * @return - odczytana z komunikatu wartosc (<code>int</code>)
    */
    native int FreeMessageUnPackInt();
    
    /**[native] Pobiera z komunikatu wartosc zmienno przecinkowa
    * @return - odczytana z komunikatu wartosc (<code>float</code>)
    */
    native float FreeMessageUnPackFloat();
    
    /**[native] Pobiera z komunikatu wartosc String
    * @return - odczytana z komunikatu wartosc (<code>String</code>)
    */
    native String FreeMessageUnPackString();
        
    boolean FreeMessageUnPackBoolean()
    { 
        return (FreeMessageUnPackByte() != 0);
    }
    /**[native] Powinno byc wolane na zakonczenie odbierania wiadomosci z messega.
     */
    native void FreeMessageEnd();
    
    /**[native] Do testowania gubienia pakietow w sieci. Ustala jaki procent pakietow
     * bedzie gubiony (pakiety nie zostana fizycznie wyslane w siec).
     */
    native void SetPacketLossPercent(int iPacketLossPercent);
    native void SetReadPacketLossPercent(int iPacketLossPercent);
    
    /**[native] For testing purpourse. Adds latency to server and client
     * packet receive/send thread.
     */
    native void SetAdditionalLatency(float fAddLatency);
    
    /**[native] Pobiera z replication identyfikator aktualnie replikowanego
     * klienta
     * jesli nie mamy aktualnie replikacji lub cos sie nie 
     * powiedzie to zwraca -1
     */
    native int GetCurrentClientHostId();
    
    /**
     * System kamer do oceny relevantnosci (istotnosci) obiektow
     * 
     * Uwaga! Na razie TO NIE DZIALA!
     * W przyszlosci bedzie to tak, ze serwer bedzie ocenial
     * ktore obiekty sa istotne dla poszczegolnych klientow
     * i na tej podstawie beda replikowane (bedzie nie) obiekty
     * w sieci
     */
  
    /**
     * Metoda zwraca informacje czy obiekt ma stala relevantnosci
     * Jesli tak to wpisywana mu jest relevantnosc 1.0f
     * TODO: zastanowic sie czy przyda sie w przyszlosci 
     * zeby obiekt mial relevantnosc stala ale < 1.0f;
     */
    public boolean IsRelevantConstans(){ return true; }
    
    /**[native] Ustawia na serwerze (wpisuje do replication managera) 
     * Obiekt glowny klienta o identyfikatorze nId
     * Obiektem tym moze byc Player (aktualnie tak jest i liczymy
     * na podstawie tego odleglosci od obiektow) lub najlepiej kamera
     * dzieki ktorej bedziemy mogli okreslic zakres widocznosci klienta (w przyszlosci
     * tak bedzie zrobione)
     */
    native void SetClientMainObject( int nId, ControlObject obj );
             
    
    /**[native] Pobiera z replication managera pozycja aktualnie replikowanego
     * klienta
     * W przyszlosci najprawdopodobniej nie bedzie to raczej wykorzystywane 
     * bo bedziemy oceniac widocznosc obiektu na podstawie kamery
     */
    native Vector GetCurrentClientPosition();
    
    /** Pobiera odleglosc obiektu od glownego obiektu (kamery?) aktualnie
     * przetwarzanego klienta, 
     * metoda do wykorzystania na serwerze w metodzie GetRelevantLevel()
     * podczas sprawdzania czy obiekt sieciowy jest w poblizu aktualnie 
     * przetwarzanego klienta
     * @return zwraca odleglosc od klienta (<code>float</code>)
     * dla obiektu ktory nie jest ControlObjectem zwroci -1.0f
     * W przyszlosci najprawdopodobniej nie bedzie to raczej wykorzystywane 
     * bo bedziemy oceniac widocznosc obiektu na podstawie kamery
     */
    native float GetCurrentClientDistance();
    
    /**[native] Ustawia na serwerze (wpisuje do replication managera) 
     * kamere, przez ktory patrzy klient o identyfikatorze nId
     */
    native void SetClientCamera( int nId, GameObject camera );
    
    /***
     * 
     * Zakladanie gry i podlaczanie sie do gry
     * 
     */
	
	/**[native] Tworzy serwer o podanej nazwie, numerach portow ( do danych gry 
	 * i komunikacyjny) oraz maksymalnej liczbie kilentow
	 * @param strName nazwa serwera (<code>String</code>)
	 * @param nGamePort numer portu danych gry (<code>int</code>)
	 * @param nMaxConnections maksymalna liczba klientow (<code>int</code>)
	 * @param strPassword haslo gry sieciowej (<code>String</code>)
	 * @param nLANServer okresla, czy server jest tylko dla klientow z LAN (1 = TAK, 0 = NIE) (<code>int</code>)
     * @return powodzenie akcji utorzenia serwera (1 = TAK, 0 = NIE) (<code>int</code>)
	 * */
	final native static int CreateServer( String strName, int nGamePort, int nMaxConnections, String strPassword, int nLANServer );
	
    /**[native] Ustawia stan servera w kodzie w CPP
     * @param iServerState - stan servera do ustawienia (<code>int</code>)
     * @param sMap - aktualna mapa na serwerze (<code>String</code>)
     */
    final native static void SetServerState(int iServerState, String sMap);
    
    /**[native] Sprawdzenie, czy jest sens wisiec z zaladowana mapa w oczekiwaniu
     * na wejscie do gry. Tutaj tylko wyslanie zapytania, potem trzeba skorzystac
     * z procki CheckServerState
     * @param sMap - mapa, ktora client ma zaladowana (<code>String</code>)
     */
    final native static void CheckServerStateStart(String sMap);
    
    /**[native] Sprawdzenie, czy przyszla odpowiedz, na nasze OSTATNIE zapytanie 
     * dokonane przy pomocy CheckServerStateStart
     * @return odpowiedz servera (0 = NIE, nie przylaczysz sie do gry, 1 = OK - czekaj na dolaczenie, 2 = WAITING - jeszcze server nie odpowiedzial) (<code>int</code>)
     */
    final native static int CheckServerState();
    
    /**TODO COM
     */
	final native static void ShutdownServer();
    
	/**[native] Tworzy klienta.	
    * @return powodzenie akcji utorzenia klienta (1 = TAK, 0 = NIE) (<code>int</code>)
    * */
	final native static int CreateClient(int nGamePort);

	/**[native] Usuwa klienta.*/
	final native static void DeleteClient();

	/**[native] Kontrola schizofrenii. Jestem klientem.*/
	final native static boolean AmIClient();

	/**[native] Kontrola schizofrenii. Jestem serwerem.*/
	final native static boolean AmIServer();

	/**[native] Wstepna inicjalizacja klienta - tworzy jego obiekt i zapamietuje
	 * w nim podstawowe parametry polaczenia z serwerem
	 * @param strAddr adres serwera (<code>String</code>)
	 * @param nGamePort numer portu danych gry (<code>int</code>)
     * @return powodzenie akcji tworzenia hosta (1 = TAK, 0 = NIE) (<code>int</code>)
     * */
	final native static int InitClientHost( String strAddr, int nGamePort);
    
    /**[native] Proste ustawienie parametrow polaczenia z serwerem. Nie tworzy obiektu klienta
     */
    final native static void SetClientConnectionData(String strAddr, int nGamePort);
	
	/**[native] Nawiazanie polaczenia z serwerem
     * @param strMapFilePath sciezka do mapy, ktora jest grana na serwerze (<code>String</code>)
     * @param strPassword haslo dostepu do danej gry sieciowej (<code>String</code>)
 	 * @param iWishListQueryNum numer zapytania, pozwalajacy potem serwerowi zweryfikowac odpowiedz (<code>int</code>)
	 * @param iCRC wyliczona suma CRC, o ktora bylo zapytanie (<code>int</code>)
	 * @return zwraca, czy rozpoczety zostal proces laczenia z serwerem (1 = TAK, 0 = NIE) (<code>int</code>)
	 */
	final native static int Join(String strMapFilePath, String strPassword, int iWishListQueryNum, int iCRC);
	
	/**[native] Zwraca stan laczenia z serwerem
	 * @return zwracana wartosc interpretowac jako pola obiektu EConnectingStatus
	 */
	final native static int GetConnectingStatus();
    
    /**TODO COM
     */
	final native static void DisconnectFromServer();

	/**[native] Ustawia podana przepustowosc sieci.
	 * @param nBytePerSec przepustowosc sieci w bajtach na sekunde
	 * */
	final native static void SetNetworkBandwidth(int nBytePerSec);
    
	/**[native] Zwraca przepustowosc sieci. 
	 * @return przepustowosc sieci w bajtach na sekunde. -1 gdy brak aktywnego hosta (odpowiednio clienta lub servera)
	 * */
	final native static int GetNetworkBandwidth();
    
    /**[native] Zwraca IP obiektu serwera albo klienta
     * @return IP serwera albo klienta, pusty ciag, jesli nie ma zadnego hosta
     */
    final native static String GetLocalHostIP();
    
    /**[native] Sends signal from Client to Server that client is alive and
     * client can be disconnected if he's not responding
     */
    final native static void SendClientAliveSignal();
    
    final native static void EnableLogSum(boolean bEnable);
    
    final native static String GetSelectedNetworkInterface();

    /** Gets disconect time set in Network.scr script.
     * @return time in seconds
     */
    public static native float GetDisconnectTime();
	
    /** Gets packet loss value. For server always returns 0.f. Should be
     * called for some GameObject.
     * @return packet loss in percent
     */
    public native float GetPacketLoss();
    
    /** Gets packet loss value for specified host.
     * Works only on server. On client returns 0. Should be called for
     * some GameObject.
     * @param nHostID id of host to check its packet loss
     * @return packet loss in percent
     */
    public native float GetPacketLoss(int nHostID);
    
    /*************************************************
     * 
     *      Obsgluga sieci - koniec
     * 
     *************************************************/

    
    ///////////////////////////////////////////
    // obsluga sprite'ow na domyslnym levelu //
    ///////////////////////////////////////////
    
    /**[native] Dodaje {@link Sprite} do modulu do najwyszego 
	 * poziomu w hierarchii
	 * @param sprite dodawany sprite (<code>Sprite</code>)
	 * @see #RemoveSprite(Sprite sprite)*/
	native void AddSprite(Sprite sprite);

	/**[native] Usuwa sprite z modulu.
	 * @param usuwany sprite (<code>Sprite</code>)
	 * @see #AddSprite(Sprite sprite)
	 * */
	native void RemoveSprite(Sprite sprite);

    /**[native] Wymusza czyszczenie ekranu przez sprite managera */
	public native final void SpriteClearScreen(boolean bClear);
    
    /**[native] Ustawia kolor uzywany przy czyszczeniu ekranu przez sprite manager */
	public native final void SpriteClearColor(float r, float g, float b);
    
    public native final void SetNumSpriteSurfaces(int iNumSurfaces);
    
    //////////////////////
    // obsluga dzwiekow //
    //////////////////////
    
    /**[native] Pobranie identyfikatora dzwieku. Powoduje wczytanie dzwieku z dysku.
     * Identyfikator ten nie ma nic wspolnego z identyfikatorem dzwieku zwracanym przez
     * metode ControlObject.AttachSound !!!!!!!
     * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
     * @return identyfikator dzwieku
     */
    //native public int GetSoundID(String sSoundName); //NOT IMPLEMENTED
    
    /**[native] Kasowanie dzwieku z pamieci o podanym ID.
     * @param nSoundID identyfikator dzwieku zwrocony przez GetSoundID
     */
    //native public void ReleaseSound(int nSoundID); //NOT IMPLEMENTED
    
    /**[native] Powoduje odegranie jednorazowego dzwieku
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
	 */
	native public void PlaySound(String sSoundName);

    /**[native] Powoduje odegranie jednorazowego dzwieku
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
	 * @param fMin odleglosc od emitera dzwieku po ktorej zacznie dzwiek wygasac
     * @param fMax odleglosc od emitera dzwieku po ktorej dzwieku nie slychac
	 */
	native public void PlaySound(String sSoundName, float fMin, float fMax);
    
    /**[native] Powoduje odegranie jednorazowego dzwieku z podana glosnoscia
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
	 * @param fVolume glosnosc dzwieku
	 */
	native public void PlaySound(String sSoundName, float fVolume);

    /**[native] Powoduje odegranie jednorazowego dzwieku na podanej pozycji
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
	 * @param vPos pozycja w swiecie
	 */
	native public void PlaySound(String sSoundName, Vector vPos);
    
    /**[native] Powoduje odegranie jednorazowego dzwieku na podanej pozycji
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
	 * @param vPos pozycja w swiecie
	 * @param fMin odleglosc od emitera dzwieku po ktorej zacznie dzwiek wygasac
     * @param fMax odleglosc od emitera dzwieku po ktorej dzwieku nie slychac
	 */
	native public void PlaySound(String sSoundName, Vector vPos, float fMin, float fMax);

    void PlaySoundAtTime(String sSoundName, Vector vPos, float fTime)
    {
        //TODO: uwzglednic czas!!!! takze w pozostalych funkcjach PlaySound
        PlaySound( sSoundName, vPos );
    }
    

    /**[native] Powoduje odegranie jednorazowego dzwieku z podana glosnoscia i pozycja
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
	 * @param fVolume glosnosc dzwieku
	 * @param vPos pozycja w swiecie
	 */
	native public void PlaySound(String sSoundName, float fVolume, Vector vPos);

    /**[native] Kasuje z pamieci dzwiek odegrany za pomoca PlaySound, AttachSound itp.
     * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
     */
    native public void PurgeSound(String sSoundName);
    
    /**[native] Powoduje zaladowanie pliku z szablonami dzwiekow
	 * @param sSoundIniFile plik z parametrami dzwiekow
	 */
	native public void LoadSoundTemplates(String sSoundIniFile);

    /**[native] Ustawia zasiegi slyszalnosci dzwieku
	 * @param fMin do tej odleglosci slychac z pelna glosnoscia
	 * @param fMax powyzej tej odleglosci dzwieki sa wyciszone
	 * @return false jesli ustawienie sie powiodlo
	 */
	native public boolean SetDefaultSoundRanges(float fMin, float fMax);

    /**[native] Odtwarza wave'a
	 * @param sWaveFile - plik z dzwiekiem w formacie wave
	 * @return identyfikator wave'a
	 */
    native public int PlayWave(String sWaveFile);

    /**[native] Odtwarza wave'a od podanej pozycji
	 * @param sWaveFile - plik z dzwiekiem w formacie wave
	 * @param lPos - pozycja od ktrej naley odtwarza dzwiek
	 * @return identyfikator wave'a
	 */	
	native public int PlayWaveFromPos(String sWaveFile, long lPos);
	
    /**[native] Pobiera pozycj odtwarzania wave'a
	 * @param iWaveID - identyfikator wave'a
	 */	
	native public long GetPlayingWavePos(int iWaveID);

    /**[native] Pobiera pozycje odtwarzania wave'a (w sekundach)
     * @param iWaveID - identyfikator wave'a
     */
    native public double GetPlayingWaveTimePos(int iWaveID);
    
    /**[native] Pobiera pozycj odtwarzania wave'a
	 * @param iWaveID - identyfikator wave'a
	 * @param lPos - pozycja od ktrej naley odtwarza dzwiek
	 */		
	native public void SetPlayingWavePos(int iWaveID, long iPos);	
    
    /**[native] Zwraca dlugosc wave'a w jednostkach pozycji
	 * @param iWaveID - identyfikator wave'a
	 */		
	native public long GetPlayingWaveLen(int iWaveID);
    
    /**[native] Zatrzymuje odtwarzanie wave'a, identyfikator przestaje byc wazny (dzwiek
     * jest kasowany).
	 * @param iWaveID identyfikator wave'a
	 */
    native public void StopWave(int iWaveID);

    /**[native] Pauzuje wave'a, mozna go potem odpauzowac (UnPauseWave)*/
    native public void PauseWave(int iWaveID);
    
    /**[native] Odpauzowywuje spauzowanego wave'a*/
    native public void UnPauseWave(int iWaveID);
    
    /**[native] Sprawdzenie czy podany wave odgrywa sie
     * @param iWaveID identyfikator wave'a
     */
    native public boolean IsPlayingWave(int iWaveID);
    
    /**[native] Ustawia glosnosc wave'a
	 * @param iWaveID identyfikator wave'a
	 * @param fVolume glosnosc (0 do 1)
	 */
    native static public void SetWaveVolume(int iWaveID, float fVolume);
   
    /**[native] Zwraca glosnosc wave'a
	 * @param iWaveID identyfikator wave'a
	 * @return glosnosc (0 do 1)
	 */
    native static public float GetWaveVolume(int iWaveID);
    
    /**[native] Pobiera dlugosc wave'a w sekundach
	 * @param iWaveID identyfikator wave'a
	 * @return dlugosc w sekundach
	 */
    native static public float GetWaveDuration(int iWaveID);
    
    /**[native] Pobiera dlugosc wave'a w sekundach
	 * @param sWaveFile plik z dzwiekiem w formacie wave
	 * @return dlugosc w sekundach
	 */
    native public float GetWaveDuration(String sWaveFile);

    /**[native] Odtwarza muzyke z wave'a (moze grac tylko jedna na raz)<BR>
     * UWAGA!!! Trzeba koniecznie zawsze wylaczac muzyke przez StopMusic
	 * @param sMusicFile plik z dzwiekiem w formacie wave (moze byc skompesowany np. mp3)
	 */
    native public void PlayMusic(String sMusicFile);
    
    /**[native] Zatrzymuje odtwarzanie muzyki z wave'a
	 */
    native public void StopMusic();

    /**[native] Ustawia rodzaj srodowiska dla dzwieku (EAX)
     * @param iPreset - srodowisko (ESoundEnv)
     * @param fMorphTime - czas przejscia do danego srodowiska, gdy rowny 0
     * srodowisko ustawiane jest natychmiast
     * @returns - poprzednio ustawione srodowisko
     */
    native public int SetSoundEnvironment(int iPreset, float fMorphTime);

    /** [native] Zapisuje stan sound mixera */
    native public void SoundSaveState(FileChunk fileChunk);
    /** [native] Odczytuje stan sound mixera */
    native public void SoundLoadState(FileChunk fileChunk);
    
    ////////////////////
    // Wl/wyl kolizji //
    ////////////////////
    
    /**[native] Globalnie wlacza / wylacza kolizje z terenem przy chodzeniu
     * (wlasciwie przy przesuwaniu elipsoid)
	 * @param bOnOff true - wlaczone, false - wylaczone
	 */
    native public void TerrainBlocksMove(boolean bOnOff);
    
    /**[native] Globalnie wlacza / wylacza kolizje z drzewami przy chodzeniu
     * (wlasciwie przy przesuwaniu elipsoid)
	 * @param bOnOff true - wlaczone, false - wylaczone
	 */
    native public void TreesBlockMove(boolean bOnOff);
    
    /**[native] Globalnie wlacza / wylacza kolizje z obiektami przy chodzeniu
     * (wlasciwie przy przesuwaniu elipsoid)
	 * @param bOnOff true - wlaczone, false - wylaczone
	 */
    native public void ObjectsBlockMove(boolean bOnOff);

    /**[native] Liczniki faceow, updejtowane w kazdej ramce renderowania.
     */
    native public int GetFacesTerrain();
    native public int GetFacesTrees();
    native public int GetFacesGrass();
    native public int GetFacesObjects();
    native public int GetFacesObjectsNoLOD();
    
	
    /**[native] Zmienia pozycje kursora na ekranie, ale nie zmienia pozycji tekstury
     * kursora, nie nalezy wiec wolac tej funkcji, ale {@link Cursor#SetPos(Vector vPos)}
     * @param vPos nowa pozycja kursora, (fX i fY)*/
    native public void SetCursorPos(Vector vPos);
            
    
    // video
    
    native public String EnumResolution(int iResolutionMode, int iBPP);
    native public boolean IsResolutionPresent(int iWidth, int iHeight, int iBPP);
    native public boolean Is16bppPresent();
    native public boolean Is24bppPresent();
    
    native public void SetResolution(int iWidth, int iHeight);
    native public void SetResolutionMode(int iResolutionMode, int iBPP);
    native public void SetBPP(int iBPP);
    native public void SetTextureBPP(int iTexBPP);
    native public void SetFXQuality(int iFXQ);
    native public void SetMaterialQuality(int iMatQ);
    native public void SetTextureQuality(int iTexQ);
    native public void SetTextureFiltering(int iTexFiltering);
    native public void SetFullscreen(boolean bFullscreen);
    native public void SetLowestDisplayFreq(boolean bLowestFreq);
    native public void SetGamma(int iGamma);
    native public void SetShadows(int iShadows);

    native public int GetResolutionMode();
    native public int GetBPP();
    native public int GetTextureBPP();
    native public int GetFXQuality();
    native public int GetMaterialQuality();
    native public int GetTextureQuality();
    native public int GetTextureFiltering();
    native public boolean GetFullscreen();
    native public int GetGamma();
    native public int GetShadows();
    
    native public void ApplyVideoSettings();
    native public void SaveVideoSettings(String sConfigFile);

    // audio
    
    native public void SetAudioAPI(int iAPI);
    native public void SetAudioMixFrequency(int iMixFrequency);
    native public void SetAudioChannels(int iChannels);
    native public void AudioUseEAX(boolean bUseEAX);
    /**[native] Ustawia zmienna typu bool w ustawieniach audio gry, mozna to wykorzystac
     * do zapisywania dodatkowych ustawien dzwieku. W simvivorze zapisywane tu jest czy
     * maja sie odgrywac dzwieki newsow, po stronie c efektem jest jedynie zapisanie zmiennej, 
     * obsluge trzeba zrobic w javie*/
    native public void AudioSetExtraSounds01(boolean bPlayExtraSounds01);

    native public int GetAudioAPI();
    native public int GetAudioMixFrequency();
    native public int GetAudioChannels();
    native public boolean IsAudioUsingEAX();
    
    /**[native] Pobiera wartosc zmiennej typu bool z ustawien audio, zmienna ta jest zapisywana
     * w pliku. Moze sluzyc do pamietania dowolnej opcji audio. Obsluga musi byc po stronie javy*/
    native public boolean IsAudioPlayingExtraSounds01();
    
    /**[native] Ustala wartosc glosnosci dla efektow dzwiekowych (odpalanych przez PlaySound(...))
     * @param iVolume glosnosc 1 - 10*/
    native public void SetAudioFXVolume(int iVolume);
    /**[native] Pobiera wartosc glosnosci efektow dzwiekowych*/
    native public int GetAudioFXVolume();
    
    /**[native] Ustala wartosc glosnosci dla wave'ow (odpalanych przez PlayWave(...))
     * (streaming audio), takze glosnosc wave'ow mp3 i w filmach avi
     * @param iVolume glosnosc 1 - 10*/
    native public void SetAudioWaveVolume(int iVolume);
    /**[native] Pobiera wartosc glosnosci wave'wow*/
    native public int GetAudioWaveVolume();
    
    native static public void SetAudioCDVolume(int iVolume);
    native static public int GetAudioCDVolume();

    native static public void ForceNoEAX();
    
    /**[native] Ustawia zmiany po stronie c. 
     * @return czy zmiany wymagaly przeinicjowania systemu audio (wtedy kasowane sa wszystkie dzwieki)*/
    native public boolean ApplyAudioSettings();
    
    native public void SaveAudioSettings(String sConfigFile);

    /**[native] Ustawia czulosc myszy, po stronie c tylko dla sprite'ow (i potomkow),  
     * dla innych obiektow nalezy dzialac recznie w javie pobierajac modyfikator 
     * czulsosci funkcja GetControllerMouseSensitivityModifier()
     * @param nSensitivity wartosc od 1 do 10, 5 to ustawienie neutralne*/
    native public void SetControllerMouseSensitivity(int nSensitivity);
    
    /**[native] Pobiera czulosc myszy
     * @see #SetControllerMouseSensitivity(int nSensitivity)*/
    native public int GetControllerMouseSensitivity();
    
    /**[native] Pobiera modyfikator czulosci myszy*/
    native public float GetControllerMouseSensitivityModifier();
    native public void SaveControllerSettings(String sConfigFile);
	// sprite manager

    /** Tworzy font z TTFa o nazwie sTTFName i zapamietuje go pod nazwa sFontID
     * @param sFontID nazwa identyfikujaca fonta
     * @param sTTFName nazwa fonta TTF
     * @param iHeight wysokosc fonta
     * @param nStyle styl (kombinacja stalych STYLE_... z ESpriteText)
     * @param sFormatFile sciezka do pliku z literami do wygenerowania (jesli font jeszcze nie istnieje), jesli null, to data/panel/format.dat
     * @param bOldStyle jesli true, to wielkosc czcionki nie jest przeliczana tak jak trzeba, tylko zgodnie ze starym systemem (gry przed fim speedway)
     * @return true jesli font zostal utworzony
    */
    native public boolean LoadFont( String sFontID, String sTTFName, int iHeight, int nStyle, String sFormatFile, boolean bOldStyle);

    boolean LoadFont( String sFontID, String sTTFName, int iHeight, int nStyle)
    {
        return LoadFont( sFontID, sTTFName, iHeight, nStyle, null, false);
    }
    
    boolean LoadFont( String sFontID, String sTTFName, int iHeight)
    {
        return LoadFont( sFontID, sTTFName, iHeight, 0, null, false);
    }
    
    /** Usuwa font o id sFontID
     * @param sFontID nazwa identyfikujaca fonta
     */
	native public void ReleaseFont( String sFontID );

    /** Usuwa z pamieci teksture fonta. Tekstura zostanie doladowana nastepnym
     * razem gdy bedzie pisany tekst tym fontem.
     * @param sFontID nazwa identyfikujaca fonta
     */
    native public void ReleaseFontTexture( String sFontID );
    
    /** Usuwa z pamieci tekstury wszystkicgh fontow. Dla kazdego fonta jego
     * tekstura zostanie doladowana nastepnym razem gdy bedzie pisany tekst
     * tym fontem.
     */
    native public void ReleaseFontsTextures();
    
    /** Zmienia szerokosc znaku w podanym foncie.
     * @param sFontID nazwa identyfikujaca fonta
     * @param nCharacter znak dla ktorego zmienic szerokosc
     * @param nWidth nowa szerokosc znaku
     */
    native public void ForceFontCharacterWidth(String sFontID, int nCharacter, int nWidth);
    
    /////////////////////////////////////
    // Blokowanie przerysowywania gry
    /////////////////////////////////////
    
    /** Blokowanie przerysowywania gry. Sluzy do wykonywania operacji na enginie gry z watkow
     * innych niz gra np. okien Swinga itp.
     * Jesli masz watpliwosci co do dzialania tej metody i nie wiesz o co chodzi,
     * NIE WOLAJ tego nigdzie w grze!!!!
     */
    native public boolean BlockGameRendering();
    
    /** Odblokowanie przerysowywania gry po uzyciu metody BlockGameRendering().
     */
    native static public void UnblockGameRendering();
    
    /////////////////////////////////////
    // cd audio
    /////////////////////////////////////
    
    /**[native] Sprawdza, czy w napedzie CD jest plyta o podanej nazwie
     * @param sCDName nazwa cd
     * @return true gdy plyta o podanej nazwie jest w napedzie*/
    native public boolean CDCheckForCD(String sCDName);
    
    /**[native] Zwraca true jesli CD jest poprawnie zainicjalizowany*/
    native public boolean CDIsInitialized();
    
    /**[native] Inicjalizuje system CD*/
    native public void CDInitialize();

    /**[native] Dodaje sciezke do podanej grupy
     * @param sGroupName nazwa grupy sciezek
     * @param iTrack numer sciezki (numeruje sie od 1)*/
    native public void CDAddTrack(String sGroupName, int iTrack);

    /**[native] Odtwarza grupe sciezek o podanej nazwie*/
    native public void CDPlayGroup(String sGroupName);
    /**[native] Zatrzymuje odtwarzanie*/
    native public void CDStop();
    /**[native] Wstrzymuje odtwarzanie*/
    native public void CDPause();
    /**[native] Wznawia odtwarzanie po wstrzymaniu*/
    native public void CDResume();

    /**[native] Rejestruje obiekt na rzecz ktorego bedzie wolany handler
     * OnCDBadStatus(int iStatus), powiadamiajacy np. o wyjeciu CD podczas grania*/
    native public void CDRegisterNotificationObject(GameObject NotifyObj);

    
    
    native public boolean InitGLRenderer(int iFlags);
    
    /**[native] Wylaczenie ladowania drzew w calej grze. Przydatne w celu
     * przyspieszenia wczytywania leveli.
     */
    native public void DisableTrees();
    
    /**[native] Wlacza zgrywanie filmu do avika otwartego metoda EnableRecordingAVI
     */
    native public void StartAVIRecording();
    
    /**[native] Wylacza zgrywanie filmu do avika otwartego metoda EnableRecordingAVI
     */
    native public void StopAVIRecording();
    
	/**[native] Wlacza nagrywanie inputow (powinno byc wczesniej ustawione stale ziarno)
	 * @param sFilename - nazwa pliku z inputami
	 */
	native void StartInputRecording(String sFileName);
	
	/**[native] Zatrzymuje zapis inputow
	 */
	native void StopInputRecording();
	
	/**[native] Wlacza odczyt inputow z dysku
	 * @param sFileName = nazwa pliku z zapisanymi inputami
	 */
	native void StartInputPlaying(String sFileName);
	
	/**[native] Wylacza odczyt inputow z dysku	 
	 */
	native void StopInputPlaying();
	
	
	/**[native] ustawia flage zezwalajaca na wyswietlanie lightmap
	 * nalezy ustawiac te flage przed zaladowaniem levelu
	 * @param enable true - rysuj lightmapy
	 */
	native void SetLightmapEnable( boolean enable );
	
	/**[native] pobiera flage okreslajaca czy rysuja sie lightmapy
	 */
	native boolean GetLightmapEnable();
	
	/**[native] zamyka manager lightmap, wolac ta metode jedynie gdy nie ma juz w systemie 
	 * zaladowanych meshy z lightmapi
	 */
	native void ShutdownLightmapMgr();

	/**[native] Powoduje zuploadowanie geometrii i tekstur do pamieci karty
	 */
	native void Precache();

    /**[native] Umozliwia bezwgledne wylaczenie renderowania traw.
     * Zrobione na potrzeby Speedwaya
     * @param bRender okresla czy trawy maja sie renderowac
     */
    native void SetRenderGrass(boolean bRender);
	
    /**[native] Powoduje, ze renderowane sa tylko sprite'y */
    final native void SetRenderSpritesOnly(boolean bEnable);
    
    /**[native] Zwraca true jesli renderowane sa tylko sprite'y */
    final native boolean GetRenderSpritesOnly();
    
    /**[native] Wlacza rendering w grze */
    final native void EnableGameRendering(boolean bEnable);
    
    /**[native] Zwraca true jesli rendering w grze jest wlaczony */
    final native boolean IsGameRenderingEnabled();
    
    /** Klasa umozliwia pobieranie wartosci losowych */
    private static java.util.Random cRand = new java.util.Random(0);
    
    /** Ustawia wartosc ziarana generatora na bierzacy czas.
     */
    public static void ResetRandom()
    {
        cRand.setSeed((long)GetAbsoluteTime());
    }
    
    /** Zwraca losowa liczbe z zakresu od 0 do nMax
     */
    public static int RandomInt(int nMax)
    {
        //dodatkowe zabezpiecznie przed zarowym zakresem
        if ( nMax == 0 ) 
            return 0;
        int nRand = cRand.nextInt();
        if (nRand < 0) 
            nRand = -nRand;
        return nRand%nMax;
    }

    /** Zwraca losowa liczbe z zakresu od nMin do nMax
     */
    public static int RandomInt(int nMin, int nMax)
    {
        int nRand = cRand.nextInt();
        if(nRand < 0) nRand = -nRand;
        if (nMin == nMax)
            return nMin;
        else
            return nMin + nRand % (nMax - nMin);
    }

    /** Zwraca losowa liczbe z zakresu od 0 do fMax
     */
    public static float RandomFloat(float fMax)
    {
        return cRand.nextFloat() * fMax;
    }
	
    /** Zwraca losowa liczbe z zakresu od fMin do fMax
     */
    public static float RandomFloat(float fMin, float fMax)
    {
        return fMin + cRand.nextFloat() * (fMax - fMin);
    }
	
    /** Ustawia paremetry standardowego swiatla
     * @param vPos pozycja swiatla (lub wektor kierunku dla swiatla kierunkowego
     * @param vColor kolor swiatla
     * @param bDirLight true jesli swiatlo ma byc kierunkowe
     */
    public native void SetDefaultLight(Vector vPosition, Vector vColor, boolean bDirLight);

    /** Ustawia kolor standardowego swiatla
     * @param vColor kolor swiatla
     */
    public native void SetDefaultLightColor(Vector vColor);
    
    public void SetDefaultLightColor(float fR, float fG, float fB)
    {
        SetDefaultLightColor(new Vector(fR, fG, fB));
    }
    
   /** Dodaje swiatlo
     * @param vPos pozycja swiatla (lub wektor kierunku dla swiatla kierunkowego)
     * @param vColor kolor swiatla
     * @param bDirLight true jesli swiatlo ma byc kierunkowe
     * @return id swiatla
     */
    public native int AddLight(Vector vPosition, Vector vColor, boolean bDirLight);

    /** Usuwa swiatlo
     * @param iID id swiatla
     */
    public native void RemoveLight(int iID);
 
    /** Obraca wektor vVector o kat fAngle wokol osi vAxis
     * @param vVector wektor, ktory chcemy obrocic
     * @param fAngle kat
     * @param vAxis os obrotu, nie musi byc znormalizowana
     * @return nowy, obrocony wektor
	 */
	native public Vector RotateVector(Vector vVector, float fAngle, Vector vAxis);
    
    /** Obraca wektor vVector o kat fAngle wokol osi vAxis i zwraca wynik
     * do ostatniego parametru
     * @param vVector wektor, ktory chcemy obrocic
     * @param fAngle kat
     * @param vAxis os obrotu, nie musi byc znormalizowana
     * @param vResult wektor do ktorego trafia wynik
     * @return nowy, obrocony wektor
	 */
	native public void RotateVector(Vector vVector, float fAngle, Vector vAxis, Vector vResult);
    

    /** Tworzy obiekt klasy podanej w stringiu. Wola pierwszy zadeklarowany
     * konstruktor. Prawdopodbnie bedzie to pierwszy wystepujacy w pliku .java.
     * Gdy cos pojdzie nie tak jak trzeba to wyrzuci stack trace'a.
     * @param sClassName nazwa klasy
     * @return obiekt podanej klasy lub null gdy wystapil blad
     */
    Object GetObjectOfClass(String sClassName)
    {
        Object cReturing = null;
        try {
            Class cWeaponClass = Class.forName(sClassName);
            java.lang.reflect.Constructor []aConstructors = cWeaponClass.getDeclaredConstructors();
            cReturing = aConstructors[0].newInstance( null );
        }
        catch(Throwable e)
        {
            e.printStackTrace();
        }
        return cReturing;
    }
 
    /** Usuwa wszystkie dzwieki z calej gry (attach'owane do obiektow i streaming sound)
     */
	native public void DeleteAllSounds();
	
	//
	// benchmark
	//
	
	native public float GetCPUTime();
	native public float GetMemTime();
	native public float GetVideoTime();
    
    /** Dodaje licznik do systemu profilingu
     * @param sPage nazwa strony w konsoli, do ktorej doda sie licznik
     * @param sName nazwa licznika
     * @return zwraca id licznika
     */
    native public final int ProfileAdd(String sPage, String sName);

    /** Usuwa licznik (nie wolno go pozniej uzywac)
     * @param iProfileID id licznika
     */
    native public final void ProfileRemove(int iProfileID);

    /** Poczatek pomiaru czasu
     * @param iProfileID id licznika
     */
    native public final void ProfileBegin(int iProfileID);

    /** Koniec pomiaru czasu
     * @param iProfileID id licznika
     * @return ostatnie wskazanie licznika
     */
    native public final float ProfileEnd(int iProfileID);
	
    /** Startuje prosty licznik czasu.
     */
    native public final void ProfileIntervalStart();
    
    /** Zwraca ile czasu uplynelo od ostatniego zawolania ProfileIntervalStart().
     * @return czas w ms od ostatniego zawolania ProfileIntervalStart()
     */
    native public final float ProfileIntervalEnd();
    
	/**[native] ustawia flage engine'u okreslajaca sposob renderowania leveli
     * flaga okresla czy rendering leveli ma byc rozdzielony..
     * gdy rendering leveli rozdzielony: kazdy level rysuje sie niezaleznie od pozostalych
     * w kolejnosci w jakiej byly te levele dodawane do gry (kolejnosc dotyczy zbufora)
     * w przypadku gdy flaga wyzerowana rendering leveli jest polaczony, znaczy to ze 
     * levele rysuja sie we wspolnym zbuforze
     * flaga ta ma takze wplyw na zasade dzialania PostRender()...
     * default: true
	 */
	native public final static void IndependentLevelRendering( boolean activate );
	native public final static boolean GetIndependentLevelRendering();
	
	////////////////
	// Renderer
	////////////////

    /** [native] Dolacza podany katalog i wszystkie jego podkatalogi do texture managera.
     * Katalog dodawany jest tak, aby byl uzywany jako pierwszy
     * w kolejnosci przeszukiwania, dzieki czemu tektury z tego katalogu
     * moga 'zastapic' tekstury z innych katalogow
     */
    public static native final void AddTexturePath(String sPath);

    /** [native] Usuwa podany katalog i wszystkie jego podkatalogi z texture managera. */
    public static native final void RemoveTexturePath(String sPath);
    
	/** [native] preload materialu o podanej nazwie
	 * @param matName nazwa materialu do zaladowania
	 * @return handle na zaladowany material
	 */
	native static public final int MaterialLoad( String matName );
	
	/** [native] zwolnienie materialu 
	 * @param handle na material (otrzymany z MaterialLoad)
	 */
	native static public final void	MaterialFree( int handle );
	
	/* [native] zwalnia materialy zaladowane przez MaterialLoad
	 */
	native static public final void	MaterialFreeAll();
 
	/** [native] zwraca id materialu o podanej nazwie
	 * @param matName nazwa materialu
	 * @return id materialu
	 */
	native static public final int MaterialGetID( String matName );
	
	/** [native] Podmienia texture w materiale
	 * UWAGA! Podmiana nastepuje dla _materialu_, tzn. zmieni sie wyglad wszystkiego,
	 * co z tego materialu korzysta
	 * @param matName nazwa materialu
	 * @param pass numer passu w materiale
	 * @param tmu numer tmu w passie
	 * @param texName nazwa textury
	 * @return true jesli nastapila podmiana (nie sprawdza, czy zadana tekstura istniala
	 * - podmiana na texture defaultowa tez da true!)
	 */
	native public final boolean MaterialSetTexture(String matName, int pass, int tmu, String texName);
	
	/** [native] Podmienia cubemape w materiale
	 * UWAGA! Podmiana nastepuje dla _materialu_, tzn. zmieni sie wyglad wszystkiego,
	 * co z tego materialu korzysta
	 * @param matName nazwa materialu
	 * @param pass numer passu w materiale
	 * @param tmu numer tmu w passie
	 * @param texName* nazwa textury
	 * @return true jesli nastapila podmiana (nie sprawdza, czy zadane tekstury istnialy
	 * - podmiana na texture defaultowa tez da true!)
	 */
	native public final boolean MaterialSetCubemap(String matName, int pass, int tmu, String texXNeg, String texXPos, String texYNeg, String texYPos, String texZNeg, String texZPos);
	
	/** [native] Podmienia texture we wszystkich materialach
	 * @param sOldTexName nazwa starej tekstury
	 * @param sNewTexName nazwa nowej tekstury
	 */
	native public final void MaterialReplaceTextures(String sOldTexName, String sNewTexName);
    
	/** [native] Podmienia cubemape we wszystkich materialach
	 * @param sOldTex* nazwy starych tekstur
	 * @param sNewTex* nazwy nowych tekstur
	 */
	native public final void MaterialReplaceCubemaps(String sOldTexXNeg, String sOldTexXPos, String sOldTexYNeg, String sOldTexYPos, String sOldTexZNeg, String sOldTexZPos,
                                                     String sNewTexXNeg, String sNewTexXPos, String sNewTexYNeg, String sNewTexYPos, String sNewTexZNeg, String sNewTexZPos);
    
    ////////////////
    // Filmy
    ////////////////
    
    /**[native] Odtwarza film o podanej nazwie
     * Wola na rzecz modulu handler OnStartMovie
     * Handler powinien stworzyc widok dla filmu i zawolac SetMovieView
     * Po skonczeniu odtwarzania wolany jest na rzecz modulu handler OnEndMovie
     * @param sMovieName nazwa filmu
     * @param bLevelLogicEnabled jesli false to update logiki, fizyki i time handlerow
     * zostaje wstrzymany na czas odtwarzania filmu
     * @param bRestoreViews jesli w danym momencie film nie definiuje aktywnej kamery
     * przywracane sa wszystkie widoki levelu
     * @return true jesli OK
     */
    public native final boolean PlayMovie(String sMovieName, boolean bLevelLogicEnabled, boolean bRestoreViews);

    public boolean PMov(String sMovieName, boolean bLevelLogicEnabled, boolean bRestoreViews)
    {
        return PlayMovie(sMovieName, bLevelLogicEnabled, bRestoreViews);
    }

    /**[native] Odtwarza film o podanej nazwie bez zatrzymania po skonczeniu
     * Wola na rzecz modulu handler OnStartMovie
     * Handler powinien stworzyc widok dla filmu i zawolac SetMovieView
     * @param sMovieName nazwa filmu
     * @param bLevelLogicEnabled jesli false to update logiki, fizyki i time handlerow
     * zostaje wstrzymany na czas odtwarzania filmu
     * @param bRestoreViews jesli w danym momencie film nie definiuje aktywnej kamery
     * przywracane sa wszystkie widoki levelu
     * @return true jesli OK
     */
    public native final boolean PlayMovieNoStop(String sMovieName, boolean bLevelLogicEnabled, boolean bRestoreViews);

    /** Zatrzymuje odtwarzanie filmu
     * Wola na rzecz modulu handler OnEndMovie
     */
    public native void StopMovie();
    /** Zatrzymuje odtwarzanie filmu updateujac wszystkie sciezki do konca filmu.
     * Oznacza to m.in., ze obiekty w filmie beda mialy pozycje jak na koncu filmu.
     * Wola na rzecz modulu handler OnEndMovie
     */
    public native final void StopMovieUpdateToEnd();

    /** Wstrzymuje/wznawia odtwarzanie filmu
     */
    public native final void PauseMovie();

    /**[native] Zwraca true jesli jakis film jest aktualnie odtwarzany */
    public native final boolean IsMoviePlaying();

    /**[native] Pobiera dlugosc filmu o podanej nazwie
     * @param nazwa filmu
     * @return dlugosc filmu w sekundach lub -1 gdy blad
     */
    public native final float GetMovieLength(String sMovieName);

    /**[native] Ustawia widok dla filmu
     * @param id widoku
     */
    public native final void SetMovieView(int iViewID);
    
    ////////////////
    // Pliki
    ////////////////
    
    /** Otwiera plik o podanej nazwie i tworzy w nim poczatkowego Chunka
     * do ktorego mozna przyczepiac potomne chunki.
     * @param sFileName nazwa pliku do otwarcia.
     * @return chunk do ktorego mozna pisac
     */
    static native public FileChunk NewFile(String sFileName);
    
    /** Dodaje nowego Chunka do podanego chunka.
     * @param nChunkID identyfikator chunka
     * @param cParent chunk do ktorego ma byc podlaczony nowy chunk
     * @return chunk do ktorego mozna pisac
     */
    static native public FileChunk NewChunk(int nChunkID, FileChunk cParent);
    
    /** Otwiera podany plik i odczytuje go. Zwraca chunka glownego
     * ktory zawiera cala strukture pliku.
     * @param sFileName nazwa pliku do otwarcia
     */
    static native public FileChunk OpenFile(String sFileName);
    
    /** Odczytuje chunka z pliku zwiazanego z parentem.
     * @param cParent chunk pobrany ta metoda wczesniej lub metoda OpenFile
     */
    static native public FileChunk LoadChunk(FileChunk cParent);
    
	//*********************************************************
	//					SaveGame
    //*********************************************************
	
	/** Ustala czy obiekt jest zapisywany/odczytywany do SaveGame'a
	 */
	public boolean bSG = false;
	public boolean bSGLoaded = false;
	public boolean bSGRepeated = false;
   
    /** Atrybut ma wartosc true gdy obiekt jest aktualnie wczytywany z SaveGame'a.
     * Inaczej mowiac jak wolana jest metoda dla obiektu void Load(FileChunk cFC)
     */
	public boolean bLoadingSG = false;	
    
    /** Obiekt do robienia SaveGame'ow. Pole to jest inicjalizowane automatycznie
     * w kontruktorze SGController.
     */
	static SGController cSG = null;
    
    /** Pozwala wlaczyc i wylaczyc tryb okreslania czy obiekt ma byc
     * utworzony poprzez metode PreCreate.
     * @param bMode true wlacza wolanie PreCreate, false wylacza wolanie PreCreate
     */
    public static native final void SetPreCreateMode(boolean bMode);

	/** Tworzy obiekt na podstawie jego nazwy. Wymagany jest konstruktor bezparametrowy.
	*/
	public static Object CreateJavaObject(String sClassName)
	{
		Object object = null;
		
		try
		{
			Class classDefinition = Class.forName(sClassName);
			object = classDefinition.newInstance();
		} catch (InstantiationException e) {
		    GameObject.CrashLog("[GameObject.CreateJavaObject] = ERROR! " + e + "\n");
		} catch (IllegalAccessException e) {
		    GameObject.CrashLog("[GameObject.CreateJavaObject] = ERROR! " + e + "\n");
		} catch (ClassNotFoundException e) {
		    GameObject.CrashLog("[GameObject.CreateJavaObject] = ERROR! " + e + "\n");
		}
        
        if (object == null)
            GameObject.CrashLog("[GameObject.CreateJavaObject] Object is null !\n");
		
		return object;
	}		
	
	public State CreateState(String sStateName)
	{   
		CrashLog("[GameObject.CreateState()] : State = <"+sStateName+"> \n");
		
		State myState = null;
		
        Class InnerClass = null;         
        Class someClass = this.getClass();
		Class[] paramTypes = new Class[]{ someClass };
        
        try
        {                    
            InnerClass = Class.forName(sStateName);
            paramTypes =  new Class[]{ someClass };
        }
        catch(Throwable ee)
        {
			CrashLog("[GameObject.CreateState()] : Error! State = <"+sStateName+"> not found !\n");
            someClass = null;
        }
            
        if (someClass!=null)
        {
            Object Outer = this;
			
            try
            {
                Object[] initArgs = new Object[]{ Outer };
                Constructor innerConstr = InnerClass.getConstructor( paramTypes );
                myState = (State) innerConstr.newInstance( initArgs );
				
            }
            catch (Throwable eee) 
            { 
                LogR("Blad przy przelaczaniu do stanu <"); LogR(sStateName); LogR(">\n"); 
                eee.printStackTrace();
            }
        }
        else
        {
            LogR("No such state <"+sStateName+"> in class tree <"+getClass().getName()+">\n");
        }           
		
		return myState;
	}
	
	public State SGLoadState(FileChunk cFC)
	{
		if (cFC.LoadInt() == -1)
		{
			FileChunk.Log("[SGLoadState] = null \n");
			return null;
		}
	
		String sStateName = cFC.LoadString();
		FileChunk.Log("[SGLoadState] " + sStateName + "\n");
		State s = CreateState(sStateName);		
		
		// FileChunk.Log("[SGLoadState] ClassName = <"+this.getClass().getName()+"> Name = <"+this.GetName()+"> State = <"+sStateName+"> \n");
		
		if (s != null) s.SGLoad(cFC);
		return s;
	}
	
	public void SGSaveState(FileChunk cFC, State s)
	{
		if (s == null)
		{
			cFC.SaveInt(-1);
			FileChunk.Log("[SGSaveState] = null \n");
		}
		else
		{
			cFC.SaveInt(1);
			FileChunk.Log("[SGSaveState] " + s.getClass().getName() + "\n");
			
			String sCurrentState = s.getClass().getName();
					
			cFC.SaveString(sCurrentState);		
			s.SGSave(cFC);
		}		
	}
        
    /** Metoda wczytuje dane z SaveGame'a z dla podanego Chunka. 
     * Wolana gdy obiekt ma ustawione pole bSG. Metoda do pokrycia. 
     * Powinno si wywolac super.SGLoadChunk
     * @param nChk identyfikator odczytywanego chunka identyczny z cFC.GetID()
     * @param cFC chunk do ktorego zapisuje sie dane obiektu.
     */
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[GameObject.SGLoadChunk] Name = <"+GetName()+"> ClassName = <"+getClass().getName()+"> \n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunks._CHK_GAME_OBJECT)
        {
			bSGLoaded = true;
			
			if (cFC.LoadBool())
				NativeSGLoad(cFC);
		
			if (cFC.LoadInt() == -1)
			{
				cFC.delete();
				CurrentState = null;
				FileChunk.Log("[GameObject.SGLoadChunk] - END\n");
				return;
			}
		
			String sStateName = cFC.LoadString();
			
            if(sStateName == null)
            {
                CrashLog("<"+GetName()+":"+GetUniqueID()+"> state null!!\n");
            }
            else
            {
				FileChunk.Log("[SGLoadChunk].CurrentState = <" + sStateName + ">\n");
			    int nDolar = sStateName.indexOf('$');
			    sStateName = sStateName.substring(nDolar+1);						
			    //FileChunk.Log("[SGLoadChunk] SetStateNoEnter\n");
			    SetStateNoEnter(sStateName);
			    //FileChunk.Log("[SGLoadChunk] SGLoad\n");
			    if(CurrentState!=null) CurrentState.SGLoad(cFC);
			    //FileChunk.Log("[SGLoadChunk].After setting state\n");
            }
		}
		
		cFC.delete();
		FileChunk.Log("[GameObject.SGLoadChunk] - END\n");
    }
    
    /** Metoda zapisuje dane do SaveGame'a dla podanego Chunka. 
     * Wolana gdy obiekt ma ustawione pole bSG. Metoda do pokrycia. 
     * Powinno si wywolac super.SGSaveChunk
     * @param cFC chunk do ktorego zapisuje sie dane obiektu.
     */
    public void SGSaveChunk(FileChunk cFCParent)
    {	
		int nUniqueID = GetUniqueID();
		
		if (nUniqueID != -1)
			FileChunk.Log("[GameObject.SGSaveChunk] Name = <"+GetName()+"> ClassName = <"+getClass().getName()+"> \n");
		else
			FileChunk.Log("[GameObject.SGSaveChunk] Object with UniqueID = <-1> ClassName = <"+getClass().getName()+"> \n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunks._CHK_GAME_OBJECT, cFCParent);
		
		if (bSGRepeated)
			FileChunk.Log("[GameObject.SGSaveChunk] SG-WARNING! Object was already saved ! \n");
		
		bSGRepeated = true;
		
		cFC.SaveBool(nUniqueID != -1);
			
		if (nUniqueID != -1)
			NativeSGSave(cFC);
		
		SGSaveState(cFC,CurrentState);
		
		cFC.delete();
		
		FileChunk.Log("[GameObject.SGSaveChunk] - END\n");
    }
	
    /** Metoda zapisuje dane trzymane dla obiektu po stronie CPP.
     * Metoda sama rozpoznaje typ obiektu i w zaleznosci od typu
     * odcztuje odpowiednio dane.
     * @return zwraca true gdy operacja zostanie przeprowadzona bez bledu
     */
    public native boolean NativeSGLoad(FileChunk cFC);

    /** Metoda zapisuje dane trzymane dla obiektu po stronie CPP.
     * Metoda sama rozpoznaje typ obiektu i w zaleznosci od typu
     * odcztuje odpowiednio dane.
     * @return zwraca true gdy operacja zostanie przeprowadzona bez bledu
     */
    public native boolean NativeSGSave(FileChunk cFC);
	
	public native void SaveMovieState(FileChunk cFC);
	public native void LoadMovieState(FileChunk cFC);
	public native void UpdateMovieState();		
						
    /** Metoda wywoluje odpowiednie akcje po odczytaniu danych. 
     * Metoda do pokrycia. Przewanie powinno si wywolac super.SGUpdate
     */
    public void SGUpdate()
    {
		if ((CurrentState != null) && !(this instanceof Module))
			CurrentState.SGUpdate(); // !!! przejscie z OnEnter
    }	

    ///////////////////////////////////////////////////////
    // Globalne parametry update'u fizyki i animacji
    ///////////////////////////////////////////////////////

    /**[native] Ustawia iterwal, co jaki update'uja sie animacje obiektow
     * calkowicie nieistotnych tzn. o nieistotnosci rownej 1
     * czas pomiedzy updatami = nieistotnosc * interwal
     */
    public native final void SetMaxUpdateInterval(float fInterval);

    /**[native] Ustawia iterwal, co jaki update'uje sie fizyka obiektow
     * calkowicie nieistotnych tzn. o nieistotnosci rownej 1
     * czas pomiedzy updatami = nieistotnosc * interwal
     */
    public native final void SetMaxPhysicsUpdateInterval(float fInterval);
	
    /**[native] kasuje nieuzywane mesh'e
	 */
	native void DeleteUnusedMeshes();

    
    /**[native] Wylaczenie/wlaczenie renderingu wody */
    native public void EnableWater(boolean bEnable);
 
    
    /**Zwraca liczbe obiektow danej klasy (jedynie tych, ktore sa tez po stronie c)*/
    String CountMe()
    {
        GameObject tab[] = GetObjectsOfClass(this.getClass().getName());
        if (tab == null)
            return (getClass().getName() + ": " + 0);
        else
            return (getClass().getName() + ": " + tab.length);
    }

    String CountOfClass(String sClass)
    {
        GameObject tab[] = GetObjectsOfClass(sClass);
        if (tab == null)
            return (sClass + ": " + 0);
        else
            return (sClass + ": " + tab.length);
    }
    
    String CountFromClass(String sClass)
    {
        GameObject tab[] = GetObjectsFromClass(sClass);
        if (tab == null)
            return (sClass + ": " + 0);
        else
            return (sClass + ": " + tab.length);
    }
    /**[native] Znajduje na levelu obiekt o podanej nazwie. Szuka tez wsrod obiektow
     * znajdujacych sie tylko w filmach
     */
    public native final ControlObject GetMovieObject(String sObjectName);
    
    ///////////////////////////////////////////////////////
    // SYSTEM MISJI
    ///////////////////
 
    /** Metoda ustawia atrybut o podanej nazwie na podana wartosc.
     * Podczas ustawiania jest sprawdzana zgodnosc typu, gdy nie udalo sie
     * ustawic metoda zwraca false. UWAGA metoda dziala tylko w edytorze!
     * @param sName nazwa atrybutu
     * @return true gdy atrybut zostal poprawnie ustawiony
     */
    public native boolean SetEditorAttribute(String sName, GameObject cObj);
 
    /** Sprawdza czy obiekt na rzecz ktorego jest zawolane mozna skastowac 
     * na klase o podanej nazwie.
     * @param sBaseClass nazwa klasy na ktora kastujemy
     * @return true gdy obiekt jest klasyy sBaseClass
     */
    public native boolean IsAssignable(String sBaseClass);
 
    /** Wolane dla nowo dodanego obiektu w edytorze
     */
    public void OnEditorObjectAdd(ControlObject[] aSelectedObject) {}
    
    /** Wolane przed skasowaniem obiektu w edytorze
     */
    public void OnEditorObjectDel() {}
    
    /** Obliczenie wartosci wyrazenia skryptowego
     * @param sExpression wyrazenie do obliczenia
     * @return wynik obliczenia wyrazenia
     */
    static public native int EvaluateExpression(Script sExpression);
    
    /** Przygotowanie do obliczenia wyrazenia.
     */
    static public native boolean PreEvaluate();
    
    /** Obliczenie wartosci wyrazenia skryptowego dla zwyklego Stringa
     * @param sExpression wyrazenie do obliczenia
     */
    public int Eval(String sExpression) { return EvaluateExpression(new Script(sExpression)); }
        
    /** Ustawia wartosc podanej zmiennej.
     * @param sName nazwa zmiennej
     * @return wartosc zmiennej
     */
    static public native int GetVariableValue(String sName);
    
    /** Ustawia wartosc podanej zmiennej.
     * @param sName nazwa zmiennej
     * @param nVal nowa wartosc do ustawienia
     */
    static public native void SetVariableValue(String sName, int nVal);
    
    /** Metoda uruchamia podany lancuch akcji.
     * @param aChain tablica z akcjami do wykonania. W praktyce bedzie sie podawalo
     * atrybut ktory zawiera akcje.
     */
    static public void InvokeActionChain(Action[] aChain)
    {
        if(aChain == null) return;
        
        Action cFirst = null;
        
        // zapisujemy wszystkim akcjom z lancucha ich nastepniki z tablicy
        for(int i=0; i<aChain.length; ++i)
        {
            Action cAct = aChain[i];
            if(cAct != null)
            {
                if(cFirst == null)
                    cFirst = cAct;

                // ustawiamy nastepnik akcji
                cAct.cNext = null;
                if(i+1 < aChain.length)
                {
                    for(int j=i+1; j<aChain.length; ++j)
                        if(aChain[j] != null)
                        {
                            cAct.cNext = aChain[j];
                            break;
                        }
                }
            }
        }

        // uruchamiamy pierwsza akcje
        if(cFirst != null)
            cFirst.Invoke();
    }
    
    /////////////////////////////////////////////////////////////////////////////

    /** Metoda zwraca nazwe STATYCZNEGO PUBLICZNEGO pola typu int podanej klasy 
     *  o zadanej wartosci (pierwszego pola jakie znajdzie)
     * 
     *  (przydatne przy debugowaniu, gdy wartosc stalej wyliczeniowej niewiele mowi)
     * 
     *  @param cClass klasa w ktorej zadeklarowane jest to pole
     *  @param nValue wartosc dla ktorej poszukujemy nazwy pola
     *  @param sNamePrefix od jakiego ciagu znakow powinna zaczynac sie nazwa pola
     *  @return nazwa pola albo info o braku
     */
    public static String GetFieldNameForIntValue(Class cClass, int nValue, String sNamePrefix)
    {
        Field[] aFields = cClass.getFields();

        for (int i=0; i<aFields.length; i++)
        {
                
            try {
                if (Modifier.isStatic(aFields[i].getModifiers()) &&
                    (sNamePrefix == null || aFields[i].getName().startsWith(sNamePrefix)) &&
                    aFields[i].getInt(null) == nValue)
                {
                    return aFields[i].getName();
                }
            }
            catch (IllegalAccessException e)
            {
            }
        }
        return "field with value " + nValue + " not found";
    }
    
    /** Pochodna spline'a catmull-rom dla parametru T i 4 punktow ABCD
     */
    static float CalcSplineD(float A, float B, float C, float D, float T)
    {
        float   fa, fb, fc;

        fa = 0.5f*(3*B + D - A -3*C);
        fb = A + 2*C - 0.5f*(5*B + D);
        fc = 0.5f*(C - A);

        return 3*T*T*fa + 2*T*fb + fc;
    }

    ///////////////////////////////////////////////////////////////////////////
    // definiowanie wizualizacji do debugowania logiki gry
    
    /** Pozwala wlaczac i wylaczac renderowanie wizualizacji debugowej.
     */
    public native void SetRenderDebugVis(boolean bRender);
    
    /** Definiuje linie rysyowana miedzy obiektem a jego atrybutem.
     * @param sAttributeName nazwa atrybutu do ktorego ma byc rysowana linia
     * @param vColor kolor lini
     */
    public native static void DefineDebugLine(String sAttributeName, Vector vColor);
    
    /** Definiuje linie rysyowana miedzy dwoma atrybutami obiektu.
     * @param sAttributeName1 nazwa atrybutu od ktorego ma byc rysowana linia
     * @param sAttributeName2 nazwa atrybutu do ktorego ma byc rysowana linia
     * @param vColor kolor lini
     */
    public native static void DefineDebugLine(String sAttributeName1, String sAttributeName2, Vector vColor);
    
    /** Definiuje linie rysyowana miedzy obiektem a elementami tablicy - atrybutu podanego.
     * @param sAttributeName nazwa atrybutu do ktorego ma byc rysowana linia, atrybut musi byc tablica
     * @param vColor kolor lini
     */
    public native static void DefineDebugLineArray(String sAttributeName, Vector vColor);
    
    /** Definiuje strzalke rysyowana miedzy obiektem a jego atrybutem.
     * @param sAttributeName nazwa atrybutu do ktorego ma byc rysowana linia
     * @param vColor kolor lini
     * @param bDirection kierynek lini =false grot przy obiekcie =true grot przy atrybucie
     */
    public native static void DefineDebugArrow(String sAttributeName, Vector vColor, boolean bDirection);
    
    /** Definiuje strzalke rysyowana miedzy dwoma atrybutami obiektu.
     * @param sAttributeName1 nazwa atrybutu od ktorego ma byc rysowana linia
     * @param sAttributeName2 nazwa atrybutu do ktorego ma byc rysowana linia
     * @param vColor kolor lini
     * @param bDirection kierynek lini =false grot przy attr1 =true grot przy attr2
     */
    public native static void DefineDebugArrow(String sAttributeName1, String sAttributeName2, Vector vColor, boolean bDirection);

    /** Definiuje strzalke rysyowana miedzy obiektem a elementami tablicy - atrybutu.
     * @param sAttributeName nazwa atrybutu do ktorego ma byc rysowana linia, ma byc typu tablicowego
     * @param vColor kolor lini
     * @param bDirection kierynek lini =false grot przy obiekcie =true grot przy atrybucie
     */
    public native static void DefineDebugArrowArray(String sAttributeName, Vector vColor, boolean bDirection);
    
    /**[native]
     * Zwraca tekst duzymi literami, uwzgledniajac specyfike jezyka*/
    public static native String TextToUpperCase(String sText);
    
    /**[native]
     * Zwraca tekst malymi literami, uwzgledniajac specyfike jezyka*/
    public static native String TextToLowerCase(String sText);
    
    /** Metoda wolana dla klasy w celu zdefiniowania za pomoca powyzszych metod
     * sposobu wizualizacji atrybnutow klasy
     */
    //public static void DefineVisualisation()
    //{
    //}

    /** Wlacza kolorowe mipmapy
     * @param iTMU - na ktorym tmu podmieniac textury
     * (-1 wylacza kolorowanie)
     */
    public native final void UseColorMips(int iTMU);

	/////////////////////////////////
	// DEBUG-owanie
	/** znacznik trybu debygowania GameObject'a	 */
	boolean m_bDebug = true;

	/** Wlacza/wylacza tryb debugowania (wypisywanie logow DEBUG-a).
	 *  @param bSet wlaczenie/wylaczenie trybu DEBUG	 */
	void SetDebug(boolean bSet)
	{
		m_bDebug = bSet;
	}
	
	/** Wypisuje do crash.log tekst z nazwa klasy , jesli ustawiony tryb DEBUG. 
	 *  @param sText wyswietlany tekst*/
	void DEBUG(String sText)
	{
		if(m_bDebug)
			CrashLog("["+((Object)this).getClass().getName()+"->DEBUG] " + sText  + "\n");								
	}	
	
	/** Wypisuje do crash.log tekst*, jesli globalny znacznik  w grze Game._bDbg
	 *  jest ustawiony na true. 
	 *  @param sText wyswietlany tekst*/
	static void DEBUG_GLOBAL(String sText)
	{
		if(Game._bDbg)
			CrashLog("[DEBUG] " + sText  + "\n");								
	}
	/** wypisuje do crash.log tekst z nazwa klasy jesli  globalny znacznik  w grze Game._bDbg
	 *  jest ustawiony na true.
	 *  @param sSource dowolny tekst identyfikujacy miejsce w kodzie
	 *  @param sText wyswietlany tekst*/ 
	static void DEBUG_GLOBAL(String sSource, String sText)
	{
		if(Game._bDbg)
			CrashLog("["+sSource+"->DEBUG] " + sText  + "\n");								
	}			
	
	
	/** wyswietla tekskt na ekranie jesli obiekt jest przypisany do moudulu.
	 * @param sText wyswietlany tekst
	 * @param nPos pozycja wyswietlanego tekstu (Tools._INFO_POS_TOP, Tools._INFO_POS_BOTTOM ,
	 *			   Tools._INFO_POS_CENTER )	 */
	void ShowInfo(String sText, int nPos)
	{
		if(!m_bDebug)
			return;
		Module cModule = GetModule();
		if(cModule!=null)	
			Tools.ShowInfo(sText, cModule, nPos);							
	}
	
	/** wyswietla tekskt na srodku ekranu jesli obiekt jest przypisany do moudulu.	
	 * @param sText wyswietlany tekst  */
	void ShowCenterInfo(String sText)
	{
		ShowInfo(sText, Tools._INFO_POS_CENTER);
	}
	
	/** wyswietla tekskt na gorze ekranu jesli obiekt jest przypisany do moudulu.
	 * @param sText wyswietlany tekst  */
	void ShowTopInfo(String sText)
	{
		ShowInfo(sText, Tools._INFO_POS_TOP);
	}
	
	/** wyswietla tekskt na dole ekranu jesli obiekt jest przypisany do moudulu	 
	 * @param sText wyswietlany tekst  */
	void ShowBottomInfo(String sText)
	{
		ShowInfo(sText, Tools._INFO_POS_BOTTOM);
	}	
	
	//////////////////////////////////
	// ASERCJE
	
	/** wypisuje tekst dla asercji.
     * @param sText wyswietlany tekst  */
	static void ASSERT(String sText)
	{
		if(Game.m_bAssertOn)
			GameObject.CrashLog("[ASSERTION] "+sText+" \n"  );
	}
	
	/** wypisuje tekst dla asercji z podaniem lokalizacji wystapienia
	 *  @param sSource dowolny tekst identyfikujacy miejsce asercjiw kodzie
	 *  @param sText wyswietlany tekst*/ 
	static void ASSERT(String sSource, String sText)
	{
		if(Game.m_bAssertOn)
			GameObject.CrashLog("["+sSource+"->ASSERTION] "+sText+" \n"  );
	}	
	
	/** wypisuje tekst dla asercji z podaniem obiektu w ktorym wystapila
	 *  @param cSource obiekt w ktorym zostala wywolana asercja
	 *  @param sText wyswietlany tekst
	 * */ 
	static void ASSERT(Object cSource, String sText)
	{
			ASSERT( ((Object)cSource).getClass().getName(), sText);
	}	
	
	/** wypisuje tekst dla asercji i wychodzi z systemu
	 *  @param sText wyswietlany tekst 
	 */
	static void ASSERT_EXIT(String sText)
	{
		if(!Game.m_bAssertOn)
			return;
		ASSERT(sText);
		System.exit(0);
	}	
	
	/** Wypisuje tekst dla asercji i wychodzi z systemu z podaniem lokalizacji
	 *  wystapienia.
	 *  @param sSource dowolny tekst identyfikujacy miejsce asercjiw kodzie
	 *  @param sText wyswietlany tekst
	 * */ 
	static void ASSERT_EXIT(String sSource, String sText)
	{	
		if(!Game.m_bAssertOn)
			return;		
		ASSERT(sSource,sText);
		System.exit(0);
	}	
	
	/** Wypisuje tekst dla asercji i wychodzi z systemu z podaniem obiektu w ktorym 
	 *  wystapila.
	 *  @param cSource obiekt w ktorym zostala wywolana asercja
	 *  @param sText wyswietlany tekst
	 * */ 
	static void ASSERT_EXIT(Object cSource, String sText)
	{	
		ASSERT_EXIT(((Object)cSource).getClass().getName(),sText);
	}
	
	//////////////////////////
    // Obsluga napisow bedacych sciezkami do plikow.

    /** Z nazwy pliku wyciaga sciezke zakonczona '\'.
     *  @param filename pelna nazwa pliku do rozbicia
     *  @return sciezka zakonczona znakiem '\' :
     *      "C:\DOS\command.com" zwroci "C:\DOS\".
     */
    public static String GetPathFromFilename( String sFilename )
    {
        if( sFilename == null )
            return null;
        int lastSlash = sFilename.lastIndexOf( '\\' );
        if( lastSlash != -1 )
            return sFilename.substring( 0, lastSlash + 1 );
        return null;
    }

    /** Z pelnej nazwy pliku wyciaga wylacznie nazwe pliku(bez rozszerzenia).
     *  @param filename pelna nazwa pliku do rozbicia
     *  @return zwraca nazwe pliku :
     *      "C:\DOS\command.com" zwroci "command".
     */
    public static String GetNameFromFilename( String sFilename )
    {
        if( sFilename == null )
            return null;

        int lastDot = sFilename.lastIndexOf( '.' );
        int lastSlash = sFilename.lastIndexOf( '\\' );

        int nameStart = lastSlash + 1;
        int nameEnd = lastDot - 1;
        if( nameStart < 0 )
            return null;
        if( nameStart >= sFilename.length() )
            return null;
        if( nameEnd < 0 )
            nameEnd = sFilename.length() - 1;

        return sFilename.substring( nameStart, nameEnd + 1 );
    }

    /** Z pelnej nazwy pliku wyciaga rozszerzenie z kropka.
     *  @param filename pelna nazwa pliku do rozbicia
     *  @return rozszerzenie(z kropka!) :
     *      "C:\DOS\command.com" zwroci ".com".
     */
    public static String GetExtFromFilename( String sFilename )
    {
        if( sFilename == null )
            return null;

        int lastDot = sFilename.lastIndexOf( '.' );

        if( lastDot != -1 )
            return sFilename.substring( lastDot );
        return null;
    }

    /** Z pelnej nazwy pliku wyciaga nazwe z rozszerzeniem.
     *  @param filename pelna nazwa pliku do rozbicia
     *  @return nazwa pliku z rozszerzeniem :
     *      "C:\DOS\command.com" zwroci "command.com".
     */
    public static String GetNameExtFromFilename( String sFilename )
    {
        if( sFilename == null )
            return null;
        int nameStart = sFilename.lastIndexOf( '\\' ) + 1;
        if( nameStart < sFilename.length() )
            return sFilename.substring( nameStart );
        return null;
    }

    //////////////////////////
    // MAPA TYPU TERENU
    //////////////////////////

    /**[native] Ustawia typ terenu w punkcie (x,z)
     * @param wspolrzedna x pozycji
     * @param wspolrzedna z pozycji
     * @param wartosc do ustawienia
     */
    public native void SetTerrainType(float x, float z, int value);

    /**[native] Pobiera typ terenu w punkcie (x,z)
     * @param wspolrzedna x pozycji
     * @param wspolrzedna z pozycji
     * @return typ terenu lub -1 gdy pozycja poza mapa
     */
    public native int GetTerrainType(float x, float z);

    /**[native] Sprawdza czy w punkcie (x,z) jest dany typ terenu
     * Przy sprawdzaniu wykonywane jest filtrowanie wartosci 
     * (percentage closer filtering)
     * @param wspolrzedna x pozycji
     * @param wspolrzedna z pozycji
     * @param wartosc do sprawdzenia
     * @return -1 gdy pozycja poza mapa, 0 gdy nie ma danej wartosci, 1 gdy jest
     */
    public native int GetTerrainTypeSmooth(float x, float z, int value);

    /////////////////////////////
    // DRZEWA
    /////////////////////////////
    
    /**[native] Pobiera drzewa w podanym promieniu od punktu vPos. Identyfikatory drzew
     * zwracane sa w tablicy aTreeArray. Drzewa zwracane sa posortowane po odleglosci
     * od zadanego punktu.
     * @param vPos pozycja punktu
     * @param fRadius promien okregu, z ktorego wybierane sa drzewa
     * @param aTreeArray tablica identyfikatorow drzew (allokowana przez uzytkownika!)
     * @param iMaxNumTrees ile maksymalnie zwrocic drzew - wartosc ta nie moze byc wieksza
     * od wielkosci tablicy aTreeArray!
     * @return ilosc drzew umieszczonych w tablicy
     */
    public native int GetNearestTreesInRadius(Vector vPos, float fRadius, int [] aTreeArray, int iMaxNumTrees);

    /**[native] Pobiera pozycje drzewa iTree do wektora vPos */
    public native void GetTreePos(int iTree, Vector vPos);

    /**[native] Pobiera extenty drzewa iTree do wektorow vMin, vMax */
    public native void GetTreeExtents(int iTree, Vector vMin, Vector vMax);

    /**[native] Liczy przeciecie promienia z drzewem
     * @param iTree identyfikator drzewa
     * @param vStart poczatek promienia
     * @param vEnd koniec promienia (po wykryciu kolizji docinany do punktu kolizji)
     * @param cColl struktura z dodatkowymi wynikami kolizji (moze byc null)
     * @return true jesli wystapila kolizja
     */
    public native boolean RaytraceTree(int iTree, Vector vStart, Vector vEnd, Collision cColl);

    /**[native] Pobiera obiekty w podanym promieniu od punktu vPos. Obiekty
     * zwracane sa w tablicy aTreeArray i sa posortowane po odleglosci
     * od zadanego punktu.
     * @param vPos pozycja punktu
     * @param fRadius promien okregu, z ktorego wybierane sa obiekty
     * @param aObjectArray tablica obiektow (allokowana przez uzytkownika!)
     * @param iMaxNumObjects ile maksymalnie zwrocic obiektow - wartosc ta
     * nie moze byc wieksza od wielkosci tablicy aObjectArray!
     * @param cExclude obiekt do pominiecia
     * @return ilosc obiektow umieszczonych w tablicy
     */
    public native int GetNearestObjectsInRadius(Vector vPos, float fRadius, ControlObject [] aObjectArray, int iMaxNumObjects, ControlObject cExclude);

    /**[native] Zwraca true gdy obiekt byl stworzony na podstawie danych odczytanych
     * z pliku levelu, czyli jest obiektem ustawionym w edytorze.
     */
    public native boolean IsCreatedDuringLoading();
    
	///////////////////
	/** Zwraca opis dzialania obiektu na podstawie jego atrybutow. 
	 */
	public String GetDescription()
	{
		return "GameObject::GetDescription()";
	}
	
    public int compareTo(Object o) {
		GameObject obj = (GameObject)o;		
		int nID = GetUniqueID();
		int nObjID = obj.GetUniqueID();
		if (nID == nObjID)
			return 0;
		else if (nID < nObjID)
			return -1;
			
		return 1;
    }    	
    
    /**[native] Pobiera z rejestru wartosc klucza typu REG_SZ
     * @param KeyName nazwa klucza
     * @param KeyValue nazwa wartosci klucza
     * @return - odczytana wartosc klucza
     */
    native String RegKeyReadString(String KeyName,String KeyValue);
    
    /**[native] Liczy CRC dla czesci podanego pliku
     * @param sFileName sciezka do pliku (<code>String</code>)
     * @param iStart poczatek czesci pliku do liczenia
     * @param iEnd   koniec        -||-
     * @return - policzone CRC
     */
    public static native int FileCRC32(String sFileName, int iStart, int iEnd);

    /**[native] Pobiera globalny identyfikator klasy */
    public static native int GetClassGUID(String sClassName);

    /**[native] Pobiera nazwe klasy dla podanego globalnego identyfikatora */
    public static native String GetClassByGUID(int iClassGUID);
    
    /**[native] pobiera string okreslajacy locale (windowsowe) dla programu */
    public static native String GetLocale();

    /**[native] pobiera string okreslajacy wersje jezykowa gry */
    public static native String GetLocaleID();
    
    public String GetLoc()
    {
        return GetLocale();
    }
	
	native public void SetSafeMode( boolean enable );
    
    
    /////////////////////////////////////////////////////////////////////
    // All Seeing Eye
    // ASE will call handlers on Module:
    // void ASEQuery_OnWantStatus();
    // void ASEQuery_OnWantRules();
    // void ASEQuery_OnWantPlayers();
    
    /** Initializes ASE
     * @param bInternet
     * @return error code
     */
    public native int ASEQuery_Initialize(boolean bInternet);
    
    public native void ASEQuery_Shutdown();

    /** Can be called only once during handlers called for module.
     * Shows ASE server rule.
     */
    public native void ASEQuery_Status(String sHostName, String sGameType, String sMapName, String sGameVer, boolean bPassword, int nNumPlayers, int nMaxPlayers);
    
    /** Can be called only during handlers called for module.
     * Shows ASE server rule.
     */
    public native void ASEQuery_AddRule(String sRuleName, String sRuleValue);

    /** Can be called only during handlers called for module.
     * Shows ASE one player.
     */
    public native void ASEQuery_AddPlayer(String sName, String sTeam, String sSkin, String sScore, String sPing, String sTime);
    
    
}
