
/** Obiekt swiatla dynamicznego.
 */
public class LightObject extends EditorRenderableObject
{   
    public native void SetColor(float r, float g, float b);
    // zasieg ustawia sie poprzez skale obiektu
    //public native void SetRange(float range);
    public native void SetMinDistance(float min);
    public native void SetMaxDistance(float max);
    public native void SetAttenuationAtMax(float attenuation);
    public native void SetType(int type);
    
    public native Vector GetColor();
    public native float GetRange();
    public native float GetMinDistance();
    public native float GetMaxDistance();
    public native float GetAttenuationAtMax();
    public native int GetType();
    
    public native boolean IsDynLightEnabled();
    public native void EnableDynLight(boolean enable);
    
    LightObject()
    {
        iRenderType = EEditorRenderType._MESH;
		EditorMeshName = "data\\interface\\zarowkaD.3da";
    }
    
    // ============ PLYNNA ZMIANA PARAMETROW =============

    // parametry potrzebne do plynnej zmiany koloru
    Vector vColorInc = null;
    float fDestColorTime = 1;
    float fDestColorT = 1;

    //parametry do plynnej zmiany skali
    Vector vScaleInc = null;
    Vector vScaleCurr = null;
    float fDestScaleTime = 1;
    float fDestScaleT = 1;

    //czas updateu plynnej zmiany atrybutow
    protected float fUpdateTime = 0.05f;

    /** Plynna zmiana koloru
     */
    void SetColor(Vector vDestColor, float fDestColorTime)
    {
        if (fDestColorT < 1) DisableCallMethod("SetColorExe");
        fDestColorT = 0;
        this.fDestColorTime = fDestColorTime;        

        Vector vCurrColor = null;        
        vCurrColor = GetColor();
        vColorInc = new Vector(vDestColor);
        if (vCurrColor != null)
			vColorInc.Sub(vCurrColor);
		else
			CrashLog("[LightObject.SetColor] Error! LighObject was not created in engine !");
        vColorInc.Mul(fUpdateTime / fDestColorTime);

        CallMethodEvery("SetColorExe", fUpdateTime);
    }

    protected void SetColorExe()
    {
        fDestColorT += fUpdateTime / fDestColorTime;
        if (fDestColorT > 1) fDestColorT = 1;

        Vector vCurColor = GetColor();
        
        vCurColor.Add(vColorInc);
        
        if (vCurColor.fX < 0) vCurColor.fX = 0;
        else if (vCurColor.fX > 1) vCurColor.fX = 1;
        
        if (vCurColor.fY < 0) vCurColor.fY = 0;
        else if (vCurColor.fY > 1) vCurColor.fY = 1;
        
        if (vCurColor.fZ < 0) vCurColor.fZ = 0;
        else if (vCurColor.fZ > 1) vCurColor.fZ = 1;
        
        SetColor(vCurColor.fX, vCurColor.fY, vCurColor.fZ);

        if (fDestColorT == 1) DisableCallMethod("SetColorExe");
    }

    /** Plynna zmiana zasiegu
     */
    void SetScale(float fDestScale, float fDestScaleTime)
    {
        if (fDestScaleT < 1) 
            DisableCallMethod("SetScaleExe");
        fDestScaleT = 0;
        this.fDestScaleTime = fDestScaleTime;

        vScaleCurr = GetScale();        
        vScaleInc = new Vector(fDestScale, fDestScale, fDestScale);
        vScaleInc.Sub(vScaleCurr);
        vScaleInc.Mul( fUpdateTime / fDestScaleTime);

        CallMethodEvery("SetScaleExe", fUpdateTime);        
    }

    protected void SetScaleExe()
    {
        fDestScaleT += fUpdateTime / fDestScaleTime;
        if (fDestScaleT > 1) fDestScaleT = 1;

        SetScale(1f/vScaleCurr.fX, 1f/vScaleCurr.fY, 1f/vScaleCurr.fZ); 
        vScaleCurr.Add(vScaleInc);
        SetScale(vScaleCurr); 

        if (fDestScaleT == 1) 
            DisableCallMethod("SetScaleExe");
    }

	public boolean UpdateDataByMatrix(){ return false;}
	public boolean UpdateMatrixByData(){ return false;}
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[LightSignal.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunks._CHK_LIGHT_OBJECT)
        {
			super.SGLoadChunk(cFC);
			
			vColorInc = cFC.LoadVector();
			fDestColorTime = cFC.LoadFloat();
			fDestColorT = cFC.LoadFloat();
					
			vScaleInc = cFC.LoadVector();
			vScaleCurr = cFC.LoadVector();
			fDestScaleTime = cFC.LoadFloat();
			fDestScaleT = cFC.LoadFloat();					
        }
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[LightSignal.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunks._CHK_LIGHT_OBJECT, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		cFC.SaveVector(vColorInc);
		cFC.SaveFloat(fDestColorTime);
		cFC.SaveFloat(fDestColorT);
			
		cFC.SaveVector(vScaleInc);
		cFC.SaveVector(vScaleCurr);
		cFC.SaveFloat(fDestScaleTime);
		cFC.SaveFloat(fDestScaleT);		
						
		cFC.delete();
	}				
	
}
