/**Klasa opisujaca przycisk podobny do windowsowego.*/
class Button extends Window 
{
    
    
    /**Referencja do jakichs danych, do dowolnego wykorzystania*/
    Object cData = null;
    /**Typ przycisku (zwykly lub typu checkbutton).*/
    int nBtnType;
    /**Stan przycisku.*/
    int nBtnState = EButtonFlags.BTN_STATE_UNCHECKED;
    /**Numer tekstury w stanie normalnym*/
    int nUncheckedTex = 0;
    /**Numer tekstury w stanie wcisnietym/zaznaczonym*/
    int nCheckedTex = 0;
    /**Numer tekstury wyszarzonej (przycisk nieaktywny)*/
    int nDisabledTex = 0;
    /**Numer tekstury w stanie podswietlenia*/
    int nHighlightedTex = 0;
    /**Fragment tekstury do wykorzystania.*/
    Rect rUncheckedTex = new Rect(), rCheckedTex = new Rect(), rDisabledTex = new Rect(), rHighlightedTex = new Rect();
    /**Identyfikator przycisku.*/
    int nID;
    /**Dodatkowy identyfikator przycisku.*/
    int nIDExtra;
    boolean bCallingPressButton = false;
    /**Tekst wyswietlany na przycisku*/
    String sText = "";
    /**Dodatkowy tekst, moze byc wykorzystany do przechowywania np tooltipa*/
    String sTextExtra = "";
    /**Okresla czy kursor myszy jest w obrebie przycisku*/
    boolean bMouseInside = false;

    boolean bDoubleClick = false;
    /**Pamieta czas ostatniego klikniecia przycisku, potrzebne przy dblclicku*/
    float fClickTime = 0.0f;
    
    float fDblClckTime = 0.3f;
    
    void SetData(Object cRef)
    {
        cData = cRef;
    }
    
    Object GetData()
    {
        return cData;
    }
    
    
    /**Ustala dane przycisku. Musi byc wywolana po jego utworzeniu, zeby 
     * przycisk mogl funkcjonowac. Polozenie przycisku ustalane jest
     * za pomoca {@link Sprite#SetRelativePos(Vector v)}, wiec zeby bylo
     * ok, trzeba ja wywolywac po dodaniu buttona do hierarchii
     * sprite'ow ({@link Sprite#AddChild(Sprite sprite)})
     * @param data dane potrzebne przy tworzeniu przycisku, {@link UIElement}  
     * @module modul w ktorym znajduje sie przycisk
     */
    void Init(UIElement data, Module module)
    {
        m_cElementData = data;
        SetModule(module);
        nID = data.nID;
        nIDExtra = data.nIDExtra;
        nBtnType = data.nFlags & EButtonFlags.BTN_TYPE_MASK;
        if ((data.nFlags & EButtonFlags.BTN_STATE_MASK) != 0)
        {
            nBtnState = data.nFlags & EButtonFlags.BTN_STATE_MASK;
        }
        if (data.sUncheckedTex.length() > 0) 
        {
            nUncheckedTex = AddTexture(data.sUncheckedTex, true, true);
            rUncheckedTex = data.rUncheckedTex;
        }
        if (data.sCheckedTex.length() > 0) 
        {
            nCheckedTex = AddTexture(data.sCheckedTex, true, true);
            rCheckedTex = data.rCheckedTex; 
        }
        if (data.sHighlightedTex.length() > 0)
			nHighlightedTex = AddTexture(data.sHighlightedTex, true, true);
        rHighlightedTex = data.rHighlightedTex;
        if (data.sDisabledTex.length() > 0)
			nDisabledTex = AddTexture(data.sDisabledTex, true, true);
        rDisabledTex = data.rDisabledTex;
        
        if (nBtnState == EButtonFlags.BTN_STATE_UNCHECKED)
        {
            if (nUncheckedTex != 0)
				SetTexture(nUncheckedTex, rUncheckedTex);
        }
        else
        {
            if (nCheckedTex != 0)
				SetTexture(nCheckedTex, rCheckedTex);
        }
        
        sText = data.sText;
        SetSize(data.vSize);
        SetRelativePos(data.vPos);
        ProcessMouse(true);
        Show();
		((Sprite)this).SetTextPosFlags(ESpriteText.HORIZ_MIDDLE | ESpriteText.VERT_MIDDLE);
        SetState("Idle");
    }
    /**Ustawia stan zaznaczenia/wcisniecia przycisku
     * @param bCheck jesli true to przycisk wcisniety/zaznaczony
     */
    void SetCheck(boolean bCheck)
    {
        if ((nBtnState & EButtonFlags.BTN_STATE_DISABLED) == EButtonFlags.BTN_STATE_DISABLED) return;
        if (bCheck)
        {
            if (nCheckedTex != 0)
				SetTexture(nCheckedTex, rCheckedTex);
            nBtnState = EButtonFlags.BTN_STATE_CHECKED;
        }
        else
        {
            if (nUncheckedTex != 0)
				SetTexture(nUncheckedTex, rUncheckedTex);
            nBtnState = EButtonFlags.BTN_STATE_UNCHECKED;
        }
    }
    
    boolean GetCheck()
    {
        return ((nBtnState & EButtonFlags.BTN_STATE_CHECKED) == EButtonFlags.BTN_STATE_CHECKED);
    }
    
    

    void OnRBUp(float fMouseX, float fMouseY)
    {
    }
    
    /**Ustala stan podswietlenia przycisku
     * @param bHighlight jesli true to podswietlamy przycisk
     * @param bNotify ustala, czy powiadomic rodzica o podswietlaniu
     * @see Window#OnButtonHighlight(Button btnSender)
     */
    void SetHighlight(boolean bHighlight, boolean bNotify)
    {
        //LogLine("SetHighlight " + bHighlight + " nBtnState: " + nBtnState + " button: " + this);
        if ((nBtnState & EButtonFlags.BTN_STATE_DISABLED) == EButtonFlags.BTN_STATE_DISABLED) return;
        if ((nBtnType == EButtonFlags.BTN_TYPE_LISTBOX_ITEM)
            || (nBtnType == EButtonFlags.BTN_TYPE_LISTBOX)) 
            return;
        if (bHighlight)
        {
            if ((nBtnState & EButtonFlags.BTN_STATE_HIGHLIGHTED ) == 0)
            {
                nBtnState = nBtnState | EButtonFlags.BTN_STATE_HIGHLIGHTED;
                if (nHighlightedTex != 0) 
                {
                    //LogLine("SetHighlight");
                    SetTexture(nHighlightedTex, rHighlightedTex);
                    if (bNotify) ButtonHighlighted(this);
                }
            }
        }
        else
        {
            if ((nBtnState & EButtonFlags.BTN_STATE_HIGHLIGHTED ) == EButtonFlags.BTN_STATE_HIGHLIGHTED)
            {
                nBtnState = nBtnState & (~EButtonFlags.BTN_STATE_HIGHLIGHTED);
                SetCheck((nBtnState & EButtonFlags.BTN_STATE_CHECKED) 
                                     == EButtonFlags.BTN_STATE_CHECKED);
                //LogLine("UnSetHighlight");
            }
        }
    }
    
    void OnMouseEnter(float fMouseX, float fMouseY)
    {
        if (!CanProcessMouseMove())
            return;
        bMouseInside = true;
        SetHighlight(true, true);
    }
    
	void OnMouseLeave(float fMouseX, float fMouseY)
	{
        if (!CanProcessMouseMove())
            return;
        bMouseInside = false;
        super.OnMouseLeave(fMouseX, fMouseY);
        if ((nBtnState & EButtonFlags.BTN_STATE_DISABLED) == EButtonFlags.BTN_STATE_DISABLED)
        {
            return;
        }        
        if ((nBtnState & EButtonFlags.BTN_STATE_PUSHED) == EButtonFlags.BTN_STATE_PUSHED)
        {
            SetHighlight(false, true);
            nBtnState = nBtnState & (~EButtonFlags.BTN_STATE_PUSHED);   
            if ((nBtnType == EButtonFlags.BTN_TYPE_PUSHBUTTON)||
                (nBtnType == EButtonFlags.BTN_TYPE_PRESSBUTTON))
            {
                SetCheck(false);
            }
        }
        else 
        {
            SetHighlight(false, true);
        }
        //LogLine("OnMouseLeave");
	}

    void OnMouseMove(float fMouseX, float fMouseY, int iButtonsFlags)
    {
        if (!CanProcessMouseMove())
            return;
        super.OnMouseMove(fMouseX, fMouseY, iButtonsFlags);
        bMouseInside = true;
        SetHighlight(true, true);
        //LogLine("OnMouseMove");
    }
    
    /**Metoda wywolywana po nacisnieciu przycisku lub checkboxa. 
     * Wywoluje handlera ktory odpala {@link Window#OnButton(Button btnSender)} lub 
     * {@link Window#OnButtonDoubleClick(Button btnSender)}
     *@param btnSender wcisniety przycisk potomny
     */
    void ButtonPushed(Button btnSender)
    {
        float fPrevTime = fClickTime;
        fClickTime = GetTime();

        if (btnSender == null) 
        {
            btnSender = this;
        }
        
        if (fClickTime - fPrevTime < fDblClckTime)
        {
            bDoubleClick = true;
            CallOnButtonDoubleClick(btnSender);
            fClickTime = 0.0f;
        }
        else
        {
            CallOnButton(btnSender);
        }
    }
   
    /**Metoda wywolywana po podswietleniu przycisku lub checkboxa. 
     * Wywoluje handlera ktory odpala {@link Window#OnButtonHighlight(Button btnSender)}
     *@param btnSender wcisniety przycisk potomny
     * @see #SetHighlight(boolean bHighlight, boolean bNotify)
     */
    void ButtonHighlighted(Button btnSender)
    {
        //Log("Button: OnButtonHighlighted id: " + btnSender.nID + "\n");
        if (btnSender == null) 
        {
            btnSender = this;
        }
        CallOnButtonHighlight(btnSender);
    }
    
    /**Metoda wywolywana po wcisnieciu przycisku lub checkboxa prawym
     * klawiszem myszki.
     * Wywoluje handlera ktory odpala {@link Window#OnButtonRB(Button btnSender)}
     *@param btnSender wcisniety przycisk potomny
     */
    void ButtonPushedRB(Button btnSender)
    {
        //Log("Button: OnButtonPushedRB id: " + btnSender.nID + "\n");
        if (btnSender == null) 
        {
            btnSender = this;
        }
        CallOnButtonRB(btnSender);
    }
    
    /**Pozwala aktywowac lub deaktywowac przycisk. Jesli przy 
     * inicjacji podalismy teksture wyszarzonego przycisku to 
     * przy deaktywacji zostanie ona wyswietlona.
     * @param bEnable jesli true to przycisk aktywny jesli false 
     * to nieaktywny*/
    void Enable(boolean bEnable)
    {
        if (bEnable)
        {            
            nBtnState = nBtnState & (~EButtonFlags.BTN_STATE_DISABLED);
            SetCheck(((nBtnState & EButtonFlags.BTN_STATE_CHECKED) == EButtonFlags.BTN_STATE_CHECKED));
            SetState("Idle");
        }
        else
        {
            nBtnState = nBtnState | EButtonFlags.BTN_STATE_DISABLED;
            //nBtnState = nBtnState & (~EButtonFlags.BTN_STATE_HIGHLIGHTED);
            if (nDisabledTex != 0)
            {
                SetTexture(nDisabledTex, rDisabledTex);
            }
            SetState("Disabled");            
        }
    }
   
    /**Stan po wcisnieciu prawego klawisza myszy.*/
    public class RMBDown extends State
    {
        public void OnEnter()
        {
            //LogLine("RMBDown onenter");
        }
        void OnRBUp(float fMouseX, float fMouseY)
        {
            ButtonPushedRB(Button.this);
            SetState("Idle");
        }
        
        void OnRBDown(float fMouseX, float fMouseY)
        {
            SetState("Idle");
        }
        
        void OnMouseEnter(float fMouseX, float fMouseY)
        {
            SetHighlight(true, true);
            SetState("Idle");
        }
        
        void OnMouseLeave(float fMouseX, float fMouseY)
        {
            (Button.this).OnMouseLeave(fMouseX, fMouseY);
            SetHighlight(false, true);
            SetState("Idle");
        }
        

    }
    
    /**Stan neutralny (np. gdy zaden z klawiszy nie jest wcisniety)*/
    public class Idle extends State 
    {
        public void OnEnter()
        {
            if (nBtnType == EButtonFlags.BTN_TYPE_PRESSBUTTON)
            {
                if (bCallingPressButton) DisableCallMethod("PressButton");
                bCallingPressButton = false;
            }
            //LogLine("Idle onenter");
        }
     
        void OnLBDown(float fMouseX, float fMouseY)
	    {
            if (!CanProcessMouseBtns())return;
            if ((nBtnType == EButtonFlags.BTN_TYPE_PUSHBUTTON)||
                (nBtnType == EButtonFlags.BTN_TYPE_PRESSBUTTON))
            {
                SetCheck(true);
            }
            nBtnState = nBtnState | EButtonFlags.BTN_STATE_PUSHED;
            SetState("LMBDown");
	    }
        
        void OnRBDown(float fMouseX, float fMouseY)
        {
            if (!CanProcessMouseBtns())return;
            SetState("RMBDown");
        }
    }
    
    /**Stan po wcisnieciu lewego klawisza myszy.*/
    public class LMBDown extends State 
    {
        //zmienna potrzebna gdy przycisk jest pressbuttonem
        float fLastOnButtonTime;
        float fPeriod = 0.1f;
        public void OnEnter()
        {
            if (nBtnType == EButtonFlags.BTN_TYPE_PRESSBUTTON)
            {
                CallMethodEvery("PressButton", 0.1f);
                bCallingPressButton = true;
            }
            //LogLine("LMBDown onenter");
        }
        void OnLBUp(float fMouseX, float fMouseY)
	    {
            if (!CanProcessMouseBtns())return;
            if ((nBtnType == EButtonFlags.BTN_TYPE_CHECKBUTTON) ||
                (nBtnType == EButtonFlags.BTN_TYPE_LISTBOX_ITEM))
            {
                if ((nBtnState & EButtonFlags.BTN_STATE_CHECKED)== EButtonFlags.BTN_STATE_CHECKED)
                {
                    SetCheck(false);
                }
                else
                {
                    SetCheck(true);
                }
            }
            else//zwykly przycisk
            {
                SetCheck(false);
            }
            ButtonPushed(Button.this);
            SetHighlight(true, false);
            SetState("Idle");
	    }   
        
        void OnRBDown(float fMouseX, float fMouseY)
        {
            SetState("Idle");
        }
        
        void OnMouseEnter(float fMouseX, float fMouseY)
        {
            if (!CanProcessMouseMove())
                return;
            SetHighlight(true, true);
            SetState("Idle");
        }
        
        void OnMouseLeave(float fMouseX, float fMouseY)
        {
            if (!CanProcessMouseMove())
                return;
            (Button.this).OnMouseLeave(fMouseX, fMouseY);
            SetHighlight(false, true);
            SetCheck(false);
            SetState("Idle");
        }
    }
    
    /**Stan po zdeaktywowaniu przycisku 
     * @see #Enable(boolean bEnable)*/
    public class Disabled extends State
    {
        void OnLBDown(float fMouseX, float fMouseY){}
        void OnRBDown(float fMouseX, float fMouseY){}
        void OnMouseEnter(float fMouseX, float fMouseY){}//LogLine("Disabled.OnMouseEnter");}
        void OnMouseLeave(float fMouseX, float fMouseY){}//LogLine("Disabled.OnMouseLeave");}
        void OnMouseMove(float fMouseX, float fMouseY, int iButtonsFlags){}//LogLine("Disabled.OnMouseMove");}
    }
    
    void PressButton()
    {
        if (bDoubleClick)
        {
            bDoubleClick = false;
        }
        else
        {
            CallOnButton(this);
        }
    }
    
    String GetText()
    {
        return sText;
    }
	
    /**Ustala tekst na przycisku, ale go nie wyswietla.
     */
    void SetText(String sNewText)
    {
        sText = sNewText;
    }
	
	public void ReleaseTextures()
	{
		super.ReleaseTextures();

		if (nUncheckedTex != 0)
		{
			ReleaseTexture(nUncheckedTex); 
			nUncheckedTex = 0;
		}
		
		if (nCheckedTex != 0)
		{
			ReleaseTexture(nCheckedTex);    
			nCheckedTex = 0;
		}
		
		if (nDisabledTex != 0)
		{
			ReleaseTexture(nDisabledTex);    
			nDisabledTex = 0;
		}
		
		if (nHighlightedTex != 0)
		{
			ReleaseTexture(nHighlightedTex);
			nHighlightedTex = 0;
		}
	}

    public void DumpHeader(ScriptWriter cWriter)
    {
        cWriter.Write("Button(\"" + "UIButton" /*getClass().getName()*/ + "\"" + ", \"ID_" + String.valueOf(nID) + "\", \"" + sText + "\", \"Specific\")");
    }
    
    public void DumpData(ScriptWriter cWriter)
    {
        super.DumpData(cWriter);
        cWriter.Write("Text(\"" + sText + "\")");
        cWriter.Write("Font(\"" + GetFont() + "\")");
        if (m_cElementData != null)
        {
            cWriter.Write("Texture(\"" + m_cElementData.sUncheckedTex + "\")");
            cWriter.Write("TextureChecked(\"" + ((UIElement)m_cElementData).sCheckedTex + "\")");
            cWriter.Write("TextureHightlight(\"" + ((UIElement)m_cElementData).sHighlightedTex + "\")");
            cWriter.Write("TextureDisabled(\"" + ((UIElement)m_cElementData).sDisabledTex + "\")");
            if ((((UIElement)m_cElementData).nFlags & EButtonFlags.BTN_TYPE_CHECKBUTTON) == EButtonFlags.BTN_TYPE_CHECKBUTTON)
                cWriter.Write("CanCheck(1)");
        }
    }
    
}

