/**Modalny dialog typu yesno, wystarczy zainicjowac, podlaczyc jako potomka i czekac w oknie rodzica 
 * na komunikat EUI.ID_YES lub EUI.ID_NO, w pliku z tekstami powinny znalezc sie teksty o 
 * identyfikatorach BtnYes i BtnNo
 */
public class YesNoDialog extends Window
{
    Button btnYes = null;
    Button btnNo = null;
    /**Odleglosc pomiedzy przyciskami*/
    int nSeparator = 10;
    String sQuestionText = null;
    Sprite sprText = null;
    
    String sBkgTex = "";
    Rect rBkgTex = new Rect(0,0,1,1);
    UIElement uiBtnData = null;

    void Init(Module module, String sBkgTex, Rect rBkgTex, UIElement uiBtnData, String sQuestion)
    {
        Init(module, sBkgTex, rBkgTex, uiBtnData, sQuestion, Fonts.GetDefaultFont());
    }
    
    /**Inicjuje okienko dialogowe, ktore po tej operacji stanie sie modalne i przechwyci mysz.
     * @param module modul, w ktorym dzialamy
     * @param sBkgTex nazwa tekstury tla
     * @param rBkgTex fragment tekstury tla, jesli null, to standardowo tekstura ustawiana poprzez
     * {@link Sprite#SetBackground(String sName)}, wiec rozmiar tekstury musi sie zgadzac z 
     * rozmiarem sprite'a 
     * @param uiBtnData dane przyciskow, nalezy wypelnic pola dotyczace tekstur i rozmiaru
     * , oba przyciski beda takie same
     * @param sQuestion pytanie,na ktore oczekujemy odpowiedzi
     * @param sFont font pytania
     */
    void Init(Module module, String sBkgTex, Rect rBkgTex, UIElement uiBtnData, String sQuestion, String sFont)
    {
        this.sBkgTex = sBkgTex;
        if (rBkgTex != null) 
            this.rBkgTex = rBkgTex;
        this.uiBtnData = uiBtnData;
        sQuestionText = sQuestion;
        SetModule(module);
        ProcessMouse(true);
        MoveTop();
        Cursor cursor = module.GetCursor();
        if (cursor != null)
        {
            MoveTop();
            cursor.MoveTop();
        }
        
        sprText = (Sprite)CreateObject("Sprite");
        AddChild(sprText);
        sprText.SetTextPosFlags(ESpriteText.HORIZ_MIDDLE | ESpriteText.VERT_MIDDLE);
        sprText.SetFont(sFont);
        sprText.DisplayText(sQuestionText);
        sprText.Show();
        
        int nTex;
        if (rBkgTex != null)
        {
            nTex = AddTexture(sBkgTex);
            if (nTex != 0)
				SetTexture(nTex, this.rBkgTex);
        }
        else
        {
            SetBackground(sBkgTex);
        }
        Show();
        btnYes = (Button)CreateObject("Button");
        AddChild(btnYes);
        AddChildWnd(btnYes, this);
        btnYes.Init(new UIElement(EUI.ID_YES, 0, 
                                  new Vector(),
                                  uiBtnData.vSize,
                                  uiBtnData.sUncheckedTex,
                                  uiBtnData.sCheckedTex,
                                  uiBtnData.sDisabledTex,
                                  uiBtnData.sHighlightedTex,
                                  EButtonFlags.BTN_TYPE_PUSHBUTTON, 
                                  ""), module);
        btnYes.DisplayText(Text.Get("&BtnYes&"));
        btnNo = (Button)CreateObject("Button");
        AddChild(btnNo);
        AddChildWnd(btnNo, this);
        btnNo.Init(new UIElement(EUI.ID_NO, 0, 
                                  new Vector(),
                                  uiBtnData.vSize,
                                  uiBtnData.sUncheckedTex,
                                  uiBtnData.sCheckedTex,
                                  uiBtnData.sDisabledTex,
                                  uiBtnData.sHighlightedTex,
                                  EButtonFlags.BTN_TYPE_PUSHBUTTON, 
                                  ""), module);
        btnNo.DisplayText(Text.Get("&BtnNo&"));
        PositionButtons();
        SetModal();
        CaptureMouse();
    }   
    
    void PositionButtons()
    {
        if ((btnYes == null)||(btnNo ==null)||(sprText == null)) 
            return;
        
        Vector vSize = GetSize();
        Vector vYesPos = new Vector((vSize.fX - btnYes.GetSize().fX - btnNo.GetSize().fX - nSeparator) / 2,
                                     (vSize.fY / 3) * 2, 0);
        btnYes.SetRelativePos(vYesPos);
        Vector vNoPos = new Vector(vYesPos.fX + btnYes.GetSize().fX + nSeparator, vYesPos.fY, 0);
        btnNo.SetRelativePos(vNoPos);
        
        Vector vTextPos = new Vector(0, vSize.fY / 4, 0);
        Vector vTextSize = new Vector(vSize.fX, vSize.fY/3, 0);
        sprText.SetSize(vTextSize);
        sprText.SetRelativePos(vTextPos);
        

    }
    
    void OnButton(Button btnSender)
    {
        ClearModal();
        ClearCaptureMouse();
        if (btnSender == null) return;
        if (wndParent != null) wndParent.CallOnButton(btnSender);
    }
    
    public void SetSize(Vector vNewSize)
    { 
        super.SetSize(vNewSize);
        Vector vPos = new Vector((GetScreenWidth() - vNewSize.fX) / 2, (GetScreenHeight() - vNewSize.fY)/2, 0);
        SetPos(vPos);
        PositionButtons();
    }
    
    void Destroy()
    {
        module.RemoveSprite(btnYes);
        module.RemoveSprite(btnNo);
        module.RemoveSprite(sprText);
    }
}
