
public class MeshObject extends GroupObject
{
    /**Wmusza zapis obiektu do save'a*/	
	public boolean bForceSG = false; //#MESH
	
    /**Nazwa pliku zawierajacego dane obiektu 3da.
    domyslny parametr dla LoadMesh() wywolywanej podczas inicjacji obiektu.
    */
    public String MeshName = "data\\interface\\Wykrzyknik.3da"; //#MESH
    /** domyslny parametr morfingu dla LoadMesh() wywolywanej 
        podczas inicjacji obiektu.
        akceptuje stale zdefiniowane w EMorphType, @see EMorphType;
        ustawienie EMorphType.SOURCE oznacza ze mesh bierze udzial w morfingu
        MeshName zawiera nazwe mesha bedacego zrodlem dla morfingu, a
        TargetMorphName zawiera nazwe mesha bedacego celem dla morfingu.
        Ladowanie obu meshy odbywa sie automatycznie podczas inicjalizacji 
        obiektu lub jezeli nazwy i/lub enum nie sa ustawione mozna zrobic to 
        "recznie" przez metody LoadMesh(name) i/lub LoadTargetMorphMesh();
    */
    public int MorphType = EMorphType.NONE;
    /** TargetMorphName zawiera nazwe mesha bedacego celem dla morfingu.*/
    public String TargetMorphName;
	
    /** aduje mesha i TargetMorphMesha.
    aduje mesha i TargetMorphMesha na podstawie MeshName, MorphType i
    TargetMorphName. Jezeli MorphType jest rowne EMorphType.NONE to ladowana
    jest jedynie geometria o nazwie MeshName. Jezeli MorphType jest rowne
    EMorphType.SOURCE to dodatkowo ladowana jest geometria TargetMorphName
    ktora bedzie sluzyla jako cel podczas morfingu @see EMorphType.
    Jezeli nazwa TargetMorphName jest nieprawidlowa mozna pozniej zaladowac
    ta geometrie za pomoca LoadTargetMorphMesh.
    */
    native void LoadMesh();

    /** Zaladowanie mesha o podanej nazwie.
    tryb morfingu jest pobierany z MorphType. @see LoadMesh();
     */
    void LoadMesh(String sMeshName)
    {
        MeshName = sMeshName;
        LoadMesh();
    }

    /** [native]Zaladuj i podepnij geometrie ktora bedzie celem morfingu.
        Uwaga: Mesh zrodlowy musi byc wczesniej zaladowany w trybie EMorphType.SOURCE!
        @see LoadMesh(), @see EMorphType.
    */
    native boolean LoadTargetMorphMesh( String sMeshName );




	/**Sciezka do pliku z bitmapa majaca przedstawiac dany obiekt na
	 * zdjeciach satelitarnych. Jesli pusta, to obiekt nie bedzie
	 * zastapiony (bedzie normalnie wyrenderowany).
	 * 
	 * w konstruktorze obiektu, ktoremu chcemy ustawic ikone wrzucamy:
	 * 
	 *  //#DEF_BEGIN
     *  ScreenshotIconPath = "data/textures/icons/texname.png";
     *  //#DEF_END
     * 
	 * */
	public String ScreenshotIconPath = ""; //#MESH



	 /**[native] Ustawienie pozycji chwilowej .
     * @param iParticle numer particla
     * @param vPos okresla pozycje
     */
    public native void PhysicsSetParticlePos(int iParticle, Vector vVal);

    /**[native] Zerowanie sil (zostaje tylko grawitacja)
     */
    public native void PhysicsParticlesResetForces();
	
	public native void UpdateAnimations();
	
    ////////////////////////////////////////////////////////////////////////
    // FUNKCJE DO MORPHINGU POMIEDZY DWIEMA GEOMETRIAMI
    ////////////////////////////////////////////////////////////////////////

	/** Morfing pomiedzy dwiema geometriami wymaga dwoch geometrii zrodlowej i docelowej.
        Pozycja rysowanej geometrii wyliczana jest na podstawie wag dla kazdej pary wierzcholkow:
            Vmorph[i] = Vsrc[i] * (1-Vweight) + Vtarget[i] * Vweight[i];
        Vmorph[i] - wierzcholek po morfingu;
        Vsrc[i] - wierzcholek w zrodlowej geometrii;
        Vtarget[i] - wierzcholek w docelowej geometrii;
        Vweight[i] - waga interpolacji dla pary wierzcholkow;
    */

    /** [native]Zmodyfikuj wagi morfingu dla wierzcholkow znajdujacych sie w sferze (center, radius).
        @param vCenter srodek sfery w ukladzie swiata.
        @param fRadius promien sfery.
        @param fStrength sila modyfikacji - moze przyjmowac wartosci ujemne.
        @param bFalloff czy sila ma zanikac im dalej od srodka sfery?
            true - im blizej srodka sfery tym wartosc modyfikacji blizsza strength im dalej tym blizsza 0:
                w' = w + strength * LengthSqr( Vorg - center ) / radius * radius;
            false - sila dziala jednakowo w calej objetosci sfery
                w' = w + strength;
    */
    native void ApplyMorphInSphere( Vector vCenter, float fRadius, float fStrength, boolean bFalloff );

    /** [native]Dodaje do lub ustawia wage we wszystkich wierzcholkach modelu.
        @param fWeight waga.
        @param bAdd jezeli true to dodaje fWeight do aktualnych wag: w' = w + fWeight,
            jezeli false to ustawia: w' = fWeight.
    */
    native void ApplyMorphToAllVertices( float fWeight, boolean bAdd );


	/**[native] ustawia parametry widocznosci sluzace do liczenia 'fade out' 
	 * musi byc: blendRange < range
	 * jesli podac range < 0 to ustawinia brane sa z klasy CLevel (po stronie cpp)
	 * (ustawienia z CLevel mozna modyfikowac z GameOject'u: SetObjectsVisiblityRange)
	 * jesli sie tej metody nie wola to ustawienia rowniez brane sa z CLevel
	 * parametr range w obecnej chwili nie ma wplywu na zakres widocznosci obiektu, a jedynie
	 * sluzy do licznia 'fade out'
	 */
	public native void SetMeshVisibilityRange( float blendRange, float range );
	
	
	/**[native] ustawiaa flage czy testowac widocznoc obiektu uwzgledniajac jego wielkosc 
	 * obiekty minijesze (w przestrzeni ekranu) niz podana wielkosc nie zostana narysowane
	 * defaultowo flaga ustawiona jest na true. zobacz: Module::SetMeshCullSize(..)
	 * @param enable - flaga testowac/nie testowac
	 */
	public native void SetMeshCullSizeEnable( boolean enable );
	
	/**[native] ustawia 'ambient' w parametrach oswietlenia mesh'a
	 * parametry te sa wspolne dla wszystkich mesh'y na scenie
	 */
	public native void SetMeshMaterialAmbient( float r, float g, float b );

	/**[native] ustawia kolor mesh'a, kolor pojawi sie w tfactorze z user colorgenem
	 * @param rgb skladowe koloru w zakresie [0, 1]
	 */
	public native void SetMeshColor( Vector rgb );

	/**[native] ustawia alfe w kolorze mesh'a, alfa pojawi sie w tfactorze z user colorgenem
	 */
	public native void SetMeshColorAlpha( float alpha );

    /**[native] Podaje rozmiar mesha.
     */
    native public float GetMeshSize();
    
    /**[native] Podaj extenty mesha w jego lokalnym ukladzie wspolrzednych
     * w postaci 2 wektorow. Wielkosc bounding box mesh to <br>
     *      vMaxExtent - vMinExtent
     * @param vMinExtent wektor minimalny (<code>Vector</code>)
     * @param vMaxExtent wektor maksymalny (<code>Vector</code>)
     */
    native void GetMeshExtents( Vector vMinExtent, Vector vMaxExtent );

    /**[native] Podaje extenty mesha w ukladzie wspolrzednych swiata
     *  w postaci 2 wektorow. 
     * @param vMinExtent wektor minimalny (<code>Vector</code>)
     * @param vMaxExtent wektor maksymalny (<code>Vector</code>)
     */
    native void GetMeshExtentsInWorld( Vector vMinExtent, Vector vMaxExtent );
    
    
    /**[native] Podaje extenty elementu w ukladzie swiata
     * w postaci 2 wektorow. 
     * @param vMinExtent wektor minimalny (<code>Vector</code>)
     * @param vMaxExtent wektor maksymalny (<code>Vector</code>)
     * @param iElem numer elementu mesha (<code>int</code>)
     */
    native void GetElementExtentsInWorld( Vector vMinExtent, Vector vMaxExtent, int iElem  );

    /**[native] Podaje extenty elementu w ukladzie lokalnym elementu
     * w postaci 2 wektorow. 
     * @param vMinExtent wektor minimalny (<code>Vector</code>)
     * @param vMaxExtent wektor maksymalny (<code>Vector</code>)
     * @param iElem numer elementu mesha (<code>int</code>)
     */
    native void GetElementExtentsLocal( Vector vMinExtent, Vector vMaxExtent, int iElem  );

    /**[native] Podaj extenty mesha we wspolrzednych ekranu
     * @param Cam kamera, dla ktorej ma byc okreslony rozmiar mesha
     * @param vMinExtent lewy gorny rog (<code>Vector</code>)
     * @param vMaxExtent prawy dolny rog (<code>Vector</code>)
     */
    native void GetScreenExtents( Camera Cam, Vector vMinExtent, Vector vMaxExtent );

    /**[native] Podaj extenty elementu mesha we wspolrzednych ekranu. 
     * @param Cam kamera, dla ktorej ma byc okreslony rozmiar elementu
     * @param vMinExtent lewy gorny rog (<code>Vector</code>)
     * @param vMaxExtent prawy dolny rog (<code>Vector</code>)
     * @param Element identyfikator elementu pobrany metoda GetElementID()
     */
    native void GetScreenExtents( Camera Cam, Vector vMinExtent, Vector vMaxExtent, int Element );
    
    /**[native] Podlacza obiekt obj do podanego node'a.
     * @param obj - podlaczany obiekt
     * @param sNode - node, do ktorego obiekt jest podlaczany
     */
    public final native void AttachChild(ControlObject obj, String sNode);
    
    /**[native] Podlacza obiekt obj do podanego node'a. Zachowuje pozycje obiektu obj w swiecie.
     * @param obj - podlaczany obiekt
     * @param sNode - node, do ktorego obiekt jest podlaczany
     */
    public final native void AttachChildAndKeepWorldPos(ControlObject obj, String sNode);

    /**[native] Podlacza obiekt obj do podanego node'a.
     * @param obj - podlaczany obiekt
     * @param iElem - meshelement, do ktorego obiekt jest podlaczany
     */
    public final native void AttachChild(ControlObject obj, int iElem);
    
    /**[native] Podlacza obiekt obj do podanego node'a. Zachowuje pozycje obiektu obj w swiecie.
     * @param obj - podlaczany obiekt
     * @param iElem - meshelement, do ktorego obiekt jest podlaczany
     */
    public final native void AttachChildAndKeepWorldPos(ControlObject obj, int iElem);

    /**[native] Ustawia elipsoide obiektu. Razem z elipsoida zmieniane sa extenty
     * obiektu na extentu opisane na elipsoidzie
     * @param fRadius - promien elipsoidy w osiach X i Z obiektu
     * @param fHeight - wysokosc elipsoidy (czyli podwojny promien w osi Y)
     * @param fYPos - przesuniecie elipsoidy w Y wzgledem pivot pointa obiektu
     */
    public final native void SetEllipsoid(float fRadius, float fHeight, float fYPos);

    /**[native] Pobiera elipsoide obiektu. 
     * @return wektor, ktrego polami sa:
     * fX - fRadius - promien elipsoidy w osiach X i Z obiektu
     * fY - fHeight - wysokosc elipsoidy (czyli podwojny promien w osi Y)
     * fZ - fYPos - przesuniecie elipsoidy w Y wzgledem pivot pointa obiektu
     */
    public final native Vector GetEllipsoid();
	
    /**[native] Ustawia elipsoide obiektu. Nie zmienia extentow obiektu.
     * 
     * UWAGA!!! jesli extenty beda mniejsze od elipsoidy to moga sie sypac kolizje
     * z zalozenia extenty zawieraja elipsoide i na tej podstawie sa badane kolizje
     * 
     * @param fRadius - promien elipsoidy w osiach X i Z obiektu
     * @param fHeight - wysokosc elipsoidy (czyli podwojny promien w osi Y)
     * @param fYPos - przesuniecie elipsoidy w Y wzgledem pivot pointa obiektu
     */
    public final native void SetEllipsoidNoExtents(float fRadius, float fHeight, float fYPos);

    /**[native] Ustawia extenty obiektu. Nie zmienia elipsoidy.
     * 
     * UWAGA!!! jesli extenty beda mniejsze od elipsoidy to moga sie sypac kolizje
     * z zalozenia extenty zawieraja elipsoide i na tej podstawie sa badane kolizje
     * 
     * @param vP1, vP2 - przeciwlegle punkty definiujace nowy extent
     */
    public final native void SetExtents(Vector vP1, Vector vP2);

    ///////////////////////////////////////////////////////////////////////////
    // ANIMATION INTERFACE 
    ///////
    
    /**[native] Inicjalizuje sekwencje animacji.
     * Dodaje nowe lub zastepuje juz zdefiniowane sekwencje animacji.
     * @param sSeqDef lancuch znakow zawierajacy definicje sekwencji animacji
     * w formacie "nazwa0(ramka_poczatkowa ramka_koncowa) nazwa1(.."
     * np: "seq0(0 50) seq1(51 99)" wstawi 2 sekwencje (<code>String</code>)
     * @return true jesli inicjacja wszystkich sekwencji przebiegla prawidlowo
     */
    public native boolean InitAnimSeqStr(String sSeqDef);

    /**[native] Inicjalizuje sekwencje animacji.
     * Dodaje nowe lub zastepuje juz zdefiniowane sekwencje animacji.
     * @param sFileName plik zawierajacy definicje sekwencji animacji
     * w formacie "nazwa0(ramka_poczatkowa ramka_koncowa) nazwa1(.."
     * np: "seq0(0 50) seq1(51 99)" wstawi 2 sekwencje (<code>String</code>)
     * @return true jesli inicjacja wszystkich sekwencji przebiegla prawidlowo
     */
    public native boolean InitAnimSeqFile(String sFileName);

    /**[native] Inicjalizuje pojedyncza sekwencje animacji.
     * @param sSeqName nazwa sekwencji (<code>String</code>)
     * @param nStartFrame ramka poczatkowa (<code>int</code>)
     * @param nEndFrame ramka koncowa (<code>int</code>)
     * @return true jesli inicjacja sie powiodla
     */
    public native boolean InitAnimSeq(String sSeqName, int nStartFrame, int nEndFrame);

    /**[native] Zwraca identyfikator animacji dla danego elementu i sekwencji.
     * Identyfikator animacji jest unikalny dla kazdej pary (nazwa_elementu, nazwa_sekwencji).
     * @param sElemName nazwa elementu (<code>String</code>)
     * @param sSeqName nazwa sekwencji (<code>String</code>)
     * @return identyfikator, gdy podany zly element to zwraca -1
     */
    public native int GetAnimID(String sElemName, String sSeqName);

    /**[native] Zwraca identyfikator animacji dla danego elementu i sekwencji.
     * Identyfikator animacji jest unikalny dla kazdej pary (nazwa_elementu(id), nazwa_sekwencji).
     * @param nElemID identyfikator elementu (<code>String</code>)
     * @param sSeqName nazwa sekwencji (<code>String</code>)
     * @return identyfikator, gdy podany zly element to zwraca -1
     */
    public native int GetAnimID(int nElemID, String sSeqName);

    /**[native] Zwraca identyfikator animacji dla danego elementu i sekwencji identyfikowanej
     * id nadanym przez uzytkownika.
     * Identyfikator animacji jest unikalny dla kazdej pary (id_elementu, id_sekwencji).
     * @param nElemID identyfikator elementu (<code>String</code>)
     * @param nUserSeqID identyfikator animacji przypisany przez uzytkownika (<code>String</code>)
     * @return identyfikator, gdy podany zly element to zwraca -1
     */
    public native int GetAnimID(int nElemID, int nUserSeqID);
    
    /**[native] Zwraca nazwe animacji o podanym ID
     * @param iAnimID identyfikator animacji
     * @return nazwa animacji albo null, gdy nieprawidlowe iAnimID
     */
    public native String GetAnimName(int iAnimID);
    
    /**[native] Nadaje podanej sekwencji animacji id nadane przez uzytkownika.
     * Gdy podany identyfikator nie jest unikalny to wywala sie asercja.
     * @param sSeqName nazwa sekwencji (<code>String</code>)
     * @param nUserSeqID unikalny identyfikator animacji podawany przez uzytkownika (<code>int</code>)
     */
    public native void SetUserSeqID(String sSeqName, int nUserSeqId);

    /**[native] Zwraca nazwe animacji o podanym userID ustawionym przez SetUserSeqID
     * @param nUserSeqId identyfikator animacji
     * @return nazwa animacji albo null, gdy nieprawidlowe nUserSeqId
     */
    public native String GetAnimNameForUserSeqID(int nUserSeqId);
    
    /** [native] Blokuje zejscie animacji w dol hierarchii.
     * Np. SetAnimBlock("Bip01 Spine1", true) zablokuje animacje, ktory byly
     * odpalone nizej w hierarchii (np. Spine) dla wszystkich elementow powyzej
     * Spine1 (np. Head).
     */
    public native void SetAnimBlock(String sElemName, boolean bBlock);

    /**[native] Pobiera ilosc klatek w calej animacji obiektu.
     * @return Ilosc klatek w animacji obiektu.
     */
    public native float GetNumFrames();
    
    /** [native] Odgrywa sekwencje animacji.<br>
     * Po zakonczeniu animacji obiekt pozostaje w ostatniej ramce swojej
     * sekwencji animacji.<br>
     * Po uruchomienu PlayAnim moga zostac wywolane nastepujace handlery:
     *  - OnAnimEnd(int iAnimID) - po zakonczeniu animacji<br>
     *  - OnAnimCycle(int iAnimID, int iCount) - po zakonczeniu cyklu animacji,
     * iCount jest numerem zakonczonego cyklu<br>
     *  - OnAnimMorph(int iAnimID) - po zakonczeniu morphingu z poprzedniej animacji<br>
     * @param iAnimID identyfikator animacji (<code>int</code>)
     * @param iCount ilosc powtorzen, nieskonczona petla dla iCount <=0 (<code>int</code>)
     * @return identyfikator animacji
     */
    public native int PlayAnim(int iAnimID, int iCount);

    /** Odgrywa sekwencje animacji z uwzglednieniem czasu 
     * @param iAnimID identyfikator animacji (<code>int</code>)
     * @param fTime bezwzgledny czas rozpoczecia animacji (<code>float</code>)
     */
    public native int PlayAnimAtTime(int iAnimID, float fTime);
  
    /** j.w.
     */
    public native int PlayAnimAtTime(String sElemName, String sSeqName, float fTime);
    
    public int PlayAnimAtTime(String sSeqName, float fTime)
    {
        return PlayAnimAtTime("", sSeqName, fTime);
    }
    
    /** [native] Odgrywa sekwencje animacji dla podanej nazwy elementu i sekwencji.<br>
     * Dziala jak {@link #PlayAnim}.
     * @param sElemName nazwa elementu. Jesli "" to animacja uruchamiana jest dla
     * elementu 0 lub roota jesli mesh jest hierarchia (np. biped z CS) (<code>String</code>)
     * @param sSeqName nazwa sekwencji (<code>String</code>)
     * @param iCount liczba powtorzen animacji (<code>int</code>)
     * @return identyfikator animacji
     */
    public native int PlayAnim(String sElemName, String sSeqName, int iCount);
    
    
    public int PlayAnim(String sSeqName, int iCount)
    {
        return PlayAnim("", sSeqName, iCount);
    }
       
    
    /**
     */
    public void PlayAnimSeq(int iStart, int iEnd)
    {
        InitAnimSeq("seq0", iStart, iEnd);
        PlayAnim("", "seq0", 1);
    }
    

    /**[native]
     * Przyklad uzycia: ustawienie reki w 10.5 klatce animacji "up"
     *
     * int animID = GetAnimID("", "up");
     * int frame = GetFirstFrame(aimID) + 10.5;
     * //int frame = GetFirstFrame(Anims[A_STAND]);
     * int elem = GetMeshElemFromBoneID(EBones._PELVIS);
     * SetFrame(frame, elem);
     * Vector pos = GetBoneJointPos(EBones._L_HAND);
     * 
     * @param fFrame
     * @param iElem
     */
    public native void SetFrame(float fFrame, int iElem);
	public native void SetFrame(String seqName, float frame, int elem);
	public native void SetFrame(int animId, float frame, int elem);

    /**[native]
     * @param iAnimID
     * @return numer pierwszej klatki sekwencji animacji
     */
    public native int GetFirstFrame(int iAnimID);

    /**[native]
     * @param iAnimID
     * @return numer ostatniej klatki sekwencji animacji
     */
    public native int GetLastFrame(int iAnimID);

    /** [native] Sprawdza czy dla podanego elementu jest odgrywana animacja
     * @param sElemName nazwa elementu (<code>String</code>)
     * @return true jesli jakas animacja jest odgrywana dla sElemName
     */
    public native boolean IsPlaying(String sElemName);

    /** [native] Sprawdza czy podana animacja jest odgrywana
     * @param iAnimID identyfikator animacji (<code>int</code>)
     * @return true jesli jakas animacja jest odgrywana dla sElemName
     */
    public native boolean IsPlaying(int iAnimID);

    /** [native] Zatrzymuje animacje dla podanego elementu
     * @param sElemName nazwa elementu (<code>String</code>)
     */
    public native void StopAnim(String sElemName);

    /** [native] Zatrzymuje animacje dla podanego elementu
     * @param nElem identyfikator elementu (<code>int</code>)
     */
    public native void StopAnim(int nElem);

    /** [native] Wznawia animacje podanego elementu jesli jakas zostala zatrzymana
     * @param sElemName nazwa elementu (<code>String</code>)
     */
    public native void StartAnim(String sElemName);

    /** [native] Wznawia animacje podanego elementu jesli jakas zostala zatrzymana
     * @param nElem identyfikator elementu (<code>int</code>)
     */
    public native void StartAnim(int nElem);

    /** [native] Ustawia czas trwania sekwencji animacji
     * @param sSeqName nazwa sekwencji dla ktorej ustawic czas trwania (<code>String</code>)
     * @param fTime czas trwania animacji [s] (<code>int</code>)
     */
    public native void SetAnimLength(String sSeqName, float fTime);

    /** [native] Pobiera dlugosc sekwencji animacji
     * @param sSeqName nazwa sekwencji animacji
     * @return dlugosc sekwencji animacji [s]
     */
    public native float GetAnimLength(String sSeqName);

    /** [native] Pobiera dlugosc sekwencji animacji
     * @param iAnimID identyfikator animacji
     * @return dlugosc sekwencji animacji [s]
     */
    public native float GetAnimLength(int iAnimID);

    /** [native] Ustawia czestotliwosc odgrywania sekwencji animacji
     * @param sSeqName nazwa sekwencji dla ktorej ustawic czestotliwosc (<code>String</code>)
     * @param fTime czestotliwosc [Hz] (<code>float</code>)
     */
    public native void SetAnimFreq(String sSeqName, float fFreq);

    /** [native] Ustawia czestotliwosc odgrywania sekwencji animacji
     * @param iAnimID identyfikator sekwencji dla ktorej ustawic czestotliwosc (<code>int</code>)
     * @param fTime czestotliwosc [Hz] (<code>float</code>)
     */
    public native void SetAnimFreq(int iAnimID, float fFreq);

    /** [native] Zwraca czestotliwosc odgrywania animacji.
     * @param sSeqName nazwa sekwencji (<code>String</code>)
     * @return czestotliwosc [Hz]
     */
    public native float GetAnimFreq(String sSeqName);

    /** [native] Zwraca czestotliwosc odgrywania animacji.
     * @param iAnimID identyfikator sekwencji (<code>int</code>)
     * @return czestotliwosc [Hz]
     */
    public native float GetAnimFreq(int iAnimID);

    /** [native] Ustawia czas morphingu z poprzedniej animacji
     * @param sElemName nazwa meshElementu dla ktorego ustawic czas
     * @param fTime czas morphingu[s] (<code>float</code>)
     * @param bPropagate jesli true, to czas zostanie ustawiony dla wszystkich elementow w hierarchii
     * ponizej podanego sElemName
     */
    public native void SetAnimMorphTime(String elemName, float fTime, boolean bPropagate);

    /** [native] Ustawia czas morphingu z poprzedniej animacji
     * @param nElemID numer meshElementu dla ktorego ustawic czas (<code>int</code>)
     * @param fTime czas morphingu[s] (<code>float</code>)
     * @param bPropagate jesli true, to czas zostanie ustawiony dla wszystkich elementow w hierarchii
     * ponizej podanego nElemID
     */
    public native void SetAnimMorphTime(int nElemID, float fTime, boolean bPropagate);
    
    /** [native] Pobiera czas morphingu z poprzedniej animacji
     * @param elemName nazwa meshElementu, dla ktorego pobierany jest czas (<code>String</code>)
     * @return czas morphingu dla podanego meshElementu
     */
    public native float GetAnimMorphTime(String elemName);

    /** [native] Pobiera czas morphingu z poprzedniej animacji
     * @param nElemID numer meshElementu, dla ktorego pobierany jest czas (<code>int</code>)
     * @return czas morphingu dla podanego meshElementu
     */
    public native float GetAnimMorphTime(int nElemID);

    /** Ustawia czas morphingu dla wszystkich elementow ponizej sElemName w hierarchii.
     */
    public void SetAnimMorphTime(String sElemName, float fTime)
    {
        SetAnimMorphTime(sElemName, fTime, false);
    }

    /** Ustawia czas morphingu z poprzedniej animacji dla wszystkich elementow mesha.
     */
    public void SetAnimMorphTime(float fTime)
    {
        SetAnimMorphTime("", fTime, true);
    }

    
    
    /**[native] Odpowiednik SetAnimType ale dla obiektow bez kosci
     * @param nElemID identyfikator MeshElementu, dla ktorego ustawiamy typa animacji
     * @param eAnimType Sposob animacji elementu (klasa {@link EAnimType}) (<code>byte></code>)
     * @param bPropagate Jesli true - sposob animacji propaguje sie na elementy
     * lezace nizej w hierarchii
     */
    public native void SetElementAnimType(int nElemID, byte eAnimType, boolean bPropagate);

    
    /** [native] Metoda sprawdzajaca czy MeshObject jest Biped'em
     */
    public native boolean IsBiped();
    
    /**[native] Odpowiednik BoneRotate dla obiektow bez kosci
     * @param nElemID identyfikator obracanego MeshElementu
     * @param vAxis os obrotu
     * @param fAngle kat o jaki obracamy [stopnie]
     * @param bApply znacznik skladania obrotu z poprzednimi (gdy false, macierz
     * obrotu jest zastepowana nowa)
     */
    public native void RotateElementWithAnim(int nElemID, Vector vAxis, float fAngle, boolean bApply);

    public void RotateElementWithAnim(int nElemID, Vector vAxis, float fAngle)
    {
        RotateElementWithAnim(nElemID, vAxis, fAngle, false);
    }

    ///////////////////////////////////////////////////////////////////////////
    // BIPED OBJECTS INTERFACE
    // ( ponizsze metody nadaja sie do uzytku tylko dla
    //   MeshObjectow ze szkieletem (biped) )
    ///////////////////////////////////////////////////////////////////////////

    /**[native] Ustawia sposob animacji kosci. Domyslnie kazda kosc jest animowana
     * tylko za pomoca sekwencji animacji. Mozna laczyc kilka sposobow animacji, np po wywolaniu:<br>
     * 
     *      SetAnimType(EBones._NECK, EAnimType._SEQUENCE || EAnimType._TRANSFORM, false);
     * 
     * polozenie szyi bedzie modyfikowane za pomoca sekwencji oraz transformacji.
     * @param eBone Identyfikator kosci (z klasy {@link EBones}) (<code>byte</code>)
     * @param eAnimType Sposob animacji kosci (klasa {@link EAnimType}) (<code>byte></code>)
     * @param bPropagate Jesli true - sposob animacji propaguje sie na kosci
     * lezace nizej w hierarchii
     */
    public native void SetAnimType(byte eBone, byte eAnimType, boolean bPropagate);
    
    /**[native] Ustawia polozenie kosci.
     * @param eBone identyfikator kosci
     * @param vDir kierunek wzdlug ktorego kosc zostanie zorientowana
     * @param vPerp kierunek prostopadly do kierunku orientacji kosci.
     * Uwaga: Funkcja odniesie skutek jesli wczesniej zostanie ustawiona animacja za pomoca tranformacji,
     * np. wykonanie: 
     * 
     *      SetAnimType(EBones._HEAD, EAnimType._TRANSFORM, false);
     *      SetBoneOrientation(EBones._HEAD, new Vector(0, 1, 0), new Vector(1, 0, 0));
     * 
     * zmieni polozenie glowy.
     */
    public native void SetBoneOrientation(byte eBone, Vector vDir, Vector vPerp);
    
    /**[native] Ustawia macierz rotacji dla kosci. Kosc musi byc animowana za pomoca
     * EAnimType._TRANSFORM.
     * @param eBone Identyfikator kosci (z klasy {@link EBones}) (<code>byte</code>)
     * @param vAxis Os obrotu (<code>Vector</code>)
     * @param fAngle Kat obrotu (stopnie) (<code>float</code>)
     * @param bApply jesli ustawiona, to macierz obrotu zostanie zlozona z juz istniejaca, inaczej
     * macierz zostanie stworzona. Jesli wiec bApply = true, to kazdorazowe wywolanie BoneRotate
     * dla danej kosci bedzie obracalo ja w stosunku do poprzedniej pozycji i np. wywolanie:
     * 
     *      BoneRotate(EBones._HEAD, new Vector(0, 1, 0), 5, true);
     *      BoneRotate(EBones._HEAD, new Vector(0, 1, 0), 5, true); 
     * 
     * obroci glowe o 10 stopni, podczas gdy po:
     * 
     *      BoneRotate(EBones._HEAD, new Vector(0, 1, 0), 5, false);
     *      BoneRotate(EBones._HEAD, new Vector(0, 1, 0), 5, false); 
     *
     * glowa bedzie obrocona o 5 stopni (<code>boolean</code>)
     */
    public native void BoneRotate(byte eBone, Vector vAxis, float fAngle, boolean bApply);

    /** (@link BoneRotate(byte eBone, Vector vAxis, float fAngle, boolean bApply))
     */
    public void BoneRotate(byte eBone, Vector vAxis, float fAngle)
    {
        BoneRotate(eBone, vAxis, fAngle, false);
    }

    /**[native] Sprawdza kierunek kosci (jej os X)
     * @param eBone Identyfikator kosci (z klasy {@link EBones}) (<code>byte</code>)
     * @return kierunek wzdluz kosci (<code>Vector</code>)
     */
    public native Vector GetBoneDirVector(byte eBone);
    
    /**[native] Sprawdza kierunek prostopadly do kosci (jej os Z)
     * @param eBone Identyfikator kosci (z klasy {@link EBones}) (<code>byte</code>)
     * @return kierunek prostopadly do kosci (<code>Vector</code>)
     */
    public native Vector GetBonePerpVector(byte eBone);

    /**[native] Sprawdza pozycje polaczenia miedzy kosciami (podstawy kosci eBone)
     * @param eBone Identyfikator kosci (z klasy {@link EBones}) (<code>byte</code>)
     * @return pozycja podstawy kosci eBone(<code>Vector</code>)
     */
    public native Vector GetBoneJointPos(byte eBone);

    /**[native] Podaje identyfikatoro kosci (zgodny z definicjami w (@link EBones))
     */
    public native byte GetBoneIDFromMeshElem(int iMeshElem);

    /**[native] Podaje identyfikator mesha na podstawie identyfikatora kosci.
     */
    public native int GetMeshElemFromBoneID(byte eBone);

    /**[native] Tworzy obiekt odpowiedzialny za symulacje fizyki ciala podczas
     * interakcji ze srodowiskiem (postrzaly, wybuchy, trzesienie ziemi..)
     * @param fBodyMass ciezar ciala (<code>float</code>)
     */
    public native void PhysicsCreate(float fBodyMass);
    
    /**[native] Tworzy obiekt odpowiedzialny za symulacje fizyki ODE ciala podczas
     * interakcji ze srodowiskiem (postrzaly, wybuchy, trzesienie ziemi..)
     * @param fBodyMass ciezar ciala (<code>float</code>)
     */
    public native void ODEBipedPhysicsCreate(float fBodyMass);
    
    /**[native] Tworzy obiekt odpowiedzialny za symulacje fizyki.
     * @param iPhysiscType: 0 - fizyka ciala, 1 - fizyka rigid body
     * @param sConfigFile: nazwa pliku konfiguracyjnego 
     * @param fBodyMass ciezar ciala
     */
    //OBSOLETE, use 3-parameter version instead
    public native void PhysicsCreate(int iPhysicsType, float fBodyMass);
    public native void PhysicsCreate(int iPhysicsTpye, String sConfigFile, float fBodyMass);

    /**[native] Aktywuje fizyke ciala (musi byc ona najpierw stworzona za
     * pomoca (@link PhysicsCreate)
     */
    public native void PhysicsOn();

    /**[native] Deaktywuje fizyke ciala.
     */
    public native void PhysicsOff();

    /**[native] Uderzenie w cialo.
     * @param eBone identyfikator kosci z klasy EBones
     * @param vPos miejsce uderzenia (np. punkt zwrocony prez engine kolizji)
     * @param vDir kierunek uderzenia (np. kierunek lotu pocisku)
     * @param fStrength wzgledna sila uderzenia
     * @param fDuration czas trwania symulacji. Po tym czasie pozycje
     * kosci sa blendowane do tych wynikajacych z animacji (w czasie trwania
     * symulacji fizyki animacje sa odgrywane). fDuration < 0 oznacza symulacje
     * w nieskonczonosc.
     */
    public native void PhysicsHit(byte eBone, Vector vPos, Vector vDir, 
                                  float fStrength, float fDuration);

    /**[native] Eksplozja kierunkowa. W kazdym miejscu sila odrzutu ma ten sam kierunek.
     * @param vCenter centrum eksplozji
     * @param vDir kierunek odrzutu
     * @param fStrength sila odrzutu, maleje liniowo do zera w odleglosci fRange od vCenter
     * @param fTime czas, przez jaki sila odrzutu dziala na symulowany obiekt
     * @param fRange zasieg eksplozji, jesli obiekt znajduje sie w zasiegu > fRange, to nie dziala
     * na niego zadna sila
     * @return void
     */
    public native void PhysicsDirExplosion(Vector vCenter, Vector vDir,
                                           float fStrength, float fTime, float fRange);

    /** [native] Eksplozja.
     * @param vCenter centrum eksplozji
     * @param fStrength sila odrzutu, maleje liniowo do zera w odleglosci fRange od vCenter
     * @param fTime czas, przez jaki sila odrzutu dziala na symulowany obiekt
     * @param fRange zasieg eksplozji, jesli obiekt znajduje sie w zasiegu > fRange, to nie dziala
     * na niego zadna sila
     * @return void
     */
    public native void PhysicsExplosion(Vector vCenter, float fStrength,
        float fTime, float fRange
        );

    /**[native] Impuls sily w obiekt z wlaczona fizyka EPhysics._RIGIDBODY.     
     * @param vPos miejsce przylozenia sily
     * @param vDir kierunek przylozenia sily
     * @param fStrength wartosc sily
     * @param fDuration czas trwania sily. Po tym czasie pozycje sila nie ma wplywu na obiekt     
     */
    public native void ParticlePhysicsHit(
        Vector vPos, Vector vDir, float fStrength, float fDuration
        );

    /**[native] Ustawienie sily przylozonej do particla.
     * Uwaga: Sila moze byc zmieniona przez powtorne wywolanie funkcji.
     * Jesli wiec chcemy nadac particlowi po prostu jakas predkosc, to
     * lepiej uzyc PhysicsSetParticleInstantVelocity.
     */
    public native void PhysicsSetParticleForce(
        int iParticle, Vector vDir, float fStrength
        );

    /**[native] Ustawienie predkosci chwilowej dla particla.
     * @param iParticle numer particla
     * @param vVal okresla kierunek i wartosc predkosci [m/s]
     */
    public native void PhysicsSetParticleInstantVelocity(
        int iParticle, Vector vVal
        );

    /**[native] Przesuwa particle zachowujac ich wzgledne polozenie.
     * @param shift Wektor przesuniecia
     */
    public native void PhysicsShiftParticles( Vector shift );
    
    /**[native] Pobiera aktualna predkosc particla o zadanym id
     * @return wektor predkosci wyrazony w jednostkach gry/s ([cm/s])
     */
    public native Vector PhysicsGetParticleSpeed(int iParticle);

    public native Vector PhysicsGetAvgParticleSpeed(int iParticle);


    public native Vector PhysicsGetAvgParticleAccel(int iParticle);
	
	/**[native] Pobiera informacje czy pozycje particli zostaly juz zainicjowane
     * @return 
     */
	public native boolean PhysicsParticlePosInited();
	
    /**[native] Pobiera aktualna pozycje particla o zadanym id
     * @return pozycja particla iPartcle
     */
    public native Vector PhysicsGetParticlePos(int iParticle);

    /**[native] 
     * @param iParticle index of particle
     * @param vDir
     * @param fDragD
     * @param fDragPerp
     * Uwaga: Do uzycia tylko z systemem particli symulujacycm cialo sztywne
     */
    public native void PhysicsSetParticleOrientation(
        int iParticle, Vector vDir, float fDragD, float fDragPerp
        );

    /**[native] Clears orientation of particle - particle will be treated as non-oriented
     * @param iParticle index of particle
     */
    public native void PhysicsClearParticleOrientation( int iParticle );
    
    /**[native] Sets parameter of particle physics engine for this object
     * @param iParameter parameter id, taken from from EParticlePhysics enum
     * @param fValue float value of the parameter
     */
    public native void PhysicsSetParameterFloat(int iParamter, float fValue);
	public native void PhysicsSetParticleParameterFloat(int iParameter, int iParticle, float fValue);

    //////////////////////////////////////////////////////////////
    // recording of physics simulation
    
    
    /**[native] Turns on regular simulation of particles
     */
    public native void PhysicsSimulate();

    /**[native] Starts recording of particle simualtion
     */
    public native void PhysicsRecord();
    
    /**[native] Dogrywa do danych obecnych w obiekcie idRecordData.
     * @param idRecordData identyfikator danych z powtorka. Musi byc wczesniej
     * uzyskany za pomoca wywolania PhysicsGetRecordData.
     * Raz uzyty identyfikator nie moze byc uzyty powtornie!
     */
    public native void PhysicsRecord( int idRecordData);

    /**[native] Zwraca identyfikator danych z nagrana powtorka
     * @return id danych lub 0 jesli nie ma particlowej fizyki albo powtorka
     * nie zostala nagrana
     */
    public native int PhysicsGetRecordData();

    /**[native] Starts playback of particle simulation from
     * last recorded data
     * @param fStart indicated where to start the playback
     * @param bRev whether to play reverse or forward
     * this value is in rage [0..1], where 0 - start of recorded data, 1 - end of recorded data
     */
    public native void PhysicsPlay(float fStart, boolean bRev);

    ///////////////////////////////////////////////////////////////////////////
    // Particle Rigid Body Interface 
    ///////
    /** PROTOTYP!!! 
        System czasteczek symulujacy bryle sztywna. */

    /** Handler wywolywany dla kazdego obiektu uczestniczacego w kolizji.
    Niezbednym warunkiem otrzymania powiadomienia o kolizji jest
    wczesniejsze ustawienie odpowiedniej flagi za pomoca SetRigidBodyCollisionNotifyFlags(). 
    Domyslnym zachowaniem systemu ciala sztywnego jest odbicie sie od przeszkody.
    Mozna to zmienic wywolujac wewnatrz Handlera metode SetRigidBodyCollisionResponse().

    @param obstacle przeszkoda na ktora natrafila bryla. Moze byc null w przypadku 
        terenu lub drzewa!
    @param eType    typ przeszkody na ktora natrafila bryla. @see EObstacleType
    @param vHitPoint    punkt trafienia we wspolrzednych swiata
    @param vVelocity    wektor predkosci z jaka porusza sie bryla, a wlasciwie predkosc
        czasteczki/czasteczek w chwili trafienia w przeszkode. [cm/s]
    */
    public void OnCollisionParticleRigidBody( MeshObject obstacle, int eType, 
        Vector vHitPoint, Vector vVelocity )
    {   CrashLog( "" ); }

    /** [native] Okresla jak ma zareagowac bryla sztywna na aktualnie zgloszona kolizje.
            @see ECollisionResponse. Mozliwe wartosci
        ECollisionResponse.IGNORE - powoduje zignorowanie kolizji - czyli bryla przenika przez przeszkode.
        ECollisionResponse.BOUNCE - powoduje odbicie sie od przeszkody.
    */
    public native void SetRigidBodyCollisionResponse( int eResponse );

    /** [native] Wlacza powiadamianie o kolizjach z wybranymi typami obiektow/przeszkod.
    @param iFlags powstaje ze zlozenia enumow typu EObstacleType. */
    public native void SetRigidBodyCollisionNotifyFlags( int iFlags );
    /** [native] Wylacza powiadamianie o kolizjach z wybranymi typami obiektow/przeszkod. 
    @param iFlags powstaje ze zlozenia enumow typu EObstacleType.
    */
    public native void ClearRigidBodyCollisionNotifyFlags( int iFlags );
    /** [native] Zwraca aktualne flagi powiadomien o kolizji. Flagi powstaja ze zlozenia
    enumow typu @see EObstacleType. i jezeli koniunkcja logiczna flag z odpowiednim enumem 
    zwroci ten enum to obiekt otrzyma powiadomienie o kolizji z takim typem.
        boolean bNotifedAboutTrees = ( GetRigidBodyCollisionNotifyFlags() & EObstacleType.TREE ) == EObstacleType.TREE;
    Domyslnie flagi ustawione sa na 0 - co oznacza nie powiadamianie.
    @return powstaje ze zlozenia enumow typu EObstacleType.
    */
    public native int GetRigidBodyCollisionNotifyFlags();




    //////////////////////////////////////////////////////////////


    /**[native] Przelacza pokazywanie zgrubnego boxa dla obiektu
     */
    public native void ShowCoarseBox(boolean bState);
    
    /**[native] Przelacza pokazywanie boxow dla elementow mesha
     */
    public native void ShowElementBoxes(boolean bState);
    
    /**[native] Przesuwa o fMove sciane o zadanym indeksie iSide (0-5) w boxie
     * podanego elementu iElementID
     */
    public native void MoveElementBoxSide(int iElementID, int iSide, float fMove);
    
    /** [native] Ustawia flage czy obiekt ma byc oswietlony.
     * Defaultowo oswietlenie dynamiczne obiektu jest wylaczone.
     * @param bSet oswietlenie (<code>boolean</code>)
     */
    public native void SetLighting(boolean bSet);
    
    /**[native] ustawia flage czy obiekt ma sie ustawiac plaszczyzna XY w kierunku 
     * observatora
     */
    public native void SetFacingTowardsObserver( boolean bSet );
    
    /**[native] metoda ustawia nty material w skinie
     * z zalozenia mesh musi miec nty material
     * TextureName - nazwa textury ktora ustawic (sciezka do pliku)
     * iMatNum - numer materialu do ktorego przypisac texture
     */
    public native void SetSkinElement( String TextureName, int iMatNum );
	
	/**[native] metoda pobiera nazwe textury z podanego materialu
	 * nazwa jest pobierana z zerowej mapy (mapa koloru)
	 * @param iMatNum - numer materialu z ktorego nalezy pobrac nazwe textury
	 */
	public native String GetSkinElement( int iMatNum );

    /**[native] Ustawia dla obiektu skina o nazwie sSkinName.
     * Efekt dzialania poprzedniego skina jest usuwany.
     * Oryginalne ustawienia przywracane sa po podaniu nazwy "Default".
     */
    public native boolean SetSkin(String sSkinName);

    /**[native] Ustawia dla obiektu skina o nazwie sSkinName.
     * Efekt dzialania poprzedniego skina jest zachowywany.
     */
    public native boolean SetSkinLeavePrevious(String sSkinName);

    /**[native] Pobiera nazwe aktualnego skina obiektu. Jesli zwrocony string
     * jest pusty, obiekt nie ma skina. */
    public native String GetSkin();

    /**[native] Powoduje zaladowanie wszystkich materialow z podanego skina */
    public native void PreloadSkin(String sSkinName);

    /**[native] Odladowuje materialy zaladowane przez PreloadSkin */
    public native void UnloadSkin(String sSkinName);
    
	/**[native] metoda pobiera ilosc materialow w obiekcie
	 */
	public native int GetNumMaterials();
	
	
	/**[native] metoda zwraca tablice nazw materialow 
	 * @param lod - poziom detalu modelu z ktorego pobrane zostana materialy
	 */
	public native String[] GetMaterials( int lod ); 
	
	
	/**[native] metoda ustawia sposob 'cullowania' dla podanego materialu
	 * @param matNum - numer materialu ktoremu nalezy ustawic 'cullowanie'
	 * @param cullMode - sposob 'cullowania' : -1 - wycianaj tylne; 0 - nie wycinaj; 1 - wycinaj przednie
	 */
	public native void SetMaterialCulling( int matNum, int cullMode );
	
	/**[native] metoda pobiera sposob 'cullowania' dla podanego materialu
	 * @param matNum - numer materialu z ktorego pobrac metode 'cullowania'
	 */
	public native int  GetMaterialCulling( int matNum );

    /**[native] Pobranie identyfikatora elementu mesha. Identyfikator moze byc nastepnie uzywany
     * przy metodach obliczajacych kolizje.
     */
    public native int GetElementID(String sElementName);
    
    /**[native] Pobranie nazwy elementu mesha. 
     * @param nElementId id elementu
     */
    public native String GetElementName(int nElementId);
	
	public native String GetMeshElementNameAttachedTo();
    
    /**[native] Pobranie pozycji mesh elementu w KURWA WE WSPOLRZEDNYCH SWIATA
     * @param nElementID numer elementu pobrany poprzez GetElementID()
     * @return dla niepoprawnego numeru iElementID zwraca null lub poprawny (<code>Vector </code>)
     */
    public native Vector GetElementPos(int nElementID);

    /**[native] Pobranie pozycji mesh elementu w KURWA WE WSPOLRZEDNYCH SWIATA do 
     * wektora podanego jako parametr
     * @param nElementID numer elementu pobrany poprzez GetElementID()
     * @param vResult pozycja (<code>Vector </code>)
     * @return dla niepoprawnego numeru iElementID zwraca false, dla poprawnego true
     */
    public native boolean GetElementPos(int nElementID, Vector vResult);
    
    /**[native]
     * @param nElementID numer elementu pobrany poprzez GetElementID()
     * @return dla niepoprawnego numeru iElementID zwraca null lub poprawny (<code>Vector </code>)
     */
    public native Vector GetElementLocalPos(int nElementID);
    
    /**[native] Ustawienie pozycji mesh elementu w ukladzie MeshObjecta
     * @param nElementID numer elementu pobrany poprzez GetElementID()
     * @param vPos nowa pozycja meshelementu
     */
    public native void SetElementLocalPos(int nElementID, Vector vPos);

    /**[native] Pobranie wektora forward mesh elementu w ukladzie MeshObjecta
     * @param nElementID numer elementu pobrany poprzez GetElementID()
     * @return dla niepoprawnego numeru iElementID zwraca null lub poprawny (<code>Vector </code>)
     */
    public native Vector GetElementForwardVector(int nElementID);

    /**[native] Pobranie wektora right mesh elementu w ukladzie MeshObjecta
     * @param nElementID numer elementu pobrany poprzez GetElementID()
     * @return dla niepoprawnego numeru iElementID zwraca null lub poprawny (<code>Vector </code>)
     */
    public native Vector GetElementRightVector(int nElementID);

    /**[native] Pobranie wektora up mesh elementu w ukladzie MeshObjecta
     * @param nElementID numer elementu pobrany poprzez GetElementID()
     * @return dla niepoprawnego numeru iElementID zwraca null lub poprawny (<code>Vector </code>)
     */
    public native Vector GetElementUpVector(int nElementID);
    
    /**[native] Metoda zwraca ilosc meshelementow w meshu obiektu
     * @return ilosc elementow w meshu
     */
    public native int GetElementsNumber();
    
    /**[native] Metoda ustawia obrot elementu w jego ukladzie lokalnym.
     * Wywolanie tej metody ustawia nowa wartosc obrotu w miejsce aktualnej. 
     * Kolejne obroty nie sumuja sie! Pozycja elementu nie ulega zmianie!
     * @param nElementID  identyfikator elementu do obrocenia
     * @param vDir os wedlug ktorej chcemy obracac
     * @param fAngle kat o ktory chcemy obrocic [w stopniach]
     */
    public native void RotateElement(int nElementID, Vector vDir, float fAngle);
    
	/**[native] Metoda ustawia obrot elementu w jego ukladzie lokalnym
	 * i powiadamia dzieci o zmianie macierzy.
	 * Wywolanie tej metody ustawia nowa wartosc obrotu w miejsce aktualnej. 
	 * Kolejne obroty nie sumuja sie! Pozycja elementu nie ulega zmianie!
	 * @param nElementID  identyfikator elementu do obrocenia
	 * @param vDir os wedlug ktorej chcemy obracac
	 * @param fAngle kat o ktory chcemy obrocic [w stopniach]
	 */
	public native void RotateElementWithChildren(int nElementID, Vector vDir, float fAngle);

    /**[native] Metoda powodujaca ustawienie macierzy lokalnej elementu
     * @param nElementID identyfikator elementu do obrocenia
     * @param vUp wektor do gory
     * @param vForward wektor do przodu
     * @param vPos pozycja elementu
     * DZIALA JESLI W OBIEKCIE SA ANIMACJE
     */
    public native void FromUpForwardPosElement(int nElementID, Vector vUp,
                                               Vector vForward, Vector vPos);
    /**[native] Metoda powodujaca ustawienie macierzy do swiata elementu
     * @param nElementID identyfikator elementu do obrocenia
     * @param vUp wektor do gory
     * @param vForward wektor do przodu
     * @param vPos pozycja elementu
     * DZIALA JESLI W OBIEKCIE SA ANIMACJE
     */
    public native void FromUpForwardPosElementWorld(int nElementID, Vector vUp,
                                                    Vector vForward, Vector vPos);

    /** [native] Ustawia macierz w swiecie na podstawie quaterniona i pozycji w swiecie.*/
    public native void ElementSetWorldMatrixFromQuatPos( int iElementID, Vector vDir, float fAngle,
        Vector vPos );

    /**Podlacza dziecko i ustawia element w element. UWAGA, po tej operacji przywracany
     * jest oryginalny pivot
     * @param nParentElem element w rodzicu
     * @param cChild dziecko 
     * @param nChildElem element w dziecku*/
    public void AttachChildElemToElem(int nParentElem, MeshObject cChild, int nChildElem)
    {
        cChild.SetPivotPointFromElement(nChildElem);
        AttachChild(cChild, nParentElem);
        //cChild.SetPivotPointFromElement(
    }
    
    /** [native] Ustawia macierz obiektu tak aby uklad lokalny obiektu pokryl sie z ukladem
     * lokalnym zadanego mesh elementu
     * UWAGA! dziala tylko wtedy, gdy obiekt nie ma parenta
     */
    public native void SetPivotPointFromElement(int iElementID);
    
    /**[native] Wlaczenie/wylaczenie renderingu dla danego obiektu
     * (dziala lokalnie na kliencie np. dla FPP)
     */
    public native void EnableRendering(boolean bRender);
	
	/**[native] Wlaczenie/wylaczenie renderingu dla danego obieketu dla danego widoku
	 * @param ViewHandle handle vidoku otrzymany przy tworzeniu widoku
	 * @param bRender flaga true/false okreslajaca rysowac/nierysowac
	 */
	public native void EnableRendering(int ViewHandle, boolean bRender);

    /**[native]
     * Mowi czy obiekt ma ustawiona flage renderingu (patrz EnableRendering(...))
     * Nie mowi czy obiekt sie renderuje w danej chwili, ale czy moze sie renderowac 
     */
    public native boolean IsRenderable();
        
    /**[native] Wylacza renderowanie mesha na wszystkich viewportach.
     */
    public native void DisableRenderingAllViews();
    
    /**[native] Metoda przelicza podana pozycje w ukladzie mesh elementu ModelObjecta do ukladu swiata
     * @param vVector wektor w ukladzie mesh elementu ModelObjecta
     * @return wektor w ukladzie swiata 
     */
    public native Vector PointLocalToWorld(int nMeshElement, Vector vVector);
    
    /**[native] Metoda przelicza podana pozycje w swiecie do ukladu mesh elementu ModelObjecta.
     * @param vVector wektor w ukladzie swiata 
     * @return wektor w ukladzie mesh elementu ModelObjecta
     */
    public native Vector PointWorldToLocal(int nMeshElement, Vector vVector);
    
    /**[native] Metoda przelicza podany wektor(w ukladzie mesh elementu ModelObjecta) do ukladu swiata.
     * @param vVector wektor w ukladzie mesh elementu ModelObjecta
     * @return wektor w ukladzie swiata 
     */
    public native Vector VectorLocalToWorld(int nMeshElement, Vector vVector);
    
    /**[native] Metoda przelicza podany wektor w ukladzie swiata do ukladu mesh elementu ModelObjecta.
     * @param vVector wektor w ukladzie swiata
     * @return wektor w ukladzie mesh elementu ModelObjecta 
     */
    public native Vector VectorWorldToLocal(int nMeshElement, Vector vVector);

    //////////////////////////////////////////
    // metody ustawiajace parametry kolizji //
    //////////////////////////////////////////
    
    /** [native] Ustawia typ kolizji z promieniami (metody Trace)
     * Defaultowo ustawiony jest typ kolizji _COARSE_BOX
     * @param iType typ kolizji (z ECollTraceFlags)
     */
    public native void SetTraceCollType(int iType);
    
    /** Metoda zwraca Vector ID'ekow elementow mesza zgodnych nazwa z podana
     *  maska.
     * @param sMask maska nazwy mesh elementu. Narazie jest to zwykly prefix do 
     *      ktorego doklejane sa kolejne numerki od 00 do 99. Dodawany jest rowniez element dokladnie
     *      odpowiadajacy masce.
     * @return zwraca Vector z identyfikatorami elementow zapisanymi jako Integer.
     *      Gdy nie ma zadnego elementu o zadanej masce to zwraca null.
     */
    public java.util.Vector GetElementsWithMask(String sMask)
    {
        java.util.Vector vID = new java.util.Vector(0);
        int nID = 0;
        String sElemName = null;
        
        // najpierw dokladamy element dokladnie odpowiadajacy masce
        nID = GetElementID(sMask);
        if (nID != -1)
        {
            vID.addElement(new java.lang.Integer(nID));
        }
        
        // szukamy pozostalych elementow
        for(int i=0; i<200; i++)
        {
            sElemName = sMask + ( (i<10) ? "0" : "" ) + i;
            
            nID = GetElementID(sElemName);
            if (nID != -1)
            {
                vID.addElement(new java.lang.Integer(nID));
                //nID =  ((java.lang.Integer)vID.elementAt(0)).intValue();
            }
        }//for
        
        return (vID.size() == 0) ? null : vID;
    }

    /**[native] Ustawienie wspolczynnika przelaczania LOD obiektow.
     * Wzor na wyliczenie wspolczynnika:
     *                          wielkosc_obiektu * pewien_lod
     * fCoeff = --------------------------------------------------------------------------
     *          odleglosc_na_jakiej_ma_sie_pewien_lod_pojawic * (ilosc_lodow - pewien_lod)
     * 
     * 
     *                                                   wielkosc_obiektu * pewien_lod
     * odleglosc_na_jakiej_ma_sie_pewien_lod_pojawic = -------------------------------------
     *                                                  fCoeff * (ilosc_lodow - pewien_lod)
     * 
     */
    native public void SetLODCoeff(float fCoeff);
    
    /**[native] Zwraca wartosc wspolczynnika wybierania LODa
     */
    native public float GetLODCoeff();
    
    /**[native] Ustawienie wielkosci mesha uzywanej do przelaczania LODow obiektow.
     * Wzor na wyliczenie wspolczynnika j.w. */
    native public void SetLODMeshSize(float fMeshSize);
    /**[native] Pobranie wielkosci mesha uzywanej do przelaczania LODow obiektow.
     * Wzor na wyliczenie wspolczynnika j.w. */
    native public float GetLODMeshSize();
    
    /**[native] Laduje wygenerowany wczesniej obrys obiektu uzywany przy pathmapie.
     * Jezeli obrys nie istnieje, zwracane jest -1. Jezeli istnieje, zwracany jest
     * jego identyfikator. Stamp sie usuwa sam przy okazji kasowania levelu.
     */
    public native int LoadStamp();


    public native int FindPortalForDoor(int iElementID);
    
   
    //////////////////////
    // obsluga dzwiekow //
    //////////////////////

    /**[native] Powoduje podlaczenie do obiektu zapetlonego dzwieku
     * @param iElementID id elementu (node'a), do ktorego ma byc przypiety dzwiek
     * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
     * @return identyfikator dzwieku
     */
    native int AttachSound(int iElementID, String sSoundName);

    /**[native] Powoduje podlaczenie do obiektu zapetlonego dzwieku
     * @param iElementID id elementu (node'a), do ktorego ma byc przypiety dzwiek
     * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
     * @param fMin odleglosc od emitera dzwieku po ktorej zacznie dzwiek wygasac
     * @param fMax odleglosc od emitera dzwieku po ktorej dzwieku nie slychac
     * @return identyfikator dzwieku
     */
    native int AttachSound(int iElementID, String sSoundName, float fMin, float fMax);
    
    /**[native] Powoduje "obciecie" (wylaczenie renderowania) czesci mesha skinowanego, ktora zostala 
     * powiazana z koscia o zadanym identyfikatorze. 
     * @param eBone identyfikator kosci (z klasy {@link EBones}) (<code>byte</code>)
     * @param bAffectChildren flaga okreslajaca czy przy usuwaniu czesci mesha 
     * podpietej pod dana kosc maja byc usuwane czesci mesha podpiete pod kosci wpiete w hierarchii pod dana kosc
     */
    
    native final boolean CutBodyPart(byte eBone, boolean bAffectChildren);
    /**[native] Powoduje przywrocenie renderowania "obcietej" czesci mesha skinowanego, ktora zostala 
     * powiazana z koscia o zadanym identyfikatorze
     * @param eBone identyfikator kosci (z klasy {@link EBones}) (<code>byte</code>)
     * @param bAffectChildren flaga okreslajaca czy przy przywracaniu maja byc brane pod uwage czesci mesha podpiete 
     * pod kosci wpiete w hierarchii pod dana kosc
     */
    native final boolean UnCutBodyPart(byte eBone, boolean bAffectChildren);
	
	/*[native] ukrywa (z renderingu) mesh element o podanym numerze
	 * metoda zwraca false jesli mesh element o podanym numerze nie istnieje
	 * @param iElementNum numer mesh element'u 
     * DZIALA JESLI W OBIEKCIE SA ANIMACJE
	 */
	native final boolean HideElement( int iElementNum );
	
	/*[native] zezwala na rysowanie mesh element'u o podanym numerze
	 * metoda zwraca false jesli mesh element o podanym numerze nie istnieje	 
	 * @param iElementNum numer mesh element'u
     * DZIALA JESLI W OBIEKCIE SA ANIMACJE
	 */
	native final boolean UnhideElement( int iElementNum );

    /*[native] Zwraca true, jesli element o podanym id nie istnieje albo jest ukryty
     * @param iElementNum numer mesh element'u
     * DZIALA JESLI W OBIEKCIE SA ANIMACJE
     */
    native final boolean IsElementHidden( int iElementNum );

	/*[native] Wlacza/wylacza kolizje mesh elementu
	 * @param iElementNum numer mesh element'u 
     * @param bEnable true jest wlaczyc
     * @return false jesli element poza zakresem
	 */
	native final boolean EnableCollisionsElement( int iElementNum, boolean bEnable );
	
    /**[native] Okresla czy czesc mesha skinowanego przypisana do kosci o zadanym identyfikatorze zostala "obcieta"
     */
    native final boolean IsBodyPartCutted(byte eBone);
    
    /**[native] Okresla czy mesh skinowany posiada "obciete" czesci
     */
    native final boolean HasBodyCuttedParts();

    /** Kopiuje pozycje mesh elementow z obiektu cSource */
    public native void CopyElementsPosition( MeshObject cSource );
    /** Kopiuje stan particli z obiektu cSource */
    public native void CopyParticleState( MeshObject cSource );
    
    
    /** 
     * Do obslugi sieci
     */
    
    /** Replikacja particli
     */   
    /** Wrzuca do _aktualnego_ komunikatu _wyjsciowego_ stan particli obiektu */
    public native void MessagePackParticleState();
    /** Odczytuje z _aktualnego_ komunikatu _wejsciowego_ stan particli obiektu 
     * odczytane dane sa przechowywane w wewnetrzym buforze, do wykorzystania
     * metoda ApplyParticleState */
    public native boolean MessageUnPackParticleState();
    /** Zastosowuje z bufora odczytany poprzednio stan kosci */
    public native boolean ApplyReceivedParticleState();
    /** Replikuje stan particli obiektu */
    public native void ReplicateParticleState();
    /** wymuszenie symulacji z czasem 0.0f */
    public native void ForceParticleSimulate();
        
    
    /** Replikacja mesh elementow
     * Uwaga! Jeszcze nie zaimplementowane!!!
     */   
    /** Wrzuca do _aktualnego_ komunikatu _wyjsciowego_ pozycje mesh elementow obiektu */
    //Uwaga! Jeszcze nie zaimplementowane!!!
    public native void MessagePackElementsPosition();
    /** Odczytuje z _aktualnego_ komunikatu _wejsciowego_ pozycje mesh elementow obiektu 
     * odczytane dane sa przechowywane w wewnetrzym buforze, do wykorzystania
     * metoda ApplyElementsPosition */
    //Uwaga! Jeszcze nie zaimplementowane!!!
    public native boolean MessageUnPackElementsPosition();
    /** Zastosowuje z bufora odczytane poprzednio pozycje mesh elementow */
    //Uwaga! Jeszcze nie zaimplementowane!!!
    public native boolean ApplyReceivedElementsPosition();
       
    
    
    //---------------------------------------------------------------------------    
    void OnCreate()
    {
		if (bForceSG == true)
			bSG = bForceSG;
        // wylaczamy fizyke dla obiektow wolno stojacych
        SetPhysicsType(EPhysics._NONE);
        SetPhysicsActive(false);
        		
        //SetFlag(EFlags.FXPARTICLE_COLLISION);
        
        //CallMethodIn("InitializeMeshNameSprite", 1);
        //ShowCoarseBox(true);
    }
    
    /**Okno z nazwa obiektu*/
    Window wndToolTip = null;
    /**Tekstura dla okna z nazwa obiektu*/
    int nWndToolTipTex = 0;
    
    /**Tworzy i inicjuje okno do pokazywania nazwy obiektu*/
    void CreateObjectNameWnd()
    {
        float fToolTipMargin = 2f;
        String sText = GetTextToShow();
        wndToolTip = (Window)FreeCamera.GetObjectNameModule().CreateObject("Window");
        wndToolTip.SetModule(FreeCamera.GetObjectNameModule());
        nWndToolTipTex = wndToolTip.AddTexture("data\\interface\\er_t_blue.png", true, true);
	    wndToolTip.SetTexture(nWndToolTipTex, new Rect(0,0,1,1));
		wndToolTip.SetPos(Vector.Zero);
		wndToolTip.ProcessMouse(false);
		FreeCamera.GetObjectNameModule().AddSprite(wndToolTip);
        wndToolTip.MoveTop();
        wndToolTip.SetTextPosFlags(ESpriteText.HORIZ_MIDDLE|ESpriteText.VERT_MIDDLE);
        wndToolTip.SetFont(Fonts.FONT_MSSANSSERIF_12);
        wndToolTip.SetTextColor(1f, 1f,1f, 1f);
        Vector vTextSize = wndToolTip.GetTextExtent(sText);
        wndToolTip.DisplayText(sText);
        vTextSize.fX += 2f * fToolTipMargin;
        vTextSize.fY += 2f * fToolTipMargin;
        wndToolTip.SetSize(vTextSize);
    }
    
    /**Daje tekst do wyswietlenia w ShowObjectName(...), standardowo jest to nazwa obiektu*/
    String GetTextToShow()
    {
        String sText = null;
        if (GetName().length() == 0)
            sText = "NONAME ";
        else
            sText = GetName();
        return sText;
    }
    
    /**Pokazuje/ukrywa sprite z nazwa obiektu, wolane w kamerze*/
    void ShowObjectName(boolean bShow, boolean bPosition, Camera cCamera)
    {
        if (!Game.bCanShowObjectNames)
            return;
        
        if (wndToolTip == null)
        {
            CreateObjectNameWnd();
            return;
        }
        
        if (!bShow)
        {
            wndToolTip.Hide();
        }
        else
        {
            if (GetModule() == null)
                return;
            Vector vPosOnScreen = this.PointToScreen(cCamera, GetPosition());
            Vector vMinExtent = new Vector(), vMaxExtent = new Vector();
            GetScreenExtents(cCamera, vMinExtent, vMaxExtent, 0 );
            MeshObject goCol = CheckForObjectsOnScreen((int)vPosOnScreen.fX, (int)vPosOnScreen.fY, cCamera);
            //sprawdzimy jeszcze czy widac srodek obiektu
            if ((goCol != this))// && IsPointOnScreen(vMinExtent))
            {
                vPosOnScreen = new Vector(vMinExtent);
                vMaxExtent.Sub(vMinExtent);
                vMaxExtent.Mul(0.5f);
                vPosOnScreen.Add(vMaxExtent);
                goCol = CheckForObjectsOnScreen((int)vPosOnScreen.fX + 10, (int)vPosOnScreen.fY - 10, cCamera);
            }
            if ((goCol != this))// && IsPointOnScreen(vMaxExtent))
            {
                vPosOnScreen = vMaxExtent;
                goCol = CheckForObjectsOnScreen((int)vPosOnScreen.fX - 10, (int)vPosOnScreen.fY - 10, cCamera);
            }
            if (goCol == this)
            {
                String sText = null;
                if (bPosition)
                    sText = GetTextToShow() + GetPosition().toString();
                else
                {
                    sText = GetTextToShow();
                }
                Vector vTipSize = wndToolTip.GetTextExtent(sText);
                wndToolTip.SetSize(vTipSize);
                wndToolTip.DisplayText(sText);
                vPosOnScreen.fX -= vTipSize.fX/2;//vHeadPosOnScreen.fX -= vTipSize.fX/2;
                vPosOnScreen.fY -= vTipSize.fY;//vHeadPosOnScreen.fY -= vTipSize.fY;
                if (vPosOnScreen.fY < GetScreenHeight())//if (vHeadPosOnScreen.fY < GetScreenHeight().fY)
                {
                    wndToolTip.SetPos(vPosOnScreen);//wndToolTip.SetPos(vHeadPosOnScreen);
                    wndToolTip.Show();   
                }
                else
                {
                    wndToolTip.Hide();
                }
            }
            else
            {
                wndToolTip.Hide();
            }

        }
    }
    
    boolean IsPointOnScreen(Vector vScreenPoint)
    {
        if ((vScreenPoint.fX >= 0) &&(vScreenPoint.fX < GetScreenWidth()) &&
            (vScreenPoint.fY >= 0) &&(vScreenPoint.fY < GetScreenHeight()))
            return true;
        else
            return false;
    }
    
    
    MeshObject CheckForObjectsOnScreen(int iXPos, int iYPos, Camera cCamera)
	{
        Module mod = GetModule();
        if (mod == null)
            return null;
		Vector vStart = GetOnScreenPoint(cCamera,iXPos, iYPos);
		Vector vScreenVector = GetOnScreenVector(cCamera,iXPos, iYPos);
		Vector vEnd = new Vector(vScreenVector);
		vEnd.Mul(100000.0f);
		vEnd.Add(vStart);
		Collision col;
		
		// przycinamy wektor do terenu, zeby za gleboko nie szukac obiektu
		//Vector vEndObj = new Vector(vEnd);
		//col = TraceTerrain(vStart, vEnd);
				
		col = TraceObjects(vStart, vEnd);

		if ((col != null) && (col.Obstacle != null))// && (m_nSelectedCount > 0))
		{
            if (col.Obstacle instanceof MeshObject)
                return (MeshObject)col.Obstacle;
            else
                return null;
		}
		else
			return null;

	}
    
    void OnDestroy()
    {
		super.OnDestroy();
	
        if (wndToolTip != null)
        {
            wndToolTip.ReleaseTextures();
            FreeCamera.GetObjectNameModule().RemoveSprite(wndToolTip);
            wndToolTip.delete();
        }
	}
    
    /**[native] Podmienia w meshu material sOldMaterial na sNewMaterial.
     * Nowy material musi byc kompatybilny na poziomie streamow z materialem
     * z jakim dany surface zostal utworzony (niekoniecznie sOldMaterial)
     * @return true jesli ok
     */
    public native boolean ReplaceMaterial(String sOldMaterial, String sNewMaterial);

    /**[native] Podmienia material sOldMaterial na sNewMaterial w zadanym mesh elemencie.
     * Nowy material musi byc kompatybilny na poziomie streamow z materialem
     * z jakim dany surface zostal utworzony (niekoniecznie sOldMaterial)
     * @return true jesli ok
     */
    public native boolean ReplaceMaterialOnElement(String sOldMaterial, String sNewMaterial, String sElementName);
    
    /**[native] Podmienia w meshu material, ktory oryginalnie (przy ladowaniu)
     * mial zadane ID, na sNewMaterial.
     * Nowy material musi byc kompatybilny na poziomie streamow z materialem oryginalnym
     * @return true jesli ok
     */
    public native boolean ReplaceOriginalMaterial(int iOriginalMaterial, String sNewMaterial);
    // KURWA A GDZIE KOMENTARZ!?!??!?!
    public native boolean ReplaceOriginalMaterial(int iOriginalMaterial, int handle );

    /**[native] Podmienia w mesh elemencie material, ktory oryginalnie (przy ladowaniu)
     * mial zadane ID, na sNewMaterial.
     * Nowy material musi byc kompatybilny na poziomie streamow z materialem oryginalnym
     * @return true jesli ok
     */
    public native boolean ReplaceOriginalMaterialOnElement(int iOriginalMaterial, String sNewMaterial, String sElementName);

    /**[native] Ustawia zasiegi widocznosci dla dynamicznej cube mapy
     * UWAGA: ustawiac po zaladowaniu mesha
     * @param fMaxRange - zasieg dla wszystkiego oprocz obiektow (domyslnie: 10000)
     * @param fObjectRange - zasieg dla obiektow (domyslnie: 10000)
     */
    public native void SetCubeMapRanges(float fMaxRange, float fObjectRange);
    
    /**[native] Wlacza updatowanie jednej sciany cubemapy na ramke */
    public native void SetCubeMapOneFace(boolean bEnable);

    /**[native] Powoduje, ze obiekt ReflectionObject bedzie sie renderowal
     * przy odbiciach (woda, cube mapy) zamiast danego obiektu
     * Podanie null wylacza podmiane
     */
    public native void SetReflectionObject(MeshObject ReflectionObject);
    
    /** Zaladowanie pliku z definicja skina mesha. Definicja sklada sie z
     * modyfikatorow materialow oraz chowania elementow mesha.
     * @param cSkinFile plik z ktorego odczytana ma byc definicja skina
     */
    public void LoadSkin(FileName cSkinFile)
    {
        ParseKeyword Root = ParseFile(cSkinFile.toString());
        if(Root != null)
        {
            ParseKeyword cK = Root;
            for(; cK != null; cK = cK.GetNext())
            {
                if(cK.Equals("Material"))
                {
                    ReplaceMaterial(cK.GetSParam(0), cK.GetSParam(1));
                }
                else
                if(cK.Equals("HideElement"))
                {
                    int nElem = GetElementID(cK.GetSParam(0));
                    HideElement(nElem);
                }
                
            }
            Root.delete();
        }
    }
    
    /////////////////////////////////////////////////////////////////////////////
    // Obsluga akcji act_Appear

    public void Disappear(ControlObject cDestinationObj)
    {
        super.Disappear(cDestinationObj);
        EnableRendering(false);
    }
    public void Appear(ControlObject cDestinationObj)
    {
        super.Appear(cDestinationObj);
        EnableRendering(true);
    }

    
    ////////////////////////////
    //  Interfejs do efektow rozwalanej szyby.
    //  czytaj FXBrokenGlassManager.rtf
    ///////////////////////////

    /** Ustawia domyslna grawitacje dla nowo utworzonych efektow.*/
    public final native void FXBrokenGlass_SetDefaultGravity( Vector vGravity );
    /** Ustawia grawitacje dla podanego efektu.*/
    public final native void FXBrokenGlass_SetGlassGravity( int iGlass, Vector vGravity );

    /** Dla podanego iMeshElementu w "model" przygotowuje do dzialania efekt rozbitej szyby.
    Po utworzeniu efekt nie jest jeszcze rysowany(Szyba poczatkowo nie jest rozbita!). 
    Aktywacja efektu nastepuje w momencie rozbicia szyby metoda @see ShotABullet() lub @see CauseAnExplosion().
    @param fAge - jak dlugo ma zyc efekt po rozbiciu [s], 0 oznacza jak dlugo sie da!
    @param fMaxFragmentSize - dlugosc [cm] najwiekszego kawalka w jednej z osi X,Y,Z.
        UWAGA: Im mniejsze kawalki tym wiecej ich sie wygeneruje i w konsekwencji
        system bedzie wolniej dzialal!
    @return id efektu ktore nalezy podac w metodach odpalajacych.

    W tej chwili nieuzyteczne, gdyz tworzenie szyb nastepuje automatycznie
    podczas ladowania 3DA (dla surface-ow uzywajacych materialu GLASS).
    Moze sie przydac w przyszlosci, gdy generowanie w realtime bedzie szybsze.
    */
    public final native int FXBrokenGlass_CreateGlass( int iMeshElement, float fAge, float fMaxFragmentSize );

    /** zwraca id szyby zdefiniowanej dla iMeshElementu.
    jezeli nie znajdzie szyby w podanym iMeshElement zwraca -1.
    */
    public final native int FXBrokenGlass_GetGlassFromElement( int iMeshElement );

    /** Wystrzel pocisk w szybe.
        meshElement do ktorego jest podczepiona szyba staje sie niewidzialny i niekolidujacy!
        @param iEffect - uchwyt szyby.
        @param vPosition - miejsce uderzenia w ukladzie swiata;
        @param vDirection - kierunek lotu pocisku w ukladzie swiata;
        @param fVelocity - szybkosc z jaka maja poruszac sie najszybsze odpryski szyby;
        @param fRadius - promien strefy na ktora oddzialywuje pocisk;
    */
    public final native void FXBrokenGlass_ShotABullet( int iEffect, Vector vPosition, Vector vDirection, float fVelocity, float fRadius );

    /** Wystrzel pocisk w szybe.
        meshElement do ktorego jest podczepiona szyba staje sie niewidzialny i niekolidujacy!
        @param iMeshElement - element dla ktorego jest zdefiniowana szyba.
        @param vPosition - miejsce uderzenia w ukladzie swiata;
        @param vDirection - kierunek lotu pocisku w ukladzie swiata;
        @param fVelocity - szybkosc z jaka maja poruszac sie najszybsze odpryski szyby;
        @param fRadius - promien strefy na ktora oddzialywuje pocisk;
        @return jezeli dla iMeshElement jest stworzona szyba to zwraca true.
        Jezeli szyba zostala trafiona, wylaczany jest dla danego elementu rendering
        oraz kolizje.
    */
    public final native boolean FXBrokenGlass_ShotABulletToElement( int iMeshElement, Vector vPosition, Vector vDirection, float fVelocity, float fRadius );

    /** Stworz eksplozje niszczaca wszystkie szyby w promieniu fRadius.
    Eksplozja ma ksztalt kuli i jej sila slabnie im dalej od srodka. 
    Przez sile eksplozji rozumiem szybkosc nadawana fragmentom szyb.
    MeshElementy do ktorych podczepione sa szyby znajdujace sie w zasiegu eksplozji
    staja sie niewidzialne i niekolidujace!
    @param vCenter - pozycja centrum eksplozji[cm].
    @param fVelocity - predkosc jaka maja osiagnac najszybsze fragmenty [cm/s].
    @param fRadius - promien sfery eksplozji [cm].
    @return ilosc szyb rozbitych przez eksplozje. Nie liczy szyb juz rozbitych!
    */
    public final native int FXBrokenGlass_CauseAnExplosion( Vector vCenter, float fVelocity, float fRadius );


    /** Stworz eksplozje niszczaca szybe w wybranym meshElemencie.
    Eksplozja ma ksztalt kuli i jej sila slabnie im dalej od srodka. 
    Przez sile eksplozji rozumiem szybkosc nadawana fragmentom szyby.
    Jezeli MeshElement znajduje sie w zasiegu eksplozji i jego szyba nie zostala
    jeszcze rozbita staje sie niewidzialny i niekolidujacy(po 0.8 sekundy)!
    @param iElement - mesh element w ktorym znajduje sie szyba
    @param vCenter - pozycja centrum eksplozji[cm].
    @param fVelocity - predkosc jaka maja osiagnac najszybsze fragmenty [cm/s].
    @param fRadius - promien sfery eksplozji [cm].
    @return ilosc szyb rozbitych przez eksplozje. Nie liczy szyb juz rozbitych!
    */
    public final native boolean FXBrokenGlass_CauseAnExplosionToElement( int iMeshElement, 
        Vector vCenter, float fVelocity, float fRadius );

    /** Odnawia wszystkie wybite szyby, by mozna je bylo ponownie rozbic.
    Na potrzeby gry sieciowej. 
    */
    public final native void FXBrokenGlass_RespawnAll();

    /** Odnawia szybe dla podanego Elementu.
    Na potrzeby gry sieciowej.
    */
    public final native void FXBrokenGlass_RespawnGlassAtElement( int iMeshElement );

    /** Ustawia wszystkie istniejace szyby w stan rozbite.
    Nie uruchamiajac efektu lecacych fragmentow.
    */
    public final native void FXBrokenGlass_BreakAllQuietly();
    /** Ustawia szybie parametr rozbita, nie uruchamiajac efektu rozbicia - sypiace odlamki itp.
    Na potrzeby  gry sieciowej. Faktycznie wylacza widocznosc i kolizje wybranego
    mesh elementu.
    */
    public final native void FXBrokenGlass_BreakGlassAtElementQuietly( int iMeshElement );

    /** Handler wywolywany w momencie rozbicia szkla.
    @params iMeshElement - ktorego szyba zostala rozbita.
    @params bBullet - kula czy eksplozja?
    @params vPosition - punkt uderzenia kuli lub centrum eksplozji.
    @param vDirection - kierunek w ktorym leciala kula.
    @param fVelocity - szybkosc z jaka powinny leciec odlamki.
    @param fRadius - promien kuli lub eksplozji.
    public void FXBrokenGlass_OnBreakGlass( int iMeshElement, boolean bBullet, Vector vPosition, 
        Vector vDirection, float fVelocity, float fRadius );
    */

	native void LoadMeshElements(FileChunk cFC);
	native void SaveMeshElements(FileChunk cFC);
	
    //*********************************************************
	//					SaveGame
    //*********************************************************

    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[MeshObject.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunks._CHK_MODEL_OBJECT)
        {
			super.SGLoadChunk(cFC);			
						
			String sMeshElement = cFC.LoadString();
			if (sMeshElement != null)
			{
				((MeshObject)GetParent()).AttachChild(this,sMeshElement);
				FileChunk.Log("[MeshObject.SGLoadChunk] MeshElement = <"+sMeshElement+">\n");
			}
			else
				FileChunk.Log("[MeshObject.SGLoadChunk] MeshElement = <null>\n");
			
			if (bForceSG)
			{
				FileChunk.Log("[MeshObject.SGSaveChunk] ForceSG !\n");
				LoadMeshElements(cFC);			
			}
		}
		
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[MeshObject.SGSaveChunk]\n");
		
		FileChunk cFC = NewChunk(ESGChunks._CHK_MODEL_OBJECT, cFCParent);
		
        super.SGSaveChunk(cFC);
				
		String sMeshElement = GetMeshElementNameAttachedTo();
		
		cFC.SaveString(sMeshElement);

		if (sMeshElement != null)
			FileChunk.Log("[MeshObject.SGSaveChunk] MeshElement = <"+sMeshElement+">\n");
		else
			FileChunk.Log("[MeshObject.SGSaveChunk] MeshElement = <null>\n");
		
		if (bForceSG)
		{
			FileChunk.Log("[MeshObject.SGSaveChunk] ForceSG !\n");
			SaveMeshElements(cFC);			
		}
				
		cFC.delete();
	}

	public String GetMEAttachedTo()
	{
		return GetMeshElementNameAttachedTo();
	}

    /// ----------- probny interfejs do popychanych traw.---------

    /// Impulsowo odpycha wszystkie trawy w promieniu fRadius, od punktu vPos.
    public native void GrassPusher_DropSphere( Vector vPos, float fRadius );
    /// Tworzy kule o promieniu fRadius odpychajaca trawe, ktorej srodek zaczepiony jest do
    /// srodka mesh elementu iMeshElement.
    /// Do jednego elementu system rejestruje jedna kule, kolejne odwolania tylko modyfikuja
    /// jej promien.
    public native void GrassPusher_RegisterPusher( int iMeshElement, float fRadius );
    /// Usuwa uprzednio zarejestrowana kule odpychajaca trawe. 
    /// @see GrassPusher_RegisterPusher().
    public native void GrassPusher_UnRegisterPusher( int iMeshElement );
    /// Usuwa wszystkie odpychacze zarejestrowane przez MeshObject
    public native void GrassPusher_UnRegisterPushersForMesh();


    /*[native] Ustawia parametry shadow mapy dla obiektu
    * fEnlarge - o ile dodatkowo rozszerzyc frustum (left, right, top, bottom) (default: 50)
    * fFarEnlarge - ile razy dalej pociagnac far frustuma, poza wyliczony z extentow obiektu (default: 1.5)
    * fRadiusXZ - o ile odsunac w plaszczyznie XZ plaszczyzne clipowania cienia (default: 20)
     */
    public native void SetShadowMapParams(float fEnlarge, float fFarEnlarge, float fRadiusXZ);
    
    /*[native] Jezeli obiekt uzywa fizyki particli typu CSBiped to funkcja zrzuca (LogR) stan particli.
     */
    public native void DumpCSBipedParticles();
}
