public class Trigger extends GameObject implements iact_EnableDisable
{
    // flaga do wypisywania niektorych logow, ktore moga zwolnic dzialanie gry
    boolean bDebug = false;
    
    /** Warunek wykonania triggera */
    public Condition cCondition = null;//#\TRIGGER;Condition for trigger execution
    
    /** Akcja wyzwalana w triggerze */
    public Action[] aActions = null;//#\TRIGGER
    
    /** Czas co ktory bedzie sprawdzany Trigger */
    public float fCheckInterval = 0.1f;//#\TRIGGER
    
    /** Ilosc wykonan triggera po ktorych zostanie usuniety.
     * Gdy -1 to dziala w nieskonczonosc. */
    public int nTimes = 1;//#\TRIGGER
    
    /** Znacznik, ze ten trigger aktualnie dziala */
    public boolean bEnabled = true;//#\TRIGGER
    
    
    void OnCreate()
    {
		bSG = true;
		
        if (bEnabled)
            CallMethodIn("Enable", 0.0f);
    }

    public void OnDestroy()
    {
        if(cCondition != null)
        {
            cCondition.delete();
            cCondition = null;
        }
    }
    
    void Check()
    {
        if(!bEnabled)
        {
            DisableCallMethods();
            return;
        }
        
        if(cCondition==null || cCondition.Evaluate())
        {
            if(nTimes != 0) InvokeActionChain(aActions);
            if(nTimes > 0) nTimes--;
        }
        
        if(nTimes == 0)
        {
            // Trigger usuwa sie
            DisableCallMethods();
            //CallMethodIn("delete", 0.0f);
        }
    }
    
    /** Metoda wolana po przyjsciu zdarzenia act_Enable */
    public void Enable()
    {
        if (nTimes != 0)
        {
            if(cCondition != null) cCondition.Clear();
            Check();
            CallMethodEvery("Check", fCheckInterval, RandomFloat(fCheckInterval));
        }
        bEnabled = true;
    }
    
    /** Metoda wolana po przyjsciu zdarzenia act_Disable */
    public void Disable()
    {
        DisableCallMethods();
        bEnabled = false;
    }
    
    /** Metoda sprawdza czy obiekt jest wlaczony czy wylaczony */
    public boolean IsEnabled()
    {
        return bEnabled;
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[Trigger.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
      
        if (cFC.GetID() == ESGChunks._CHK_TRIGGER)
        {
			super.SGLoadChunk(cFC);
			
			cCondition = (Condition)cFC.LoadObject(cCondition,this);
			//cCondition = (Condition)cFC.LoadObjectRef();	
			
			// Ilosc wykonan triggera po ktorych zostanie usuniety.
			nTimes = cFC.LoadInt();
			// Znacznik, ze ten trigger aktualnie dziala.
			bEnabled = cFC.LoadBool();			
        }
		
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Trigger.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunks._CHK_TRIGGER, cFCParent);
		
		super.SGSaveChunk(cFC);

		// Warunek wykonania triggera.
		cFC.SaveObject(cCondition);
		//cFC.SaveObjectRef(cCondition);
        
		// Ilosc wykonan triggera po ktorych zostanie usuniety.
		cFC.SaveInt( nTimes );
		// Znacznik, ze ten trigger aktualnie dziala.
		cFC.SaveBool( bEnabled );
			
		cFC.delete();
	}	
	
	public void SGUpdate()
	{
		FileChunk.Log("[Trigger.SGUpdate]\n");
		
		super.SGUpdate();
		
        if(!bEnabled)
        {
            DisableCallMethods();
            return;
        }
		
        if (nTimes == 0)
            DisableCallMethods();
	}
}
