/** Sciezka (czyli uporzadkowany zbior punktow),
 * do dowolnego uzycia.
 */
public class Path implements ISaveGame
{
	/** flagi dotyczace sciezek */
	static int _PATH_ONE_WAY = 1;
	static int _PATH_TWO_WAYS = 2;
	static int _PATH_LOOPED = 4;
	
	/** flagi sciezki (jednokierunkowa, dwukierunkowa, zamknieta) */
	int nFlags = _PATH_ONE_WAY;
	
	/** identyfikator sciezki */
	int nPathID = -1;
	
	/** nazwa sciezki */
	String sStringID = "";
	
	/** punkty sciezki */
	java.util.Vector aPoints = new java.util.Vector();
	
	boolean IsPathLooped()
	{
		return (nFlags & _PATH_LOOPED) != 0;
	}
	
	boolean IsPathOneWay()
	{
		return (nFlags & _PATH_ONE_WAY) != 0;
	}
	
	boolean IsPathTwoWays()
	{
		return (nFlags & _PATH_TWO_WAYS) != 0;
	}
	
	Path()
	{
	}
	
	void SetPathOneWay()
	{
		nFlags = _PATH_ONE_WAY;
	}
	
	Path(MapInputStream cStream)
	{
		try
		{
			nPathID = cStream.readInt();
			sStringID = cStream.readString();
			nFlags = cStream.readInt();
			switch (nFlags)
			{
				case 0:
					nFlags = _PATH_ONE_WAY;
				break;
				case 1:
					nFlags = _PATH_LOOPED;
				break;
				case 2:
					nFlags = _PATH_TWO_WAYS;
				break;
			}
			
			int nPoints = cStream.readInt();
			for (int i = 0; i < nPoints; i++)
			{
				PathPoint point = new PathPoint(cStream);
				point.LoadPathPointData(cStream);
				aPoints.addElement(point);
			}
		}
		catch (java.io.IOException e)
		{
			GameObject.CrashLog("[LoadPathData] File error while reading Path data !!!\n");
			return;
		}
		
/*		GameObject.CrashLog("nPathID:" + nPathID + "\n");
		GameObject.CrashLog("sStringID:" + sStringID + "\n");
		GameObject.CrashLog("nFlags:" + nFlags + "\n");
		GameObject.CrashLog("Points:" + nPoints + "\n");
		for (int i = 0; i < aPoints.size(); i++)
			GameObject.CrashLog( ((PathPoint) aPoints.elementAt(i)).vPosition + "\n");
*/	}
	
	public void AddPathPoint(PathPoint p)
	{
		aPoints.addElement(p);
	}
	
	public void LoadPointData(MapInputStream cStream)
	{
		try
		{
			PathPoint point = new PathPoint(cStream);
			aPoints.addElement(point);
		}
		catch (Exception e)
		{
			GameObject.CrashLog("[Path] Error in loading points data\n");
		}
	}
	
	int GetPointsNum()
	{
		return aPoints.size();
	}
	
	PathPoint GetPoint(int nNum)
	{
		if ( (nNum < 0) || (nNum >= aPoints.size()) )
			return null;
		
		return (PathPoint) aPoints.elementAt(nNum);
	}
	
	int GetPathID()
	{
		return nPathID;
	}
	
	String GetPathName()
	{
		return sStringID;
	}
	
	Vector []ConvertToVector()
	{
		Vector []points = new Vector[GetPointsNum()];
		for (int i = 0; i < GetPointsNum(); i++)
			points[i] = GetPoint(i).vPosition;
		return points;
	}
	
	public void PathToCrashLog()
	{
		GameObject.CrashLog("Path:" + this + "\n");
		GameObject.CrashLog("PathID:" + nPathID + ", StringID:" + sStringID + ", Flags:" + nFlags + "\n");
		for (int i = 0; i < aPoints.size(); i++)
		{
			PathPoint p = (PathPoint) aPoints.elementAt(i);
			GameObject.CrashLog("Point. flags:" + p.nFlags + ", pos:" + p.vPosition + ", radius:" + p.fRadius + "\n");
		}
	}
    
	public void SGSaveChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[Path.SGSaveChunk]\n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunks._CHK_PATH, cParentFC);
		
        cFC.SaveInt(nFlags);
        cFC.SaveInt(nPathID);
        cFC.SaveString(sStringID);
        cFC.SaveInt(aPoints.size());
        for (int i = 0; i < aPoints.size(); i++)
        {
            PathPoint pt = (PathPoint) aPoints.get(i);
            cFC.SaveInt(pt.nFlags);
            cFC.SaveVector(pt.vPosition);
            cFC.SaveFloat(pt.fRadius);
            cFC.SaveString(pt.sTextField);
        }
        
        cFC.delete();
    }
    
	public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[Path.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunks._CHK_PATH)
        {	
            nFlags = cFC.LoadInt();
            nPathID = cFC.LoadInt();
            sStringID = cFC.LoadString();
            int iTemp = cFC.LoadInt();
            aPoints.clear();
            
            for (int i = 0; i < iTemp; i++)
            {
                int nFlags = cFC.LoadInt();
                Vector vPos = cFC.LoadVector();
                float fRadius = cFC.LoadFloat();
                String sTextField = cFC.LoadString();
                PathPoint pt = new PathPoint(vPos, fRadius, nFlags);
                pt.sTextField = sTextField;
                aPoints.add(pt);
            }
        }
		
		cFC.delete();
    }
    
	public void SGUpdate()
    {
		FileChunk.Log("[Path.SGSaveChunk]\n");
    }
}
