/** Klasa obsluguje zapisywanie i wczytywanie stanu gry
 */
import java.io.*;
import java.util.*;
import java.util.zip.*;

public class SGController
{
    static public SGController inst = new SGController();
    
    private SGController()
    {
        GameObject.cSG = this;
        FileChunk.cSG = this;
    }
	
	public static int _SAVE_OK = 0;
	public static int _WRONG_SAVE_VERSION_NUMBER = 1;
	public static int _DECOMPRESSION_FAILED = 2;
	public static int _WRONG_CRC32_FOR_MAP_FILE = 3;
	public static int _PRELOAD_FAILED = 4;
	
	static private int _SG_BUFFER_SIZE = 65000;
	
	static String _SG_RESTARTSAVELOAD_FILE = "save/restart.sav";	
	
	static private byte buffer[] = new byte[_SG_BUFFER_SIZE];


    // identyfikatory stanu obiektu
    static int _SG_NONE = 0;
	static int _SG_PRE_LOAD = 1;
	static int _SG_PRE_LOAD_FINISHED = 2;
    static int _SG_LOAD = 3;
    static int _SG_SAVE = 4;
	
	static float m_fStartTime = 0.f;
	static float m_fEndTime = 0.f;
    
    /** Aktualny stan obiektu. 
     * Okresla czy gra jest odczytywana czy zapisywana
     */
    static int nState = _SG_NONE;	
	
	long nReadVersion = -1;
	int  nNextIDValue = -1;
    
    boolean IsLoading() { return nState==_SG_LOAD; }
    boolean IsSaving()  { return nState==_SG_SAVE; }	
    
    /** Glowny chunk pliku ze stanem gry */
    FileChunk cMainFC = null;	
    /** Obiekt otrzymujacy dane naglowkowe  */
    SGPreObject cPreObject = null;		
    /** Modul do ktorego sa odczytywane/zachowywane dane */
    Module cModule = null;	
    
    /** Tablica z danymi obiektow
     */
    java.util.Vector vObjects = null;
    
    /** Obsluga wyjatku krytycznego. Wstrzymanie odczytywania save'a
     */	
    void CriticalError(String sErrorText)
    {
        // wypisujemy tekst bledu
        GameObject.CrashLog("[SGController.CriticalError] "+sErrorText+"\n");
        Thread.currentThread().dumpStack();
        
        // wylaczamy w obiekcie odczytywanie save'a
        GameObject.SetPreCreateMode(false);
        
        nState = _SG_NONE;
    }

    //*********************************************************
    //					Odczytywanie SaveGame
    //*********************************************************
    
    /** Pobranie obiektu z tablicy obiektow po identyfikatorze.
     */
    SGObject GetObjectByID(int nID)
    {
        // szukamy obiektu w tablicy obiektow
        if (vObjects != null)
        {
            for(int i=0; i < vObjects.size(); ++i)
            {
                SGObject cSGObj = (SGObject)vObjects.elementAt(i);
                if (cSGObj.nID == nID)
                    return cSGObj;
            }
        }
        return null;
    }		
    	
	private boolean DecompressFile(String sFileName)
	{
		int nRead = 0;
		boolean bRet = true;
		
        try
        {			
			FileOutputStream jTempOut = new FileOutputStream(sFileName+".temp");
			
			FileInputStream jSave = new FileInputStream(sFileName);
			ZipInputStream jSaveZip = new ZipInputStream(jSave);
			
			ZipEntry jEntry = jSaveZip.getNextEntry();				
							
			while ((nRead = jSaveZip.read(buffer,0,_SG_BUFFER_SIZE)) != -1)
				jTempOut.write(buffer,0,nRead);
			
			jSaveZip.closeEntry();
			
			jSaveZip.close();
			jSave.close();
			
			jTempOut.close();
		}
        catch (IOException e)
        { 
			GameObject.CrashLog("SG-ERROR! File <"+sFileName+"> was not decompressed !\n");
			bRet = false;
        }				
		
		return bRet;
	}
    
    boolean bPreLoad = false;
    
    public int PreLoad(String sFileName, SGPreObject cPreObject)
    {
		if (nState != _SG_NONE)
		{
			GameObject.CrashLog("[SGController.PreLoad] SG-ERROR! State of SGController is different from expected !\n");
			Thread.dumpStack();				
			return _PRELOAD_FAILED;
		}

		/*
		if (GameObject.GetAbsoluteTime()-m_fEndTime < 0.1f)
		{
			GameObject.CrashLog("[SGController.Save] SG-ERROR! You cannot save now !\n");
			Thread.dumpStack();				
			return _PRELOAD_FAILED;
		}
		*/
		
        GameObject.CrashLog("---------------------------- \n");
        GameObject.CrashLog("[SGController.PreLoad] : <" + sFileName + ">\n");
        GameObject.CrashLog("---------------------------- \n");
		
		nState = _SG_PRE_LOAD;
		FileChunk.Log("[SGController.PreLoad] State! = <loading> \n");
		
		m_fStartTime = GameObject.GetAbsoluteTime();
				
		if (DecompressFile(sFileName) == false)
			return _DECOMPRESSION_FAILED;
        
        bPreLoad = true;
        
        this.cPreObject = cPreObject;
        
        PreLoadChunks(sFileName+".temp");
		
		FileChunk.Log("[SGController.PreLoad] State! = <none> \n");
        
        bPreLoad = false;
		
		nState = _SG_PRE_LOAD_FINISHED;
		
		return _SAVE_OK;
    }
	
	public int IsValidSaveVersion(String sFileName,long nValidNumber)
	{
        GameObject.CrashLog("---------------------------- \n");
        GameObject.CrashLog("[SGController.IsValidSaveVersion] : <" + sFileName + ">\n");
        GameObject.CrashLog("---------------------------- \n");        	
		
		File check = new File(sFileName);
		if (check.length() == 0)
			return _DECOMPRESSION_FAILED;
		if (DecompressFile(sFileName) == false)
			return _DECOMPRESSION_FAILED;
		
		FileChunk cFCMain = GameObject.OpenFile(sFileName+".temp");
        FileChunk cFCVersion = GameObject.LoadChunk(cFCMain);
		long nReadNumber = cFCVersion.LoadLong();		
        if (nReadNumber != nValidNumber)
		{
			cFCMain.delete();				
			File file = new File(sFileName+".temp");
			if (file.delete() == false)
			    GameObject.CrashLog("TempFile was not removed\n");					
			return _WRONG_SAVE_VERSION_NUMBER;
		}		
        cFCVersion.delete();
		FileChunk.Log("[SGController.SGController.IsValidSaveVersion] Skipping chunk <"+cFCMain.GetID()+">!\n");
		cFCMain.delete();							
		
		File file = new File(sFileName+".temp");
		if (file.delete() == false)
            GameObject.CrashLog("TempFile was not removed\n");		
		
		return _SAVE_OK;
	}
    
    /** Odczytuje stan gry z podanego pliku do podanego levelu.
     * @param sFileName nazwa pliku ze stanem gry
     * @param cPreLoadingObject obiekt, ktory zaladuje dane ogolne 
     * dotyczace calej gry, a nie tylko konkretnego levelu.
     */
    public void Load(String sFileName, Module cModule)
    {
		if (nState != _SG_PRE_LOAD_FINISHED)
		{
			GameObject.CrashLog("[SGController.Load] SG-ERROR! State of SGController is different from expected !\n");
			Thread.dumpStack();				
			return;
		}
		
		DumpDangerousObjects(cModule);
				
        GameObject.CrashLog("---------------------------- \n");
        GameObject.CrashLog("[SGController.Load] : <" + sFileName + ">\n");
        GameObject.CrashLog("---------------------------- \n");        
        
        nState = _SG_LOAD;
		FileChunk.Log("[SGController.Load] State! = <loading> \n");
        this.cModule = cModule;
		
		GameObject[] aObjs = cModule.GetObjectsFromClassModule("GameObject", cModule);
		
		FileChunk.Log("SG-INFO Number of gameobjects on module = <"+aObjs.length+"> \n");
		
		// Odrejestrowanie obiektow z mapy interakcji - same przywroce sie do niej tylko te ktore powinny
		for (int i=0; i < aObjs.length; i++)
			if (aObjs[i] instanceof ControlObject)
				((ControlObject)aObjs[i]).UnregisterInInteractionMap();
		        
        // Musi by wczesniej zaczytana tablica obiektow !
        PreLoadChunks(sFileName+".temp");
		
		aObjs = cModule.GetObjectsFromClassModule("GameObject", cModule);
        
        // Interpretujemy chunki 
        LoadChunks(sFileName+".temp");               				
        
		// Kasujemy wszystkie obiekty, ktre maja ustawiony atrybu do zapisu, ale nie zostaly odczytane
        for (int i=0; i < aObjs.length; i++)
            if ((aObjs[i].bSG == true) && (aObjs[i].bSGLoaded == false) && (!aObjs[i].IsCreatedDuringLoading()))
            {
                // kasujemy obiekty, ktore pozostaja po odczytaniu save'a
				FileChunk.Log("[SGController.Load] Removing object ID = <"+aObjs[i].GetUniqueID()+"> ClassName = <"+aObjs[i].getClass().getName()+"> Name = <"+aObjs[i].GetName()+">\n");
				aObjs[i].SetUniqueID();
				DeactivateObject(aObjs[i]);
            }
            else
			{
                aObjs[i].bSGLoaded = false;
			}
		
		// Wywolanie update na wszystkich obiektach nie-enginowych		
        FileChunk.Log("[SGController.Load] : Objects updating\n");
        SGObjectsRepository.SGUpdate();
		
		// Ustawienie wszystkim obiektom, e podlegaja odczytywaniu		
        for (int i=0; i < vObjects.size(); i++)
        {
            GameObject obj = ((SGObject)vObjects.elementAt(i)).cObj;
            if (obj != null) 
				obj.bLoadingSG = true;
        }				
		this.cModule.bLoadingSG = true;
		
		// Wywolanie update na wszystkich obiektach enginowych		
        for (int i=0; i < vObjects.size(); i++)
        {
            GameObject obj = ((SGObject)vObjects.elementAt(i)).cObj;
            if (obj != null) 
				obj.SGUpdate();
        }				
		this.cModule.SGUpdate();
		
		// Ustawienie wszystkim obiektom, e nie podlegaja odczytywaniu		
        for (int i=0; i < vObjects.size(); i++)
        {
            GameObject obj = ((SGObject)vObjects.elementAt(i)).cObj;
            if (obj != null) 
				obj.bLoadingSG = false;
        }						
		this.cModule.bLoadingSG = false;
		
		// Aktywacja na module - zabezpieczenie						
		for (int i=0; i < vObjects.size(); i++)
		{
			SGObject cSGObj = (SGObject)vObjects.elementAt(i);
			if ((cSGObj != null) && (cSGObj.cObj != null))
			{
				if ((cSGObj.bActiveOnModule == true) && (cSGObj.cObj.IsActiveOnModule() == false))					
				{
					cSGObj.cObj.DeactivateOnModuleAlways();
					if (cSGObj.bGridIndependent)
						((ControlObject)cSGObj.cObj).ActivateOnModuleGridIndependent();
					else
						cSGObj.cObj.ActivateOnModule();
					FileChunk.Log("[SGController.LoadObjectRef] SG-WARNING! Object <"+cSGObj.cObj.GetName()+":"+cSGObj.cObj.getClass().getName()+"> was not activated !\n");          
				}
				else if ((cSGObj.bActiveOnModule == false) && (cSGObj.cObj.IsActiveOnModule() == true))											
				{
					cSGObj.cObj.DeactivateOnModuleAlways();
					FileChunk.Log("[SGController.LoadObjectRef] SG-WARNING! Object <"+cSGObj.cObj.GetName()+":"+cSGObj.cObj.getClass().getName()+"> was not deactivated !\n");                
				}
			}
		}
		
        this.cModule = null;	
        this.cPreObject = null;
		nState = _SG_NONE;		
		
		// Skasowanie pliku tymczasowego i utworzenie save'a do restartu misji        			
		FileChunk.Log("[SGController.Load] State! = <none> \n");		
		
		File file = new File(sFileName+".temp");
		if (file.delete() == false)
            GameObject.CrashLog("TempFile was not removed\n");
		
		 if (!sFileName.equals(_SG_RESTARTSAVELOAD_FILE))
		 {
			CopyFile(sFileName + ".dat",_SG_RESTARTSAVELOAD_FILE);
			CopyFile(sFileName + ".lev",_SG_RESTARTSAVELOAD_FILE+".lev");
		 }
        		
		 DumpDangerousObjects(cModule);
		 
		 aObjs = null;
		 
		 m_fEndTime = GameObject.GetAbsoluteTime();
    }
	
	void CopyFile(String sFileSrc, String sFileDst)
	{
        try {
            File fFileDst = new File(sFileDst);
			if (!fFileDst.exists())
				if (!fFileDst.createNewFile())            
				{
					GameObject.CrashLog("[SGController.CopyFile] SG-ERROR! DstFile <"+sFileDst+"> was not created ! \n");
					return;            
				}
			
            File fFileSrc = new File(sFileSrc);
			if (!fFileSrc.exists())
			{
				GameObject.CrashLog("[SGController.CopyFile] SG-ERROR! SrcFile <"+sFileSrc+"> doesn't exist ! \n");
				return;            			
			}
			
			//przywracamy save restartowy dla loadowanej gry
            DataInputStream cRestartIn = new DataInputStream(new FileInputStream(sFileSrc) );
            DataOutputStream cRestartOut = new DataOutputStream( new FileOutputStream(sFileDst));
            byte[] tBuffer = new byte[1024];
            do{
                int nRead = cRestartIn.read(tBuffer);
                if (nRead > 0)
                    cRestartOut.write(tBuffer, 0, nRead);
                else
                    break;
            }
            while(true);
			cRestartIn.close();
			cRestartOut.close();
        } catch(Exception e) {
            GameObject.CrashLog("[SGController.CopyFile] SG-ERROR! File <"+sFileSrc+"> was not copied to <"+sFileDst+"> ! \n");
        }					
	}
    
    /** Odczytuje tablic informacyjna o obiektach : 
     * unikalny identyfikator + nazwa klasy
     */
    void LoadObjectsTable(FileChunk cFC)
    {
        FileChunk.Log("[SGController.LoadObjectsTable]\n");
        
        // odczytujemy liczbe obiektow
        int nSGObjects = cFC.LoadInt();
        vObjects = new java.util.Vector(nSGObjects);
        
        FileChunk.Log("<Number of objects> = "+nSGObjects+"\n+");

        // odczytujemy informacje o obiektach
        for(int i=0; i < nSGObjects; i++)
        {            
            SGObject cSGObj = new SGObject(cFC.LoadInt(), cFC.LoadString(),cFC.LoadBool(),cFC.LoadBool());
            vObjects.addElement(cSGObj);            
            FileChunk.Log("[Object] UniqueID = <"+cSGObj.nID+"> ClassName = <"+cSGObj.sClassName+">\n");
        }
    }
    
    private void PreLoadChunks(String sFileName)
    {
        FileChunk cMain = GameObject.OpenFile(sFileName);

        FileChunk.Log("[SGController.PreLoadChunks] <<< PRELOAD - BEGIN >>>\n");
        FileChunk.Log("[SGController.PreLoadChunks] MainChunk data to read <"+cMain.GetDataToRead()+">\n");
        
        // czytamy poki sa dane!
        while(cMain.GetDataToRead() != 0)
        {
            FileChunk.Log("---------------------------- \n");
            
            // odczytujemy kolejnego Chunka
            FileChunk cFC = GameObject.LoadChunk(cMain);
            
            FileChunk.Log("[SGController.PreLoadChunks] Chunk <"+cFC.GetID()+"> data in chunk <"+cFC.GetDataToRead()+">\n");
            
            switch( cFC.GetID() )
            {
			case ESGChunks._CHK_SAVE_VERSION_NUMBER:
				nReadVersion = cFC.LoadLong();
				break;
			case ESGChunks._CHK_SAVE_ADDINS:
				nNextIDValue = cFC.LoadInt();
				//RemoveObjectsOverNextID(cModule);
				if (cModule.GetNextIDValue() < nNextIDValue)
					cModule.SetNextIDValue(nNextIDValue);					
				break;				
            case ESGChunks._CHK_OBJECTS_TABLE:
				//if (bPreLoad) return;
                LoadObjectsTable(cFC);
                break;

            case ESGChunks._CHK_PRECREATE_OBJECT:
                if (bPreLoad)
                {
                    cPreObject.PreLoad(cFC,cModule);
                    FileChunk.Log("[SGController.PreLoadChunks] Skipping chunk <"+cFC.GetID()+">!\n");
                    cMain.delete();
                    return;
                }
                FileChunk.Log("[SGController.PreLoadChunks] Skipping chunk <"+cFC.GetID()+">!\n");
                break;				
                
            case ESGChunks._CHK_OBJECTS_REPOSITORY:
				//if (bPreLoad) return;
                SGObjectsRepository.SGLoadChunk(cFC);
                break;		
                
            case ESGChunks._CHK_EXPRESSION_VAR_MAN:
				// if (bPreLoad) return;
                cModule.LoadExpressionVarMan(cFC);
                break;
                
            case ESGChunks._CHK_INPUT:
				//if (bPreLoad) return;
                //cModule.LoadInput(cFC);
                break;				
                
            case ESGChunks._CHK_OBJECT:
				//if (bPreLoad) return;
                FileChunk.Log("[SGController.PreLoadChunks] Skipping chunk <"+cFC.GetID()+">!\n");
                break;
                
            default:
                GameObject.CrashLog("[SGController.PreLoadChunks] Unrecognized chunk <"+cFC.GetID()+">!\n");
            }
            
            // kasujemy chunka tzn. przechodzimy na jego koniec
            cFC.delete();
        }		
        
        cMain.delete();
        
        FileChunk.Log("[SGController.PreLoadChunks] <<< PRELOAD - END >>>\n");
    }
    
    /** Metoda czyta kolejne chunki i wywoluje dla nich odpowiednie metody
     */
    private void LoadChunks(String sFileName)
    {
        // otwieramy plik z Savem
        FileChunk cMain = GameObject.OpenFile(sFileName);

        FileChunk.Log("----------------------------------------------------------------------- \n");
        FileChunk.Log("[SGController.LoadChunks]				ONLY OBJECTS					 \n");
        FileChunk.Log("[SGController.LoadChunks] MainChunk data to read <"+cMain.GetDataToRead()+">\n");
        FileChunk.Log("----------------------------------------------------------------------- \n");
        
        // czytamy poki sa dane!
        while(cMain.GetDataToRead() != 0)
        {
            FileChunk.Log("---------------------------- \n");
            
            // odczytujemy kolejnego Chunka
            FileChunk cFC = GameObject.LoadChunk(cMain);
            
            FileChunk.Log("[SGController.LoadChunks] Chunk <"+cFC.GetID()+"> data in chunk <"+cFC.GetDataToRead()+">\n");
            
            switch( cFC.GetID() )
            {                
			case ESGChunks._CHK_SAVE_VERSION_NUMBER:
            case ESGChunks._CHK_PRECREATE_OBJECT:
            case ESGChunks._CHK_OBJECTS_TABLE:
            case ESGChunks._CHK_OBJECTS_REPOSITORY:
            case ESGChunks._CHK_EXPRESSION_VAR_MAN:
                FileChunk.Log("[SGController.LoadChunks] Skipping chunk <"+cFC.GetID()+">!\n");
                break;
                
            case ESGChunks._CHK_OBJECT:
				//FileChunk.Log("----------------------------------------------------------------------- \n");
                LoadObject(cFC);
				//FileChunk.Log("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ \n");
				//DumpDangerousObjects(cModule);
				//FileChunk.Log("----------------------------------------------------------------------- \n");
                break;
                
            default:
                FileChunk.Log("[SGController.LoadChunks] Unrecognized chunk <"+cFC.GetID()+">!\n");
            }
            
            // kasujemy chunka tzn. przechodzimy na jego koniec
            cFC.delete();
        }
        
        // zamykamy glownego chunka, co powoduje zamkniecie pliku
        cMain.delete();		
        
        FileChunk.Log("----------------------------------------------------------------------- \n");
        FileChunk.Log("[SGController.LoadChunks]				ONLY OBJECTS - END				 \n");
        FileChunk.Log("----------------------------------------------------------------------- \n");
    }	

    /** Wczytywanie obiektu.
     */		
    GameObject LoadObject(FileChunk cFCObj)
    {
        FileChunk.Log("[SGController.LoadObject]\n");
        
        GameObject obj = LoadGameObjectRef(cFCObj);
        
        if (obj == null) return null;
        
        if (!obj.IsCreatedFromEngine())
            GameObject.CrashLog("[SGController.LoadObject] SG-ERROR! Object was not created from engine!!!\n");

        // czytamy dane obiektu
        obj.bLoadingSG = true;       
        
        // ladujemy kolejne chunki z chunka obiektu
        if(cFCObj.GetDataToRead() != 0)
        {
            FileChunk.Log("cFCObj.GetDataToRead()="+cFCObj.GetDataToRead()+"\n");
            // odczytujemy kolejnego Chunka
            obj.SGLoadChunk(cFCObj);
        }
        
        if (cFCObj.GetDataToRead() != 0)
            GameObject.CrashLog("[SGController.LoadObject] ERROR !!! Data was not read : <"+cFCObj.GetDataToRead()+">\n");
        
        obj.bLoadingSG = false;
        
        return obj;
    }
    
    //*********************************************************
    //					Zapisywanie SaveGame
    //*********************************************************
	
	private boolean CompressFile(String sFileName)
	{
		int nRead = 0;
		boolean bRet = true;
		
        File fFile = new File(sFileName+".temp");			
        if (!fFile.exists())
            return false;	
		if (fFile.length() == 0)
		{
			fFile.delete();
			return false;				
		}
		fFile = null;
		
        try
        {				
			FileInputStream jTempIn = new FileInputStream(sFileName+".temp");
			
			FileOutputStream jSave = new FileOutputStream(sFileName,false);
			ZipOutputStream jSaveZip = new ZipOutputStream(jSave);
			jSaveZip.setLevel(9);				
			
			ZipEntry jEntry = new ZipEntry("data");
			jEntry.setMethod(ZipEntry.DEFLATED);
			jSaveZip.putNextEntry(jEntry);
			
			while ((nRead = jTempIn.read(buffer,0,_SG_BUFFER_SIZE)) != -1)
				jSaveZip.write(buffer,0,nRead);
			
			jSaveZip.closeEntry();
			
			jSaveZip.close();
			jSave.close();
			
			jTempIn.close();
			
			File file = new File(sFileName+".temp");
			file.delete();
		}
        catch (IOException e)
        { 
			GameObject.CrashLog("SG-ERROR! File <"+sFileName+"> was not compressed !\n");
			bRet = false;
        }			
		
		
        fFile = new File(sFileName);			
        if (fFile.exists() && (fFile.length() == 0))
		{
			fFile.delete();				
            return false;				
		}
		fFile = null;
		
		return bRet;
	}
	
	public void SaveObjectAfterParents(GameObject obj)
	{
		// drugi raz nie zapisujemy tego samego obiektu
		if (obj.bSGRepeated == true)
			return;
		
		// zapisanie w pierwszej kolejnosci rodzicw
		if (obj instanceof ControlObject)
		{
			ControlObject parent = ((ControlObject)obj).GetParent();		
			if (parent != null)		
			{
				parent.bSG = true;
				SaveObjectAfterParents(parent);
			}
		}
		
        FileChunk.Log("[Object] UniqueID = <"+obj.GetUniqueID()+"> ClassName = <" + obj.getClass().getName() + ">\n");	
               
        FileChunk cFC = GameObject.NewChunk(ESGChunks._CHK_OBJECT, cMainFC);
        // zapisujemy co to za obiekt       
		cFC.SaveInt((obj instanceof Module) ? 0 : obj.GetUniqueID());
        cFC.SaveString(obj.getClass().getName());		
        // zapisujemy dane obiektu
        obj.SGSaveChunk(cFC);                   
        cFC.delete();		
		
		FileChunk.Log("---------------------------- \n");
	}
	
	public void SaveGameObjects(boolean bReset)
	{
        SaveObjectsTable(false,bReset);					
		for (int nSize = -1, nTurn=0; nSize != vObjects.size();nTurn++)
		{			
			FileChunk.Log(" BEGIN TURN = <"+nTurn+"> \n");
			
			nSize = vObjects.size();
			for(int i=0; i < vObjects.size(); i++)
				SaveObjectAfterParents(((SGObject)vObjects.elementAt(i)).cObj);			            				
			SaveObjectsTable(false,false);
			
			FileChunk.Log(" END TURN = <"+nTurn+"> \n");
		} 				
	}
    
    /** Zapisuje stan gry do podanego pliku z podanego levelu.
     */
    public void Save(String sFileName, Module cModule, SGPreObject cPreObject)
    {
		if (nState != _SG_NONE)
		{
			GameObject.CrashLog("[SGController.Save] SG-ERROR! State of SGController is different from expected !\n");
			Thread.dumpStack();				
			return;
		}
		
		/*
		if (GameObject.GetAbsoluteTime()-m_fEndTime < 0.1f)
		{
			GameObject.CrashLog("[SGController.Save] SG-ERROR! You cannot save now !\n");
			Thread.dumpStack();				
			return;			
		}
		*/
		
		m_fStartTime = GameObject.GetAbsoluteTime();
				
        GameObject.CrashLog("---------------------------- \n");
        //GameObject.CrashLog("[SGController.Save] : <" + sFileName + "> Version = <"+cModule.GetSaveVersion()+"> Level = <"+((ChromeModuleSingle)cModule).cGameInitInfo.cLevels.GetCurrent().sSaveName+">\n");
		GameObject.CrashLog("[SGController.Save] : <" + sFileName + "> Version = <"+cModule.GetSaveVersion()+"> \n");
        GameObject.CrashLog("---------------------------- \n");
		
		DumpDangerousObjects(cModule);
        
		// zresetowanie repozytorium obiektw nie osadzonych w engine
        SGObjectsRepository.Reset();

        nState = _SG_SAVE;
		
		FileChunk.Log("[SGController.Save] State! = <saving> \n");		
		
        this.cModule = cModule;
        this.cPreObject = cPreObject;				

        // glowny chunk pliku
        cMainFC = GameObject.NewFile(sFileName+".temp");
		
		// zapisanie wresji save'a
        FileChunk cFCVersion = GameObject.NewChunk(ESGChunks._CHK_SAVE_VERSION_NUMBER, cMainFC);
        cFCVersion.SaveLong(cModule.GetSaveVersion());
        cFCVersion.delete();
		        
        // zapisujemy obiekt specjalny
        if (cPreObject != null)
        {
			FileChunk cFCPreSave = GameObject.NewChunk(ESGChunks._CHK_PRECREATE_OBJECT, cMainFC);
            FileChunk.Log("[SGController.Save] : Special information about save \n");       
            cPreObject.PreSave(cFCPreSave);           
			cFCPreSave.delete();
        }		
		else
			GameObject.CrashLog("[SGController.Save] SG-ERROR ! pre-object = null \n");			

        FileChunk cFCAddIns = GameObject.NewChunk(ESGChunks._CHK_SAVE_ADDINS, cMainFC);
        cFCAddIns.SaveInt(cModule.GetNextIDValue());
        cFCAddIns.delete();				
        
		// zapisanie wartosci skryptowych
        FileChunk cFCSave = GameObject.NewChunk(ESGChunks._CHK_EXPRESSION_VAR_MAN, cMainFC);		
        cModule.SaveExpressionVarMan(cFCSave);
        cFCSave.delete();
        		
		// zapisanie wszystkich obiektw pochodnych GameObject osadzonych w engine
		// z atrybutem ustawionym na zapis
		SaveGameObjects(true);
		        
		// zapisanie obiektw, ktre nie sa osadzone w enginie, a zostay zapisane ich 
		// referencje przy zapisie powyszych obiektw
        SGObjectsRepository.SGSaveChunk(cMainFC,true);
		
		// W wyniku zapisania powyzszych obiektw moga pojawic si nowe obiekty pochodne
		// enginowe GameObject, ktre potrzebuja zapisania i naley to ponownie sprawdzi
		SaveGameObjects(false);
		SaveObjectsTable(true,false);		
		
		SGObjectsRepository.SGSaveChunk(cMainFC,false);
        
        // zamykamy plik z save'em
        cMainFC.delete();
		
        this.cModule = null;
        this.cPreObject = cPreObject;			
        		
		// kompresja save'a
		CompressFile(sFileName);
		
		nState = _SG_NONE;
		
		FileChunk.Log("[SGController.Save] State! = <none> \n");						

		// przekopiowanie pliku restartu misji
		if (!sFileName.equals(_SG_RESTARTSAVELOAD_FILE))
		   CopyFile(_SG_RESTARTSAVELOAD_FILE,sFileName + ".dat");
		
		
		DumpDangerousObjects(cModule);
		
		m_fEndTime = GameObject.GetAbsoluteTime();
    }	

    /** Zapisuje tablic informacyjna o obiektach
     * unikalny identyfikator + nazwa klasy
     */	
    void SaveObjectsTable(boolean bWriteTable, boolean bResetObjects)
    {
        FileChunk.Log("[SGController.SaveObjectsTable]\n");		
        
        // wyciagamy wszystkie GameObjecty i budujemy tablice obiektow
        GameObject[] aObjs = cModule.GetObjectsFromClassModule("GameObject", cModule);
		
		FileChunk.Log("SG-INFO Number of GameObjects on module = <"+aObjs.length+"> \n");
		
		for(int i=0; i < aObjs.length; i++) 
		{
			if (aObjs[i].bSG && (aObjs[i] instanceof ControlObject))
			{
				ControlObject[] aChildren = ((ControlObject)aObjs[i]).GetChildren();
				if (aChildren != null)
				{
					for (int j=0; j < aChildren.length; j++)
						aChildren[j].bSG = true;
				}
			}
		}
        
        // liczymy ile jest obiektow zapisywalnych
        int nSGObjects = 0;
		
        for(int i=0; i < aObjs.length; i++) 
        {
            if (bResetObjects) aObjs[i].bSGRepeated = false;
            if (aObjs[i].bSG) ++nSGObjects;
        }

		FileChunk cFC = null;
		
		if (bWriteTable)
		{							
			cFC = GameObject.NewChunk(ESGChunks._CHK_OBJECTS_TABLE, cMainFC);        
			// zapisujemy ilosc elementow
			cFC.SaveInt(nSGObjects);
		}
        
        vObjects = new java.util.Vector(nSGObjects);

		// Tylko do kocowych testw zawartosci leveli 
		List listObjs = Arrays.asList(aObjs);
		Collections.sort(listObjs);						
		aObjs = (GameObject[])listObjs.toArray();		
		
		if (bResetObjects)
		{
			GameObject.CrashLog("<<< ERRORS SECTION - BEGIN >>>\n");
				
			for(int i=0; i < aObjs.length; i++)
			{
				for(int j=i+1; j < aObjs.length; j++)
					if (aObjs[i].GetUniqueID() == aObjs[j].GetUniqueID())
					{
						GameObject.CrashLog("[Object-1] UniqueID = <"+aObjs[i].GetUniqueID()+"> ClassName = <"+aObjs[i].getClass().getName()+"> Name <"+aObjs[i].GetName()+"> ThisID <"+aObjs[i].GetThisID()+">\n");
						GameObject.CrashLog("[Object-2] UniqueID = <"+aObjs[j].GetUniqueID()+"> ClassName = <"+aObjs[j].getClass().getName()+"> Name <"+aObjs[j].GetName()+"> ThisID <"+aObjs[j].GetThisID()+">\n");
					}
					else
						break;
			}
		
			GameObject.CrashLog("<<< ERRORS SECTION - END >>>\n");
		}
		        
        // zapisujemy obiekty
        for(int i=0; i < aObjs.length; i++)
        {
            if(aObjs[i].bSG)
            {
                SGObject cSGObj = new SGObject(aObjs[i]);
                vObjects.addElement(cSGObj);
                
				if (bWriteTable)
				{					
					cFC.SaveInt( cSGObj.nID );
					cFC.SaveString( cSGObj.sClassName );
					cFC.SaveBool( cSGObj.bActiveOnModule );
					cFC.SaveBool( cSGObj.bGridIndependent );
					FileChunk.Log("[Object] UniqueID = <"+cSGObj.nID+"> ClassName = <"+cSGObj.sClassName+">\n");
				}                                
            }
        }

		if (bWriteTable)
			cFC.delete();			
		
        FileChunk.Log("---------------------------- \n");
    }
	
	void DeactivateObject(GameObject obj)
	{
		if (obj == null) return;			
		
		if (obj.IsCreatedDuringLoading())
		{
			GameObject.CrashLog("[SGController.LoadObjectRef] SG-ERROR! Object was not deleted !!! Name = <"+obj.GetName()+"> ID = <"+obj.GetUniqueID()+"> \n");
			return;
		}
		
		if (obj instanceof Sprite)
			((Sprite)obj).RemoveSprite((Sprite)obj);		
				
        obj.DisableCallMethods();
				
		if (obj instanceof ControlObject)
		{
			if (((ControlObject)obj).GetParent() != null)
				((ControlObject)obj).DetachFromParent();
			((ControlObject)obj).DetachAllSounds();						
		}
        obj.DeactivateOnModuleAlways();
        if (obj instanceof MeshObject)
            ((MeshObject)obj).EnableRendering(false);

		/* w razie niebezpieczestwa zakomentowa - begin */
		obj.delete();									
		/* w razie niebezpieczestwa zakomentowa - end */		
	}

    /** Zapisanie referencji do obiektu.
     */			
    void SaveObjectRef(FileChunk cFC, ISaveGame obj)
    {
        boolean bSaveType = false;
        
        if (obj instanceof GameObject) 
        {	
            bSaveType = true;
            
            // obiekty GameObject, ktre nie zostay stworzone przez CreateObject
            // sa traktowane, tak jak obiekty, ktore nie dziedzicza po GameObject
            if (((GameObject)obj).GetUniqueID() == -1)
                bSaveType = false;				
        }
        
        // informacja potrzebana do odczytu, aby zastosowac wlasciwa metode odczytu
        if (bSaveType == true)
            cFC.SaveBool(true); 
        else
            cFC.SaveBool(false); 		   
        
        // jesli referencja nie wskazuje na zaden obietkt to zapisz -1
        if (obj == null)
        {
            cFC.SaveInt(-1);
            FileChunk.Log("[SGController.SaveObjectRef] = null\n");		
        }		
        // w przeciwnym wypadku: unikalny identyfikator i nazwe klasy
        else
        {
            if (bSaveType == true)
            {
                if (((GameObject)obj).bSG == false) 
                {
                    if (((GameObject)obj).IsCreatedDuringLoading() == false)
                    {                        
						((GameObject)obj).bSG = true;
						FileChunk.Log("[SGController.SaveObjectRef] SG-INFO! Mark <bSG> changed from 'false' to 'true' ! UniqueID = <" + ((GameObject)obj).GetUniqueID() +"> ClassName = <" + obj.getClass().getName() + ">\n");                        
					}
					else
					{						
						FileChunk.Log("[SGController.SaveObjectRef] SG-WARNING! Object created in editor! Failed trying mark <bSG> for UniqueID = <" + ((GameObject)obj).GetUniqueID() +"> ClassName = <" + obj.getClass().getName() + ">\n");
					}
                }
                cFC.SaveInt(((GameObject)obj).GetUniqueID());
                cFC.SaveString(obj.getClass().getName());
                FileChunk.Log("[SGController.SaveObjectRef] UniqueID = <" + ((GameObject)obj).GetUniqueID() +"> ClassName = <" + obj.getClass().getName() + ">\n");
            }
            else
            {
                int nCode = SGObjectsRepository.AddObject(obj);
                cFC.SaveInt(nCode);				
                FileChunk.Log("[SGController.SaveObjectRef*] UniqueID = <" + nCode +"> ClassName = <" + obj.getClass().getName() + ">\n");
            }			
        }		
    }    

    /** Odczytanie referencji na obiekt.
     */				
    ISaveGame LoadObjectRef(FileChunk cFC)
    {
        if (cFC.LoadBool() == true)
            return LoadGameObjectRef(cFC);
        else
            return LoadSimpleObjectRef(cFC);
    }
    
    /** Odczytanie referencji na obiekt dziedziczace po GameObject.
     */			
    GameObject LoadGameObjectRef(FileChunk cFCObj)
    {
        // odczytujemy dane obiektu
        int nID = cFCObj.LoadInt();
        
        if (nID == -1) 
        {
            FileChunk.Log("[SGController.LoadGameObjectRef] Reference is <null>\n");
            return null;
        }
        
        String sClassName = cFCObj.LoadString();
        
        FileChunk.Log("[SGController.LoadGameObjectRef] UniqueID = <"+nID+"> ClassName = <"+sClassName+"> LoadGameObjectRef\n");
        
        if (nID == 0)
        {
            FileChunk.Log("[SGController.LoadGameObjectRef] Module : " + sClassName + " loaded\n");
            return cModule;
        }
        
        // odczytujemy dane obiektu
        SGObject cSGObj = GetObjectByID(nID);
        
        if (cSGObj == null)
        {	
            // szukamy obiektu, bo mogl stworzyc sie poza wczytywaniem
            GameObject cObj = cModule.GetObjectByID(nID);
            
            if (cObj != null) 
            {
                if (cObj.getClass().getName().compareTo(sClassName) != 0)
                {
                    FileChunk.Log("[SGController.LoadObjectRef] Object (+) <"+cObj.getClass().getName()+"> deleted and created new <" + sClassName + ">\n");
					cObj.SetUniqueID();						
					if (cObj.bSG != true) //$ !!! nie powinienem usuwa obiektw, ktre sa do zapisu i tak na kocu te ktre nie zostay odczytane to zostana usuniete
						DeactivateObject(cObj);
                    cObj = cModule.CreateObjectNoInit(sClassName,nID);						
                    cObj.InitObject();
                }
                
                return cObj;
            }
            
            // ta sytuacja nie moze sie nigdy zdazyc!
            GameObject.CrashLog("[SGController.LoadObjectRef] nID="+nID+" sClassName="+sClassName+"\n");
            GameObject.CrashLog("[SGController.LoadObjectRef] SG-ERROR! This will never happen!\n");
            CriticalError("Error file reading !");
            return null;
        }
        
        if(cSGObj.cObj == null)
        {
            // szukamy obiektu, bo mogl stworzyc sie poza wczytywaniem
            cSGObj.cObj = cModule.GetObjectByID(nID);
            
            if (cSGObj.cObj==null)
            {
                // nie ma obiektu, trzeba stworzyc nowy o klasie sClassName
                FileChunk.Log("[SGController.LoadObjectRef] Object was not found sClassName=<"+sClassName+"> nID="+nID+"\n");
                // tworzymy i przypisujemy obiekt do modulu, ktory jest ladowany
                cSGObj.cObj = cModule.CreateObjectNoInit(sClassName,nID);
                cSGObj.cObj.InitObject();
            }
            else
            {
                if (cSGObj.cObj.getClass().getName().compareTo(sClassName) != 0)
                {
                    if (cSGObj.cObj.IsCreatedDuringLoading())
                    {
                        GameObject.CrashLog("[SGController.LoadObjectRef] SG-ERROR!!! That is impossible! Object to move was created in editor and can not be renumerted or deleted.  \n");
                    }
                    else if (cSGObj.cObj.bSG == false)
                    {                        
                        cSGObj.cObj.SetUniqueID();
                        FileChunk.Log("[SGController.LoadObjectRef] Object (++) <"+cSGObj.cObj.GetName()+":"+cSGObj.cObj.getClass().getName()+"> renumerated becouse of <" + sClassName + ">\n");						
                        cSGObj.cObj = cModule.CreateObjectNoInit(sClassName,nID);
                        cSGObj.cObj.InitObject();
                    }
                    else
                    {
                        FileChunk.Log("[SGController.LoadObjectRef] Object (++) <"+cSGObj.cObj.GetName()+":"+cSGObj.cObj.getClass().getName()+"> deleted and created new <" + sClassName + ">\n");                
						
                        cSGObj.cObj.SetUniqueID();
						if (cSGObj.cObj.bSG != true) //$ !!! nie powinienem usuwa obiektw, ktre sa do zapisu i tak na kocu te ktre nie zostay odczytane to zostana usuniete
							DeactivateObject(cSGObj.cObj);						
                        cSGObj.cObj = cModule.CreateObjectNoInit(sClassName,nID);						
                        cSGObj.cObj.InitObject();										
						
                    }
                }
                else
                    FileChunk.Log("[SGController.LoadObject] Object loaded with module.\n");
            }
        }
        
        if (sClassName.compareTo(cSGObj.cObj.getClass().getName()) != 0)
            GameObject.CrashLog("[SGController.LoadObject] SG-ERROR! Wrong class! Expected <"+sClassName+"> but <"+cSGObj.cObj.getClass().getName()+"> !.\n");
        
		if (cSGObj.cObj != null)
		{
			cSGObj.cObj.bSGLoaded = true;
			//if (cSGObj.cObj.IsActiveOnModule() == false)
			//	FileChunk.Log("[SGController.LoadObjectRef] SG-ERROR!!! Object <"+cSGObj.cObj.GetName()+":"+cSGObj.cObj.getClass().getName()+"> was not activated !!!\n");                
		}
			
        return cSGObj.cObj;
    }		
    
    /** Odczytanie referencji na obiekt nie dziedziczace po GameObject.
     */				
    ISaveGame LoadSimpleObjectRef(FileChunk cFC)
    {
        int nObjectID = cFC.LoadInt();
        
        if (nObjectID == -1)
        {
            FileChunk.Log("[SGController.LoadSimpleObjectRef*] = null\n");		
            return null;
        }
        
        FileChunk.Log("[SGController.LoadSimpleObjectRef*] UniqueID = <" + nObjectID + ">   \n");
        
        ISaveGame object = SGObjectsRepository.GetObject(nObjectID);
        
        if (object == null)
            GameObject.CrashLog("[SGController.LoadSimpleObjectRef*] SG-ERROR!: object is null\n");
        else        
            FileChunk.Log("[SGController.LoadSimpleObjectRef*] UniqueID = <" + nObjectID + "> ClassName = <" + object.getClass().getName() + ">\n");
        
        return object;
    }    	
    
    /** Zapisanie wektora referencji do obiektow.
     */		
    void SaveObjectsVectorRef(FileChunk cFC, java.util.Vector vRefs)
    {	
        if (vRefs == null)
        {
            cFC.SaveInt(-1);
            return;
        }
        
        int nSize = vRefs.size();
        FileChunk.Log("[SGController.SaveObjectsVectorRef] Size = <" + nSize +">\n");
        cFC.SaveInt(nSize);
        for (int i=0; i < nSize; i++)
            SaveObjectRef(cFC,(ISaveGame)vRefs.elementAt(i));	
    }
    
    /** Odczytanie wektora referencji na obiekty.
     */			
    java.util.Vector LoadObjectsVectorRef(FileChunk cFC)
    {
        int nSize = cFC.LoadInt();
        
        if (nSize == -1) return null;
        
        FileChunk.Log("[SGController.LoadObjectsVectorRef] Size = <" + nSize +">\n");
        java.util.Vector vRefs = new java.util.Vector();
        vRefs.setSize(nSize);
        FileChunk.Log("[SGController.LoadObjectsVectorRef] Size = <" + vRefs.size()+">\n");
        for (int i=0; i < nSize; i++)
            vRefs.setElementAt(LoadObjectRef(cFC),i);
        
        return vRefs;
    }	
	
	void RemoveObjectsOverNextID(Module mod)
	{		
		GameObject[] aObjs = cModule.GetObjectsFromClassModule("GameObject", cModule);
		for(int i=0; i < aObjs.length; i++)
			if (aObjs[i].GetUniqueID() > nNextIDValue)
				DeactivateObject(aObjs[i]);
	}	
	
	void DumpDangerousObjects(Module mod)
	{
		/*
		
		GameObject.CrashLog("<<< DANGEROUS OBJECTS - BEGIN >>>\n");
		
        // wyciagamy wszystkie GameObjecty i budujemy tablice obiektow
        GameObject[] aObjs = Module.GetObjectsFromClassModule("GameObject", mod);
		
		FileChunk.Log("SG-INFO Number of GameObjects on module = <"+aObjs.length+"> \n");
		        		
		List listObjs = Arrays.asList(aObjs);
		Collections.sort(listObjs);						
		aObjs = (GameObject[])listObjs.toArray();		
		
				
		for(int i=0; i < aObjs.length; i++)
		{
			for(int j=i+1; j < aObjs.length; j++)
				if (aObjs[i].GetUniqueID() == aObjs[j].GetUniqueID())
				{
					if (aObjs[i] == aObjs[j])
						GameObject.CrashLog("[Object-1] and [Object-2] have the same reference ! \n");
					GameObject.CrashLog("[Object-1] UniqueID = <"+aObjs[i].GetUniqueID()+"> ClassName = <"+aObjs[i].getClass().getName()+"> Name <"+aObjs[i].GetName()+">\n");
					GameObject.CrashLog("[Object-2] UniqueID = <"+aObjs[j].GetUniqueID()+"> ClassName = <"+aObjs[j].getClass().getName()+"> Name <"+aObjs[j].GetName()+">\n");
				}
				else
					break;
		}
		
		GameObject.CrashLog("<<< DANGEROUS OBJECTS - END >>>\n");	
		*/
	}
}

/** Obiekt zapisany na dysku - jego reprezentacja w tablicy obiektow zapisywanych.
 */
class SGObject implements Comparable
{
    // identyfikator unikalny obiektu z C++
    int nID = -1;
    
    // obiekt
    GameObject cObj = null;
    
    // nazwa klasy do utworzenia obiektu jesli cObj == null
    String sClassName = null;

	// czy jest aktywny na module
	boolean bActiveOnModule = false;
	boolean bGridIndependent = false;
    
    /** Utworzenie obiektu przy zapisie
     */
    SGObject(GameObject obj)
    {
        cObj = obj;
        nID = cObj.GetUniqueID();
		bActiveOnModule = obj.IsActiveOnModule();
		bGridIndependent = false;
		if (obj instanceof ControlObject) 
			bGridIndependent = ((ControlObject)obj).IsGridIndependent();			
        //HACK
        if (cObj instanceof Module)
            nID = 0;
        sClassName = cObj.getClass().getName();
        if (sClassName == null)
            GameObject.CrashLog("[SGObject.Save] SG-ERROR! sClassName = null \n");
    }
    
    /**Utworzenie obiektu przy odczycie
     */
    SGObject(int id, String className, boolean bActive, boolean bGrid)
    {
        nID = id;
        cObj = null;
		bActiveOnModule = bActive;
		bGridIndependent = bGrid;
        if (className == null)
            GameObject.CrashLog("[SGObject.Load] SG-ERROR! className = <null> \n");				
        sClassName = new String(className);
        if (sClassName == null)
            GameObject.CrashLog("[SGObject.Load] SG-ERROR! sClassName = <null> \n");						
    }
    

    public int compareTo(Object o) {
		SGObject obj = (SGObject)o;		
		return (nID == obj.nID) ? 1 : 0;
    }    
}
