public class LaserPointer extends MeshObject
{
    final static String _BEAM_RED           = "LaserBeamRed",
                        _BEAM_BLUE          = "LaserBeamBlue",
                        _BEAM_YELLOW        = "LaserBeamYellow",
                        _BEAM_WHITE         = "LaserBeamWhite",
                        _BEAM_GREEN         = "LaserBeamGreen",
                        _BEAM_YELLOW_NDT    = "LaserBeamYellowNDT";

    
    String m_sFXBeamName = _BEAM_RED;
    int m_nCurSequence = -1;
    float m_fRange = 10000;
    
    /**Czy promien ma byc zawsze jeden, czy "stare" moga zostawac (na czas trwania sekwencji
     * m_sFXBeamName) */
    boolean m_bSingleBeam = true;
    
    
    void OnCreate()
    {
        super.OnCreate();
        EnableRendering(false);
        ClearFlag(EFlags._COLLISION);
    }
    
    
    //parametry dynamicznego fxa, ktorym jest promien lasera
    public float fFXSizeX = 2.0f;
    public float fFXSizeY = m_fRange/2f;
    public float fFXZero = 0f;
    public float fFXPosY = 0f;
    
    
    
    void SetSize(float fSize)
    {
        if (fSize > 0)
            fFXSizeX = fSize;
    }
    
    void SetRange(float fRange)
    {
        m_fRange = fRange;
    }
    
    /**Wskazuje do zadanego punktu, nie bierze pod uwage m_fRange*/
    void PointTo(Vector vPoint)
    {
        //float fDist = vStart.Distance(vEnd);
        Vector vPos = GetPositionVectorVolatile();
        Vector vDir = vPoint.Sub2(vPos);
        float fDist = vPos.Distance(vPoint);
        vDir.Normalize();
        
        fFXSizeY = fDist;
        fFXPosY = fDist/2f;
        ExecuteBeam(vPos, vDir);
    }
    
    /**Wskazuje w kierunku vDir (bierze pod uwage m_fRange)*/
    void PointTowards(Vector vDir)
    {
        vDir.Normalize();
        fFXSizeY = m_fRange;
        fFXPosY = m_fRange/2f;
        ExecuteBeam(GetPositionVectorVolatile(), vDir);
    }
    
    /**Wskazuje w kierunku forwarda lasera w zadanym zakresie*/
    void PointForward(float fRange)
    {
        if (fRange <= 0)
            fRange = m_fRange;
        fFXSizeY = fRange;
        fFXPosY = fRange/2f;
        ExecuteBeam(GetPositionVectorVolatile(), GetForwardVectorVolatile());
    }
    
    /**Wskazuje w kierunku forwarda lasera w zadanym zakresie, z przyataczowaniem do lasera, 
     * jesli laser sie poruszy to beam tez*/
    void PointForwardAttach(float fRange)
    {
        if (fRange <= 0)
            fRange = m_fRange;
        fFXSizeY = fRange;
        fFXPosY = fRange/2f;
        ExecuteBeamAttach(Vector.Zero, VectorWorldToLocal(GetForwardVectorVolatile()));
    }
    
    /**Wskazuje od punktu do punktu*/
    void PointFromTo(Vector vStart, Vector vEnd)
    {
        fFXSizeY = vStart.Distance(vEnd);
        fFXPosY = fFXSizeY/2f;
        Vector vDir = vEnd.Sub2(vStart);
        vDir.Normalize();
        ExecuteBeam(vStart, vDir);
    }
    
    /**Wskazuje od punktu do punktu, fx atachuje sie do lasera, wektory musza byc lokalne*/
    void PointFromToAttach(Vector vStart, Vector vEnd)
    {
        fFXSizeY = vStart.Distance(vEnd);
        fFXPosY = fFXSizeY/2f;
        Vector vDir = vEnd.Sub2(vStart);
        vDir.Normalize();
        ExecuteBeamAttach(vStart, vDir);
    }
    
    /**Wskazuje od zadanego punktu w zadanym kierunku*/
    void PointFromTowards(Vector vStart, Vector vDir)
    {
        vDir.Normalize();
        fFXSizeY = m_fRange;
        fFXPosY = m_fRange/2f;
        ExecuteBeam(vStart, vDir);
    }
    
    private void ExecuteBeam(Vector vPos, Vector vDir)
    {
        if (m_nCurSequence != -1 && m_bSingleBeam)
            DeleteFXSequence(m_nCurSequence);
        m_nCurSequence = ExecuteFX(m_sFXBeamName, vPos, vDir);
    }

    private void ExecuteBeamAttach(Vector vPos, Vector vDir)
    {
        if (m_nCurSequence != -1 && m_bSingleBeam)
            DeleteFXSequence(m_nCurSequence);
        m_nCurSequence = ExecuteFX(this, m_sFXBeamName, vPos, vDir);
    }
    
    public void delete()
    {
        if (m_nCurSequence != -1)
            DeleteFXSequence(m_nCurSequence);
        super.delete();
    }
    
}

