
/**Klasa reprezentujaca obiekt, ktory moze byc uzyty przez Actora.
 */
public class TriggerObject extends MeshObject implements iact_EnableDisable
{
    /** Nazwa elementu ktory ma byc aktywny. Na jej podstawie jest ustalany numerek.
     */
    public String ActiveElement=null;//#TRIGGER
    
    /** Akcje wolane gdy na obiekcie zrobi sie use */
    public Action[] aOnExecute = null;//#\TRIGGER
    
    /* Flaga mowi czy obiekt jest aktywny tzn. czy mozna go uruchomic */
    public boolean bActiveTrigger = true;//#\TRIGGER
    
    /** Ustawienie tej flagi powoduje liczenie kolizji z calym obiektem,
     * ale podswietlanie tylko aktywnego elementu. Wprowadzone, aby latwiej
     * bylo celowac w obiekty np. AI.
     */
    public boolean bForceWholeObjectCollision = false;//#TRIGGER
    
    /** Tekst pokazywany na tipie gdy gracz podswietli triggera */
    public String sTip = "";//#TRIGGER;Tip shown when trigger is highlighted
    public String sAdditionalTip = "";//#TRIGGER;Additional tip shown when trigger is highlighted
    
    /** Tablica elementow aktywnych w TriggerObjectie
     */
    java.util.Vector aActiveElements = null;
    
    /** Element, ktory jest aktualnie aktywny. Ustawiane w metodzie CanActivate
     */
    int nActiveElement=-1;
    
    /** Flaga ustawiana po skasowaniu obiektu ( w OnDestroy )
     * po to, by mozna bylo debugowac ewentualne proby dostepu
     * do obiektu po jego skasowaniu
     */
    boolean bDeleted = false;
    
    /////////////////////////////////////////////////////////////////////////////
    // Obsluga przez humanoidalne AI

    ///////////////////////////////////////////////////////
    // INTERNAL EXECUTUION MODES
    public final static int _IEM_NONE    = 0;
    public final static int _IEM_EXECUTE = 1;
        
    /** Sposob, w jaki obslugiwany jest TriggerObject  */
    public EnumExecuteMode cExecuteMode = new EnumExecuteMode(EnumExecuteMode._GENERIC);//#TRIGGER
    
    /** Zapamietana rola aktualnego executora (dla TriggerObjectow, ktore moga byc 
     *  obslugiwane przez kilku executorow) */
    protected int nExecutorRole = 0;

    /** Nazwy dummies'ow (maska) wyznaczajacych pozycje executora
     */
    public String ExecutorOriginDummy = null;
    
    /** Tablica dummies'ow z pozycji executora
     */
    protected java.util.Vector aExecutorOriginDummies = null;
    
    /** Domyslna odleglosc z jakiej powinien uzywac TriggerObjecta executor 
     *  (gdy nie sa zdefiniowane dummies'y)
     */
    public float fDefExecutionDistance = 70.f;//#TRIGGER

    void OnCreate()
    {
		bSG = true;
        //PeH tak chcial, by MK :-)
        super.OnCreate();
        //LogD("Object <"+GetName()+">\n");
        
        nActiveElement=-1;
        
        if(ActiveElement!=null)
        {
            aActiveElements = GetElementsWithMask(ActiveElement);
            
            if( aActiveElements==null )
            {
                LogD("Any of active elements with mask <"+ActiveElement+"> not found!\n");
                
                SetTraceCollType(ECollTraceFlags._HULL);
            }
            else
            {
                // ustawienie kolizji per element, zeby mozna bylo wskazywac konkretne elementy
                SetTraceCollType(ECollTraceFlags._ELEMENT_BOX);
            }
        }
        
        if (ExecutorOriginDummy!=null)
        {
            aExecutorOriginDummies = GetElementsWithMask(ExecutorOriginDummy);   
        }
        
        SetFlag(EFlags._COLLISION);
        SetFlag(EFlags._BLOCK_TRACE);
    }
      
    void OnDestroy()
    {
		super.OnDestroy();
        bDeleted = true;
    }
    /////////////////////////////////////////////////////////////////////////////

    /** Pobranie liczby dostepnych rol, w jakich moze wystepowac przyszly executor */
    int GetExecutorRolesCount()
    {
        return 1;
    }
    
    /** Ustawienie roli, w jakiej chce wystepowac obiekt odwolujacy sie do TriggerObjecta
     *  @param nRole aktualnie wymagana rola (0 <= nRole < GetExecutorRolesCount())
     *  @return true, jesli ustawienie roli powiodlo sie
     */
    boolean SetExecutorRole(int nRole)
    {
        if (nRole >= 0 && nRole < GetExecutorRolesCount())
        {
            nExecutorRole = nRole;            
            return true;
        }
        else
            return false;
    }
    
    /** Uzycie obiektu
     * @param executor Actor wykonujacy akcje (<code>Actor</code>)
     */
    void Execute( Actor executor )
    {
        //CrashLog( "[TriggerObject.Execute] Executed by " + executor + "\n" );
        Module cModule = GetModule();
        if (cModule != null)
            cModule.TriggerExecuted(this, executor);
        
        InvokeActionChain(aOnExecute);
    }
    
    /** Pobranie pozycji, z jakiej nalezy uzywac TriggerObjecta 
     *  (dla roznego typu humanoidalnych AI)
     */
    Vector GetExecutionOrigin()
    {
        Vector vPos = null;
        if (nExecutorRole >= 0 &&
            aExecutorOriginDummies != null &&
            nExecutorRole < aExecutorOriginDummies.size())
        {
            // jest stablicowana pozycja
            vPos = GetElementPos(((Integer)aActiveElements.elementAt(nExecutorRole)).intValue());
        }

        if (vPos != null)
            vPos = PointLocalToWorld(vPos);
        else
        {
            // przesun sie od srodka w kierunku przeciwnym od execution dir (lokalnie)
            vPos = GetExecutionDir();
            float fLen = vPos.Len();
            if (fLen > 0.001);
                vPos.Mul(- fDefExecutionDistance / fLen);
            vPos.Add(GetPositionVectorVolatile());
        }
        
        return vPos;
    }
    
    /** Pobranie kierunku, do jakiego nalezy sie obrocic aby uzywac TriggerObjecta 
     *  (dla roznego typu humanoidalnych AI)
     */
    Vector GetExecutionDir()
    {
        Vector vDir = null;
        if (nExecutorRole >= 0 &&
            aExecutorOriginDummies != null &&
            nExecutorRole < aExecutorOriginDummies.size())
        {
            // jest stablicowana pozycja
            vDir = GetElementForwardVector(((Integer)aActiveElements.elementAt(nExecutorRole)).intValue());
        }

        if (vDir != null)
            vDir = VectorLocalToWorld(vDir);
        else
        {
            vDir = GetForwardVector();
            vDir.Negate();
        }
        return vDir;
    }
    
    
    /** Zwraca rodzaj triggera 
     * UWAGA: Prawdopodobnie trzeba bedzie to rozbudowac o mozliwosc definiowania 
     *        roznych ExecutionMode'ow dla roznych Roli executora */
    int GetTriggerExecutionMode()
    {
        return cExecuteMode.GetValue();
    }
	
	void SetTriggerExecutionMode(int iMode)
	{
		cExecuteMode = new EnumExecuteMode(iMode);
	}
    
    //////////////////////////////////////////////////////////////////////////////

    public boolean bExtendedExecutionMode = false;
    public ExecutionInfo[] aExecutionSteps = null;
    /** TODO: Comment
     */
    public void InitExecutionActions()
    {
        if (aExecutionSteps == null || aExecutionSteps.length == 0)
        {
            bExtendedExecutionMode = false;
            return;
        }
        
        bExtendedExecutionMode = true;
        if (aExecutionSteps.length > 1)
        {
            for (int i = 0; i < aExecutionSteps.length - 1; i++)
            {
                aExecutionSteps[i].cNext = aExecutionSteps[i + 1];
            }
        }
        aExecutionSteps[aExecutionSteps.length - 1].cNext = null;
    }

    /** TODO: Comment
     */
    public void StartExecution(Actor executor)
    {
        if (!bExtendedExecutionMode)
        {
            Execute(executor);
            return;
        }
        ExecutionInfo currentExecutionInfo = null;
        if (aExecutionSteps != null && aExecutionSteps.length > 0)
            currentExecutionInfo = GetFirstValidExecutionInfo(aExecutionSteps[0]);
        executor.TriggerObjectNotification(this, currentExecutionInfo);
    }

    /** TODO: Comment
     */
    public void ExecutionStepDone(Actor executor, ExecutionInfo finishedStep)
    {
        if (finishedStep != null)
        {
            if (finishedStep.iInternalExecutionMode == _IEM_EXECUTE)
                Execute(executor);
            executor.TriggerObjectNotification(this, GetNextValidExecutionInfo(finishedStep));
        }
    }

    /** TODO: Comment
     *  INFO: Do pokrycia 
     */
    public ExecutionInfo GetFirstValidExecutionInfo(ExecutionInfo currentInfo)
    {
        return currentInfo;
    }

    /** TODO: Comment
     *  INFO: Do pokrycia 
     */
    public ExecutionInfo GetNextValidExecutionInfo(ExecutionInfo currentInfo)
    {
        return currentInfo.cNext;
    }
    
    
    
    
    
    /** Podaje opis obiektu
     * @return string z opisem obiektu (<code>String</code>)
     */
    String GetTip()
    {
        return sTip;
    }
    
    String GetAdditionalTip()
    {
        return sAdditionalTip;
    }
    
    /** Zwraca rodzaj operacji, ktore da sie na obiekcie wykonac.
     *  (w Chrome tylko 1 akcja, okreslona umownie, bo co zrobi Execute to wie
     * tylko ona).
     * @return rodzaje operacji dostepnych na tym obiekcie (zwykle flagi)
     */
    byte GetExecuteMethod()
    {
        return 0;
    }
    
    /** Podaje extenty tej czesci mesha, ktora jest triggerem
     * (np. klamka dla obiektu drzwi).
     * @param vMinExtent tutaj zwracany jest 1. wierzcholek bounding boxa (<code>Vector</code>)
     * @param vMaxExtent tutaj zwracany jest 2. wierzcholek bounding boxa (<code>Vector</code>)
     */
    void GetTriggerExtents(Camera cam, Vector vMinExtent, Vector vMaxExtent )
    {
        if(nActiveElement > -1)
        {
            // aktywny jest jeden element
            GetScreenExtents(cam, vMinExtent, vMaxExtent, nActiveElement);
        }
        else
        {
            // aktywny jest caly obiekt
            GetScreenExtents(cam, vMinExtent, vMaxExtent);
        }
    }

    
    /**Stwierdzenie czy moze byc uruchomiony trigger dla tego obiektu.
     * Metode nalezy pokryc.
     * @param Activator obiekt, ktory dobiera sie do triggered object
     */
    public boolean IsActive(ControlObject Activator)
    {
        return bActiveTrigger;
    }

    /**Stwierdzenie czy moze byc uruchomiony trigger dla tego obiektu. 
     * Metode nalezy pokryc. Wprowadzona w celu rozrozniania aktywnosci poszczegolnych
     * elemtow aktywnych.
     * @param Activator obiekt, ktory dobiera sie do triggered object
     * @param nCheckingElement element dla ktorego ustalana jest aktywnosc obiektu
     */
    public boolean IsActive(ControlObject Activator, int nCheckingElement)
    {
        return true;
    }
    
    /** Zwraca true gdy obiekt moze byc aktywowany. Sprawdza czy trafiony zostal element
     * aktywny.
     * @param nHitElement meshElement ktory zostal trafiony kolizja podczas sprawdzania aktywowania
     */
    boolean CanActivate(int nHitElement)
    {
        if(aActiveElements != null)
        {
            // obiekt ma aktywne elementy, trzeba sprawdzic ktory sie aktywuje
            for(int i=0; i<aActiveElements.size(); i++)
            {
                int nElementID = ((Integer)aActiveElements.elementAt(i)).intValue();
                //Log("Sprawdzam element <"+nElementID+"> z wartoscia <"+nHitElement+">\n");
                if( nElementID == nHitElement )
                {
                    nActiveElement = nElementID;
                    return true;
                }
            }
            
            // wymuszamy podswietlenie pierwszego aktywnego elementu
            if( bForceWholeObjectCollision )
            {
                nActiveElement = ((Integer)aActiveElements.elementAt(0)).intValue();
                return true;
            }
            else
                return false;
        }

        nActiveElement = -1;
        // nie ma obiekt aktywnych elementow to oznacza ze caly jest aktywny
        return true;
    }

    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[TriggerObject.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunks._CHK_TRIGGER_OBJECT)
        {
			super.SGLoadChunk(cFC);
			
			// Nazwa elementu ktory ma byc aktywny. Na jej podstawie jest ustalany numerek.
			ActiveElement = cFC.LoadString();
            nActiveElement = cFC.LoadInt();
            bActiveTrigger = cFC.LoadBool();
			bForceWholeObjectCollision = cFC.LoadBool();
			
			// aActiveElements = GameObject.cSG.LoadObjectsVectorRef(cFC);
            
            nExecutorRole = cFC.LoadInt();
            int arraySize = cFC.LoadInt();
            if (arraySize < 0)
                aExecutionSteps = null;
            else
            {
                aExecutionSteps = new ExecutionInfo[arraySize];
                for (int i = 0; i < arraySize; i++)
                    aExecutionSteps[i] = (ExecutionInfo)cFC.LoadObjectRef();
            }
        }
		
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[TriggerObject.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunks._CHK_TRIGGER_OBJECT, cFCParent);
		
		super.SGSaveChunk(cFC);

		// Nazwa elementu ktory ma byc aktywny. Na jej podstawie jest ustalany numerek.
		cFC.SaveString( ActiveElement );		
        cFC.SaveInt( nActiveElement );
        cFC.SaveBool( bActiveTrigger );	
		cFC.SaveBool( bForceWholeObjectCollision );
		
		// GameObject.cSG.SaveObjectsVectorRef(cFC,aActiveElements);

        cFC.SaveInt(nExecutorRole);
        if (aExecutionSteps != null)
        {
            cFC.SaveInt(aExecutionSteps.length);
            for (int i = 0; i < aExecutionSteps.length; i++)
                cFC.SaveObjectRef(aExecutionSteps[i]);
        }
        else
            cFC.SaveInt(-1);
			
		cFC.delete();
	}
        
    public void Enable()
    {
        bActiveTrigger = true;
    }
    
    public void Disable()
    {
        bActiveTrigger = false;
    }
    
    public boolean IsEnabled()
    {
        return bActiveTrigger;
    }
    
    ////////////////
    // SIEC
    ////////////////
    
    // tu bedziemy przechowywac przychodzace dane
    Actor m_cActionExecutor   = null;
    int   m_nActionExecutorId = -1;
    float m_fExecuteTime      = -1.0f;
    
    /** Funkcja uzywana po odebraniu komunikatu o zmiania stanu obiektu 
     * Sprawdza czy podany fTime jest "nowszy" niz m_fExecuteTime, czyli
     * jesli tak, to m_fExecuteTime przyjmuje nowa wartosc i zwraca true
     * co oznacza ze mozna komunikat zinterpretowac i "zawolac" zmiane stanu
     * jesli funkcja zwraca false, to oznacza ze przyszedl komunikat "z przeszlosci"
     * i trzeba go zignorowa
     * w DisableAllWaitingMethod(); 
     */
    boolean CheckExecuteTime( float fTime )
    {
        // HACK: tymczasowy: FIXIT
        if ( m_fExecuteTime >= fTime )
        {
            LogD( " HACK: FALSE(m_fExecuteTime "+m_fExecuteTime+",fTime "+fTime+") -> TRUE\n");
            m_fExecuteTime = fTime - 0.01f;
        }
            
        if ( m_fExecuteTime < fTime )
        {
             m_fExecuteTime = fTime;
             DisableAllWaitingMethod();
             return true;
        }
        
        return false;
    }
    
    /**
     * tu trzeba wpisac zablokowanie wszystkich mozliwych metod, 
     * ktore "wisza" (moga wisiec) podczas przelaczania poprzedniego stanu 
     * (przelaczanie stanu moglo sie jeszcze nie skonczyc, wiec trzeba to 
     * przerwac)
     */
    void DisableAllWaitingMethod()
    {
    }
    
    /** Proba uzycia obiektu
     * @param executor Actor wykonujacy akcje (<code>Actor</code>)
     */
    void ActionExecute( Actor executor )
    {
        // zapamietajmy sobie Executora, zeby wiedziec co wywolac w ExecuteDo()
        m_cActionExecutor   = executor;
        m_nActionExecutorId = (executor!=null) ? executor.GetNetId() : -1;
        
        if (Role == ERole._AUTHORITY)
        {
            //TryToExecute();
            ExecuteDo();
            m_fExecuteTime = GetTime();
            MsgSendClientState();
        }
        else
        {
            MsgSendServerExecuteRequest();
        }
    }
    
    /**
     * Weryfikacja mozliwosci wykonania akcji na obiekcie
     */
    boolean ServerVerifyActionExecute( Actor cExecutor, float fMaxDistance )
    {
        //sprawdzmy czy executor nie jest za daleko
        Vector vPos = GetPositionVectorVolatile();
        return (vPos.Distance( cExecutor.GetPosition() ) <= fMaxDistance);
    }
        
    void ServerExecuteRequestFromClient()
    {
        if ( CanExecuteRequestFromClient() )
        {
            ExecuteDo();
            // dla niektorych triggerow bedzie mozna wybrac SendClientExecute();
            MsgSendClientState();
        }
    }
    
    boolean CanExecuteRequestFromClient()
    {
        if ( bDeleted )
        {
            CrashLog("ERROR: "+this+" deleted! ExecuteRequest cancelled\n");
            return false;
        }
                        
        m_cActionExecutor = (m_nActionExecutorId == -1)?
                             null : 
                             GetActorByNetId( m_nActionExecutorId );
        //TODO: czy executor moze byc null?
        if ( m_cActionExecutor != null )
        {
             if ( ! m_cActionExecutor.ServerVerifyActionExecute( this ) )
             {
                 LogR(""+m_cActionExecutor+" cant execute "+this+" ExecuteRequest cancelled\n");
                 return false;
             }
        }
        return true;
    }
    
    // metoda wolajaca prawdziwe Execute() z paremetrami globalnymi (odczytanymi w ActionExecute)
    // jezeli gdzies ma byc wykonane Execute z parametrami to trzeba to przeciazyc
    void ExecuteDo()
    {
        Execute( m_cActionExecutor );
    }
    
    // metoda wolajaca ExecuteDo() 
    // wczesniej sprawdza czy wszystkie potrzebne obiekty (w tym przypadku Executor) 
    // sa obecne (co ma znaczenie na kliecie)
    void TryToExecute()
    {
        if ( m_nActionExecutorId == -1 )
        {
            m_cActionExecutor = null;
        }
        else
        {
            m_cActionExecutor = GetActorByNetId( m_nActionExecutorId );
            if ( m_cActionExecutor == null )
            {
                CallMethodIn( "TryToExecute", 0.1f );
                return;
            }
        }
        DisableCallMethod( "TryToExecute" );
        ExecuteDo();
    }
    
    public void ReplicationInit()
    {
    }
    
    public float GetRelevantLevel()
    {
        return 1f;
    }
   
    public void OnReplicateServer()
    {
        if ( bNetInit )
        {
            // podczas inicjalizacji, zawsze musimy pochwalic
            // sie swoim aktualnym stanem
		    MsgSendClientState();
        }
    }
    
    
    final int MSG_SERVER_EXECUTE_REQUEST = 0; //klient wysyla do serwera requesta o Execute
    final int MSG_CLIENT_EXECUTE         = 1; //serwer kaze klientowi wykonac akcje    
    final int MSG_CLIENT_STATE           = 2; //serwer wysyla klientowi stan obiektu
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case MSG_SERVER_EXECUTE_REQUEST:    
            LogD(" MSG_SERVER_EXECUTE_REQUEST\n");
            MsgRecvServerExecuteRequest();
            break;
            
        case MSG_CLIENT_EXECUTE:
            LogD(" MSG_CLIENT_EXECUTE\n");
            MsgRecvClientExecute();
            break;
            
        case MSG_CLIENT_STATE:
            LogD(" MSG_CLIENT_STATE\n");
            MsgRecvClientState();
            break;
            
        default:
            MessageCancel();
        }
    }
    
    // kazdy obiekt powinien zadbac sam o to, aby umiec wyslac swoj stan
    void MsgSendClientState()
    {
        //CrashLog("ERROR: "+this+" MsgSendClientState nie pokryta!\n");
    }
    void MsgRecvClientState()
    {
        //CrashLog("ERROR: "+this+" MsgRecvClientState nie pokryta!\n");
    }

    // zapisanie do komunikatu czasu wykonania ostatniej akcji
    void PackExecuteTime()
    {
        MessagePackFloat( m_fExecuteTime );
    }
    
    // odczytanie z komunikatu czasu wykonania ostatniej akcji
    float UnPackExecuteTime()
    {
        float fTime = MessageUnPackFloat();
        return fTime;
    }
    
    // wyslanie z klienta na serwera requesta wykonania akci
    void MsgSendServerExecuteRequest()
    {
        MessageNew( MSG_SERVER_EXECUTE_REQUEST );
        MessagePackData();
        MessageSend();
    }
    
    // odebranie komunikatu o requescie i wykonanie (proba) executa
    void MsgRecvServerExecuteRequest()
    {
        MessageUnPackData();
        MessageEnd();
        
        ServerExecuteRequestFromClient();
    }
    
    // wyslanie klientowi infa o tym, aby wykonal exekuta (chyba nie bedziemy
    // z tego korzystac, bowiem serwer klientowi bedzie replikowac
    // stan obiektu
    void MsgSendClientExecute()
    {
        MessageNew( MSG_CLIENT_EXECUTE );
        MessagePackData();
        MessageSend();
    }
    
    // odebranie na kliencie wymuszenia executa
    void MsgRecvClientExecute()
    {
        MessageUnPackData();
        MessageEnd();
        
        TryToExecute();
    }
    
    // wyslanie danych
    void MessagePackData()
    {
        MessagePackInt( m_cActionExecutor.GetNetId() );
    }
    
    // odczyt danych
    void MessageUnPackData()
    {
        m_nActionExecutorId    = MessageUnPackInt();
    }
    
    
    Actor GetActorByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            Actor actor = (Actor) object;
            return actor;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from Actor\n");
        }
        
        return null;
    }
    
    /**
     * Na potrzeby sieci, zwraca pozycje obiektu z ktora jest
     * aktualnie renderowany
     * W grze singlowej zwraca poprostu pozycje
     */
    
    /**
     * Na potrzeby sieci, zwraca pozycje obiektu z ktora jest
     * aktualnie renderowany
     * W grze singlowej zwraca aktualne wektory
     */
    public void GetRenderPosition( Vector vPos, Vector vDir, Vector vRight )
    {
        if ( vPos != null )
            GetPosition( vPos );
        if ( vDir != null )
            GetForwardVector( vDir );
        if ( vRight != null )
            GetRightVector( vRight );
    }
}

class ExecutionInfo implements ISaveGame
{
    public static final float _TIME_NOT_FORCED = -1f;
    
    public ExecutionInfo()
    {
        //SG
    }
    
    public ExecutionInfo(int executionMode, int internalMode)
    {
        this(executionMode, internalMode,
             null, null, 
             _TIME_NOT_FORCED, 0);
    
    }

    public ExecutionInfo(int executionMode, int internalMode, 
                         Vector destPos, Vector destDir)
    {
        this(executionMode, internalMode,
             destPos, destDir, 
             _TIME_NOT_FORCED, 0);
    }

    public ExecutionInfo(int executionMode, int internalMode, 
                         Vector destPos, Vector destDir,
                         float forcedExecTime, float delayTime)
    {
        iExecutionMode = executionMode;
        iInternalExecutionMode = internalMode; 
        vDestPos = destPos; 
        vDestDir = destDir; 
        fForcedExecTime = forcedExecTime; 
        fDelayTime = delayTime; 
    }
    
    // sposb uywania (najczciej rodzaj animacji) - wartosc z EnumExecuteMode
    int iExecutionMode = EnumExecuteMode._GENERIC; 
    // identyfikator akcji do wykonania przez TriggerObject (do odpalania animacji lub pomijania niepotrzebnych akcji)
    int iInternalExecutionMode = 0; 
    // miejsce, do ktrego powinien przesun si Executor. Jeeli null, ta akcja nie wymaga przesunicia
    Vector vDestPos = null; 
    // kierunek, w jakim powinien ustawi si Executor, jeeli null, akcja nie wymaga obrotu
    Vector vDestDir = null; 
    // Wymuszony czas wykonania akcji przez TriggerObject. Jeeli < 0 to czas wykonania zaley tylko od Executora.
    float fForcedExecTime = _TIME_NOT_FORCED; 
    // Dodatkowy czas (wait) po wykonaniu akcji.
    float fDelayTime = 0; 
    // Wewnetrzna referencja na nastepne ExecutionInfo w ciagu akcji dla TriggerObjecta
    ExecutionInfo cNext = null; 
    
    
    //*********************************************************
    //                       SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {
        FileChunk.Log("[ExecutionInfo.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunks._CHK_EXECUTION_INFO)
        {
            iExecutionMode = cFC.LoadInt();
            iInternalExecutionMode = cFC.LoadInt();
            vDestPos = cFC.LoadVector();
            vDestDir = cFC.LoadVector();
            fForcedExecTime = cFC.LoadFloat();
            fDelayTime = cFC.LoadFloat(); 
            cNext = (ExecutionInfo)cFC.LoadObjectRef();
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[ExecutionInfo.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunks._CHK_EXECUTION_INFO, cFCParent);

        cFC.SaveInt(iExecutionMode);
        cFC.SaveInt(iInternalExecutionMode);
        cFC.SaveVector(vDestPos);
        cFC.SaveVector(vDestDir);
        cFC.SaveFloat(fForcedExecTime);
        cFC.SaveFloat(fDelayTime); 
        cFC.SaveObjectRef(cNext);

        cFC.delete(); 
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[ExecutionInfo.SGUpdate]\n");
    }
   
}
